(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



BeginPackage["StockPlots`"];

Unprotect["`*"]; 
ClearAll["`*"]; 


getFinancialData::usage="GetFinancialData[name, {property1,property2...},iterator] will generate a data set where each record will contain {datelist, propertyvalues..}. The iterator is the standard {start,end, period} used in FinancialData. This is basically a method of retrieving a number of property values at once.";

PricePlot::usage="PricePlot[data, {label, label color}] will generate a plot of closing prices. The label is the label to appear in the plot, typically the name of the stock. Options are the same as DateListPlot.";

OHLCPlot::usage="OHLCPlot[data, {label, label color}] will generate an OHLC (Open High Low Close) plot of data, which must have the form {{date, open, high, low, close, volume}..}. The label is the label to appear in the plot, typically the name of the stock. Options are the same as DateListPlot.";

CandlestickPlot::usage="CandlestickPlot[data, {label, label color}] will generate a candlestick plot of data which must have the form {{date, open, high, low, close, volume}..}. The label is the label to appear in the pot, typically the name of the stock. Options are the same as DateListPlot.";

VolumePlot::usage="VolumePlot[data] will generate a plot of trading volume. The data is a list of stock data with the form {dates, open, high, low, close, volume}";

MACDPlot::usage="MACDPlot[data] will generate an MACD plot of the stock data. The data is a list of stock data with the form {dates, open, high, low, close, volume}.";

RSIPlot::usage="RSIPlot[data, num, range] will generate an RSI plot of the data. The data is a list of stock data with the form {dates, open, high, low, close, volume}, num is the number of days to use for an exponential moving average, range is the range between 0 and 100 to highlight on the plot, the default value is {20, 80}.";

ADIPlot::usage="ADIPlot[data, range] will generate an ADI plot. The data is a list of stock data with the form {dates, open, high, low, close, volume}, range is an option for PlotRange, the default value is Automatic.";

BollingerData::usage="Generates Bollinger bands from data which can be overlayed onto stock price plots. The data is a list of stock data with the form {dates, open, high, low, close, volume}.";

StockPlot::usage="StockPlot[data, {label, label color}] will generate a plot of price with a plot of violume underneath. The data is a list of stock data with the form {dates, open, high, low, close, volume}. PlotType possibilities are \"Price\", \"OHLC\", and \"Candlestick\". Options specific for the price and volume plots can be entered as options for PriceStyling and VolumeStyling respectively.";

makeLabel;

plotLabel;

Bollinger;

PlotType;

PriceStyling;

VolumeStyling;

Technicals;

BollingerBands;


Begin["`Private`"];


Clear[getFinancialData];

SetAttributes[getFinancialData, ReadProtected];

(* getFinancialData was written by David Park and posted on Mathgroup *)

SyntaxInformation[getFinancialData]={"ArgumentsPattern"->{_,{_,__},_}};

getFinancialData[name_String,{first_String,rest__String},iterator_List]:=Module[{work,other},
work=Transpose@FinancialData[name,first,iterator];
other=FinancialData[name,#,iterator,"Value"]&/@{rest};
Transpose[Join[work,other]]
];


Clear[plotLabel];

(* label to appear in top left hand corner of small plots *)
plotLabel[label_String]:=Inset[
Panel[
Grid[{{label}},Alignment-> Left],Appearance-> None,Background->White],Offset[{0,0},Scaled[{0.05,.95}]],{Left,Top}];

plotLabel[label_String,args___]:=Inset[
Panel[
Grid[{{Style[label,args]}},Alignment-> Left],Appearance-> None,Background->White],Offset[{0,0},Scaled[{0.05,.95}]],{Left,Top}];


Clear[makeLabel];

(* volume label *)
makeLabel/:MakeBoxes[makeLabel[x_,y_],StandardForm]:=RowBox[{SuperscriptBox[MakeBoxes[x,StandardForm],MakeBoxes[y,StandardForm]]}];


Clear[PricePlot];

SetAttributes[PricePlot, ReadProtected];

SyntaxInformation[PricePlot]={"ArgumentsPattern"->{_,{_,_},OptionsPattern[]}};

Options[PricePlot]=Join[Options[DateListPlot],{Bollinger->"False"}];

SetOptions[PricePlot,
AspectRatio->1/GoldenRatio,
Frame->True,
FrameTicks->Automatic,
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",11,Plain],
FrameTicks->{Automatic,Automatic,Automatic,Automatic},
GridLinesStyle->{Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]],Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]]},
GridLines->Automatic,
ImageSize->600,
ImagePadding->{{50,10},{20,2}},
Joined->True,
PlotStyle->{Black,Red,ColorData["HTML"]["DarkGreen"],Blue},
(* x range padding is large due to dates being in AbsoluteTime *)
PlotRangePadding-> {{200000,200000},{0,5}}
];

PricePlot[data:{{_List,_,_,_,_,_}..},{label_String,(color_RGBColor|color_GrayLevel|color_Hue|color_CMYKColor)},opts:OptionsPattern[]]:=Module[{close,min,max,bollinger,opts1,opts2},

opts1=FilterRules[{opts},Options[DateListPlot]];
opts2=FilterRules[{Options[PricePlot]},Options[DateListPlot]];

close=data[[All,{1,5}]];
min=Min[close[[All,2]]];
max=Max[close[[All,2]]];

Which[
OptionValue[Bollinger]=="False",
DateListPlot[close,
Epilog->plotLabel[label, 14,color],
PlotRange->{All,{min,max}},
opts1,
opts2
],
OptionValue[Bollinger]=="True",
bollinger=BollingerData[data,20];
DateListPlot[{close,bollinger[[All,{1,2}]],bollinger[[All,{1,3}]],bollinger[[All,{1,4}]]},
Epilog->plotLabel[label, 14,color],
opts1,
PlotRange->{All,{Min[min,bollinger[[All,2]]],Max[max,bollinger[[All,4]]]}},
opts2
]
]
]


Clear[OHLCPlot];

SetAttributes[OHLCPlot, ReadProtected];

SyntaxInformation[OHLCPlot]={"ArgumentsPattern"->{_,{_,_},OptionsPattern[]}};

Options[OHLCPlot]=Join[Options[DateListPlot],{Bollinger->"False"}];

SetOptions[OHLCPlot,
AspectRatio->1/GoldenRatio,
Frame->True,
FrameTicks->Automatic,
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",11,Plain],
FrameTicks->{Automatic,Automatic,Automatic,Automatic},
GridLinesStyle->{Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]],Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]]},
GridLines->Automatic,
ImageSize->600,
ImagePadding->{{50,10},{20,2}},
PlotStyle->Black,
PlotRange->All,
(* x range padding is large due to dates being in AbsoluteTime *)
PlotRangePadding-> {200000,5}
];

OHLCPlot[data:{{_List,_,_,_,_,_}..},{label_String,(color_RGBColor|color_GrayLevel|color_Hue|color_CMYKColor)},opts:OptionsPattern[]]:=Module[{opts1,opts2,dates,open,high,low,close,volume,min,max,optsPS=OptionValue[PlotStyle],bollinger},

opts1=FilterRules[{opts},Options[DateListPlot]];
opts2=FilterRules[{Options[OHLCPlot]},Options[DateListPlot]];
optsPS=Cases[{optsPS},_RGBColor|__GrayLevel|__Hue|__CMYKColor,\[Infinity]];

{dates,open,high,low,close,volume}=Transpose[data];
max=Max[high];
min=Min[low];

(* add back dates *)
{open,high,low,close}=Transpose[{dates,#}]&/@{open,high,low,close};

Which[
OptionValue[Bollinger]=="False",
DateListPlot[{open,close,high,low},
Epilog->plotLabel[label, 14,color],
Filling->{3->{{4},{Directive[optsPS[[1]],AbsoluteThickness[1]]}}},
Joined->False,
PlotMarkers->{Style["\[SkeletonIndicator] ",{optsPS}],Style[" \[SkeletonIndicator]",optsPS],"","","",""},
opts1,
PlotRange->{All,{min,max}},
opts2
],
OptionValue[Bollinger]=="True",
bollinger=BollingerData[data,20];
DateListPlot[{open,close,high,low,bollinger[[All,{1,2}]],bollinger[[All,{1,3}]],bollinger[[All,{1,4}]]},
Epilog->plotLabel[label, 14,color],
Filling->{3->{{4},{Directive[optsPS[[1]],AbsoluteThickness[1]]}}},
Joined->{False,False,False,False,True,True,True},
PlotMarkers->{Style["\[SkeletonIndicator] ",{optsPS}],Style[" \[SkeletonIndicator]",optsPS],"","","","","","",""},
opts1,
PlotRange->{All,{Min[min,bollinger[[All,2]]],Max[max,bollinger[[All,4]]]}},
opts2
]
]
]


Clear[CandlestickPlot];

SetAttributes[CandlestickPlot, ReadProtected];

SyntaxInformation[CandlestickPlot]={"ArgumentsPattern"->{_,_,OptionsPattern[]}};

Options[CandlestickPlot]=Join[Options[DateListPlot],{Bollinger->"False"}];

SetOptions[CandlestickPlot,
AspectRatio->1/GoldenRatio,
Frame->True,
FrameTicks->Automatic,
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",11,Plain],
FrameTicks->{Automatic,Automatic,Automatic,Automatic},
GridLinesStyle->{Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]],Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]]},
GridLines->Automatic,
ImageSize->600,
ImagePadding->{{50,10},{20,2}},
PlotRange->All,
(* x range padding is large due to dates being in AbsoluteTime *)
PlotRangePadding-> {200000,5}
];

CandlestickPlot[data:{{_List,_,_,_,_,_}..},{label_String,(color_RGBColor|color_GrayLevel|color_Hue|color_CMYKColor)},opts:OptionsPattern[]]:=Module[{opts1,opts2,dates,open,high,low,close,volume,max,min,openFall,closeFall,openRise, closeRise,bollinger},

opts1=FilterRules[{opts},Options[DateListPlot]];
opts2=FilterRules[{Options[OHLCPlot]},Options[DateListPlot]];

{dates,open,high,low,close,volume}=Transpose[data];
max=Max[high];
min=Min[low];

(* add back dates *)
{open,high,low,close}=Transpose[{dates,#}]&/@{open,high,low,close};

(* create lists: Open on falling day, Close on Falling day, Open in rising day, Close on rising day *)

{openFall,closeFall,openRise, closeRise}=Map[If[#[[2]]>=#[[5]],{{#[[1]],#[[2]]},{#[[1]],#[[5]]},{#[[1]],Null},{#[[1]],Null}},{{#[[1]],Null},{#[[1]],Null},{#[[1]],#[[2]]},{#[[1]],#[[5]]}}]&,data]//Transpose;



Which[
OptionValue[Bollinger]=="False",
DateListPlot[{high,low,openFall,closeFall,openRise, closeRise},
Epilog->plotLabel[label, 14,color],
Filling->{
1->{{2},Directive[Black,AbsoluteThickness[1]]},
3->{{4},Directive[Red,AbsoluteThickness[2.5]]},
5->{{6},Directive[Darker[Green,0.2],AbsoluteThickness[2.5]]}},
Joined->False,
PlotMarkers->{"","","","","","",""},
opts1,
PlotRange->{All,{min,max}},
opts2
],
OptionValue[Bollinger]=="True",

bollinger=BollingerData[data,20];

DateListPlot[{high,low,openFall,closeFall,openRise,closeRise,bollinger[[All,{1,2}]],bollinger[[All,{1,3}]],bollinger[[All,{1,4}]]},
Epilog->plotLabel[label, 14,color],
Filling->{
1->{{2},Directive[Black,AbsoluteThickness[1]]},
3->{{4},Directive[Red,AbsoluteThickness[2.5]]},
5->{{6},Directive[Darker[Green,0.2],AbsoluteThickness[2.5]]}},
Joined->{False,False,False,False,False,False,True,True,True},
PlotMarkers->{"","","","","","","","",""},
opts1,
PlotRange->{All,{Min[min,bollinger[[All,2]]],Max[max,bollinger[[All,4]]]}},
opts2
]
]
]


Clear[VolumePlot];

SetAttributes[VolumePlot, ReadProtected];

SyntaxInformation[VolumePlot]={"ArgumentsPattern"->{_,OptionsPattern[]}};

Options[VolumePlot]=Options[DateListPlot];

SetOptions[VolumePlot,
AspectRatio-> 1/(4*GoldenRatio),
FillingStyle->{1->{{Thick,Darker[Red,0.25]}},2->{{Thick,ColorData["HTML"]["Green"]}}},
FrameTicks->Automatic,
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",11,Plain],
GridLines->Automatic,
GridLinesStyle->Directive[ GrayLevel[0.9],Dashing[{Small, Small}]],
ImageSize->600,
ImagePadding->{{50,10},{20,4}},
Joined->False,
PlotMarkers->{"",""},
(* x range padding is large due to dates being in AbsoluteTime *)
PlotRangePadding-> {{200000,200000},{0,1}}
];

VolumePlot[data:{{_List,_,_,_,_,_}..},opts:OptionsPattern[]]:=Module[{dates,open,high,low,close,volume,maxVolume,volume1,temp,exponent,maxVolumeScale,volumeFall,volumeRise,label},

{dates,open,high,low,close,volume}=Transpose[data];

(*volumes*)
maxVolume=Max[volume];

(* get exponent for volume label *)
exponent=IntegerLength[maxVolume]-1;
maxVolumeScale=1.1*(maxVolume*10^-exponent);
volume1=Transpose[{dates,volume*10^-exponent}];

(* create list of rescaled prices *)
temp=Transpose[{open,close,volume1[[All,2]]}];

(* create lists: volume on falling day, volume on rising day. Needed so colours can differ on winning and losing days *)
{volumeFall,volumeRise}=Transpose@Map[If[#[[1]]>=#[[2]],{#[[3]],Null},{Null,#[[3]]}]&,temp];

volumeFall=Transpose[{dates,volumeFall}];
volumeRise=Transpose[{dates,volumeRise}];

label="Vol (\[Times] "<>ToString[makeLabel[10,-exponent],StandardForm]<>")";

DateListPlot[{volumeFall,volumeRise},
Epilog->plotLabel[label,Bold,Black],
Filling->{1->0,2->0},
PlotRange->All,
opts,
Options[VolumePlot]
]
]



Clear[MACDPlot];

SetAttributes[MACDPlot, ReadProtected];

SyntaxInformation[MACDPlot]={"ArgumentsPattern"->{_}};

Options[MACDPlot]=Options[DateListPlot];

SetOptions[MACDPlot,
AspectRatio-> 1/(4*GoldenRatio),
Axes-> True,
Frame-> True,
FrameTicks->{Automatic,None,Automatic,None},
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",9,Plain],
GridLines->{Automatic,None},
GridLinesStyle->Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]],
ImageSize-> 600,
ImagePadding->{{50,10},{1,10}},
PlotRangePadding-> {200000,1}
];

MACDPlot[data:{{_List,_,_,_,_,_}..}]:=Module[{macd, signal, hist,tmp1=data[[All,1]],tmp2=data[[All,5]]},

(* using 12 and 26 day moving average -> 2/13 and 2/27 *)
macd=Transpose[{tmp1,ExponentialMovingAverage[tmp2,2./13.]-ExponentialMovingAverage[tmp2,2./27.]}];
(* using 9 day moving average of MACD -> 2/10 *)
signal=Transpose[{tmp1,ExponentialMovingAverage[macd[[All,2]],2./10.]}];
hist=Transpose[{tmp1,macd[[All,2]]-signal[[All,2]]}];

DateListPlot[{macd,signal,hist},
Filling->{3->{0,{{AbsoluteThickness[1.],Black}}}},
Epilog->plotLabel["MACD",Bold,ColorData["HTML"]["DarkRed"]],
Joined->{True,True,False},
PlotMarkers->{"","","",""},
PlotStyle->{{AbsoluteThickness[1.],Blue},{AbsoluteThickness[1.],Red},Black},
Options[MACDPlot]
]
]


Clear[RSIPlot];

SetAttributes[RSIPlot, ReadProtected];

SyntaxInformation[RSIPlot]={"ArgumentsPattern"->{_,_,{_,_}}};

Options[RSIPlot]=Options[DateListPlot];

SetOptions[RSIPlot,
AspectRatio->  1/(4*GoldenRatio),
Frame-> True,
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",9,Plain],
GridLinesStyle->{Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[.5]],Directive[GrayLevel[0.45],Dashed,AbsoluteThickness[0.5]]},
ImageSize-> 600,
ImagePadding->{{50,10},{1,1}},
PlotRangePadding-> {200000,0},
PlotStyle->{RGBColor[0.6,0.24,0.4429],AbsoluteThickness[1.5]}
];

RSIPlot[data:{{_List,_,_,_,_,_}..},num_Integer,range_:{20,80}]:=Module[{tmp=data[[All,5]],up,down,emaUp,emaDown,tmp2,num1=2./(num+1.),tmp3=range},

(* todays close minus yesterdays close *)
tmp=ListCorrelate[{-1,1},tmp];

(* U and D*)
up=tmp/.x_?Negative-> 0;
down=-(tmp/.x_?Positive-> 0);

(* EMA of U *)
emaUp=ExponentialMovingAverage[up,num1];
(* EMA of D *)
emaDown=ExponentialMovingAverage[down,num1];

tmp=MapThread[(100.*#1/(#1+Abs[#2])&),{emaUp,emaDown}];

DateListPlot[Transpose[{Drop[data[[All,1]],num-1],Drop[tmp,num-2]}],
Axes-> True,
AxesOrigin->{Automatic,50},
AxesStyle->GrayLevel[0.5],
Epilog->plotLabel["RSI",Bold,ColorData["HTML"]["DarkRed"]],
FrameTicks->{{{tmp3[[1]],50,tmp3[[2]]},None},{None,None}},
GridLines->{Automatic,{tmp3[[1]],tmp3[[2]]}},
Joined->{True,True,False},
PlotRange->{{data[[1,1]],data[[-1,1]]},{0,100}},
Options[RSIPlot]
]
]


Clear[ADIPlot];

SetAttributes[ADIPlot, ReadProtected];

SyntaxInformation[ADIPlot]={"ArgumentsPattern"->{_,___,OptionsPattern[]}};

Options[ADIPlot]=Options[DateListPlot];

SetOptions[ADIPlot,
AspectRatio-> 1/(4*GoldenRatio),
FrameTicks->{Automatic,None,Automatic,None},
FrameTicksStyle->Directive[Black,FontFamily-> "Helvetica",11,Plain],
GridLines->{Automatic,None},
GridLinesStyle->Directive[GrayLevel[0.85],Dashed,AbsoluteThickness[0.5]],
ImageSize-> 600,
ImagePadding->{{50,10},{15,1}},
Joined->True,
PlotRangePadding->{200000,0},
PlotStyle->{RGBColor[0.24,0.6,0.34],AbsoluteThickness[1.5]}
];

ADIPlot[data_,range_:Automatic,opts:OptionsPattern[]]:=Module[{dates,open,high,low,close,volume,CLV,index},

{dates,open,high,low,close,volume}=Transpose[data];

(* calculate CLV *)
CLV=(2*close-low-high)/(high-low);

(* calculate accumulated distance *)
index=Rest[FoldList[Plus,0,CLV*volume]];

DateListPlot[Transpose[{dates,index}],
Epilog->plotLabel["ADI",Bold,ColorData["HTML"]["DarkRed"]],
PlotRange->range,
opts,
Options[ADIPlot]
]
];


Clear[BollingerBands,BollingerData];

SetAttributes[BollingerData, ReadProtected];

SyntaxInformation[BollingerData]={"ArgumentsPattern"->{_,_}};

BollingerBands[ls_List]:=Module[{tmp1,tmp2},
tmp1=Mean[ls];
tmp2=2.*StandardDeviation[ls];
{tmp2+tmp1,tmp1,tmp1-tmp2}
];

BollingerData[data_List,num_Integer]:=Module[{newDates,upperBB,middleBB,lowerBB, temp},

newDates=Drop[data[[All,{1,5}]],num-1];

temp=Partition[data[[All,5]],num,1];
{upperBB,middleBB,lowerBB}=Transpose[BollingerBands/@temp];

(* {date, closing price, percentB, lowerBB, middleBB, upperBB, bandwidth} *)

MapThread[{#1[[1]],#2,#4,#3,(#1[[2]]-#2)/(#3-#2),(#3-#2)/#4}&,{newDates,lowerBB,upperBB,middleBB}]

];


Clear[StockPlot];

SetAttributes[StockPlot, ReadProtected];

SyntaxInformation[StockPlot]={"ArgumentsPattern"->{_,{_,_},OptionsPattern[]}};

Options[StockPlot]={PlotType->"Price",PriceStyling-> {},VolumeStyling-> {},Technicals->"False",Bollinger->"False"};

StockPlot[data:{{_List,_,_,_,_,_}..},{label_String,(color_RGBColor|color_GrayLevel|color_Hue|color_CMYKColor)},opts:OptionsPattern[]]:=Module[{p1,p2,p3,p4,p5},

Switch[OptionValue[PlotType],
"Price",p1=PricePlot[data,{label,color},ImagePadding->{{50,10},{1,2}},Bollinger->OptionValue[Bollinger],OptionValue[PriceStyling]];p2=VolumePlot[data,OptionValue[VolumeStyling]];,
"OHLC",p1=OHLCPlot[data,{label,color},ImagePadding->{{50,10},{1,2}},Bollinger->OptionValue[Bollinger],OptionValue[PriceStyling]];p2=VolumePlot[data,OptionValue[VolumeStyling]];,
"Candlestick",p1=CandlestickPlot[data,{label,color},ImagePadding->{{50,10},{1,2}},Bollinger->OptionValue[Bollinger],OptionValue[PriceStyling]];
p2=VolumePlot[data,OptionValue[VolumeStyling],FillingStyle->{{Black,Thick}}];
];

Which[ OptionValue[Technicals]==="False",
Graphics[
{
Inset[p1,ImageScaled[{.5,.61}],Automatic,Scaled[{1,1}]],
Inset[p2,ImageScaled[{.5,.11}],Automatic,Scaled[{1,1}]]
},
ImageSize-> 600,
PlotRange->{{0,1},{0,.74}}
],
OptionValue[Technicals]==="True",
p3=MACDPlot[data];
p4=RSIPlot[data,27,{20,80}];
p5=ADIPlot[data];
Graphics[
{
Inset[p1,ImageScaled[{.5,.765}],Automatic,Scaled[{1,1}]],
Inset[p2,ImageScaled[{.5,.465}],Automatic,Scaled[{1,1}]],
Inset[p3,ImageScaled[{.5,.3237}],Automatic,Scaled[{1,1}]],
Inset[p4,ImageScaled[{.5,.203}],Automatic,Scaled[{1,1}]],
Inset[p5,ImageScaled[{.5,.075}],Automatic,Scaled[{1,1}]]
},
ImageSize-> 600,
PlotRange->{{0,1},{0,1.25}}
]
]
]


Protect["`*"];

End[];

EndPackage[];
