(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    972519,      23822]
NotebookOptionsPosition[    965996,      23610]
NotebookOutlinePosition[    968005,      23671]
CellTagsIndexPosition[    967565,      23658]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["StockPlots examples", "Title"],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 CellChangeTimes->{{3.4627280329733963`*^9, 3.462728034827755*^9}}],

Cell[TextData[{
 "In order to make some \[OpenCurlyDoubleQuote]nice\[CloseCurlyDoubleQuote] \
stock plots I started with some of the ideas in this thread\[LongDash] ",
 ButtonBox["http://forums.wolfram.com/mathgroup/archive/2008/Nov/msg00456.\
html",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://forums.wolfram.com/mathgroup/archive/2008/Nov/msg00456.html"],
     None},
  ButtonNote->
   "http://forums.wolfram.com/mathgroup/archive/2008/Nov/msg00456.html"],
 " \[LongDash] and the function ",
 StyleBox["getFinancialData", "MB"],
 " written by David Park and went from there.\n\nThe functions in this \
package, ",
 StyleBox["ADIPlot", "MB"],
 ", ",
 StyleBox["CandlestickPlot", "MB"],
 ", ",
 StyleBox["getFinancialData", "MB"],
 ", ",
 StyleBox["MACDPlot", "MB"],
 ", ",
 StyleBox["OHLCPlot", "MB"],
 ", ",
 StyleBox["PricePlot", "MB"],
 ", ",
 StyleBox["RSIPlot", "MB"],
 ", and ",
 StyleBox["StockPlot", "MB"],
 ", and their usage, are shown in examples below."
}], "Text",
 CellChangeTimes->{{3.4626418131800327`*^9, 3.462641813634438*^9}, {
  3.462728067128889*^9, 3.462728075180912*^9}, {3.462728147607929*^9, 
  3.462728190779027*^9}, {3.4627282320083017`*^9, 3.462728407268589*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initializations", "Section",
 CellChangeTimes->{{3.462642924612701*^9, 3.462642928522793*^9}},
 CellID->2051879694],

Cell[BoxData[
 RowBox[{"Get", "[", "\"\<StockPlots`\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.462641250241763*^9, 3.462641256442813*^9}},
 CellID->1649613532]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic Examples", "Section",
 CellChangeTimes->{{3.462642936890996*^9, 3.4626429382822847`*^9}, {
  3.4627189124414988`*^9, 3.462718913339872*^9}}],

Cell["Get appropriately formatted data:", "Text",
 CellChangeTimes->{{3.46271948971524*^9, 3.462719495999708*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "getFinancialData"}]], "Input",
 CellChangeTimes->{{3.4627197974969683`*^9, 3.462719797743634*^9}, 
   3.462720026574115*^9, 3.462723115818244*^9}],

Cell[BoxData[
 StyleBox["\<\"GetFinancialData[name, {property1,property2...},iterator] will \
generate a data set where each record will contain {datelist, \
propertyvalues..}. The iterator is the standard {start,end, period} used in \
FinancialData. This is basically a method of retrieving a number of property \
values at once.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728505223374*^9},
 CellTags->"Info3462710504-4166664"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"goog1", "=", 
   RowBox[{"getFinancialData", "[", 
    RowBox[{"\"\<GOOG\>\"", ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<Open\>\"", ",", "\"\<High\>\"", ",", "\"\<Low\>\"", ",", 
       "\"\<Close\>\"", ",", "\"\<Volume\>\""}], "}"}], ",", 
     RowBox[{"DatePlus", "[", 
      RowBox[{"-", "360"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.4627194993070374`*^9}],

Cell[TextData[{
 "The ",
 StyleBox["StockPlot", "MB"],
 " comprises other plots that can be plotted as standalone plots."
}], "Text",
 CellChangeTimes->{{3.46271954166644*^9, 3.462719560897118*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PricePlot"}]], "Input",
 CellChangeTimes->{{3.46271961845997*^9, 3.462719620955353*^9}, {
   3.462719729528303*^9, 3.462719731277814*^9}, {3.462722665640987*^9, 
   3.462722667726878*^9}, 3.4627230438164263`*^9, 3.462723099112727*^9}],

Cell[BoxData[
 StyleBox["\<\"PricePlot[data, {label, label color}] will generate a plot of \
closing prices. The label is the label to appear in the plot, typically the \
name of the stock. Options are the same as DateListPlot.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728532030878*^9},
 CellTags->"Info3462710531-6531378"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PricePlot", "[", 
  RowBox[{"goog1", ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Red"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.462723618070805*^9, 3.4627236388537903`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJxVl3lwU1UUxp+iKIsatYqi0qDjWFwgUFR0RFNRijKOoYisjhnAAgUhqAUt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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{3.435*^9, 300.},
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"Google\"", 14, 
            RGBColor[1, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{Automatic, Automatic}, {{{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"Jan\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"Apr\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"Jul\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}, {{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3447532800, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3463344000, 
       GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4475328*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.463344*^9, 
       GrayLevel[0.8]}}, Automatic}],
  GridLinesStyle->{
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]], 
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]]},
  ImagePadding->{{50, 10}, {20, 2}},
  ImageSize->600,
  PlotRange->{All, {257.44, 499.06}},
  PlotRangeClipping->True,
  PlotRangePadding->{{200000, 200000}, {0, 5}},
  Ticks->None]], "Output",
 CellChangeTimes->{3.462723639630514*^9, 3.462728535149611*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "OHLCPlot"}]], "Input",
 CellChangeTimes->{{3.4627231299996843`*^9, 3.462723133288547*^9}}],

Cell[BoxData[
 StyleBox["\<\"OHLCPlot[data, {label, label color}] will generate an OHLC \
(Open High Low Close) plot of data, which must have the form {{date, open, \
high, low, close, volume}..}. The label is the label to appear in the plot, \
typically the name of the stock. Options are the same as DateListPlot.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728570376354*^9},
 CellTags->"Info3462710570-5772976"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OHLCPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", "100"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Red"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.462723679464281*^9, 3.462723703255252*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmX+QlVUZx68ihU6jW7NqYbJXq2ll0lmFMTORvUnuEhnLApK21aaCYg17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   
   "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
     {GrayLevel[0], AbsoluteThickness[1], LineBox[{201, 401}], 
      LineBox[{202, 402}], LineBox[{203, 403}], LineBox[{204, 404}], 
      LineBox[{205, 405}], LineBox[{206, 406}], LineBox[{207, 407}], 
      LineBox[{208, 408}], LineBox[{209, 409}], LineBox[{210, 410}], 
      LineBox[{211, 411}], LineBox[{212, 412}], LineBox[{213, 413}], 
      LineBox[{214, 414}], LineBox[{215, 415}], LineBox[{216, 416}], 
      LineBox[{217, 417}], LineBox[{218, 418}], LineBox[{219, 419}], 
      LineBox[{220, 420}], LineBox[{221, 421}], LineBox[{222, 422}], 
      LineBox[{223, 423}], LineBox[{224, 424}], LineBox[{225, 425}], 
      LineBox[{226, 426}], LineBox[{227, 427}], LineBox[{228, 428}], 
      LineBox[{229, 429}], LineBox[{230, 430}], LineBox[{231, 431}], 
      LineBox[{232, 432}], LineBox[{233, 433}], LineBox[{234, 434}], 
      LineBox[{235, 435}], LineBox[{236, 436}], LineBox[{237, 437}], 
      LineBox[{238, 438}], LineBox[{239, 439}], LineBox[{240, 440}], 
      LineBox[{241, 441}], LineBox[{242, 442}], LineBox[{243, 443}], 
      LineBox[{244, 444}], LineBox[{245, 445}], LineBox[{246, 446}], 
      LineBox[{247, 447}], LineBox[{248, 448}], LineBox[{249, 449}], 
      LineBox[{250, 450}], LineBox[{251, 451}], LineBox[{252, 452}], 
      LineBox[{253, 453}], LineBox[{254, 454}], LineBox[{255, 455}], 
      LineBox[{256, 456}], LineBox[{257, 457}], LineBox[{258, 458}], 
      LineBox[{259, 459}], LineBox[{260, 460}], LineBox[{261, 461}], 
      LineBox[{262, 462}], LineBox[{263, 463}], LineBox[{264, 464}], 
      LineBox[{265, 465}], LineBox[{266, 466}], LineBox[{267, 467}], 
      LineBox[{268, 468}], LineBox[{269, 469}], LineBox[{270, 470}], 
      LineBox[{271, 471}], LineBox[{272, 472}], LineBox[{273, 473}], 
      LineBox[{274, 474}], LineBox[{275, 475}], LineBox[{276, 476}], 
      LineBox[{277, 477}], LineBox[{278, 478}], LineBox[{279, 479}], 
      LineBox[{280, 480}], LineBox[{281, 481}], LineBox[{282, 482}], 
      LineBox[{283, 483}], LineBox[{284, 484}], LineBox[{285, 485}], 
      LineBox[{286, 486}], LineBox[{287, 487}], LineBox[{288, 488}], 
      LineBox[{289, 489}], LineBox[{290, 490}], LineBox[{291, 491}], 
      LineBox[{292, 492}], LineBox[{293, 493}], LineBox[{294, 494}], 
      LineBox[{295, 495}], LineBox[{296, 496}], LineBox[{297, 497}], 
      LineBox[{298, 498}], LineBox[{299, 499}], LineBox[{300, 500}]}}, {
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 501], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 502], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 503], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 504], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 505], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 506], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 507], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 508], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 509], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 510], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 511], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 512], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 513], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 514], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 515], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 516], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 517], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 518], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 519], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 520], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 521], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 522], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 523], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 524], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 525], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 526], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 527], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 528], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 529], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 530], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 531], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 532], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 533], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 534], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 535], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 536], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 537], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 538], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 539], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 540], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 541], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 542], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 543], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 544], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 545], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 546], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 547], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 548], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 549], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 550], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 551], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 552], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 553], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 554], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 555], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 556], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 557], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 558], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 559], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 560], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 561], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 562], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 563], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 564], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 565], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 566], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 567], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 568], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 569], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 570], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 571], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 572], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 573], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 574], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 575], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 576], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 577], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 578], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 579], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 580], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 581], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 582], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 583], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 584], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 585], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 586], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 587], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 588], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 589], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 590], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 591], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 592], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 593], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 594], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 595], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 596], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 597], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 598], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 599], InsetBox[
       StyleBox["\<\"\[SkeletonIndicator] \"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 600]}, 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 601], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 602], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 603], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 604], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 605], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 606], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 607], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 608], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 609], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 610], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 611], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 612], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 613], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 614], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 615], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 616], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 617], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 618], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 619], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 620], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 621], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 622], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 623], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 624], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 625], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 626], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 627], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 628], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 629], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 630], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 631], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 632], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 633], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 634], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 635], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 636], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 637], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 638], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 639], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 640], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 641], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 642], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 643], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 644], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 645], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 646], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 647], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 648], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 649], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 650], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 651], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 652], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 653], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 654], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 655], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 656], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 657], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 658], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 659], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 660], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 661], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 662], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 663], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 664], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 665], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 666], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 667], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 668], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 669], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 670], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 671], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 672], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 673], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 674], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 675], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 676], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 677], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 678], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 679], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 680], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 681], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 682], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 683], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 684], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 685], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 686], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 687], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 688], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 689], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 690], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 691], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 692], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 693], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 694], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 695], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 696], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 697], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 698], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 699], InsetBox[
       StyleBox["\<\" \[SkeletonIndicator]\"\>",
        StripOnInput->False,
        FrontFaceColor->GrayLevel[0],
        BackFaceColor->GrayLevel[0],
        GraphicsColor->GrayLevel[0],
        FontColor->GrayLevel[0]], 700]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 701],
       "InsetString"], 
      TagBox[InsetBox["", 702],
       "InsetString"], 
      TagBox[InsetBox["", 703],
       "InsetString"], 
      TagBox[InsetBox["", 704],
       "InsetString"], 
      TagBox[InsetBox["", 705],
       "InsetString"], 
      TagBox[InsetBox["", 706],
       "InsetString"], 
      TagBox[InsetBox["", 707],
       "InsetString"], 
      TagBox[InsetBox["", 708],
       "InsetString"], 
      TagBox[InsetBox["", 709],
       "InsetString"], 
      TagBox[InsetBox["", 710],
       "InsetString"], 
      TagBox[InsetBox["", 711],
       "InsetString"], 
      TagBox[InsetBox["", 712],
       "InsetString"], 
      TagBox[InsetBox["", 713],
       "InsetString"], 
      TagBox[InsetBox["", 714],
       "InsetString"], 
      TagBox[InsetBox["", 715],
       "InsetString"], 
      TagBox[InsetBox["", 716],
       "InsetString"], 
      TagBox[InsetBox["", 717],
       "InsetString"], 
      TagBox[InsetBox["", 718],
       "InsetString"], 
      TagBox[InsetBox["", 719],
       "InsetString"], 
      TagBox[InsetBox["", 720],
       "InsetString"], 
      TagBox[InsetBox["", 721],
       "InsetString"], 
      TagBox[InsetBox["", 722],
       "InsetString"], 
      TagBox[InsetBox["", 723],
       "InsetString"], 
      TagBox[InsetBox["", 724],
       "InsetString"], 
      TagBox[InsetBox["", 725],
       "InsetString"], 
      TagBox[InsetBox["", 726],
       "InsetString"], 
      TagBox[InsetBox["", 727],
       "InsetString"], 
      TagBox[InsetBox["", 728],
       "InsetString"], 
      TagBox[InsetBox["", 729],
       "InsetString"], 
      TagBox[InsetBox["", 730],
       "InsetString"], 
      TagBox[InsetBox["", 731],
       "InsetString"], 
      TagBox[InsetBox["", 732],
       "InsetString"], 
      TagBox[InsetBox["", 733],
       "InsetString"], 
      TagBox[InsetBox["", 734],
       "InsetString"], 
      TagBox[InsetBox["", 735],
       "InsetString"], 
      TagBox[InsetBox["", 736],
       "InsetString"], 
      TagBox[InsetBox["", 737],
       "InsetString"], 
      TagBox[InsetBox["", 738],
       "InsetString"], 
      TagBox[InsetBox["", 739],
       "InsetString"], 
      TagBox[InsetBox["", 740],
       "InsetString"], 
      TagBox[InsetBox["", 741],
       "InsetString"], 
      TagBox[InsetBox["", 742],
       "InsetString"], 
      TagBox[InsetBox["", 743],
       "InsetString"], 
      TagBox[InsetBox["", 744],
       "InsetString"], 
      TagBox[InsetBox["", 745],
       "InsetString"], 
      TagBox[InsetBox["", 746],
       "InsetString"], 
      TagBox[InsetBox["", 747],
       "InsetString"], 
      TagBox[InsetBox["", 748],
       "InsetString"], 
      TagBox[InsetBox["", 749],
       "InsetString"], 
      TagBox[InsetBox["", 750],
       "InsetString"], 
      TagBox[InsetBox["", 751],
       "InsetString"], 
      TagBox[InsetBox["", 752],
       "InsetString"], 
      TagBox[InsetBox["", 753],
       "InsetString"], 
      TagBox[InsetBox["", 754],
       "InsetString"], 
      TagBox[InsetBox["", 755],
       "InsetString"], 
      TagBox[InsetBox["", 756],
       "InsetString"], 
      TagBox[InsetBox["", 757],
       "InsetString"], 
      TagBox[InsetBox["", 758],
       "InsetString"], 
      TagBox[InsetBox["", 759],
       "InsetString"], 
      TagBox[InsetBox["", 760],
       "InsetString"], 
      TagBox[InsetBox["", 761],
       "InsetString"], 
      TagBox[InsetBox["", 762],
       "InsetString"], 
      TagBox[InsetBox["", 763],
       "InsetString"], 
      TagBox[InsetBox["", 764],
       "InsetString"], 
      TagBox[InsetBox["", 765],
       "InsetString"], 
      TagBox[InsetBox["", 766],
       "InsetString"], 
      TagBox[InsetBox["", 767],
       "InsetString"], 
      TagBox[InsetBox["", 768],
       "InsetString"], 
      TagBox[InsetBox["", 769],
       "InsetString"], 
      TagBox[InsetBox["", 770],
       "InsetString"], 
      TagBox[InsetBox["", 771],
       "InsetString"], 
      TagBox[InsetBox["", 772],
       "InsetString"], 
      TagBox[InsetBox["", 773],
       "InsetString"], 
      TagBox[InsetBox["", 774],
       "InsetString"], 
      TagBox[InsetBox["", 775],
       "InsetString"], 
      TagBox[InsetBox["", 776],
       "InsetString"], 
      TagBox[InsetBox["", 777],
       "InsetString"], 
      TagBox[InsetBox["", 778],
       "InsetString"], 
      TagBox[InsetBox["", 779],
       "InsetString"], 
      TagBox[InsetBox["", 780],
       "InsetString"], 
      TagBox[InsetBox["", 781],
       "InsetString"], 
      TagBox[InsetBox["", 782],
       "InsetString"], 
      TagBox[InsetBox["", 783],
       "InsetString"], 
      TagBox[InsetBox["", 784],
       "InsetString"], 
      TagBox[InsetBox["", 785],
       "InsetString"], 
      TagBox[InsetBox["", 786],
       "InsetString"], 
      TagBox[InsetBox["", 787],
       "InsetString"], 
      TagBox[InsetBox["", 788],
       "InsetString"], 
      TagBox[InsetBox["", 789],
       "InsetString"], 
      TagBox[InsetBox["", 790],
       "InsetString"], 
      TagBox[InsetBox["", 791],
       "InsetString"], 
      TagBox[InsetBox["", 792],
       "InsetString"], 
      TagBox[InsetBox["", 793],
       "InsetString"], 
      TagBox[InsetBox["", 794],
       "InsetString"], 
      TagBox[InsetBox["", 795],
       "InsetString"], 
      TagBox[InsetBox["", 796],
       "InsetString"], 
      TagBox[InsetBox["", 797],
       "InsetString"], 
      TagBox[InsetBox["", 798],
       "InsetString"], 
      TagBox[InsetBox["", 799],
       "InsetString"], 
      TagBox[InsetBox["", 800],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 801],
       "InsetString"], 
      TagBox[InsetBox["", 802],
       "InsetString"], 
      TagBox[InsetBox["", 803],
       "InsetString"], 
      TagBox[InsetBox["", 804],
       "InsetString"], 
      TagBox[InsetBox["", 805],
       "InsetString"], 
      TagBox[InsetBox["", 806],
       "InsetString"], 
      TagBox[InsetBox["", 807],
       "InsetString"], 
      TagBox[InsetBox["", 808],
       "InsetString"], 
      TagBox[InsetBox["", 809],
       "InsetString"], 
      TagBox[InsetBox["", 810],
       "InsetString"], 
      TagBox[InsetBox["", 811],
       "InsetString"], 
      TagBox[InsetBox["", 812],
       "InsetString"], 
      TagBox[InsetBox["", 813],
       "InsetString"], 
      TagBox[InsetBox["", 814],
       "InsetString"], 
      TagBox[InsetBox["", 815],
       "InsetString"], 
      TagBox[InsetBox["", 816],
       "InsetString"], 
      TagBox[InsetBox["", 817],
       "InsetString"], 
      TagBox[InsetBox["", 818],
       "InsetString"], 
      TagBox[InsetBox["", 819],
       "InsetString"], 
      TagBox[InsetBox["", 820],
       "InsetString"], 
      TagBox[InsetBox["", 821],
       "InsetString"], 
      TagBox[InsetBox["", 822],
       "InsetString"], 
      TagBox[InsetBox["", 823],
       "InsetString"], 
      TagBox[InsetBox["", 824],
       "InsetString"], 
      TagBox[InsetBox["", 825],
       "InsetString"], 
      TagBox[InsetBox["", 826],
       "InsetString"], 
      TagBox[InsetBox["", 827],
       "InsetString"], 
      TagBox[InsetBox["", 828],
       "InsetString"], 
      TagBox[InsetBox["", 829],
       "InsetString"], 
      TagBox[InsetBox["", 830],
       "InsetString"], 
      TagBox[InsetBox["", 831],
       "InsetString"], 
      TagBox[InsetBox["", 832],
       "InsetString"], 
      TagBox[InsetBox["", 833],
       "InsetString"], 
      TagBox[InsetBox["", 834],
       "InsetString"], 
      TagBox[InsetBox["", 835],
       "InsetString"], 
      TagBox[InsetBox["", 836],
       "InsetString"], 
      TagBox[InsetBox["", 837],
       "InsetString"], 
      TagBox[InsetBox["", 838],
       "InsetString"], 
      TagBox[InsetBox["", 839],
       "InsetString"], 
      TagBox[InsetBox["", 840],
       "InsetString"], 
      TagBox[InsetBox["", 841],
       "InsetString"], 
      TagBox[InsetBox["", 842],
       "InsetString"], 
      TagBox[InsetBox["", 843],
       "InsetString"], 
      TagBox[InsetBox["", 844],
       "InsetString"], 
      TagBox[InsetBox["", 845],
       "InsetString"], 
      TagBox[InsetBox["", 846],
       "InsetString"], 
      TagBox[InsetBox["", 847],
       "InsetString"], 
      TagBox[InsetBox["", 848],
       "InsetString"], 
      TagBox[InsetBox["", 849],
       "InsetString"], 
      TagBox[InsetBox["", 850],
       "InsetString"], 
      TagBox[InsetBox["", 851],
       "InsetString"], 
      TagBox[InsetBox["", 852],
       "InsetString"], 
      TagBox[InsetBox["", 853],
       "InsetString"], 
      TagBox[InsetBox["", 854],
       "InsetString"], 
      TagBox[InsetBox["", 855],
       "InsetString"], 
      TagBox[InsetBox["", 856],
       "InsetString"], 
      TagBox[InsetBox["", 857],
       "InsetString"], 
      TagBox[InsetBox["", 858],
       "InsetString"], 
      TagBox[InsetBox["", 859],
       "InsetString"], 
      TagBox[InsetBox["", 860],
       "InsetString"], 
      TagBox[InsetBox["", 861],
       "InsetString"], 
      TagBox[InsetBox["", 862],
       "InsetString"], 
      TagBox[InsetBox["", 863],
       "InsetString"], 
      TagBox[InsetBox["", 864],
       "InsetString"], 
      TagBox[InsetBox["", 865],
       "InsetString"], 
      TagBox[InsetBox["", 866],
       "InsetString"], 
      TagBox[InsetBox["", 867],
       "InsetString"], 
      TagBox[InsetBox["", 868],
       "InsetString"], 
      TagBox[InsetBox["", 869],
       "InsetString"], 
      TagBox[InsetBox["", 870],
       "InsetString"], 
      TagBox[InsetBox["", 871],
       "InsetString"], 
      TagBox[InsetBox["", 872],
       "InsetString"], 
      TagBox[InsetBox["", 873],
       "InsetString"], 
      TagBox[InsetBox["", 874],
       "InsetString"], 
      TagBox[InsetBox["", 875],
       "InsetString"], 
      TagBox[InsetBox["", 876],
       "InsetString"], 
      TagBox[InsetBox["", 877],
       "InsetString"], 
      TagBox[InsetBox["", 878],
       "InsetString"], 
      TagBox[InsetBox["", 879],
       "InsetString"], 
      TagBox[InsetBox["", 880],
       "InsetString"], 
      TagBox[InsetBox["", 881],
       "InsetString"], 
      TagBox[InsetBox["", 882],
       "InsetString"], 
      TagBox[InsetBox["", 883],
       "InsetString"], 
      TagBox[InsetBox["", 884],
       "InsetString"], 
      TagBox[InsetBox["", 885],
       "InsetString"], 
      TagBox[InsetBox["", 886],
       "InsetString"], 
      TagBox[InsetBox["", 887],
       "InsetString"], 
      TagBox[InsetBox["", 888],
       "InsetString"], 
      TagBox[InsetBox["", 889],
       "InsetString"], 
      TagBox[InsetBox["", 890],
       "InsetString"], 
      TagBox[InsetBox["", 891],
       "InsetString"], 
      TagBox[InsetBox["", 892],
       "InsetString"], 
      TagBox[InsetBox["", 893],
       "InsetString"], 
      TagBox[InsetBox["", 894],
       "InsetString"], 
      TagBox[InsetBox["", 895],
       "InsetString"], 
      TagBox[InsetBox["", 896],
       "InsetString"], 
      TagBox[InsetBox["", 897],
       "InsetString"], 
      TagBox[InsetBox["", 898],
       "InsetString"], 
      TagBox[InsetBox["", 899],
       "InsetString"], 
      TagBox[InsetBox["", 900],
       "InsetString"]}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{3.432*^9, 250.},
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"Google\"", 14, 
            RGBColor[1, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{Automatic, Automatic}, {{{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"Oct\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"Nov\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"Dec\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"Jan\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"Feb\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3433017600, 3.4330176*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3435696000, 3.435696*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3438288000, 3.438288*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3440966400, 3.4409664*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3443644800, 3.4436448*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3433017600, 3.4330176*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3435696000, 3.435696*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3438288000, 3.438288*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3440966400, 3.4409664*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3443644800, 3.4436448*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3434486400, 
       GrayLevel[0.8]}, {3437078400, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3442435200, 
       GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4344864*^9, 
       GrayLevel[0.8]}, {3.4370784*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4424352*^9, 
       GrayLevel[0.8]}}, Automatic}],
  GridLinesStyle->{
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]], 
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]]},
  ImagePadding->{{50, 10}, {20, 2}},
  ImageSize->600,
  PlotRange->{All, {247.3, 425.08}},
  PlotRangeClipping->True,
  PlotRangePadding->{200000, 5},
  Ticks->None]], "Output",
 CellChangeTimes->{{3.4627236833803883`*^9, 3.462723703935038*^9}, 
   3.462728572488009*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CandlestickPlot"}]], "Input",
 CellChangeTimes->{{3.462723186556629*^9, 3.462723186914515*^9}}],

Cell[BoxData[
 StyleBox["\<\"CandlestickPlot[data, {label, label color}] will generate a \
candlestick plot of data which must have the form {{date, open, high, low, \
close, volume}..}. The label is the label to appear in the pot, typically the \
name of the stock. Options are the same as DateListPlot.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728585095744*^9},
 CellTags->"Info3462710584-6153215"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CandlestickPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "100"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Red"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.46272371673803*^9, 3.462723740378599*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmn1sllcZxt+NhclX1gRwBhVaP0YHstWBwY/h+gYnQ6a24EwHOLqxdYyQ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   
   "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
     {GrayLevel[0], AbsoluteThickness[1], LineBox[{1, 482}], 
      LineBox[{2, 483}], LineBox[{3, 484}], LineBox[{4, 485}], 
      LineBox[{5, 486}], LineBox[{6, 487}], LineBox[{7, 488}], 
      LineBox[{8, 489}], LineBox[{9, 490}], LineBox[{10, 491}], 
      LineBox[{11, 492}], LineBox[{12, 493}], LineBox[{13, 494}], 
      LineBox[{14, 495}], LineBox[{15, 496}], LineBox[{16, 497}], 
      LineBox[{17, 498}], LineBox[{18, 499}], LineBox[{19, 500}], 
      LineBox[{20, 501}], LineBox[{21, 502}], LineBox[{22, 503}], 
      LineBox[{23, 504}], LineBox[{24, 505}], LineBox[{25, 506}], 
      LineBox[{26, 507}], LineBox[{27, 508}], LineBox[{28, 509}], 
      LineBox[{29, 510}], LineBox[{30, 511}], LineBox[{31, 512}], 
      LineBox[{32, 513}], LineBox[{33, 514}], LineBox[{34, 515}], 
      LineBox[{35, 516}], LineBox[{36, 517}], LineBox[{37, 518}], 
      LineBox[{38, 519}], LineBox[{39, 520}], LineBox[{40, 521}], 
      LineBox[{41, 522}], LineBox[{42, 523}], LineBox[{43, 524}], 
      LineBox[{44, 525}], LineBox[{45, 526}], LineBox[{46, 527}], 
      LineBox[{47, 528}], LineBox[{48, 529}], LineBox[{49, 530}], 
      LineBox[{50, 531}], LineBox[{51, 532}], LineBox[{52, 533}], 
      LineBox[{53, 534}], LineBox[{54, 535}], LineBox[{55, 536}], 
      LineBox[{56, 537}], LineBox[{57, 538}], LineBox[{58, 539}], 
      LineBox[{59, 540}], LineBox[{60, 541}], LineBox[{61, 542}], 
      LineBox[{62, 543}], LineBox[{63, 544}], LineBox[{64, 545}], 
      LineBox[{65, 546}], LineBox[{66, 547}], LineBox[{67, 548}], 
      LineBox[{68, 549}], LineBox[{69, 550}], LineBox[{70, 551}], 
      LineBox[{71, 552}], LineBox[{72, 553}], LineBox[{73, 554}], 
      LineBox[{74, 555}], LineBox[{75, 556}], LineBox[{76, 557}], 
      LineBox[{77, 558}], LineBox[{78, 559}], LineBox[{79, 560}], 
      LineBox[{80, 561}], LineBox[{81, 562}], LineBox[{82, 563}], 
      LineBox[{83, 564}], LineBox[{84, 565}], LineBox[{85, 566}], 
      LineBox[{86, 567}], LineBox[{87, 568}], LineBox[{88, 569}], 
      LineBox[{89, 570}], LineBox[{90, 571}], LineBox[{91, 572}], 
      LineBox[{92, 573}], LineBox[{93, 574}], LineBox[{94, 575}], 
      LineBox[{95, 576}], LineBox[{96, 577}], LineBox[{97, 578}], 
      LineBox[{98, 579}], LineBox[{99, 580}], 
      LineBox[{100, 
       581}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
     {RGBColor[1, 0, 0], AbsoluteThickness[2.5], LineBox[{202, 446}], 
      LineBox[{203, 447}], LineBox[{204, 448}], LineBox[{205, 449}], 
      LineBox[{206, 450}], LineBox[{207, 451}], LineBox[{211, 452}], 
      LineBox[{212, 453}], LineBox[{213, 454}], LineBox[{214, 455}], 
      LineBox[{215, 456}], LineBox[{216, 457}], LineBox[{217, 458}], 
      LineBox[{218, 459}], LineBox[{219, 460}], LineBox[{220, 461}], 
      LineBox[{221, 462}], LineBox[{222, 463}], LineBox[{223, 464}], 
      LineBox[{224, 465}], LineBox[{225, 466}], LineBox[{226, 467}], 
      LineBox[{229, 468}], LineBox[{230, 469}], LineBox[{231, 470}], 
      LineBox[{232, 471}], LineBox[{233, 472}], LineBox[{235, 473}], 
      LineBox[{236, 474}], LineBox[{237, 475}], LineBox[{238, 476}], 
      LineBox[{240, 477}], LineBox[{241, 478}], LineBox[{242, 479}], 
      LineBox[{243, 480}], 
      LineBox[{244, 481}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
     {RGBColor[0., 0.8, 0.], AbsoluteThickness[2.5], LineBox[{295, 401}], 
      LineBox[{349, 402}], LineBox[{297, 403}], LineBox[{351, 404}], 
      LineBox[{299, 405}], LineBox[{353, 406}], LineBox[{303, 407}], 
      LineBox[{357, 408}], LineBox[{358, 409}], LineBox[{359, 410}], 
      LineBox[{360, 411}], LineBox[{361, 412}], LineBox[{310, 413}], 
      LineBox[{364, 414}], LineBox[{365, 415}], LineBox[{314, 416}], 
      LineBox[{368, 417}], LineBox[{369, 418}], LineBox[{370, 419}], 
      LineBox[{371, 420}], LineBox[{372, 421}], LineBox[{373, 422}], 
      LineBox[{322, 423}], LineBox[{376, 424}], LineBox[{325, 425}], 
      LineBox[{379, 426}], LineBox[{327, 427}], LineBox[{381, 428}], 
      LineBox[{382, 429}], LineBox[{383, 430}], LineBox[{384, 431}], 
      LineBox[{332, 432}], LineBox[{386, 433}], LineBox[{387, 434}], 
      LineBox[{335, 435}], LineBox[{389, 436}], LineBox[{338, 437}], 
      LineBox[{392, 438}], LineBox[{340, 439}], LineBox[{394, 440}], 
      LineBox[{395, 441}], LineBox[{396, 442}], LineBox[{397, 443}], 
      LineBox[{398, 444}], LineBox[{399, 445}]}}, {
     {GrayLevel[0], 
      TagBox[InsetBox["", 201],
       "InsetString"], 
      TagBox[InsetBox["", 208],
       "InsetString"], 
      TagBox[InsetBox["", 209],
       "InsetString"], 
      TagBox[InsetBox["", 210],
       "InsetString"], 
      TagBox[InsetBox["", 227],
       "InsetString"], 
      TagBox[InsetBox["", 228],
       "InsetString"], 
      TagBox[InsetBox["", 234],
       "InsetString"], 
      TagBox[InsetBox["", 239],
       "InsetString"], 
      TagBox[InsetBox["", 245],
       "InsetString"], 
      TagBox[InsetBox["", 246],
       "InsetString"], 
      TagBox[InsetBox["", 247],
       "InsetString"], 
      TagBox[InsetBox["", 248],
       "InsetString"], 
      TagBox[InsetBox["", 255],
       "InsetString"], 
      TagBox[InsetBox["", 256],
       "InsetString"], 
      TagBox[InsetBox["", 257],
       "InsetString"], 
      TagBox[InsetBox["", 274],
       "InsetString"], 
      TagBox[InsetBox["", 275],
       "InsetString"], 
      TagBox[InsetBox["", 281],
       "InsetString"], 
      TagBox[InsetBox["", 286],
       "InsetString"], 
      TagBox[InsetBox["", 292],
       "InsetString"], 
      TagBox[InsetBox["", 293],
       "InsetString"], 
      TagBox[InsetBox["", 294],
       "InsetString"], 
      TagBox[InsetBox["", 301],
       "InsetString"], 
      TagBox[InsetBox["", 302],
       "InsetString"], 
      TagBox[InsetBox["", 309],
       "InsetString"], 
      TagBox[InsetBox["", 313],
       "InsetString"], 
      TagBox[InsetBox["", 321],
       "InsetString"], 
      TagBox[InsetBox["", 324],
       "InsetString"], 
      TagBox[InsetBox["", 337],
       "InsetString"], 
      TagBox[InsetBox["", 347],
       "InsetString"], 
      TagBox[InsetBox["", 354],
       "InsetString"], 
      TagBox[InsetBox["", 355],
       "InsetString"], 
      TagBox[InsetBox["", 362],
       "InsetString"], 
      TagBox[InsetBox["", 366],
       "InsetString"], 
      TagBox[InsetBox["", 374],
       "InsetString"], 
      TagBox[InsetBox["", 377],
       "InsetString"], 
      TagBox[InsetBox["", 390],
       "InsetString"], 
      TagBox[InsetBox["", 400],
       "InsetString"], 
      TagBox[InsetBox["", 582],
       "InsetString"], 
      TagBox[InsetBox["", 583],
       "InsetString"], 
      TagBox[InsetBox["", 584],
       "InsetString"], 
      TagBox[InsetBox["", 585],
       "InsetString"], 
      TagBox[InsetBox["", 586],
       "InsetString"], 
      TagBox[InsetBox["", 587],
       "InsetString"], 
      TagBox[InsetBox["", 588],
       "InsetString"], 
      TagBox[InsetBox["", 589],
       "InsetString"], 
      TagBox[InsetBox["", 590],
       "InsetString"], 
      TagBox[InsetBox["", 591],
       "InsetString"], 
      TagBox[InsetBox["", 592],
       "InsetString"], 
      TagBox[InsetBox["", 593],
       "InsetString"], 
      TagBox[InsetBox["", 594],
       "InsetString"], 
      TagBox[InsetBox["", 595],
       "InsetString"], 
      TagBox[InsetBox["", 596],
       "InsetString"], 
      TagBox[InsetBox["", 597],
       "InsetString"], 
      TagBox[InsetBox["", 598],
       "InsetString"], 
      TagBox[InsetBox["", 599],
       "InsetString"], 
      TagBox[InsetBox["", 600],
       "InsetString"], 
      TagBox[InsetBox["", 601],
       "InsetString"], 
      TagBox[InsetBox["", 602],
       "InsetString"], 
      TagBox[InsetBox["", 603],
       "InsetString"], 
      TagBox[InsetBox["", 604],
       "InsetString"], 
      TagBox[InsetBox["", 605],
       "InsetString"], 
      TagBox[InsetBox["", 606],
       "InsetString"], 
      TagBox[InsetBox["", 607],
       "InsetString"], 
      TagBox[InsetBox["", 608],
       "InsetString"], 
      TagBox[InsetBox["", 609],
       "InsetString"], 
      TagBox[InsetBox["", 610],
       "InsetString"], 
      TagBox[InsetBox["", 611],
       "InsetString"], 
      TagBox[InsetBox["", 612],
       "InsetString"], 
      TagBox[InsetBox["", 613],
       "InsetString"], 
      TagBox[InsetBox["", 614],
       "InsetString"], 
      TagBox[InsetBox["", 615],
       "InsetString"], 
      TagBox[InsetBox["", 616],
       "InsetString"], 
      TagBox[InsetBox["", 617],
       "InsetString"], 
      TagBox[InsetBox["", 618],
       "InsetString"], 
      TagBox[InsetBox["", 619],
       "InsetString"], 
      TagBox[InsetBox["", 620],
       "InsetString"], 
      TagBox[InsetBox["", 621],
       "InsetString"], 
      TagBox[InsetBox["", 622],
       "InsetString"], 
      TagBox[InsetBox["", 623],
       "InsetString"], 
      TagBox[InsetBox["", 624],
       "InsetString"], 
      TagBox[InsetBox["", 625],
       "InsetString"], 
      TagBox[InsetBox["", 626],
       "InsetString"], 
      TagBox[InsetBox["", 627],
       "InsetString"], 
      TagBox[InsetBox["", 628],
       "InsetString"], 
      TagBox[InsetBox["", 629],
       "InsetString"], 
      TagBox[InsetBox["", 630],
       "InsetString"], 
      TagBox[InsetBox["", 631],
       "InsetString"], 
      TagBox[InsetBox["", 632],
       "InsetString"], 
      TagBox[InsetBox["", 633],
       "InsetString"], 
      TagBox[InsetBox["", 634],
       "InsetString"], 
      TagBox[InsetBox["", 635],
       "InsetString"], 
      TagBox[InsetBox["", 636],
       "InsetString"], 
      TagBox[InsetBox["", 637],
       "InsetString"], 
      TagBox[InsetBox["", 638],
       "InsetString"], 
      TagBox[InsetBox["", 639],
       "InsetString"], 
      TagBox[InsetBox["", 640],
       "InsetString"], 
      TagBox[InsetBox["", 641],
       "InsetString"], 
      TagBox[InsetBox["", 642],
       "InsetString"], 
      TagBox[InsetBox["", 643],
       "InsetString"], 
      TagBox[InsetBox["", 644],
       "InsetString"], 
      TagBox[InsetBox["", 645],
       "InsetString"], 
      TagBox[InsetBox["", 646],
       "InsetString"], 
      TagBox[InsetBox["", 647],
       "InsetString"], 
      TagBox[InsetBox["", 648],
       "InsetString"], 
      TagBox[InsetBox["", 649],
       "InsetString"], 
      TagBox[InsetBox["", 650],
       "InsetString"], 
      TagBox[InsetBox["", 651],
       "InsetString"], 
      TagBox[InsetBox["", 652],
       "InsetString"], 
      TagBox[InsetBox["", 653],
       "InsetString"], 
      TagBox[InsetBox["", 654],
       "InsetString"], 
      TagBox[InsetBox["", 655],
       "InsetString"], 
      TagBox[InsetBox["", 656],
       "InsetString"], 
      TagBox[InsetBox["", 657],
       "InsetString"], 
      TagBox[InsetBox["", 658],
       "InsetString"], 
      TagBox[InsetBox["", 659],
       "InsetString"], 
      TagBox[InsetBox["", 660],
       "InsetString"], 
      TagBox[InsetBox["", 661],
       "InsetString"], 
      TagBox[InsetBox["", 662],
       "InsetString"], 
      TagBox[InsetBox["", 663],
       "InsetString"], 
      TagBox[InsetBox["", 664],
       "InsetString"], 
      TagBox[InsetBox["", 665],
       "InsetString"], 
      TagBox[InsetBox["", 666],
       "InsetString"], 
      TagBox[InsetBox["", 667],
       "InsetString"], 
      TagBox[InsetBox["", 668],
       "InsetString"], 
      TagBox[InsetBox["", 669],
       "InsetString"], 
      TagBox[InsetBox["", 670],
       "InsetString"], 
      TagBox[InsetBox["", 671],
       "InsetString"], 
      TagBox[InsetBox["", 672],
       "InsetString"], 
      TagBox[InsetBox["", 673],
       "InsetString"], 
      TagBox[InsetBox["", 674],
       "InsetString"], 
      TagBox[InsetBox["", 675],
       "InsetString"], 
      TagBox[InsetBox["", 676],
       "InsetString"], 
      TagBox[InsetBox["", 677],
       "InsetString"], 
      TagBox[InsetBox["", 678],
       "InsetString"], 
      TagBox[InsetBox["", 679],
       "InsetString"], 
      TagBox[InsetBox["", 680],
       "InsetString"], 
      TagBox[InsetBox["", 681],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 682],
       "InsetString"], 
      TagBox[InsetBox["", 683],
       "InsetString"], 
      TagBox[InsetBox["", 684],
       "InsetString"], 
      TagBox[InsetBox["", 685],
       "InsetString"], 
      TagBox[InsetBox["", 686],
       "InsetString"], 
      TagBox[InsetBox["", 687],
       "InsetString"], 
      TagBox[InsetBox["", 688],
       "InsetString"], 
      TagBox[InsetBox["", 689],
       "InsetString"], 
      TagBox[InsetBox["", 690],
       "InsetString"], 
      TagBox[InsetBox["", 691],
       "InsetString"], 
      TagBox[InsetBox["", 692],
       "InsetString"], 
      TagBox[InsetBox["", 693],
       "InsetString"], 
      TagBox[InsetBox["", 694],
       "InsetString"], 
      TagBox[InsetBox["", 695],
       "InsetString"], 
      TagBox[InsetBox["", 696],
       "InsetString"], 
      TagBox[InsetBox["", 697],
       "InsetString"], 
      TagBox[InsetBox["", 698],
       "InsetString"], 
      TagBox[InsetBox["", 699],
       "InsetString"], 
      TagBox[InsetBox["", 700],
       "InsetString"], 
      TagBox[InsetBox["", 701],
       "InsetString"], 
      TagBox[InsetBox["", 702],
       "InsetString"], 
      TagBox[InsetBox["", 703],
       "InsetString"], 
      TagBox[InsetBox["", 704],
       "InsetString"], 
      TagBox[InsetBox["", 705],
       "InsetString"], 
      TagBox[InsetBox["", 706],
       "InsetString"], 
      TagBox[InsetBox["", 707],
       "InsetString"], 
      TagBox[InsetBox["", 708],
       "InsetString"], 
      TagBox[InsetBox["", 709],
       "InsetString"], 
      TagBox[InsetBox["", 710],
       "InsetString"], 
      TagBox[InsetBox["", 711],
       "InsetString"], 
      TagBox[InsetBox["", 712],
       "InsetString"], 
      TagBox[InsetBox["", 713],
       "InsetString"], 
      TagBox[InsetBox["", 714],
       "InsetString"], 
      TagBox[InsetBox["", 715],
       "InsetString"], 
      TagBox[InsetBox["", 716],
       "InsetString"], 
      TagBox[InsetBox["", 717],
       "InsetString"], 
      TagBox[InsetBox["", 718],
       "InsetString"], 
      TagBox[InsetBox["", 719],
       "InsetString"], 
      TagBox[InsetBox["", 720],
       "InsetString"], 
      TagBox[InsetBox["", 721],
       "InsetString"], 
      TagBox[InsetBox["", 722],
       "InsetString"], 
      TagBox[InsetBox["", 723],
       "InsetString"], 
      TagBox[InsetBox["", 724],
       "InsetString"], 
      TagBox[InsetBox["", 725],
       "InsetString"], 
      TagBox[InsetBox["", 726],
       "InsetString"], 
      TagBox[InsetBox["", 727],
       "InsetString"], 
      TagBox[InsetBox["", 728],
       "InsetString"], 
      TagBox[InsetBox["", 729],
       "InsetString"], 
      TagBox[InsetBox["", 730],
       "InsetString"], 
      TagBox[InsetBox["", 731],
       "InsetString"], 
      TagBox[InsetBox["", 732],
       "InsetString"], 
      TagBox[InsetBox["", 733],
       "InsetString"], 
      TagBox[InsetBox["", 734],
       "InsetString"], 
      TagBox[InsetBox["", 735],
       "InsetString"], 
      TagBox[InsetBox["", 736],
       "InsetString"], 
      TagBox[InsetBox["", 737],
       "InsetString"], 
      TagBox[InsetBox["", 738],
       "InsetString"], 
      TagBox[InsetBox["", 739],
       "InsetString"], 
      TagBox[InsetBox["", 740],
       "InsetString"], 
      TagBox[InsetBox["", 741],
       "InsetString"], 
      TagBox[InsetBox["", 742],
       "InsetString"], 
      TagBox[InsetBox["", 743],
       "InsetString"], 
      TagBox[InsetBox["", 744],
       "InsetString"], 
      TagBox[InsetBox["", 745],
       "InsetString"], 
      TagBox[InsetBox["", 746],
       "InsetString"], 
      TagBox[InsetBox["", 747],
       "InsetString"], 
      TagBox[InsetBox["", 748],
       "InsetString"], 
      TagBox[InsetBox["", 749],
       "InsetString"], 
      TagBox[InsetBox["", 750],
       "InsetString"], 
      TagBox[InsetBox["", 751],
       "InsetString"], 
      TagBox[InsetBox["", 752],
       "InsetString"], 
      TagBox[InsetBox["", 753],
       "InsetString"], 
      TagBox[InsetBox["", 754],
       "InsetString"], 
      TagBox[InsetBox["", 755],
       "InsetString"], 
      TagBox[InsetBox["", 756],
       "InsetString"], 
      TagBox[InsetBox["", 757],
       "InsetString"], 
      TagBox[InsetBox["", 758],
       "InsetString"], 
      TagBox[InsetBox["", 759],
       "InsetString"], 
      TagBox[InsetBox["", 760],
       "InsetString"], 
      TagBox[InsetBox["", 761],
       "InsetString"], 
      TagBox[InsetBox["", 762],
       "InsetString"], 
      TagBox[InsetBox["", 763],
       "InsetString"], 
      TagBox[InsetBox["", 764],
       "InsetString"], 
      TagBox[InsetBox["", 765],
       "InsetString"], 
      TagBox[InsetBox["", 766],
       "InsetString"], 
      TagBox[InsetBox["", 767],
       "InsetString"], 
      TagBox[InsetBox["", 768],
       "InsetString"], 
      TagBox[InsetBox["", 769],
       "InsetString"], 
      TagBox[InsetBox["", 770],
       "InsetString"], 
      TagBox[InsetBox["", 771],
       "InsetString"], 
      TagBox[InsetBox["", 772],
       "InsetString"], 
      TagBox[InsetBox["", 773],
       "InsetString"], 
      TagBox[InsetBox["", 774],
       "InsetString"], 
      TagBox[InsetBox["", 775],
       "InsetString"], 
      TagBox[InsetBox["", 776],
       "InsetString"], 
      TagBox[InsetBox["", 777],
       "InsetString"], 
      TagBox[InsetBox["", 778],
       "InsetString"], 
      TagBox[InsetBox["", 779],
       "InsetString"], 
      TagBox[InsetBox["", 780],
       "InsetString"], 
      TagBox[InsetBox["", 781],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 782],
       "InsetString"], 
      TagBox[InsetBox["", 783],
       "InsetString"], 
      TagBox[InsetBox["", 784],
       "InsetString"], 
      TagBox[InsetBox["", 785],
       "InsetString"], 
      TagBox[InsetBox["", 786],
       "InsetString"], 
      TagBox[InsetBox["", 787],
       "InsetString"], 
      TagBox[InsetBox["", 788],
       "InsetString"], 
      TagBox[InsetBox["", 789],
       "InsetString"], 
      TagBox[InsetBox["", 790],
       "InsetString"], 
      TagBox[InsetBox["", 791],
       "InsetString"], 
      TagBox[InsetBox["", 792],
       "InsetString"], 
      TagBox[InsetBox["", 793],
       "InsetString"], 
      TagBox[InsetBox["", 794],
       "InsetString"], 
      TagBox[InsetBox["", 795],
       "InsetString"], 
      TagBox[InsetBox["", 796],
       "InsetString"], 
      TagBox[InsetBox["", 797],
       "InsetString"], 
      TagBox[InsetBox["", 798],
       "InsetString"], 
      TagBox[InsetBox["", 799],
       "InsetString"], 
      TagBox[InsetBox["", 800],
       "InsetString"], 
      TagBox[InsetBox["", 801],
       "InsetString"], 
      TagBox[InsetBox["", 802],
       "InsetString"], 
      TagBox[InsetBox["", 803],
       "InsetString"], 
      TagBox[InsetBox["", 804],
       "InsetString"], 
      TagBox[InsetBox["", 805],
       "InsetString"], 
      TagBox[InsetBox["", 806],
       "InsetString"], 
      TagBox[InsetBox["", 807],
       "InsetString"], 
      TagBox[InsetBox["", 808],
       "InsetString"], 
      TagBox[InsetBox["", 809],
       "InsetString"], 
      TagBox[InsetBox["", 810],
       "InsetString"], 
      TagBox[InsetBox["", 811],
       "InsetString"], 
      TagBox[InsetBox["", 812],
       "InsetString"], 
      TagBox[InsetBox["", 813],
       "InsetString"], 
      TagBox[InsetBox["", 814],
       "InsetString"], 
      TagBox[InsetBox["", 815],
       "InsetString"], 
      TagBox[InsetBox["", 816],
       "InsetString"], 
      TagBox[InsetBox["", 817],
       "InsetString"], 
      TagBox[InsetBox["", 818],
       "InsetString"], 
      TagBox[InsetBox["", 819],
       "InsetString"], 
      TagBox[InsetBox["", 820],
       "InsetString"], 
      TagBox[InsetBox["", 821],
       "InsetString"], 
      TagBox[InsetBox["", 822],
       "InsetString"], 
      TagBox[InsetBox["", 823],
       "InsetString"], 
      TagBox[InsetBox["", 824],
       "InsetString"], 
      TagBox[InsetBox["", 825],
       "InsetString"], 
      TagBox[InsetBox["", 826],
       "InsetString"], 
      TagBox[InsetBox["", 827],
       "InsetString"], 
      TagBox[InsetBox["", 828],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 829],
       "InsetString"], 
      TagBox[InsetBox["", 830],
       "InsetString"], 
      TagBox[InsetBox["", 831],
       "InsetString"], 
      TagBox[InsetBox["", 832],
       "InsetString"], 
      TagBox[InsetBox["", 833],
       "InsetString"], 
      TagBox[InsetBox["", 834],
       "InsetString"], 
      TagBox[InsetBox["", 835],
       "InsetString"], 
      TagBox[InsetBox["", 836],
       "InsetString"], 
      TagBox[InsetBox["", 837],
       "InsetString"], 
      TagBox[InsetBox["", 838],
       "InsetString"], 
      TagBox[InsetBox["", 839],
       "InsetString"], 
      TagBox[InsetBox["", 840],
       "InsetString"], 
      TagBox[InsetBox["", 841],
       "InsetString"], 
      TagBox[InsetBox["", 842],
       "InsetString"], 
      TagBox[InsetBox["", 843],
       "InsetString"], 
      TagBox[InsetBox["", 844],
       "InsetString"], 
      TagBox[InsetBox["", 845],
       "InsetString"], 
      TagBox[InsetBox["", 846],
       "InsetString"], 
      TagBox[InsetBox["", 847],
       "InsetString"], 
      TagBox[InsetBox["", 848],
       "InsetString"], 
      TagBox[InsetBox["", 849],
       "InsetString"], 
      TagBox[InsetBox["", 850],
       "InsetString"], 
      TagBox[InsetBox["", 851],
       "InsetString"], 
      TagBox[InsetBox["", 852],
       "InsetString"], 
      TagBox[InsetBox["", 853],
       "InsetString"], 
      TagBox[InsetBox["", 854],
       "InsetString"], 
      TagBox[InsetBox["", 855],
       "InsetString"], 
      TagBox[InsetBox["", 856],
       "InsetString"], 
      TagBox[InsetBox["", 857],
       "InsetString"], 
      TagBox[InsetBox["", 858],
       "InsetString"], 
      TagBox[InsetBox["", 859],
       "InsetString"], 
      TagBox[InsetBox["", 860],
       "InsetString"], 
      TagBox[InsetBox["", 861],
       "InsetString"], 
      TagBox[InsetBox["", 862],
       "InsetString"], 
      TagBox[InsetBox["", 863],
       "InsetString"], 
      TagBox[InsetBox["", 864],
       "InsetString"], 
      TagBox[InsetBox["", 865],
       "InsetString"], 
      TagBox[InsetBox["", 866],
       "InsetString"], 
      TagBox[InsetBox["", 867],
       "InsetString"], 
      TagBox[InsetBox["", 868],
       "InsetString"], 
      TagBox[InsetBox["", 869],
       "InsetString"], 
      TagBox[InsetBox["", 870],
       "InsetString"], 
      TagBox[InsetBox["", 871],
       "InsetString"], 
      TagBox[InsetBox["", 872],
       "InsetString"], 
      TagBox[InsetBox["", 873],
       "InsetString"], 
      TagBox[InsetBox["", 874],
       "InsetString"], 
      TagBox[InsetBox["", 875],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 876],
       "InsetString"], 
      TagBox[InsetBox["", 877],
       "InsetString"], 
      TagBox[InsetBox["", 878],
       "InsetString"], 
      TagBox[InsetBox["", 879],
       "InsetString"], 
      TagBox[InsetBox["", 880],
       "InsetString"], 
      TagBox[InsetBox["", 881],
       "InsetString"], 
      TagBox[InsetBox["", 882],
       "InsetString"], 
      TagBox[InsetBox["", 883],
       "InsetString"], 
      TagBox[InsetBox["", 884],
       "InsetString"], 
      TagBox[InsetBox["", 885],
       "InsetString"], 
      TagBox[InsetBox["", 886],
       "InsetString"], 
      TagBox[InsetBox["", 887],
       "InsetString"], 
      TagBox[InsetBox["", 888],
       "InsetString"], 
      TagBox[InsetBox["", 889],
       "InsetString"], 
      TagBox[InsetBox["", 890],
       "InsetString"], 
      TagBox[InsetBox["", 891],
       "InsetString"], 
      TagBox[InsetBox["", 892],
       "InsetString"], 
      TagBox[InsetBox["", 893],
       "InsetString"], 
      TagBox[InsetBox["", 894],
       "InsetString"], 
      TagBox[InsetBox["", 895],
       "InsetString"], 
      TagBox[InsetBox["", 896],
       "InsetString"], 
      TagBox[InsetBox["", 897],
       "InsetString"], 
      TagBox[InsetBox["", 898],
       "InsetString"], 
      TagBox[InsetBox["", 899],
       "InsetString"], 
      TagBox[InsetBox["", 900],
       "InsetString"], 
      TagBox[InsetBox["", 901],
       "InsetString"], 
      TagBox[InsetBox["", 902],
       "InsetString"], 
      TagBox[InsetBox["", 903],
       "InsetString"], 
      TagBox[InsetBox["", 904],
       "InsetString"], 
      TagBox[InsetBox["", 905],
       "InsetString"], 
      TagBox[InsetBox["", 906],
       "InsetString"], 
      TagBox[InsetBox["", 907],
       "InsetString"], 
      TagBox[InsetBox["", 908],
       "InsetString"], 
      TagBox[InsetBox["", 909],
       "InsetString"], 
      TagBox[InsetBox["", 910],
       "InsetString"], 
      TagBox[InsetBox["", 911],
       "InsetString"], 
      TagBox[InsetBox["", 912],
       "InsetString"], 
      TagBox[InsetBox["", 913],
       "InsetString"], 
      TagBox[InsetBox["", 914],
       "InsetString"], 
      TagBox[InsetBox["", 915],
       "InsetString"], 
      TagBox[InsetBox["", 916],
       "InsetString"], 
      TagBox[InsetBox["", 917],
       "InsetString"], 
      TagBox[InsetBox["", 918],
       "InsetString"], 
      TagBox[InsetBox["", 919],
       "InsetString"], 
      TagBox[InsetBox["", 920],
       "InsetString"], 
      TagBox[InsetBox["", 921],
       "InsetString"], 
      TagBox[InsetBox["", 922],
       "InsetString"], 
      TagBox[InsetBox["", 923],
       "InsetString"], 
      TagBox[InsetBox["", 924],
       "InsetString"], 
      TagBox[InsetBox["", 925],
       "InsetString"], 
      TagBox[InsetBox["", 926],
       "InsetString"], 
      TagBox[InsetBox["", 927],
       "InsetString"], 
      TagBox[InsetBox["", 928],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 929],
       "InsetString"], 
      TagBox[InsetBox["", 930],
       "InsetString"], 
      TagBox[InsetBox["", 931],
       "InsetString"], 
      TagBox[InsetBox["", 932],
       "InsetString"], 
      TagBox[InsetBox["", 933],
       "InsetString"], 
      TagBox[InsetBox["", 934],
       "InsetString"], 
      TagBox[InsetBox["", 935],
       "InsetString"], 
      TagBox[InsetBox["", 936],
       "InsetString"], 
      TagBox[InsetBox["", 937],
       "InsetString"], 
      TagBox[InsetBox["", 938],
       "InsetString"], 
      TagBox[InsetBox["", 939],
       "InsetString"], 
      TagBox[InsetBox["", 940],
       "InsetString"], 
      TagBox[InsetBox["", 941],
       "InsetString"], 
      TagBox[InsetBox["", 942],
       "InsetString"], 
      TagBox[InsetBox["", 943],
       "InsetString"], 
      TagBox[InsetBox["", 944],
       "InsetString"], 
      TagBox[InsetBox["", 945],
       "InsetString"], 
      TagBox[InsetBox["", 946],
       "InsetString"], 
      TagBox[InsetBox["", 947],
       "InsetString"], 
      TagBox[InsetBox["", 948],
       "InsetString"], 
      TagBox[InsetBox["", 949],
       "InsetString"], 
      TagBox[InsetBox["", 950],
       "InsetString"], 
      TagBox[InsetBox["", 951],
       "InsetString"], 
      TagBox[InsetBox["", 952],
       "InsetString"], 
      TagBox[InsetBox["", 953],
       "InsetString"], 
      TagBox[InsetBox["", 954],
       "InsetString"], 
      TagBox[InsetBox["", 955],
       "InsetString"], 
      TagBox[InsetBox["", 956],
       "InsetString"], 
      TagBox[InsetBox["", 957],
       "InsetString"], 
      TagBox[InsetBox["", 958],
       "InsetString"], 
      TagBox[InsetBox["", 959],
       "InsetString"], 
      TagBox[InsetBox["", 960],
       "InsetString"], 
      TagBox[InsetBox["", 961],
       "InsetString"], 
      TagBox[InsetBox["", 962],
       "InsetString"], 
      TagBox[InsetBox["", 963],
       "InsetString"], 
      TagBox[InsetBox["", 964],
       "InsetString"], 
      TagBox[InsetBox["", 965],
       "InsetString"], 
      TagBox[InsetBox["", 966],
       "InsetString"], 
      TagBox[InsetBox["", 967],
       "InsetString"], 
      TagBox[InsetBox["", 968],
       "InsetString"], 
      TagBox[InsetBox["", 969],
       "InsetString"], 
      TagBox[InsetBox["", 970],
       "InsetString"], 
      TagBox[InsetBox["", 971],
       "InsetString"], 
      TagBox[InsetBox["", 972],
       "InsetString"], 
      TagBox[InsetBox["", 973],
       "InsetString"], 
      TagBox[InsetBox["", 974],
       "InsetString"], 
      TagBox[InsetBox["", 975],
       "InsetString"], 
      TagBox[InsetBox["", 976],
       "InsetString"], 
      TagBox[InsetBox["", 977],
       "InsetString"], 
      TagBox[InsetBox["", 978],
       "InsetString"], 
      TagBox[InsetBox["", 979],
       "InsetString"], 
      TagBox[InsetBox["", 980],
       "InsetString"], 
      TagBox[InsetBox["", 981],
       "InsetString"]}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{3.45*^9, 380.},
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"Google\"", 14, 
            RGBColor[1, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{Automatic, Automatic}, {{{
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3451334400, 3.4513344*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3454012800, 3.4540128*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3456604800, 3.4566048*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3459283200, 3.4592832*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3461961600, 3.4619616*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
       NCache[3451334400, 3.4513344*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3454012800, 3.4540128*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3456604800, 3.4566048*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3459283200, 3.4592832*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
       NCache[3461961600, 3.4619616*^9], 
       FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
  GridLines->NCache[{{{3450124800, 
       GrayLevel[0.8]}, {3452803200, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3458073600, 
       GrayLevel[0.8]}, {3460752000, 
       GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
       GrayLevel[0.8]}, {3.4528032*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.4580736*^9, 
       GrayLevel[0.8]}, {3.460752*^9, 
       GrayLevel[0.8]}}, Automatic}],
  GridLinesStyle->{
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]], 
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]]},
  ImagePadding->{{50, 10}, {20, 2}},
  ImageSize->600,
  PlotRange->{All, {384.69, 501.98999999999995`}},
  PlotRangeClipping->True,
  PlotRangePadding->{200000, 5},
  Ticks->None]], "Output",
 CellChangeTimes->{{3.4627237232796593`*^9, 3.462723741041065*^9}, 
   3.462728587808609*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "VolumePlot"}]], "Input",
 CellChangeTimes->{{3.462723154628779*^9, 3.462723158249152*^9}}],

Cell[BoxData[
 StyleBox["\<\"VolumePlot[data] will generate a plot of trading volume. The \
data is a list of stock data with the form {dates, open, high, low, close, \
volume}\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728592594623*^9},
 CellTags->"Info3462710591-9739051"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VolumePlot", "[", "goog1", "]"}]], "Input",
 CellChangeTimes->{{3.462723764210731*^9, 3.462723769594969*^9}, 
   3.4627238004912977`*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmflfjekbx8/QMGSmyIRshywhlG00Y+kpW2Emsg8zaRgSOvaxTQfVWFKH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   
   "], {{{}, {}, {}, {}, {}, 
     {RGBColor[0.75, 0., 0.], Thickness[Large], LineBox[{372, 1}], 
      LineBox[{373, 2}], LineBox[{374, 3}], LineBox[{375, 4}], 
      LineBox[{376, 5}], LineBox[{377, 6}], LineBox[{378, 7}], 
      LineBox[{379, 8}], LineBox[{380, 9}], LineBox[{381, 10}], 
      LineBox[{382, 11}], LineBox[{383, 12}], LineBox[{384, 13}], 
      LineBox[{385, 14}], LineBox[{386, 15}], LineBox[{387, 16}], 
      LineBox[{388, 17}], LineBox[{389, 18}], LineBox[{390, 19}], 
      LineBox[{391, 20}], LineBox[{392, 21}], LineBox[{393, 22}], 
      LineBox[{394, 23}], LineBox[{395, 24}], LineBox[{396, 25}], 
      LineBox[{397, 26}], LineBox[{398, 27}], LineBox[{399, 28}], 
      LineBox[{400, 29}], LineBox[{401, 30}], LineBox[{402, 31}], 
      LineBox[{403, 32}], LineBox[{404, 33}], LineBox[{405, 34}], 
      LineBox[{406, 35}], LineBox[{407, 36}], LineBox[{408, 37}], 
      LineBox[{409, 38}], LineBox[{410, 39}], LineBox[{411, 40}], 
      LineBox[{412, 41}], LineBox[{413, 42}], LineBox[{414, 43}], 
      LineBox[{415, 44}], LineBox[{416, 45}], LineBox[{417, 46}], 
      LineBox[{418, 47}], LineBox[{419, 48}], LineBox[{420, 49}], 
      LineBox[{421, 50}], LineBox[{422, 51}], LineBox[{423, 52}], 
      LineBox[{424, 53}], LineBox[{425, 54}], LineBox[{426, 55}], 
      LineBox[{427, 56}], LineBox[{428, 57}], LineBox[{429, 58}], 
      LineBox[{430, 59}], LineBox[{431, 60}], LineBox[{432, 61}], 
      LineBox[{433, 62}], LineBox[{434, 63}], LineBox[{435, 64}], 
      LineBox[{436, 65}], LineBox[{437, 66}], LineBox[{438, 67}], 
      LineBox[{439, 68}], LineBox[{440, 69}], LineBox[{441, 70}], 
      LineBox[{442, 71}], LineBox[{443, 72}], LineBox[{444, 73}], 
      LineBox[{445, 74}], LineBox[{446, 75}], LineBox[{447, 76}], 
      LineBox[{448, 77}], LineBox[{449, 78}], LineBox[{450, 79}], 
      LineBox[{451, 80}], LineBox[{452, 81}], LineBox[{453, 82}], 
      LineBox[{454, 83}], LineBox[{455, 84}], LineBox[{456, 85}], 
      LineBox[{457, 86}], LineBox[{458, 87}], LineBox[{459, 88}], 
      LineBox[{460, 89}], LineBox[{461, 90}], LineBox[{462, 91}], 
      LineBox[{463, 92}], LineBox[{464, 93}], LineBox[{465, 94}], 
      LineBox[{466, 95}], LineBox[{467, 96}], LineBox[{468, 97}], 
      LineBox[{469, 98}], LineBox[{470, 99}], LineBox[{471, 100}], 
      LineBox[{472, 101}], LineBox[{473, 102}], LineBox[{474, 103}], 
      LineBox[{475, 104}], LineBox[{476, 105}], LineBox[{477, 106}], 
      LineBox[{478, 107}], LineBox[{479, 108}], LineBox[{480, 109}], 
      LineBox[{481, 110}], LineBox[{482, 111}], LineBox[{483, 112}], 
      LineBox[{484, 113}], LineBox[{485, 114}], LineBox[{486, 115}], 
      LineBox[{487, 116}], LineBox[{488, 117}], LineBox[{489, 118}], 
      LineBox[{490, 119}], LineBox[{491, 120}], LineBox[{492, 121}], 
      LineBox[{493, 122}], LineBox[{494, 123}], LineBox[{495, 124}], 
      LineBox[{496, 125}]}, {}, 
     {RGBColor[0, 0.5019607843137255, 0], Thickness[Large], 
      LineBox[{249, 126}], LineBox[{250, 127}], LineBox[{251, 128}], 
      LineBox[{252, 129}], LineBox[{253, 130}], LineBox[{254, 131}], 
      LineBox[{255, 132}], LineBox[{256, 133}], LineBox[{257, 134}], 
      LineBox[{258, 135}], LineBox[{259, 136}], LineBox[{260, 137}], 
      LineBox[{261, 138}], LineBox[{262, 139}], LineBox[{263, 140}], 
      LineBox[{264, 141}], LineBox[{265, 142}], LineBox[{266, 143}], 
      LineBox[{267, 144}], LineBox[{268, 145}], LineBox[{269, 146}], 
      LineBox[{270, 147}], LineBox[{271, 148}], LineBox[{272, 149}], 
      LineBox[{273, 150}], LineBox[{274, 151}], LineBox[{275, 152}], 
      LineBox[{276, 153}], LineBox[{277, 154}], LineBox[{278, 155}], 
      LineBox[{279, 156}], LineBox[{280, 157}], LineBox[{281, 158}], 
      LineBox[{282, 159}], LineBox[{283, 160}], LineBox[{284, 161}], 
      LineBox[{285, 162}], LineBox[{286, 163}], LineBox[{287, 164}], 
      LineBox[{288, 165}], LineBox[{289, 166}], LineBox[{290, 167}], 
      LineBox[{291, 168}], LineBox[{292, 169}], LineBox[{293, 170}], 
      LineBox[{294, 171}], LineBox[{295, 172}], LineBox[{296, 173}], 
      LineBox[{297, 174}], LineBox[{298, 175}], LineBox[{299, 176}], 
      LineBox[{300, 177}], LineBox[{301, 178}], LineBox[{302, 179}], 
      LineBox[{303, 180}], LineBox[{304, 181}], LineBox[{305, 182}], 
      LineBox[{306, 183}], LineBox[{307, 184}], LineBox[{308, 185}], 
      LineBox[{309, 186}], LineBox[{310, 187}], LineBox[{311, 188}], 
      LineBox[{312, 189}], LineBox[{313, 190}], LineBox[{314, 191}], 
      LineBox[{315, 192}], LineBox[{316, 193}], LineBox[{317, 194}], 
      LineBox[{318, 195}], LineBox[{319, 196}], LineBox[{320, 197}], 
      LineBox[{321, 198}], LineBox[{322, 199}], LineBox[{323, 200}], 
      LineBox[{324, 201}], LineBox[{325, 202}], LineBox[{326, 203}], 
      LineBox[{327, 204}], LineBox[{328, 205}], LineBox[{329, 206}], 
      LineBox[{330, 207}], LineBox[{331, 208}], LineBox[{332, 209}], 
      LineBox[{333, 210}], LineBox[{334, 211}], LineBox[{335, 212}], 
      LineBox[{336, 213}], LineBox[{337, 214}], LineBox[{338, 215}], 
      LineBox[{339, 216}], LineBox[{340, 217}], LineBox[{341, 218}], 
      LineBox[{342, 219}], LineBox[{343, 220}], LineBox[{344, 221}], 
      LineBox[{345, 222}], LineBox[{346, 223}], LineBox[{347, 224}], 
      LineBox[{348, 225}], LineBox[{349, 226}], LineBox[{350, 227}], 
      LineBox[{351, 228}], LineBox[{352, 229}], LineBox[{353, 230}], 
      LineBox[{354, 231}], LineBox[{355, 232}], LineBox[{356, 233}], 
      LineBox[{357, 234}], LineBox[{358, 235}], LineBox[{359, 236}], 
      LineBox[{360, 237}], LineBox[{361, 238}], LineBox[{362, 239}], 
      LineBox[{363, 240}], LineBox[{364, 241}], LineBox[{365, 242}], 
      LineBox[{366, 243}], LineBox[{367, 244}], LineBox[{368, 245}], 
      LineBox[{369, 246}], LineBox[{370, 247}], LineBox[{371, 248}]}}, {
     {Hue[0.67, 0.6, 0.6], 
      TagBox[InsetBox["", 497],
       "InsetString"], 
      TagBox[InsetBox["", 498],
       "InsetString"], 
      TagBox[InsetBox["", 499],
       "InsetString"], 
      TagBox[InsetBox["", 500],
       "InsetString"], 
      TagBox[InsetBox["", 501],
       "InsetString"], 
      TagBox[InsetBox["", 502],
       "InsetString"], 
      TagBox[InsetBox["", 503],
       "InsetString"], 
      TagBox[InsetBox["", 504],
       "InsetString"], 
      TagBox[InsetBox["", 505],
       "InsetString"], 
      TagBox[InsetBox["", 506],
       "InsetString"], 
      TagBox[InsetBox["", 507],
       "InsetString"], 
      TagBox[InsetBox["", 508],
       "InsetString"], 
      TagBox[InsetBox["", 509],
       "InsetString"], 
      TagBox[InsetBox["", 510],
       "InsetString"], 
      TagBox[InsetBox["", 511],
       "InsetString"], 
      TagBox[InsetBox["", 512],
       "InsetString"], 
      TagBox[InsetBox["", 513],
       "InsetString"], 
      TagBox[InsetBox["", 514],
       "InsetString"], 
      TagBox[InsetBox["", 515],
       "InsetString"], 
      TagBox[InsetBox["", 516],
       "InsetString"], 
      TagBox[InsetBox["", 517],
       "InsetString"], 
      TagBox[InsetBox["", 518],
       "InsetString"], 
      TagBox[InsetBox["", 519],
       "InsetString"], 
      TagBox[InsetBox["", 520],
       "InsetString"], 
      TagBox[InsetBox["", 521],
       "InsetString"], 
      TagBox[InsetBox["", 522],
       "InsetString"], 
      TagBox[InsetBox["", 523],
       "InsetString"], 
      TagBox[InsetBox["", 524],
       "InsetString"], 
      TagBox[InsetBox["", 525],
       "InsetString"], 
      TagBox[InsetBox["", 526],
       "InsetString"], 
      TagBox[InsetBox["", 527],
       "InsetString"], 
      TagBox[InsetBox["", 528],
       "InsetString"], 
      TagBox[InsetBox["", 529],
       "InsetString"], 
      TagBox[InsetBox["", 530],
       "InsetString"], 
      TagBox[InsetBox["", 531],
       "InsetString"], 
      TagBox[InsetBox["", 532],
       "InsetString"], 
      TagBox[InsetBox["", 533],
       "InsetString"], 
      TagBox[InsetBox["", 534],
       "InsetString"], 
      TagBox[InsetBox["", 535],
       "InsetString"], 
      TagBox[InsetBox["", 536],
       "InsetString"], 
      TagBox[InsetBox["", 537],
       "InsetString"], 
      TagBox[InsetBox["", 538],
       "InsetString"], 
      TagBox[InsetBox["", 539],
       "InsetString"], 
      TagBox[InsetBox["", 540],
       "InsetString"], 
      TagBox[InsetBox["", 541],
       "InsetString"], 
      TagBox[InsetBox["", 542],
       "InsetString"], 
      TagBox[InsetBox["", 543],
       "InsetString"], 
      TagBox[InsetBox["", 544],
       "InsetString"], 
      TagBox[InsetBox["", 545],
       "InsetString"], 
      TagBox[InsetBox["", 546],
       "InsetString"], 
      TagBox[InsetBox["", 547],
       "InsetString"], 
      TagBox[InsetBox["", 548],
       "InsetString"], 
      TagBox[InsetBox["", 549],
       "InsetString"], 
      TagBox[InsetBox["", 550],
       "InsetString"], 
      TagBox[InsetBox["", 551],
       "InsetString"], 
      TagBox[InsetBox["", 552],
       "InsetString"], 
      TagBox[InsetBox["", 553],
       "InsetString"], 
      TagBox[InsetBox["", 554],
       "InsetString"], 
      TagBox[InsetBox["", 555],
       "InsetString"], 
      TagBox[InsetBox["", 556],
       "InsetString"], 
      TagBox[InsetBox["", 557],
       "InsetString"], 
      TagBox[InsetBox["", 558],
       "InsetString"], 
      TagBox[InsetBox["", 559],
       "InsetString"], 
      TagBox[InsetBox["", 560],
       "InsetString"], 
      TagBox[InsetBox["", 561],
       "InsetString"], 
      TagBox[InsetBox["", 562],
       "InsetString"], 
      TagBox[InsetBox["", 563],
       "InsetString"], 
      TagBox[InsetBox["", 564],
       "InsetString"], 
      TagBox[InsetBox["", 565],
       "InsetString"], 
      TagBox[InsetBox["", 566],
       "InsetString"], 
      TagBox[InsetBox["", 567],
       "InsetString"], 
      TagBox[InsetBox["", 568],
       "InsetString"], 
      TagBox[InsetBox["", 569],
       "InsetString"], 
      TagBox[InsetBox["", 570],
       "InsetString"], 
      TagBox[InsetBox["", 571],
       "InsetString"], 
      TagBox[InsetBox["", 572],
       "InsetString"], 
      TagBox[InsetBox["", 573],
       "InsetString"], 
      TagBox[InsetBox["", 574],
       "InsetString"], 
      TagBox[InsetBox["", 575],
       "InsetString"], 
      TagBox[InsetBox["", 576],
       "InsetString"], 
      TagBox[InsetBox["", 577],
       "InsetString"], 
      TagBox[InsetBox["", 578],
       "InsetString"], 
      TagBox[InsetBox["", 579],
       "InsetString"], 
      TagBox[InsetBox["", 580],
       "InsetString"], 
      TagBox[InsetBox["", 581],
       "InsetString"], 
      TagBox[InsetBox["", 582],
       "InsetString"], 
      TagBox[InsetBox["", 583],
       "InsetString"], 
      TagBox[InsetBox["", 584],
       "InsetString"], 
      TagBox[InsetBox["", 585],
       "InsetString"], 
      TagBox[InsetBox["", 586],
       "InsetString"], 
      TagBox[InsetBox["", 587],
       "InsetString"], 
      TagBox[InsetBox["", 588],
       "InsetString"], 
      TagBox[InsetBox["", 589],
       "InsetString"], 
      TagBox[InsetBox["", 590],
       "InsetString"], 
      TagBox[InsetBox["", 591],
       "InsetString"], 
      TagBox[InsetBox["", 592],
       "InsetString"], 
      TagBox[InsetBox["", 593],
       "InsetString"], 
      TagBox[InsetBox["", 594],
       "InsetString"], 
      TagBox[InsetBox["", 595],
       "InsetString"], 
      TagBox[InsetBox["", 596],
       "InsetString"], 
      TagBox[InsetBox["", 597],
       "InsetString"], 
      TagBox[InsetBox["", 598],
       "InsetString"], 
      TagBox[InsetBox["", 599],
       "InsetString"], 
      TagBox[InsetBox["", 600],
       "InsetString"], 
      TagBox[InsetBox["", 601],
       "InsetString"], 
      TagBox[InsetBox["", 602],
       "InsetString"], 
      TagBox[InsetBox["", 603],
       "InsetString"], 
      TagBox[InsetBox["", 604],
       "InsetString"], 
      TagBox[InsetBox["", 605],
       "InsetString"], 
      TagBox[InsetBox["", 606],
       "InsetString"], 
      TagBox[InsetBox["", 607],
       "InsetString"], 
      TagBox[InsetBox["", 608],
       "InsetString"], 
      TagBox[InsetBox["", 609],
       "InsetString"], 
      TagBox[InsetBox["", 610],
       "InsetString"], 
      TagBox[InsetBox["", 611],
       "InsetString"], 
      TagBox[InsetBox["", 612],
       "InsetString"], 
      TagBox[InsetBox["", 613],
       "InsetString"], 
      TagBox[InsetBox["", 614],
       "InsetString"], 
      TagBox[InsetBox["", 615],
       "InsetString"], 
      TagBox[InsetBox["", 616],
       "InsetString"], 
      TagBox[InsetBox["", 617],
       "InsetString"], 
      TagBox[InsetBox["", 618],
       "InsetString"], 
      TagBox[InsetBox["", 619],
       "InsetString"], 
      TagBox[InsetBox["", 620],
       "InsetString"], 
      TagBox[InsetBox["", 621],
       "InsetString"]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      TagBox[InsetBox["", 622],
       "InsetString"], 
      TagBox[InsetBox["", 623],
       "InsetString"], 
      TagBox[InsetBox["", 624],
       "InsetString"], 
      TagBox[InsetBox["", 625],
       "InsetString"], 
      TagBox[InsetBox["", 626],
       "InsetString"], 
      TagBox[InsetBox["", 627],
       "InsetString"], 
      TagBox[InsetBox["", 628],
       "InsetString"], 
      TagBox[InsetBox["", 629],
       "InsetString"], 
      TagBox[InsetBox["", 630],
       "InsetString"], 
      TagBox[InsetBox["", 631],
       "InsetString"], 
      TagBox[InsetBox["", 632],
       "InsetString"], 
      TagBox[InsetBox["", 633],
       "InsetString"], 
      TagBox[InsetBox["", 634],
       "InsetString"], 
      TagBox[InsetBox["", 635],
       "InsetString"], 
      TagBox[InsetBox["", 636],
       "InsetString"], 
      TagBox[InsetBox["", 637],
       "InsetString"], 
      TagBox[InsetBox["", 638],
       "InsetString"], 
      TagBox[InsetBox["", 639],
       "InsetString"], 
      TagBox[InsetBox["", 640],
       "InsetString"], 
      TagBox[InsetBox["", 641],
       "InsetString"], 
      TagBox[InsetBox["", 642],
       "InsetString"], 
      TagBox[InsetBox["", 643],
       "InsetString"], 
      TagBox[InsetBox["", 644],
       "InsetString"], 
      TagBox[InsetBox["", 645],
       "InsetString"], 
      TagBox[InsetBox["", 646],
       "InsetString"], 
      TagBox[InsetBox["", 647],
       "InsetString"], 
      TagBox[InsetBox["", 648],
       "InsetString"], 
      TagBox[InsetBox["", 649],
       "InsetString"], 
      TagBox[InsetBox["", 650],
       "InsetString"], 
      TagBox[InsetBox["", 651],
       "InsetString"], 
      TagBox[InsetBox["", 652],
       "InsetString"], 
      TagBox[InsetBox["", 653],
       "InsetString"], 
      TagBox[InsetBox["", 654],
       "InsetString"], 
      TagBox[InsetBox["", 655],
       "InsetString"], 
      TagBox[InsetBox["", 656],
       "InsetString"], 
      TagBox[InsetBox["", 657],
       "InsetString"], 
      TagBox[InsetBox["", 658],
       "InsetString"], 
      TagBox[InsetBox["", 659],
       "InsetString"], 
      TagBox[InsetBox["", 660],
       "InsetString"], 
      TagBox[InsetBox["", 661],
       "InsetString"], 
      TagBox[InsetBox["", 662],
       "InsetString"], 
      TagBox[InsetBox["", 663],
       "InsetString"], 
      TagBox[InsetBox["", 664],
       "InsetString"], 
      TagBox[InsetBox["", 665],
       "InsetString"], 
      TagBox[InsetBox["", 666],
       "InsetString"], 
      TagBox[InsetBox["", 667],
       "InsetString"], 
      TagBox[InsetBox["", 668],
       "InsetString"], 
      TagBox[InsetBox["", 669],
       "InsetString"], 
      TagBox[InsetBox["", 670],
       "InsetString"], 
      TagBox[InsetBox["", 671],
       "InsetString"], 
      TagBox[InsetBox["", 672],
       "InsetString"], 
      TagBox[InsetBox["", 673],
       "InsetString"], 
      TagBox[InsetBox["", 674],
       "InsetString"], 
      TagBox[InsetBox["", 675],
       "InsetString"], 
      TagBox[InsetBox["", 676],
       "InsetString"], 
      TagBox[InsetBox["", 677],
       "InsetString"], 
      TagBox[InsetBox["", 678],
       "InsetString"], 
      TagBox[InsetBox["", 679],
       "InsetString"], 
      TagBox[InsetBox["", 680],
       "InsetString"], 
      TagBox[InsetBox["", 681],
       "InsetString"], 
      TagBox[InsetBox["", 682],
       "InsetString"], 
      TagBox[InsetBox["", 683],
       "InsetString"], 
      TagBox[InsetBox["", 684],
       "InsetString"], 
      TagBox[InsetBox["", 685],
       "InsetString"], 
      TagBox[InsetBox["", 686],
       "InsetString"], 
      TagBox[InsetBox["", 687],
       "InsetString"], 
      TagBox[InsetBox["", 688],
       "InsetString"], 
      TagBox[InsetBox["", 689],
       "InsetString"], 
      TagBox[InsetBox["", 690],
       "InsetString"], 
      TagBox[InsetBox["", 691],
       "InsetString"], 
      TagBox[InsetBox["", 692],
       "InsetString"], 
      TagBox[InsetBox["", 693],
       "InsetString"], 
      TagBox[InsetBox["", 694],
       "InsetString"], 
      TagBox[InsetBox["", 695],
       "InsetString"], 
      TagBox[InsetBox["", 696],
       "InsetString"], 
      TagBox[InsetBox["", 697],
       "InsetString"], 
      TagBox[InsetBox["", 698],
       "InsetString"], 
      TagBox[InsetBox["", 699],
       "InsetString"], 
      TagBox[InsetBox["", 700],
       "InsetString"], 
      TagBox[InsetBox["", 701],
       "InsetString"], 
      TagBox[InsetBox["", 702],
       "InsetString"], 
      TagBox[InsetBox["", 703],
       "InsetString"], 
      TagBox[InsetBox["", 704],
       "InsetString"], 
      TagBox[InsetBox["", 705],
       "InsetString"], 
      TagBox[InsetBox["", 706],
       "InsetString"], 
      TagBox[InsetBox["", 707],
       "InsetString"], 
      TagBox[InsetBox["", 708],
       "InsetString"], 
      TagBox[InsetBox["", 709],
       "InsetString"], 
      TagBox[InsetBox["", 710],
       "InsetString"], 
      TagBox[InsetBox["", 711],
       "InsetString"], 
      TagBox[InsetBox["", 712],
       "InsetString"], 
      TagBox[InsetBox["", 713],
       "InsetString"], 
      TagBox[InsetBox["", 714],
       "InsetString"], 
      TagBox[InsetBox["", 715],
       "InsetString"], 
      TagBox[InsetBox["", 716],
       "InsetString"], 
      TagBox[InsetBox["", 717],
       "InsetString"], 
      TagBox[InsetBox["", 718],
       "InsetString"], 
      TagBox[InsetBox["", 719],
       "InsetString"], 
      TagBox[InsetBox["", 720],
       "InsetString"], 
      TagBox[InsetBox["", 721],
       "InsetString"], 
      TagBox[InsetBox["", 722],
       "InsetString"], 
      TagBox[InsetBox["", 723],
       "InsetString"], 
      TagBox[InsetBox["", 724],
       "InsetString"], 
      TagBox[InsetBox["", 725],
       "InsetString"], 
      TagBox[InsetBox["", 726],
       "InsetString"], 
      TagBox[InsetBox["", 727],
       "InsetString"], 
      TagBox[InsetBox["", 728],
       "InsetString"], 
      TagBox[InsetBox["", 729],
       "InsetString"], 
      TagBox[InsetBox["", 730],
       "InsetString"], 
      TagBox[InsetBox["", 731],
       "InsetString"], 
      TagBox[InsetBox["", 732],
       "InsetString"], 
      TagBox[InsetBox["", 733],
       "InsetString"], 
      TagBox[InsetBox["", 734],
       "InsetString"], 
      TagBox[InsetBox["", 735],
       "InsetString"], 
      TagBox[InsetBox["", 736],
       "InsetString"], 
      TagBox[InsetBox["", 737],
       "InsetString"], 
      TagBox[InsetBox["", 738],
       "InsetString"], 
      TagBox[InsetBox["", 739],
       "InsetString"], 
      TagBox[InsetBox["", 740],
       "InsetString"], 
      TagBox[InsetBox["", 741],
       "InsetString"], 
      TagBox[InsetBox["", 742],
       "InsetString"], 
      TagBox[InsetBox["", 743],
       "InsetString"], 
      TagBox[InsetBox["", 744],
       "InsetString"]}, {}}}],
  AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
  AxesOrigin->{3.435*^9, 0.2},
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-7\\)\\))\"", Bold, 
            GrayLevel[0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{Automatic, Automatic}, {{{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"Jan\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"Apr\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"Jul\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}, {{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3447532800, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3463344000, 
       GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4475328*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.463344*^9, 
       GrayLevel[0.8]}}, Automatic}],
  GridLinesStyle->Directive[
    GrayLevel[0.9], 
    Dashing[{Small, Small}]],
  ImagePadding->{{50, 10}, {20, 4}},
  ImageSize->600,
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{200000, 200000}, {0, 1}},
  Ticks->None]], "Output",
 CellChangeTimes->{3.462723801898098*^9}]
}, Open  ]],

Cell[TextData[{
 "The following 3 types of plots are used by technical traders. (",
 ButtonBox["http://en.wikipedia.org/wiki/Category:Technical_analysis",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://en.wikipedia.org/wiki/Category:Technical_analysis"], None},
  ButtonNote->"http://en.wikipedia.org/wiki/Category:Technical_analysis"],
 ")"
}], "Text",
 CellChangeTimes->{{3.4627286110424423`*^9, 3.4627286290970373`*^9}, {
  3.462789092843791*^9, 3.462789094299199*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "MACDPlot"}]], "Input",
 CellChangeTimes->{{3.462723143280879*^9, 3.462723147272822*^9}}],

Cell[BoxData[
 StyleBox["\<\"MACDPlot[data] will generate an MACD plot of the stock data. \
The data is a list of stock data with the form {dates, open, high, low, \
close, volume}.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728602054475*^9},
 CellTags->"Info3462710601-5758356"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MACDPlot", "[", "goog1", "]"}]], "Input",
 CellChangeTimes->{3.46272382185179*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmnlcTN/j/0eSZIuoSDUK7xRJKWXJRCUhSYlS76FFaOONEjFEtiQlS6GR
rCVJKlTGllKSnRSjEllqkiVbfve8zudh7n08vv/+/nP/6fF8nGnmdc892/Pc
M3BBiIufAo/He6zI45G/zMWftavRhie/nCMJD/0xVONt2wSGRccJSw7c6mTV
XcBw1l3CRyWKihHdCEt/MCxaEnljkUMXwqER8Ux5VMQ264/4f3Ea4ZTcdOWE
H5cZrqwgPG1h3eoUNQn5wW8MC0KiQxMH8Qmb6Ccw33d5ySTl0EGEJYcZlsQX
DnVZp0tYVkb43IPJJqnahPlfGBavs00NyBhI2Jm/u9GGP9azVE8EFjkyLOo3
8syk/gMIq5YyzJvQ4+Jdiz6EBa2k/P1DI3MN5AnVTmTyOO2L3ChAuXgyw7xn
+f068dQJVy4l3H5v9D17LcJCGcMiW+X2rtr4vbj+e8j99bg9/kp/5Lcl3HOs
zlElTeQPZlgw4pu2kVpf5N+Pz8eEFYl7E87S2MuUvz/+7m0Sfl9qw7Ao0vdW
cWkP5A9kWPIhWGOFIvIK9hBerq1S/08/5JcwzDvOXzk6B/XDm7Cv0UZYNmNf
boge6ncRw2Kj7q73zIYgfwLDgtZ5jT0MhyF/IcO88w/dzwwZjvyvyef1b9nl
vAM7++9nvm/NfT/dayNQv3EMZ32VRqkcMUH+iwybDOtrMXSxKfLXM8zbpZ2r
ZW2O/D2TGJ4c81/+TwvU7w6GBV5TqsMjLVG/eQxLzzU51QeORv5ahmWyGOPP
m/D/QovkRhvVhOIvr/LMUJ85DIeW3C0ZOHYU6vMFw1kJBl8zM1Du3OUAUz8R
JWEr0pFHZMawavH3vAA1cJYXwyaaUXERqcgvqGaYPyfiUn6hEepT6SCTL12n
o1sE6ktswrDUx2hTThqed6UHwzy38jHp3XWQdxPDogOvbT8cw/OL63iI4ZD0
hoy53VGfwxnm6Z/OMD7+ifQHmTthfsMRywVlhPkbCIeP3aQ6uYL0H+cMwprv
k9XW1RKWGqYwrHzLd/3XUvJ5VTfCST6vtTNeExasI+wbbSiY/Iaw+D7h3s4d
otSbCZu4iBm2OHl+aelnwsI1hAXXdj0ybCUcd4ywtWLBsFENyPedsOrQaovE
RowHqw4zvOrF4SnfO2I8OMKwaMaigA7avTAe3GZYkBGsdv54b8KqbYQ/yMZM
OdWHsEAvlfm80qPJkcUorxSnkv5x/o2NiyphXhlhV79/h4VhvDH5TLiXSoeP
OgqEhbpHyP3sTHjYrZnkiZtCmCfUSTzqRPI6fyQ8r8J1fz0Pz3tAGsPDu30+
/gmcZU/49pM5vSb+Ip+XhhJ2MIx/6yshHNpMeEzOiaCxHzF+9TvK5H37Y9iM
yk7IO4lwlmvJm/F9kTeYYZ7u5iflPlrIu49hicIhz3wvHcIS9WNMeY+kO54i
PmGZgGGB8flehbMGEuYvIbzygH2rsR5h50SGxUEzhZGegwiLLjPMX2JU5X7Y
APVpfZzhhIhZi+qHoT4DGBbuDHJYmQ4OjSflnzZnd9AEiwsYFo+NKB6kC65s
IKwweoPXV0PCcTtPML9f3MFgrec/yHuBYZ6x623XpsHIW8cwvzbRUFyPfPwe
J5n705sRV/Ea95MVw7CgLk73UZ9+eP65DPNyPwd/qFFH3pek/N7JtAfH1ZC3
6ymm/m7ZGQ3+2QN5zRnmFQ9fpCFWRn3mELZ8MfxKA+YPk+eENbVGhxuVERYq
pzOs8Wz56Bq03zhThkWzJw/fPpz2r3mEt2gcOZaK/uf8jHzeZ4Ca3030T1Gn
DOb+dywJtQvGfJM1gmHJXaevEgXML9K5DPMuXSriLcb4rrqRlDu22st+a6D/
K5xmvn/QuarInXQ+GMawIMn9V7KxKvq/OykfqfJUK7gzYZP1hDsbzdQxViAs
TGeYV51d4xSK+VA2NJPhkg3mpaa4P74r4dfrXJZnf0P/X5tJ5qf5Wt4r0N9E
JwjXJDcem4b6y7qHchWrM6Hof4KZZxh2c3reGgIOXc0w7/QSG8NW9DfxUYYF
wpOn7z1G+628w7BkiPie9/b+qP/vhJMKd97UxfONC89iPu89Q63qA9qrJJVh
6V2P7S+TwbJywlaxZwecpe35K8OiTOP0n2P00R5SzjLtU3h2fvIAtCfVWwwL
Tsd67ZOifQs+MSwWGYeaDR6CvDrZTHs7kXbRaR7au9iB8Jta7fVCtFeTm9lk
flJ6GLZ3KMaDFoYF/NgRTZ3BcVrnmHLHsF6qWfh/iR3D4tC6R5p2YFkIKTfa
22VrMFjUxLDgof4h/Ty0/yzNHOZ+FOtri6ORRzqRYWHK78ptCWDVIFK+JHTL
KjV8XrCXlKd+HDrpMcor+55nvj92a7tKPsp5AoYFqhUjzKfh90wWMyxtEGQr
7AALdzMsPGSm8u1ffD6uiGFx+evt116B+eNzmfJZ5RGL9fF554WETbM7Nx/C
74l2MSzy23Wk9CTqO+sSw1Il5S2vyvD8pK8Y5jV4SJU3aqN+ffMY/rRd/7Xh
ANRvLMOi8YJRX0MxflXmMyy41GWGlyJtD3WEs3Q2TvTRRP7u+cz/Lz9ydKIU
7Ud2nmEBL3Fgia4G8kpJ+bQWz0xLfN5Z5QLTnowObloxF98nGsUwL81KOusV
8qieY1h8rnuXG8fRfgQ1hIMnqC97jvsJ7XyR+X43pzzTV2g/4pEM82dPSxpz
APVR6UnKK2SLv/czQn1WkfI9xSX1z8BxipeY8m6DGuoOgyXGDPNk2UPXnEB7
ks0h5RZJ5vaKaD/8KIal+76EGpSgfrM6FDB8sjbh6Wrano0I86r8VGNRv6qz
GZaM3NpZRYr6E4gYFp02MG7uj/oIPUXKq/dO8+1M54uhhczvzzUbcGck+q/J
LIZFhhvFkZHo38JIUn7qZeJz7++Y344TLj+gqbUS46HkLvn8u5POOqEdUL/O
RaS/z8yqi1NE/UYwzNuZ+l/GP5ivstIIB0UFueTh+6UVhK8N0tg96yvmt7DL
DIsn9HMpek248jDh/ho61gfekfGJV064zHW7zpTvWD98Ify6dZJ75jesH/gS
hitf5zWUYf0gO0RY+YnqrX1XsZ4pJRw6MLe+GN/v3CohvnDXVqEbzat9heGC
hf7vnmD+z5rMsOBaU4fknhivBMWEv6RElN0Eh8oIVxxdXLuQjmf9r5L1w5Xq
uocY7yptGZZ0Gvzt8iSsN3ghDIuMEnuqNqE9xn0grFBzzDwLz0uicY35/+wB
Om6fwDKba2R9YjVl0lm0V34gKX/0/DzfH/3FeQ8p3164+EciWNrnOlOe+aG7
q6su2sMEhgXWYzZn3kb7ECwi5bk+N+w00L5DExiWvJneYv4b5eJChqUbvQJ6
VKDcZNwNprznviUGBnpoD/43SP08CLpYjvK4OMK+cQqZPVAuuUg+7x8TeysN
LKtnmD90d1NtNFjkU8yUH9rkv1mMfFk7CI9S/XxaH/crzWNY9O+uzv6v6Hxd
S8r9B69ZHov6EnS7ydzPJJWa+Tp0fNhG2GPTHpsm9F/eeYZFQQZfp87E/Zi8
YFhiX/5k0nGwsEsJc/+HczaUL6bziRnDkqvL3nbLQB5+NsOiPZ0XNi/G9zlX
Ex7ew90kGOsJkVIp83v1t30O66P/ZJkwzLvxSzXjG56/1KOU1MfvPWp2tD10
vMV8fukLc6/jaE+VwxmWXFLgp7+m68s5pDzKary6HtYjJhsYFl0NW2fWFd8v
4ZWR9UKGeKkA/VdmWEbGb4Hp4Ejk4bsRFj3ucnA+1nfO6xiWTB130XI1no/o
JMPiA938unbBeKFqUM58Xq3+ctg7Ot+5MMz/fnH1tjUYX/5P/x61eUFTeNP9
P/6tbtE1uWIu1rvw75BB7ya4NqH/wr+HODQ5dZ74izD8+90slVg7v59//NtN
33HGAbMW9G/i35KxKzrMby9C/yb+7esZ5VElxHoE/n21cJvEcHpXCe9//n38
zrVLrpa9CMO/M3y+q7w+B/+Ef5s0qmfVPsB6CP7NS+kY9mUSfBT+PX3jUy2r
h2D4t/UkjYfbzsOH4d9ry470kyZifQX/tl53ePiod/g++HfZ9QnXDAvk/p1q
4XU8Ow2fh3+fl07PT7iK74d/ew+12brhMxj+vXbb0/+2xoPh38vSMxZMysDv
w78N/J76X12J74N/C88paPnW4/fg31UBE+IifeHr8O/gqFrb2rd0/4D496uG
zSm111AO/87779ivuR74Pvh39OQnnXf1wH4A/Lv3WU3PHxfg5/DvybaNuh2i
6f4B8W+lWE0j96f6yE/8uyg81adjzWDUL/Hv3mqrbj18/A/ql/i33oSB489c
HIr8xL+NbWerdWyFv8K/54zPKxI8M0Z+4t+FQ0xNjsrgu/Bv6zKL9+b/wYfh
394Jnv4fl8Kf4d+ZE0w951aD4d+xUY1qPR/K/fvSVw3LVY1g+PeFupzrzjVy
/96cvOnNIDEY/v0otGPvcmcw/NspPnfXlpv4ffj3xPq8D6blI1GfxL8v3Tyz
1HcNzUv8uyj6Use219iPgH9PXX7KbVCTIfIS/67VtdrX4QDqB/7d98WMTXUe
dD+J+HeR1fUfdeVY/8O/ffYf44/vjvU+/PuqU53p5f/wPOHfyS13bDudw/oe
/m03bucyflf0D/h33vXFVevvKiE/8e94t571X8M6Ii/x76vpuSf7DeyA50/8
2yLvhdmxa7//+LdRoEB6xvrXH/9u3nn73+sHv/3x75gWq0rH5hbC8G/JSb6v
fdBTwtS/g5M1G8xfYjwg/p00w6ThfcVnwvDvbVO7Vzn84gl4//Nv5QnC9k79
qc8S/w6yXPD5XUBnwvDvbY8XPG47BB+DfwdP2Zhh6IJy+Ld14q3RCeaYv+Hf
7bXX+0cY4vvh37cytMZd0GrF+EX8W3Wb3oGxCsgH/za5uX7E5RWZyEv8W9Pv
o7Hr+j0Yv4h/aydZZigG3vnj30rLg+0FBXR9QvxbQe9I8/01WB/Bv5PO1Qzo
vgP3A/9eYLhNeaAP9g/g31emPCto98D8AP9eP937buRBjPfwb6cg2ZcbcXS9
Svx7alSEy8EyzHfw75Yp99c5d6XrPeLfN7dfmaR3UA/1Sfx72lzH579TML7D
v1emz07118H4Dv/ONr5efXkm1vPw79rBz36Fd8P6Ff4d+6I2Ra1R7t/L9BIm
XowCw78T+34P1dpL5wvi34qvHqt2W471MPx79qixp/LT6fqU+He43mr7tk7I
B//2j/yv1nMBXY8Q/1aP6+Zm4kx9gPh3eJBn8tVGzG/w72LRzZaib6gf+Ld3
/PX1vSRYX8G/HSvcT5VaqCA/8e+THo98l9q1Y31K/DvnfmKngofPsd4j/h1W
1FZ1fflHtF/i3+ru0WffRyui/xP/9vZvkYXe74b+Rfx7iFZ0tLAj5hv4d+/t
5lL1RdgfhX/HrtR6sWs85h/4t2b9jqnf6f4p/Pv+wBjLdmv8P/z70OgZipVj
ehKGf2tWL26YfBH9F/4trJrUYe/jTuj/xL+TEt+7V1WgP8K/HaaVHtZTlSE/
8W+hx9Z1G+cfR/8j/n1G4exAy4K35H7h3/a9dWOepmG9Dv9uzF68qQNdr8O/
Ow/Yr958Rgntgfi3S8zF8rXXsZ8O/x5zsrTlzSfUN/y772mPn2Fb6fqE+Ldi
uFLABzu0V/h3XI3KwtK1dL1K/Ftg+GZPYB1dnxL/3rH6dopxLF2fEv8Ovxis
O7Kctgfi3yKHkgXtNXR9Svw7yOBK1XJDtCf4t/ebm+tPJaN9w7+n9gqOv5+E
9gf/PhOxfOoER7RP+PfcxzbHFG6A4d8dvhjX3e9F2zPx7wqzU3m73MHw76mb
r77KXCz37wP3F2j2Hw+Gf19bcP6IiRYY/r1gpM2ho/3k/m28euRF7260fxH/
Lpq6cn6Osdy/rY+5DqpYCIZ/x/fLEyxLBcO/tyQtujMkUe7fO9JmGNgflfu3
znzzFFERGP7dtm19/18baX7i3/bOOotiLlNfJP7t12N6yN1Y1A/8e2bzqpAT
pnR8IP5d5V5QffAb+if8e2y/gwp736P+4d+Nx9q3vpHh+cC/39ysjMv5gecH
/+bn3Zw/+gv6L/x7R3zhwIH78Lzh32l5u4Xj8zF+wb9rKn4u7LuMjmfEvx2L
H07b0gyGf69uULfRW4Tvg3+n1nsfqV2F34N/56SY7x/ZRn2F+Pctr6D9/uuQ
H/7d12Zg3NvRuD/4t+OmFckDLuP+4d+1Vn6nwt7T9kD8W8vo2bX0dNoeiH8X
JIxavldG65v4t5exQXen87Q9EP8OOy2K7VhD92OIf89tadNVou0P/t3D/pIo
pRddXxP/3rfGbNNeL+SDf+8oPvTkAN0PgX//GNYxbF497hf+nRpbveVRNcY/
+HfA5pIrFgrwIfh39Mpnn1c9wPwB/za61vr7VgV8Av5tpO4ze1mHnqhf4t/N
N0N2anSh78uIf8dsEOjJjLoiP/Fv/TPaG9/0wvwK/8794DFz6VLMp/BvfeFV
zTMfsZ6Hfw9okX3ddA770fDv4R91u04/Vof9UOLf8z7EmLiMwnwJ/65z6124
zvoG9hOJf8ee72NsoHDvj3/fu3tmTfp4zKfw7yZtf+1RxV8wPxP/7u2xpVu0
G10vEP/2DFMbO3s38sG/S9e+mxt4H+sB+PcYyyNKr0ZiPoB/O4zwvlPZE/cP
/7Y4v7Jg9gHUF/zbzrtoZtR9Wp/Evw8pH4qQHoOvw79NlQtD/6mm8zPxb4P2
L/UfDeFj8G/x9PzfHgKMf/Dv6Td3KkhO4fnBvwdtKrxrZ0P3Y4h/RzUM9vpw
BP0D/p3roJKulIvnD/92Cfmu2bUePgf/jo1fcjT7G9o//Pv9t0stKXOpvxL/
VqmeZtDlLBj+bew95MjPPnQ/lPi3N7/Luq/e1AeJf8evfNls1RMM/9bzPbA9
Zh7dnyP+rS1ynPz5Cn4f/q2y+qTOCrpfB/9e3UUWltYFDP8Oaei+1mu03L9j
HLWMFzXQ9kz8e1vL7cyHu/F98G/lgFmtI5RxP/Bv4em5RZ8N5P6dtGKN/ZU7
1GeJfxsv9K8bX4Dvg3+3T8vco6OI+oN/+5W5144xx3wD/1Z821tcfgjPA/69
csnDbY1KeF7wbxeH9BMrb9P9L+LfWq7zi0tt6fqC+PdPi/KZdlZy/741YdiP
/ofp/iLx79o7pSm5/uiP8G/PIf5aU3/T9kD8e8ERi+EXTOj+IvHv2E8+/X7H
IS/8e06Z7e5ES9wv/Ptao2Cz+0/U///p32bw79Y//h3x8al+/AK6n0b8u6R3
Wf3TDegf8O+q2G7jyp0+//HvOZvXDrDch/d58G/D+aJIFx8lCe9//t0rbP6/
G/Tl77+Pdw50/BQKP4V/6/Ljh6y2go/Av02jW59934H1EfzbQJyzKr4afgH/
5g+rN6jp3IUw/FuU6Tf6UDjK4d/7ThYkuRrU4n0h8e821UGB9d+xHwD/bls9
Oi+dMvxbtnmKKNLtFdbfxL+FVjUm5zfexPqb+LdsTGbUT9UP8Bfi33ucN21W
bsb7Nfi3wDVtVUAY3k/Cv6V5r+faFT3B+ET8O+5l8uCrJe8xPhH/FvuOKuw5
B/UN/+ZZPYoonUrfdxL/DjgxKiOgHfsd8G+ph9Xje/5YX8K/nwRnuHblY78S
/j3o9puXKxbCn+Dfyw/Fzsw/jPUd/HvetI5JLp54XwT//uj1Ol7LAb4H//40
PWJuVhLWk/Dv624zrs/Xhv/Bvy2PLgoachrrSfh3m2SD4d7heD8P//Y1OHH2
0FWsP+HfzqWdPT/YYD8A/t1tfUD36SbYf4B/Kzs4SroOBcO/VRX32+58jO+H
f/P7pJsX7sf+K/ybt1ampz20EPVJ/FswfLnxoMUPsN9K/FukaFDc4FyC8Z74
t1QzTaMpsQntmfi3MGOb2PwRrW/i37LK5C6hBlifwr/jrjxcl7MY8xn8O7Bk
veT1T4zP8O8CO/6v6Gj0d/j3noQtoQcr0B/h37ZHtr3ruwv9Ef69WMlxr0wZ
DP8uMPSa9jME/ZP6d8BmTyNTun4l/r3BsHveVz/0d+rfy6fnv1bAfh38+0by
xHknluC8Cfx79PNdwogGOl8R/x606ln2jHL4Ifx7o9ONey25dP4k/h2oEPQz
4BOdP4l/z57z/UD2P/XYbyf+/fzlno7zOn9A/cK/VzlZN1xo//P+2zLWJ/jj
P5j/4N/Ca+mOW99jvoN/nzO5ca51IeZD+PeX2AfJU9wx38O/225P3fVpGfLC
v4eU9cwfEYZ88O8nLV34FULkg3/nv3xzura8Hs+f+Peaw7ePXqpvw34B/LtE
fbnvMPQ/+LefafXHFctU0D6Jf2+806fawhztH/495pmry2Xz9j/vv4/sVtbs
a37tz/vvVdM9hlQlv//j37+SJBZrr/7+49+/7zzet3A8nd+Jf5vZaTraH8T8
Dv/WeXY2c8YctB/499q0s9eXTsXn4d/t2pppbWqoH/i3b3vcMJsk1Cf8++0Z
5+qLj/B84d/eE7NyegyDD8G/s3ss042NpecJiH8bBNo/eK+O34d/n9C7IQhs
oOsT4t/5Zk+OJZjj/Qf8W/bqud4gd4wn8G/JUP2fmdl3MV4R/+ZPbp7lPrsR
+y/Ev08kDzSrSoAPwr9Vu+grfDHB+1z49xOVskUWm6i/Ev+WancYpzQP9Q//
1pwZISi2AsO/M9zurx2bBv+Ef8v6bL0dZAaGf8f4pUzTFqH/w78PXegqOd+A
8Qf+/SpjQXZ2PMYn+Pdbm6fNAw2wXwn/vvQq1Echju7HEv9eYOZdbWaI/Uf4
9/lmtS199qMc/r1L59Fa5f10f4v4d86EJaGC2/Bz+Hd++5ckP88vmM+If8eF
FFb/u/MtYfi3xCl76cd4tFf497RBA0yu6GI9Cf8eJ3Tw9+mH/gj/NqnWaxYo
Yn8a/i1VOrD72jY8L/j3dGsLt+MSrJfh32nx9oEhkWD491uFpB/DTsnff7vu
O7nI2RT+DP+uGbjjS8QO+DX8W+N44C+XlVi/wr9XpV2wzf1N19vEv42vbHpi
7ULPHxD/jtH08ZyWiO+Dfz/S2eIangaGf4e7e3ul12E9AP8WDtNve1GM8ZO+
//Z7u7bfYPq+jfi3/zbDyaqz8Xn4N79sXduBmXS8gn9L95k/GID/h3+Xt+9Y
t/AmzUv8W/n3klAPNdQP/PuE9o0+eq/QH+HfW4bd5od9wPoD/h1q6H7uURbG
e/i37OTksTNG4fnAv/n/3dFZEYL9OPr+O3KWprH2NcwHxL9Fj5+e8b/1COMD
3n8H5s+++KkS6wfi3zxl9UnuWVsIw795CwN1S3ZcRHsg/i1wCEi9b3gX7QHv
vwXDQ17GVKM9EP+WnLQa2cOlBvmJf/P6tCfV7sH/w78liioNc3QlWB8Q/xbF
FbxaOaYU8xnxb57au7lXr1RjP4b4t8nshcPUMzEewr/76PQ5eKYr1j/wb9uW
kb92XsX4B/8OCHtqdDKO7s8S/37f2Ho4qBLjJ/zbwbFC7c4K7M/Cv8XeJaYl
8zFewr8F0Q7hjUb4PPx71JRUnu9LnEeBf/O9tAaH/8R6BP5dOfL74PatGF/g
38Jn4y0uzHuB9Qzx7z4de9um7Md4BP8e1zKhd8dotA/4t8mjBeGKOWhP8O85
L1d3dP4P7RX+nTHKesmELegf8O85k2zP82k5/Dt8yo+mrIcKaL/Evw3+KTKO
asDv0fffBh7q78egPcC/xWdMo8+l1aA+iX/nD5A9mNEF74fg3/VXVr95NIKO
D8S/83OqWhdFK2N8I/4t3Lsmf52Ajm/Ev3OEU0tP6WJ9Cv/+wrt/t3EP1l/w
7wWrkvI1dOj4Rvy70kps2f4A6yn4d7fNAf82BOH/4d+PksafCRtO5zvi37PX
60/TfonzdPDvgrpRIRmbcN4N/m0y+9O9lfewXoV/2yert4yrRHuBfwd2Vd9z
oSPGd/h3/JcGtYVStBf4d4/nqTYua5WQl/h3V9mV+iIJ9vPh3xlB336FZn/F
fEH8+6Px54a0d/fRHoh/LzM0tj68veaPf2s0OZYZr/r5x7+7LB5QVOCE8Qr+
HTzM7euZUjxv+PfgFcMuCn7Q9kD8O3is5EtWG84Hwb9jygr9brRi/QD/Ds1/
bOfdjPEX/r37/bhjy47g/+HfKx3Gn/K9gPEG/r3l6Pcd98yx3wD/tvzWPD/E
FetD+Lfw35RJee8/IT/ef3/tk7F+NnwB/l2eFDbc+Rh+D/4dWvPs47wA5IN/
f+obnRtuiHL4t4G+/fPdA/H/8O/8RQov7KV0fUb8u8Ss981wD5TDvwVDY64c
H4n9Cvi3cLNl2vQq+An8m79VmPv85xvkx/vvmKy23LXwB/g3/7+3pT7NdWgP
xL9VnVK3ZLhhPoN/L99Q3z5uGfo7/Ntgpuo2hSc4zwD/zvKZlzk05znaB/Fv
fkRTj6jqYtQv8W9xVEaHlS+RB/6d9cJVPe42GP4tSvEauOsRxj/4Ny9DL+rV
Ajq+Ef8Wt0z/qj0d4y38O79BONLCkr7vIf7dlrq+7NN69D/49+5+AXZjm34i
P/FvVXWnQfX+8B3495M3j2eNXAuGfzuYpuoZ78X9wL+fRFtPOdD3JcYz4t88
6zGGjcrYz4Z/C80U5iz+/BT1SfxbppCdsPwc6hf+vXvhjQuTt9LzacS/x72J
ig3djvYE//bdunj+CAW0X/i3c67S43PjMP/Bv5fbvLlbxkN7p+fN5Rc9by6/
6Hlz+QXfZjE9by6/6Hlz+UXPm7Oub1yGb7MYvs1i+DaL4dsspu+7WfkduUzP
m8sv+DY7P/Ftdv7JXKbnzeUXfJvF9Lw5K78tl+Hb7Pz7uUx9W37Bt9n5A7kM
32bnl3AZvs1C+DY7fwKX6XlzVv7XXIZvsxi+zc5/kcv0vDkrP86byy/4Novh
2+z8tVymvi2/4Nsshm+z8xLfZuc14zJ8m8XwbRbDt9l5TbhMz5uzrk1chm+z
mJ43Z+V35zI9b87Kn8Flet5cftHz5qz867hMz5vLL3reXH7R8+asvMe4TM+b
yy963lx+wbfZ+W5zGb7Nzkd8m8X0vDnrKuMyPW/Oyovz5qy8U7hMz5uz8uK8
OSuvPZfpeXP5Rc+byy/4NjvvJC7T8+asvPu4TM+byy/4Novh2+z8iVyGb7MY
vs1i+DY7fzyX4dvs/A1chm+z817gMnybnZf4Novh2yym581ZeV9yGb7NzmvO
ZXreXH7R8+byi543Z+U35TJ8m8X0vLn8gm+z84/gMj1vzsq/kcvwbRbDt9n5
3bkM32bnT+cyPW8uv+h5c1b+tVyGb7Pz3+MyfJvF9Lw5K/9RLsO32fm/cxm+
zWL4Njt/OZfh2yyGb7MYvs3O+4nL8G12Xgcu0/Pm8gu+zc6L8+asvHZcpufN
5Rd8m8XwbXb+iVymvs3Kv5fL9Lw5+wNchm+z8+/mMnybxfBtFlPfZuXfxWX4
Njv/Ky7T8+byC77Nzp/PZfg2Oz/Om8sv+DY7r5TL8G123lFchm+zGL7Nzkt8
m513JJfh2yyGb7MYvs1iet6clX8Ol+HbLIZvsxi+zc4/m8vwbXb+U1ym583l
F3ybnT+Sy/S8OSv/XS7Dt1lMz5uz8qdxmZ43l1/0vLn8oufNWVc5l+l5c1Ze
nDeXX/S8ufyi581ZeVu5DN9m553MZfg2i+Hb7Pw4b87Kb8tl+DYL4dsspufN
WfltuEzPm7Py7+EyPW8uv+Db7PyLuAzfZucv5DJ8m8XwbXb+OC7Dt9n567kM
32YxfJudP4/L8G12fuLbLIZvsxi+zc7/gsv0vDkrvxmX4dsshm+z8xPfZuc3
4TJ8m8XwbXbe4VyGb7PzbuAyPW8uv+Db7LxuXIZvs/Oe5DJ8m8Xwbfb3/X3f
/fd999/33X/fd/993/33fTfv7/vuv++7/77v/vu+++/77r/vu/++7/77vvvv
++7/r++7/x91VtS+
   "], {{{}, {}, 
     {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
      LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 
       19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
        37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 
       54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
        72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 
       89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 
       105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 
       119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 
       133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 
       147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 
       161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 
       175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 
       189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 
       203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 
       217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 
       231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 
       245, 246, 247, 248}]}, 
     {RGBColor[1, 0, 0], AbsoluteThickness[1.], LineBox[CompressedData["
1:eJwV0UNgEAAAQNEt2/aybdtcdi3btm3btm3bto1jrtfh3f7tB4V0Ce4cGBAQ
8I3v/OAnv/jNH/4SIAokFKEJQ1jCEZ4IRCQSkYlCVKIRnRjEJBaxiUNc4hGf
BCQkEYlJQlKSkZwUBJGSVKQmDWlJR3oykJFMZCYLWclGdnKQk1zkJg95yUd+
ClCQQhSmCEUpRnFKUJJSlKYMZSlHeSpQkUpUpgpVqUZ1ahBMTWpRmzrUpR71
aUBDGtGYJjSlGc0JoQUtaUVr2tCWdrSnAx3pxP+5XehKN7rTg570ojd96Es/
+jOAgQxiMEMYyjCGM4KRjGI0YxjLOMYzgYlMYjJTmMo0pjODmcxiNnOYyzzm
s4CFLGIxS1jKMpazgpWsYjVrWMs61rOBjWxiM1vYyja2s4Od7GI3e9jLPvZz
gIMc4jBHOMoxjnOCk5ziNGc4yznOc4GLXOIyV7jKNa5zg5vc4jZ3uMs97vOA
hzziMU94yjOe84KXvOI1b3jLO97zgY984jNf+Mo/BX97eQ==
       
       "]]}, {}, {}, {}, {}, {}, 
     {GrayLevel[0], AbsoluteThickness[1.], LineBox[{749, 502}], 
      LineBox[{750, 503}], LineBox[{751, 504}], LineBox[{752, 505}], 
      LineBox[{753, 506}], LineBox[{754, 507}], LineBox[{755, 508}], 
      LineBox[{756, 509}], LineBox[{757, 510}], LineBox[{758, 511}], 
      LineBox[{763, 516}], LineBox[{764, 517}], LineBox[{765, 518}], 
      LineBox[{772, 525}], LineBox[{773, 526}], LineBox[{774, 527}], 
      LineBox[{775, 528}], LineBox[{776, 529}], LineBox[{777, 530}], 
      LineBox[{778, 531}], LineBox[{779, 532}], LineBox[{780, 533}], 
      LineBox[{781, 534}], LineBox[{782, 535}], LineBox[{783, 536}], 
      LineBox[{784, 537}], LineBox[{785, 538}], LineBox[{786, 539}], 
      LineBox[{788, 541}], LineBox[{818, 571}], LineBox[{819, 572}], 
      LineBox[{820, 573}], LineBox[{821, 574}], LineBox[{822, 575}], 
      LineBox[{823, 576}], LineBox[{824, 577}], LineBox[{840, 593}], 
      LineBox[{841, 594}], LineBox[{842, 595}], LineBox[{843, 596}], 
      LineBox[{844, 597}], LineBox[{845, 598}], LineBox[{846, 599}], 
      LineBox[{847, 600}], LineBox[{848, 601}], LineBox[{849, 602}], 
      LineBox[{850, 603}], LineBox[{851, 604}], LineBox[{852, 605}], 
      LineBox[{853, 606}], LineBox[{854, 607}], LineBox[{855, 608}], 
      LineBox[{856, 609}], LineBox[{857, 610}], LineBox[{858, 611}], 
      LineBox[{859, 612}], LineBox[{889, 642}], LineBox[{890, 643}], 
      LineBox[{891, 644}], LineBox[{892, 645}], LineBox[{896, 649}], 
      LineBox[{899, 652}], LineBox[{900, 653}], LineBox[{901, 654}], 
      LineBox[{902, 655}], LineBox[{903, 656}], LineBox[{904, 657}], 
      LineBox[{905, 658}], LineBox[{906, 659}], LineBox[{907, 660}], 
      LineBox[{908, 661}], LineBox[{909, 662}], LineBox[{910, 663}], 
      LineBox[{921, 674}], LineBox[{922, 675}], LineBox[{923, 676}], 
      LineBox[{924, 677}], LineBox[{925, 678}], LineBox[{926, 679}], 
      LineBox[{927, 680}], LineBox[{928, 681}], LineBox[{929, 682}], 
      LineBox[{930, 683}], LineBox[{931, 684}], LineBox[{932, 685}], 
      LineBox[{933, 686}], LineBox[{934, 687}], LineBox[{935, 688}], 
      LineBox[{936, 689}], LineBox[{937, 690}], LineBox[{938, 691}], 
      LineBox[{939, 692}], LineBox[{940, 693}], LineBox[{941, 694}], 
      LineBox[{966, 719}], LineBox[{967, 720}], LineBox[{968, 721}], 
      LineBox[{969, 722}], LineBox[{970, 723}], LineBox[{976, 729}], 
      LineBox[{977, 730}], LineBox[{978, 731}], LineBox[{979, 732}], 
      LineBox[{980, 733}], LineBox[{981, 734}], LineBox[{982, 735}], 
      LineBox[{983, 736}], LineBox[{984, 737}]}, 
     {GrayLevel[0], AbsoluteThickness[1.], LineBox[{745, 498}], 
      LineBox[{746, 499}], LineBox[{747, 500}], LineBox[{748, 501}], 
      LineBox[{759, 512}], LineBox[{760, 513}], LineBox[{761, 514}], 
      LineBox[{762, 515}], LineBox[{766, 519}], LineBox[{767, 520}], 
      LineBox[{768, 521}], LineBox[{769, 522}], LineBox[{770, 523}], 
      LineBox[{771, 524}], LineBox[{787, 540}], LineBox[{789, 542}], 
      LineBox[{790, 543}], LineBox[{791, 544}], LineBox[{792, 545}], 
      LineBox[{793, 546}], LineBox[{794, 547}], LineBox[{795, 548}], 
      LineBox[{796, 549}], LineBox[{797, 550}], LineBox[{798, 551}], 
      LineBox[{799, 552}], LineBox[{800, 553}], LineBox[{801, 554}], 
      LineBox[{802, 555}], LineBox[{803, 556}], LineBox[{804, 557}], 
      LineBox[{805, 558}], LineBox[{806, 559}], LineBox[{807, 560}], 
      LineBox[{808, 561}], LineBox[{809, 562}], LineBox[{810, 563}], 
      LineBox[{811, 564}], LineBox[{812, 565}], LineBox[{813, 566}], 
      LineBox[{814, 567}], LineBox[{815, 568}], LineBox[{816, 569}], 
      LineBox[{817, 570}], LineBox[{825, 578}], LineBox[{826, 579}], 
      LineBox[{827, 580}], LineBox[{828, 581}], LineBox[{829, 582}], 
      LineBox[{830, 583}], LineBox[{831, 584}], LineBox[{832, 585}], 
      LineBox[{833, 586}], LineBox[{834, 587}], LineBox[{835, 588}], 
      LineBox[{836, 589}], LineBox[{837, 590}], LineBox[{838, 591}], 
      LineBox[{839, 592}], LineBox[{860, 613}], LineBox[{861, 614}], 
      LineBox[{862, 615}], LineBox[{863, 616}], LineBox[{864, 617}], 
      LineBox[{865, 618}], LineBox[{866, 619}], LineBox[{867, 620}], 
      LineBox[{868, 621}], LineBox[{869, 622}], LineBox[{870, 623}], 
      LineBox[{871, 624}], LineBox[{872, 625}], LineBox[{873, 626}], 
      LineBox[{874, 627}], LineBox[{875, 628}], LineBox[{876, 629}], 
      LineBox[{877, 630}], LineBox[{878, 631}], LineBox[{879, 632}], 
      LineBox[{880, 633}], LineBox[{881, 634}], LineBox[{882, 635}], 
      LineBox[{883, 636}], LineBox[{884, 637}], LineBox[{885, 638}], 
      LineBox[{886, 639}], LineBox[{887, 640}], LineBox[{888, 641}], 
      LineBox[{893, 646}], LineBox[{894, 647}], LineBox[{895, 648}], 
      LineBox[{897, 650}], LineBox[{898, 651}], LineBox[{911, 664}], 
      LineBox[{912, 665}], LineBox[{913, 666}], LineBox[{914, 667}], 
      LineBox[{915, 668}], LineBox[{916, 669}], LineBox[{917, 670}], 
      LineBox[{918, 671}], LineBox[{919, 672}], LineBox[{920, 673}], 
      LineBox[{942, 695}], LineBox[{943, 696}], LineBox[{944, 697}], 
      LineBox[{945, 698}], LineBox[{946, 699}], LineBox[{947, 700}], 
      LineBox[{948, 701}], LineBox[{949, 702}], LineBox[{950, 703}], 
      LineBox[{951, 704}], LineBox[{952, 705}], LineBox[{953, 706}], 
      LineBox[{954, 707}], LineBox[{955, 708}], LineBox[{956, 709}], 
      LineBox[{957, 710}], LineBox[{958, 711}], LineBox[{959, 712}], 
      LineBox[{960, 713}], LineBox[{961, 714}], LineBox[{962, 715}], 
      LineBox[{963, 716}], LineBox[{964, 717}], LineBox[{965, 718}], 
      LineBox[{971, 724}], LineBox[{972, 725}], LineBox[{973, 726}], 
      LineBox[{974, 727}], LineBox[{975, 728}], LineBox[{985, 738}], 
      LineBox[{986, 739}], LineBox[{987, 740}], LineBox[{988, 741}], 
      LineBox[{989, 742}], LineBox[{990, 743}], LineBox[{991, 744}]}}, {
     {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
      TagBox[InsetBox["", 1],
       "InsetString"], 
      TagBox[InsetBox["", 2],
       "InsetString"], 
      TagBox[InsetBox["", 3],
       "InsetString"], 
      TagBox[InsetBox["", 4],
       "InsetString"], 
      TagBox[InsetBox["", 5],
       "InsetString"], 
      TagBox[InsetBox["", 6],
       "InsetString"], 
      TagBox[InsetBox["", 7],
       "InsetString"], 
      TagBox[InsetBox["", 8],
       "InsetString"], 
      TagBox[InsetBox["", 9],
       "InsetString"], 
      TagBox[InsetBox["", 10],
       "InsetString"], 
      TagBox[InsetBox["", 11],
       "InsetString"], 
      TagBox[InsetBox["", 12],
       "InsetString"], 
      TagBox[InsetBox["", 13],
       "InsetString"], 
      TagBox[InsetBox["", 14],
       "InsetString"], 
      TagBox[InsetBox["", 15],
       "InsetString"], 
      TagBox[InsetBox["", 16],
       "InsetString"], 
      TagBox[InsetBox["", 17],
       "InsetString"], 
      TagBox[InsetBox["", 18],
       "InsetString"], 
      TagBox[InsetBox["", 19],
       "InsetString"], 
      TagBox[InsetBox["", 20],
       "InsetString"], 
      TagBox[InsetBox["", 21],
       "InsetString"], 
      TagBox[InsetBox["", 22],
       "InsetString"], 
      TagBox[InsetBox["", 23],
       "InsetString"], 
      TagBox[InsetBox["", 24],
       "InsetString"], 
      TagBox[InsetBox["", 25],
       "InsetString"], 
      TagBox[InsetBox["", 26],
       "InsetString"], 
      TagBox[InsetBox["", 27],
       "InsetString"], 
      TagBox[InsetBox["", 28],
       "InsetString"], 
      TagBox[InsetBox["", 29],
       "InsetString"], 
      TagBox[InsetBox["", 30],
       "InsetString"], 
      TagBox[InsetBox["", 31],
       "InsetString"], 
      TagBox[InsetBox["", 32],
       "InsetString"], 
      TagBox[InsetBox["", 33],
       "InsetString"], 
      TagBox[InsetBox["", 34],
       "InsetString"], 
      TagBox[InsetBox["", 35],
       "InsetString"], 
      TagBox[InsetBox["", 36],
       "InsetString"], 
      TagBox[InsetBox["", 37],
       "InsetString"], 
      TagBox[InsetBox["", 38],
       "InsetString"], 
      TagBox[InsetBox["", 39],
       "InsetString"], 
      TagBox[InsetBox["", 40],
       "InsetString"], 
      TagBox[InsetBox["", 41],
       "InsetString"], 
      TagBox[InsetBox["", 42],
       "InsetString"], 
      TagBox[InsetBox["", 43],
       "InsetString"], 
      TagBox[InsetBox["", 44],
       "InsetString"], 
      TagBox[InsetBox["", 45],
       "InsetString"], 
      TagBox[InsetBox["", 46],
       "InsetString"], 
      TagBox[InsetBox["", 47],
       "InsetString"], 
      TagBox[InsetBox["", 48],
       "InsetString"], 
      TagBox[InsetBox["", 49],
       "InsetString"], 
      TagBox[InsetBox["", 50],
       "InsetString"], 
      TagBox[InsetBox["", 51],
       "InsetString"], 
      TagBox[InsetBox["", 52],
       "InsetString"], 
      TagBox[InsetBox["", 53],
       "InsetString"], 
      TagBox[InsetBox["", 54],
       "InsetString"], 
      TagBox[InsetBox["", 55],
       "InsetString"], 
      TagBox[InsetBox["", 56],
       "InsetString"], 
      TagBox[InsetBox["", 57],
       "InsetString"], 
      TagBox[InsetBox["", 58],
       "InsetString"], 
      TagBox[InsetBox["", 59],
       "InsetString"], 
      TagBox[InsetBox["", 60],
       "InsetString"], 
      TagBox[InsetBox["", 61],
       "InsetString"], 
      TagBox[InsetBox["", 62],
       "InsetString"], 
      TagBox[InsetBox["", 63],
       "InsetString"], 
      TagBox[InsetBox["", 64],
       "InsetString"], 
      TagBox[InsetBox["", 65],
       "InsetString"], 
      TagBox[InsetBox["", 66],
       "InsetString"], 
      TagBox[InsetBox["", 67],
       "InsetString"], 
      TagBox[InsetBox["", 68],
       "InsetString"], 
      TagBox[InsetBox["", 69],
       "InsetString"], 
      TagBox[InsetBox["", 70],
       "InsetString"], 
      TagBox[InsetBox["", 71],
       "InsetString"], 
      TagBox[InsetBox["", 72],
       "InsetString"], 
      TagBox[InsetBox["", 73],
       "InsetString"], 
      TagBox[InsetBox["", 74],
       "InsetString"], 
      TagBox[InsetBox["", 75],
       "InsetString"], 
      TagBox[InsetBox["", 76],
       "InsetString"], 
      TagBox[InsetBox["", 77],
       "InsetString"], 
      TagBox[InsetBox["", 78],
       "InsetString"], 
      TagBox[InsetBox["", 79],
       "InsetString"], 
      TagBox[InsetBox["", 80],
       "InsetString"], 
      TagBox[InsetBox["", 81],
       "InsetString"], 
      TagBox[InsetBox["", 82],
       "InsetString"], 
      TagBox[InsetBox["", 83],
       "InsetString"], 
      TagBox[InsetBox["", 84],
       "InsetString"], 
      TagBox[InsetBox["", 85],
       "InsetString"], 
      TagBox[InsetBox["", 86],
       "InsetString"], 
      TagBox[InsetBox["", 87],
       "InsetString"], 
      TagBox[InsetBox["", 88],
       "InsetString"], 
      TagBox[InsetBox["", 89],
       "InsetString"], 
      TagBox[InsetBox["", 90],
       "InsetString"], 
      TagBox[InsetBox["", 91],
       "InsetString"], 
      TagBox[InsetBox["", 92],
       "InsetString"], 
      TagBox[InsetBox["", 93],
       "InsetString"], 
      TagBox[InsetBox["", 94],
       "InsetString"], 
      TagBox[InsetBox["", 95],
       "InsetString"], 
      TagBox[InsetBox["", 96],
       "InsetString"], 
      TagBox[InsetBox["", 97],
       "InsetString"], 
      TagBox[InsetBox["", 98],
       "InsetString"], 
      TagBox[InsetBox["", 99],
       "InsetString"], 
      TagBox[InsetBox["", 100],
       "InsetString"], 
      TagBox[InsetBox["", 101],
       "InsetString"], 
      TagBox[InsetBox["", 102],
       "InsetString"], 
      TagBox[InsetBox["", 103],
       "InsetString"], 
      TagBox[InsetBox["", 104],
       "InsetString"], 
      TagBox[InsetBox["", 105],
       "InsetString"], 
      TagBox[InsetBox["", 106],
       "InsetString"], 
      TagBox[InsetBox["", 107],
       "InsetString"], 
      TagBox[InsetBox["", 108],
       "InsetString"], 
      TagBox[InsetBox["", 109],
       "InsetString"], 
      TagBox[InsetBox["", 110],
       "InsetString"], 
      TagBox[InsetBox["", 111],
       "InsetString"], 
      TagBox[InsetBox["", 112],
       "InsetString"], 
      TagBox[InsetBox["", 113],
       "InsetString"], 
      TagBox[InsetBox["", 114],
       "InsetString"], 
      TagBox[InsetBox["", 115],
       "InsetString"], 
      TagBox[InsetBox["", 116],
       "InsetString"], 
      TagBox[InsetBox["", 117],
       "InsetString"], 
      TagBox[InsetBox["", 118],
       "InsetString"], 
      TagBox[InsetBox["", 119],
       "InsetString"], 
      TagBox[InsetBox["", 120],
       "InsetString"], 
      TagBox[InsetBox["", 121],
       "InsetString"], 
      TagBox[InsetBox["", 122],
       "InsetString"], 
      TagBox[InsetBox["", 123],
       "InsetString"], 
      TagBox[InsetBox["", 124],
       "InsetString"], 
      TagBox[InsetBox["", 125],
       "InsetString"], 
      TagBox[InsetBox["", 126],
       "InsetString"], 
      TagBox[InsetBox["", 127],
       "InsetString"], 
      TagBox[InsetBox["", 128],
       "InsetString"], 
      TagBox[InsetBox["", 129],
       "InsetString"], 
      TagBox[InsetBox["", 130],
       "InsetString"], 
      TagBox[InsetBox["", 131],
       "InsetString"], 
      TagBox[InsetBox["", 132],
       "InsetString"], 
      TagBox[InsetBox["", 133],
       "InsetString"], 
      TagBox[InsetBox["", 134],
       "InsetString"], 
      TagBox[InsetBox["", 135],
       "InsetString"], 
      TagBox[InsetBox["", 136],
       "InsetString"], 
      TagBox[InsetBox["", 137],
       "InsetString"], 
      TagBox[InsetBox["", 138],
       "InsetString"], 
      TagBox[InsetBox["", 139],
       "InsetString"], 
      TagBox[InsetBox["", 140],
       "InsetString"], 
      TagBox[InsetBox["", 141],
       "InsetString"], 
      TagBox[InsetBox["", 142],
       "InsetString"], 
      TagBox[InsetBox["", 143],
       "InsetString"], 
      TagBox[InsetBox["", 144],
       "InsetString"], 
      TagBox[InsetBox["", 145],
       "InsetString"], 
      TagBox[InsetBox["", 146],
       "InsetString"], 
      TagBox[InsetBox["", 147],
       "InsetString"], 
      TagBox[InsetBox["", 148],
       "InsetString"], 
      TagBox[InsetBox["", 149],
       "InsetString"], 
      TagBox[InsetBox["", 150],
       "InsetString"], 
      TagBox[InsetBox["", 151],
       "InsetString"], 
      TagBox[InsetBox["", 152],
       "InsetString"], 
      TagBox[InsetBox["", 153],
       "InsetString"], 
      TagBox[InsetBox["", 154],
       "InsetString"], 
      TagBox[InsetBox["", 155],
       "InsetString"], 
      TagBox[InsetBox["", 156],
       "InsetString"], 
      TagBox[InsetBox["", 157],
       "InsetString"], 
      TagBox[InsetBox["", 158],
       "InsetString"], 
      TagBox[InsetBox["", 159],
       "InsetString"], 
      TagBox[InsetBox["", 160],
       "InsetString"], 
      TagBox[InsetBox["", 161],
       "InsetString"], 
      TagBox[InsetBox["", 162],
       "InsetString"], 
      TagBox[InsetBox["", 163],
       "InsetString"], 
      TagBox[InsetBox["", 164],
       "InsetString"], 
      TagBox[InsetBox["", 165],
       "InsetString"], 
      TagBox[InsetBox["", 166],
       "InsetString"], 
      TagBox[InsetBox["", 167],
       "InsetString"], 
      TagBox[InsetBox["", 168],
       "InsetString"], 
      TagBox[InsetBox["", 169],
       "InsetString"], 
      TagBox[InsetBox["", 170],
       "InsetString"], 
      TagBox[InsetBox["", 171],
       "InsetString"], 
      TagBox[InsetBox["", 172],
       "InsetString"], 
      TagBox[InsetBox["", 173],
       "InsetString"], 
      TagBox[InsetBox["", 174],
       "InsetString"], 
      TagBox[InsetBox["", 175],
       "InsetString"], 
      TagBox[InsetBox["", 176],
       "InsetString"], 
      TagBox[InsetBox["", 177],
       "InsetString"], 
      TagBox[InsetBox["", 178],
       "InsetString"], 
      TagBox[InsetBox["", 179],
       "InsetString"], 
      TagBox[InsetBox["", 180],
       "InsetString"], 
      TagBox[InsetBox["", 181],
       "InsetString"], 
      TagBox[InsetBox["", 182],
       "InsetString"], 
      TagBox[InsetBox["", 183],
       "InsetString"], 
      TagBox[InsetBox["", 184],
       "InsetString"], 
      TagBox[InsetBox["", 185],
       "InsetString"], 
      TagBox[InsetBox["", 186],
       "InsetString"], 
      TagBox[InsetBox["", 187],
       "InsetString"], 
      TagBox[InsetBox["", 188],
       "InsetString"], 
      TagBox[InsetBox["", 189],
       "InsetString"], 
      TagBox[InsetBox["", 190],
       "InsetString"], 
      TagBox[InsetBox["", 191],
       "InsetString"], 
      TagBox[InsetBox["", 192],
       "InsetString"], 
      TagBox[InsetBox["", 193],
       "InsetString"], 
      TagBox[InsetBox["", 194],
       "InsetString"], 
      TagBox[InsetBox["", 195],
       "InsetString"], 
      TagBox[InsetBox["", 196],
       "InsetString"], 
      TagBox[InsetBox["", 197],
       "InsetString"], 
      TagBox[InsetBox["", 198],
       "InsetString"], 
      TagBox[InsetBox["", 199],
       "InsetString"], 
      TagBox[InsetBox["", 200],
       "InsetString"], 
      TagBox[InsetBox["", 201],
       "InsetString"], 
      TagBox[InsetBox["", 202],
       "InsetString"], 
      TagBox[InsetBox["", 203],
       "InsetString"], 
      TagBox[InsetBox["", 204],
       "InsetString"], 
      TagBox[InsetBox["", 205],
       "InsetString"], 
      TagBox[InsetBox["", 206],
       "InsetString"], 
      TagBox[InsetBox["", 207],
       "InsetString"], 
      TagBox[InsetBox["", 208],
       "InsetString"], 
      TagBox[InsetBox["", 209],
       "InsetString"], 
      TagBox[InsetBox["", 210],
       "InsetString"], 
      TagBox[InsetBox["", 211],
       "InsetString"], 
      TagBox[InsetBox["", 212],
       "InsetString"], 
      TagBox[InsetBox["", 213],
       "InsetString"], 
      TagBox[InsetBox["", 214],
       "InsetString"], 
      TagBox[InsetBox["", 215],
       "InsetString"], 
      TagBox[InsetBox["", 216],
       "InsetString"], 
      TagBox[InsetBox["", 217],
       "InsetString"], 
      TagBox[InsetBox["", 218],
       "InsetString"], 
      TagBox[InsetBox["", 219],
       "InsetString"], 
      TagBox[InsetBox["", 220],
       "InsetString"], 
      TagBox[InsetBox["", 221],
       "InsetString"], 
      TagBox[InsetBox["", 222],
       "InsetString"], 
      TagBox[InsetBox["", 223],
       "InsetString"], 
      TagBox[InsetBox["", 224],
       "InsetString"], 
      TagBox[InsetBox["", 225],
       "InsetString"], 
      TagBox[InsetBox["", 226],
       "InsetString"], 
      TagBox[InsetBox["", 227],
       "InsetString"], 
      TagBox[InsetBox["", 228],
       "InsetString"], 
      TagBox[InsetBox["", 229],
       "InsetString"], 
      TagBox[InsetBox["", 230],
       "InsetString"], 
      TagBox[InsetBox["", 231],
       "InsetString"], 
      TagBox[InsetBox["", 232],
       "InsetString"], 
      TagBox[InsetBox["", 233],
       "InsetString"], 
      TagBox[InsetBox["", 234],
       "InsetString"], 
      TagBox[InsetBox["", 235],
       "InsetString"], 
      TagBox[InsetBox["", 236],
       "InsetString"], 
      TagBox[InsetBox["", 237],
       "InsetString"], 
      TagBox[InsetBox["", 238],
       "InsetString"], 
      TagBox[InsetBox["", 239],
       "InsetString"], 
      TagBox[InsetBox["", 240],
       "InsetString"], 
      TagBox[InsetBox["", 241],
       "InsetString"], 
      TagBox[InsetBox["", 242],
       "InsetString"], 
      TagBox[InsetBox["", 243],
       "InsetString"], 
      TagBox[InsetBox["", 244],
       "InsetString"], 
      TagBox[InsetBox["", 245],
       "InsetString"], 
      TagBox[InsetBox["", 246],
       "InsetString"], 
      TagBox[InsetBox["", 247],
       "InsetString"], 
      TagBox[InsetBox["", 248],
       "InsetString"]}, 
     {RGBColor[1, 0, 0], AbsoluteThickness[1.], 
      TagBox[InsetBox["", 249],
       "InsetString"], 
      TagBox[InsetBox["", 250],
       "InsetString"], 
      TagBox[InsetBox["", 251],
       "InsetString"], 
      TagBox[InsetBox["", 252],
       "InsetString"], 
      TagBox[InsetBox["", 253],
       "InsetString"], 
      TagBox[InsetBox["", 254],
       "InsetString"], 
      TagBox[InsetBox["", 255],
       "InsetString"], 
      TagBox[InsetBox["", 256],
       "InsetString"], 
      TagBox[InsetBox["", 257],
       "InsetString"], 
      TagBox[InsetBox["", 258],
       "InsetString"], 
      TagBox[InsetBox["", 259],
       "InsetString"], 
      TagBox[InsetBox["", 260],
       "InsetString"], 
      TagBox[InsetBox["", 261],
       "InsetString"], 
      TagBox[InsetBox["", 262],
       "InsetString"], 
      TagBox[InsetBox["", 263],
       "InsetString"], 
      TagBox[InsetBox["", 264],
       "InsetString"], 
      TagBox[InsetBox["", 265],
       "InsetString"], 
      TagBox[InsetBox["", 266],
       "InsetString"], 
      TagBox[InsetBox["", 267],
       "InsetString"], 
      TagBox[InsetBox["", 268],
       "InsetString"], 
      TagBox[InsetBox["", 269],
       "InsetString"], 
      TagBox[InsetBox["", 270],
       "InsetString"], 
      TagBox[InsetBox["", 271],
       "InsetString"], 
      TagBox[InsetBox["", 272],
       "InsetString"], 
      TagBox[InsetBox["", 273],
       "InsetString"], 
      TagBox[InsetBox["", 274],
       "InsetString"], 
      TagBox[InsetBox["", 275],
       "InsetString"], 
      TagBox[InsetBox["", 276],
       "InsetString"], 
      TagBox[InsetBox["", 277],
       "InsetString"], 
      TagBox[InsetBox["", 278],
       "InsetString"], 
      TagBox[InsetBox["", 279],
       "InsetString"], 
      TagBox[InsetBox["", 280],
       "InsetString"], 
      TagBox[InsetBox["", 281],
       "InsetString"], 
      TagBox[InsetBox["", 282],
       "InsetString"], 
      TagBox[InsetBox["", 283],
       "InsetString"], 
      TagBox[InsetBox["", 284],
       "InsetString"], 
      TagBox[InsetBox["", 285],
       "InsetString"], 
      TagBox[InsetBox["", 286],
       "InsetString"], 
      TagBox[InsetBox["", 287],
       "InsetString"], 
      TagBox[InsetBox["", 288],
       "InsetString"], 
      TagBox[InsetBox["", 289],
       "InsetString"], 
      TagBox[InsetBox["", 290],
       "InsetString"], 
      TagBox[InsetBox["", 291],
       "InsetString"], 
      TagBox[InsetBox["", 292],
       "InsetString"], 
      TagBox[InsetBox["", 293],
       "InsetString"], 
      TagBox[InsetBox["", 294],
       "InsetString"], 
      TagBox[InsetBox["", 295],
       "InsetString"], 
      TagBox[InsetBox["", 296],
       "InsetString"], 
      TagBox[InsetBox["", 297],
       "InsetString"], 
      TagBox[InsetBox["", 298],
       "InsetString"], 
      TagBox[InsetBox["", 299],
       "InsetString"], 
      TagBox[InsetBox["", 300],
       "InsetString"], 
      TagBox[InsetBox["", 301],
       "InsetString"], 
      TagBox[InsetBox["", 302],
       "InsetString"], 
      TagBox[InsetBox["", 303],
       "InsetString"], 
      TagBox[InsetBox["", 304],
       "InsetString"], 
      TagBox[InsetBox["", 305],
       "InsetString"], 
      TagBox[InsetBox["", 306],
       "InsetString"], 
      TagBox[InsetBox["", 307],
       "InsetString"], 
      TagBox[InsetBox["", 308],
       "InsetString"], 
      TagBox[InsetBox["", 309],
       "InsetString"], 
      TagBox[InsetBox["", 310],
       "InsetString"], 
      TagBox[InsetBox["", 311],
       "InsetString"], 
      TagBox[InsetBox["", 312],
       "InsetString"], 
      TagBox[InsetBox["", 313],
       "InsetString"], 
      TagBox[InsetBox["", 314],
       "InsetString"], 
      TagBox[InsetBox["", 315],
       "InsetString"], 
      TagBox[InsetBox["", 316],
       "InsetString"], 
      TagBox[InsetBox["", 317],
       "InsetString"], 
      TagBox[InsetBox["", 318],
       "InsetString"], 
      TagBox[InsetBox["", 319],
       "InsetString"], 
      TagBox[InsetBox["", 320],
       "InsetString"], 
      TagBox[InsetBox["", 321],
       "InsetString"], 
      TagBox[InsetBox["", 322],
       "InsetString"], 
      TagBox[InsetBox["", 323],
       "InsetString"], 
      TagBox[InsetBox["", 324],
       "InsetString"], 
      TagBox[InsetBox["", 325],
       "InsetString"], 
      TagBox[InsetBox["", 326],
       "InsetString"], 
      TagBox[InsetBox["", 327],
       "InsetString"], 
      TagBox[InsetBox["", 328],
       "InsetString"], 
      TagBox[InsetBox["", 329],
       "InsetString"], 
      TagBox[InsetBox["", 330],
       "InsetString"], 
      TagBox[InsetBox["", 331],
       "InsetString"], 
      TagBox[InsetBox["", 332],
       "InsetString"], 
      TagBox[InsetBox["", 333],
       "InsetString"], 
      TagBox[InsetBox["", 334],
       "InsetString"], 
      TagBox[InsetBox["", 335],
       "InsetString"], 
      TagBox[InsetBox["", 336],
       "InsetString"], 
      TagBox[InsetBox["", 337],
       "InsetString"], 
      TagBox[InsetBox["", 338],
       "InsetString"], 
      TagBox[InsetBox["", 339],
       "InsetString"], 
      TagBox[InsetBox["", 340],
       "InsetString"], 
      TagBox[InsetBox["", 341],
       "InsetString"], 
      TagBox[InsetBox["", 342],
       "InsetString"], 
      TagBox[InsetBox["", 343],
       "InsetString"], 
      TagBox[InsetBox["", 344],
       "InsetString"], 
      TagBox[InsetBox["", 345],
       "InsetString"], 
      TagBox[InsetBox["", 346],
       "InsetString"], 
      TagBox[InsetBox["", 347],
       "InsetString"], 
      TagBox[InsetBox["", 348],
       "InsetString"], 
      TagBox[InsetBox["", 349],
       "InsetString"], 
      TagBox[InsetBox["", 350],
       "InsetString"], 
      TagBox[InsetBox["", 351],
       "InsetString"], 
      TagBox[InsetBox["", 352],
       "InsetString"], 
      TagBox[InsetBox["", 353],
       "InsetString"], 
      TagBox[InsetBox["", 354],
       "InsetString"], 
      TagBox[InsetBox["", 355],
       "InsetString"], 
      TagBox[InsetBox["", 356],
       "InsetString"], 
      TagBox[InsetBox["", 357],
       "InsetString"], 
      TagBox[InsetBox["", 358],
       "InsetString"], 
      TagBox[InsetBox["", 359],
       "InsetString"], 
      TagBox[InsetBox["", 360],
       "InsetString"], 
      TagBox[InsetBox["", 361],
       "InsetString"], 
      TagBox[InsetBox["", 362],
       "InsetString"], 
      TagBox[InsetBox["", 363],
       "InsetString"], 
      TagBox[InsetBox["", 364],
       "InsetString"], 
      TagBox[InsetBox["", 365],
       "InsetString"], 
      TagBox[InsetBox["", 366],
       "InsetString"], 
      TagBox[InsetBox["", 367],
       "InsetString"], 
      TagBox[InsetBox["", 368],
       "InsetString"], 
      TagBox[InsetBox["", 369],
       "InsetString"], 
      TagBox[InsetBox["", 370],
       "InsetString"], 
      TagBox[InsetBox["", 371],
       "InsetString"], 
      TagBox[InsetBox["", 372],
       "InsetString"], 
      TagBox[InsetBox["", 373],
       "InsetString"], 
      TagBox[InsetBox["", 374],
       "InsetString"], 
      TagBox[InsetBox["", 375],
       "InsetString"], 
      TagBox[InsetBox["", 376],
       "InsetString"], 
      TagBox[InsetBox["", 377],
       "InsetString"], 
      TagBox[InsetBox["", 378],
       "InsetString"], 
      TagBox[InsetBox["", 379],
       "InsetString"], 
      TagBox[InsetBox["", 380],
       "InsetString"], 
      TagBox[InsetBox["", 381],
       "InsetString"], 
      TagBox[InsetBox["", 382],
       "InsetString"], 
      TagBox[InsetBox["", 383],
       "InsetString"], 
      TagBox[InsetBox["", 384],
       "InsetString"], 
      TagBox[InsetBox["", 385],
       "InsetString"], 
      TagBox[InsetBox["", 386],
       "InsetString"], 
      TagBox[InsetBox["", 387],
       "InsetString"], 
      TagBox[InsetBox["", 388],
       "InsetString"], 
      TagBox[InsetBox["", 389],
       "InsetString"], 
      TagBox[InsetBox["", 390],
       "InsetString"], 
      TagBox[InsetBox["", 391],
       "InsetString"], 
      TagBox[InsetBox["", 392],
       "InsetString"], 
      TagBox[InsetBox["", 393],
       "InsetString"], 
      TagBox[InsetBox["", 394],
       "InsetString"], 
      TagBox[InsetBox["", 395],
       "InsetString"], 
      TagBox[InsetBox["", 396],
       "InsetString"], 
      TagBox[InsetBox["", 397],
       "InsetString"], 
      TagBox[InsetBox["", 398],
       "InsetString"], 
      TagBox[InsetBox["", 399],
       "InsetString"], 
      TagBox[InsetBox["", 400],
       "InsetString"], 
      TagBox[InsetBox["", 401],
       "InsetString"], 
      TagBox[InsetBox["", 402],
       "InsetString"], 
      TagBox[InsetBox["", 403],
       "InsetString"], 
      TagBox[InsetBox["", 404],
       "InsetString"], 
      TagBox[InsetBox["", 405],
       "InsetString"], 
      TagBox[InsetBox["", 406],
       "InsetString"], 
      TagBox[InsetBox["", 407],
       "InsetString"], 
      TagBox[InsetBox["", 408],
       "InsetString"], 
      TagBox[InsetBox["", 409],
       "InsetString"], 
      TagBox[InsetBox["", 410],
       "InsetString"], 
      TagBox[InsetBox["", 411],
       "InsetString"], 
      TagBox[InsetBox["", 412],
       "InsetString"], 
      TagBox[InsetBox["", 413],
       "InsetString"], 
      TagBox[InsetBox["", 414],
       "InsetString"], 
      TagBox[InsetBox["", 415],
       "InsetString"], 
      TagBox[InsetBox["", 416],
       "InsetString"], 
      TagBox[InsetBox["", 417],
       "InsetString"], 
      TagBox[InsetBox["", 418],
       "InsetString"], 
      TagBox[InsetBox["", 419],
       "InsetString"], 
      TagBox[InsetBox["", 420],
       "InsetString"], 
      TagBox[InsetBox["", 421],
       "InsetString"], 
      TagBox[InsetBox["", 422],
       "InsetString"], 
      TagBox[InsetBox["", 423],
       "InsetString"], 
      TagBox[InsetBox["", 424],
       "InsetString"], 
      TagBox[InsetBox["", 425],
       "InsetString"], 
      TagBox[InsetBox["", 426],
       "InsetString"], 
      TagBox[InsetBox["", 427],
       "InsetString"], 
      TagBox[InsetBox["", 428],
       "InsetString"], 
      TagBox[InsetBox["", 429],
       "InsetString"], 
      TagBox[InsetBox["", 430],
       "InsetString"], 
      TagBox[InsetBox["", 431],
       "InsetString"], 
      TagBox[InsetBox["", 432],
       "InsetString"], 
      TagBox[InsetBox["", 433],
       "InsetString"], 
      TagBox[InsetBox["", 434],
       "InsetString"], 
      TagBox[InsetBox["", 435],
       "InsetString"], 
      TagBox[InsetBox["", 436],
       "InsetString"], 
      TagBox[InsetBox["", 437],
       "InsetString"], 
      TagBox[InsetBox["", 438],
       "InsetString"], 
      TagBox[InsetBox["", 439],
       "InsetString"], 
      TagBox[InsetBox["", 440],
       "InsetString"], 
      TagBox[InsetBox["", 441],
       "InsetString"], 
      TagBox[InsetBox["", 442],
       "InsetString"], 
      TagBox[InsetBox["", 443],
       "InsetString"], 
      TagBox[InsetBox["", 444],
       "InsetString"], 
      TagBox[InsetBox["", 445],
       "InsetString"], 
      TagBox[InsetBox["", 446],
       "InsetString"], 
      TagBox[InsetBox["", 447],
       "InsetString"], 
      TagBox[InsetBox["", 448],
       "InsetString"], 
      TagBox[InsetBox["", 449],
       "InsetString"], 
      TagBox[InsetBox["", 450],
       "InsetString"], 
      TagBox[InsetBox["", 451],
       "InsetString"], 
      TagBox[InsetBox["", 452],
       "InsetString"], 
      TagBox[InsetBox["", 453],
       "InsetString"], 
      TagBox[InsetBox["", 454],
       "InsetString"], 
      TagBox[InsetBox["", 455],
       "InsetString"], 
      TagBox[InsetBox["", 456],
       "InsetString"], 
      TagBox[InsetBox["", 457],
       "InsetString"], 
      TagBox[InsetBox["", 458],
       "InsetString"], 
      TagBox[InsetBox["", 459],
       "InsetString"], 
      TagBox[InsetBox["", 460],
       "InsetString"], 
      TagBox[InsetBox["", 461],
       "InsetString"], 
      TagBox[InsetBox["", 462],
       "InsetString"], 
      TagBox[InsetBox["", 463],
       "InsetString"], 
      TagBox[InsetBox["", 464],
       "InsetString"], 
      TagBox[InsetBox["", 465],
       "InsetString"], 
      TagBox[InsetBox["", 466],
       "InsetString"], 
      TagBox[InsetBox["", 467],
       "InsetString"], 
      TagBox[InsetBox["", 468],
       "InsetString"], 
      TagBox[InsetBox["", 469],
       "InsetString"], 
      TagBox[InsetBox["", 470],
       "InsetString"], 
      TagBox[InsetBox["", 471],
       "InsetString"], 
      TagBox[InsetBox["", 472],
       "InsetString"], 
      TagBox[InsetBox["", 473],
       "InsetString"], 
      TagBox[InsetBox["", 474],
       "InsetString"], 
      TagBox[InsetBox["", 475],
       "InsetString"], 
      TagBox[InsetBox["", 476],
       "InsetString"], 
      TagBox[InsetBox["", 477],
       "InsetString"], 
      TagBox[InsetBox["", 478],
       "InsetString"], 
      TagBox[InsetBox["", 479],
       "InsetString"], 
      TagBox[InsetBox["", 480],
       "InsetString"], 
      TagBox[InsetBox["", 481],
       "InsetString"], 
      TagBox[InsetBox["", 482],
       "InsetString"], 
      TagBox[InsetBox["", 483],
       "InsetString"], 
      TagBox[InsetBox["", 484],
       "InsetString"], 
      TagBox[InsetBox["", 485],
       "InsetString"], 
      TagBox[InsetBox["", 486],
       "InsetString"], 
      TagBox[InsetBox["", 487],
       "InsetString"], 
      TagBox[InsetBox["", 488],
       "InsetString"], 
      TagBox[InsetBox["", 489],
       "InsetString"], 
      TagBox[InsetBox["", 490],
       "InsetString"], 
      TagBox[InsetBox["", 491],
       "InsetString"], 
      TagBox[InsetBox["", 492],
       "InsetString"], 
      TagBox[InsetBox["", 493],
       "InsetString"], 
      TagBox[InsetBox["", 494],
       "InsetString"], 
      TagBox[InsetBox["", 495],
       "InsetString"], 
      TagBox[InsetBox["", 496],
       "InsetString"]}, 
     {GrayLevel[0], 
      TagBox[InsetBox["", 992],
       "InsetString"], 
      TagBox[InsetBox["", 993],
       "InsetString"], 
      TagBox[InsetBox["", 994],
       "InsetString"], 
      TagBox[InsetBox["", 995],
       "InsetString"], 
      TagBox[InsetBox["", 996],
       "InsetString"], 
      TagBox[InsetBox["", 997],
       "InsetString"], 
      TagBox[InsetBox["", 998],
       "InsetString"], 
      TagBox[InsetBox["", 999],
       "InsetString"], 
      TagBox[InsetBox["", 1000],
       "InsetString"], 
      TagBox[InsetBox["", 1001],
       "InsetString"], 
      TagBox[InsetBox["", 1002],
       "InsetString"], 
      TagBox[InsetBox["", 1003],
       "InsetString"], 
      TagBox[InsetBox["", 1004],
       "InsetString"], 
      TagBox[InsetBox["", 1005],
       "InsetString"], 
      TagBox[InsetBox["", 1006],
       "InsetString"], 
      TagBox[InsetBox["", 1007],
       "InsetString"], 
      TagBox[InsetBox["", 1008],
       "InsetString"], 
      TagBox[InsetBox["", 1009],
       "InsetString"], 
      TagBox[InsetBox["", 1010],
       "InsetString"], 
      TagBox[InsetBox["", 1011],
       "InsetString"], 
      TagBox[InsetBox["", 1012],
       "InsetString"], 
      TagBox[InsetBox["", 1013],
       "InsetString"], 
      TagBox[InsetBox["", 1014],
       "InsetString"], 
      TagBox[InsetBox["", 1015],
       "InsetString"], 
      TagBox[InsetBox["", 1016],
       "InsetString"], 
      TagBox[InsetBox["", 1017],
       "InsetString"], 
      TagBox[InsetBox["", 1018],
       "InsetString"], 
      TagBox[InsetBox["", 1019],
       "InsetString"], 
      TagBox[InsetBox["", 1020],
       "InsetString"], 
      TagBox[InsetBox["", 1021],
       "InsetString"], 
      TagBox[InsetBox["", 1022],
       "InsetString"], 
      TagBox[InsetBox["", 1023],
       "InsetString"], 
      TagBox[InsetBox["", 1024],
       "InsetString"], 
      TagBox[InsetBox["", 1025],
       "InsetString"], 
      TagBox[InsetBox["", 1026],
       "InsetString"], 
      TagBox[InsetBox["", 1027],
       "InsetString"], 
      TagBox[InsetBox["", 1028],
       "InsetString"], 
      TagBox[InsetBox["", 1029],
       "InsetString"], 
      TagBox[InsetBox["", 1030],
       "InsetString"], 
      TagBox[InsetBox["", 1031],
       "InsetString"], 
      TagBox[InsetBox["", 1032],
       "InsetString"], 
      TagBox[InsetBox["", 1033],
       "InsetString"], 
      TagBox[InsetBox["", 1034],
       "InsetString"], 
      TagBox[InsetBox["", 1035],
       "InsetString"], 
      TagBox[InsetBox["", 1036],
       "InsetString"], 
      TagBox[InsetBox["", 1037],
       "InsetString"], 
      TagBox[InsetBox["", 1038],
       "InsetString"], 
      TagBox[InsetBox["", 1039],
       "InsetString"], 
      TagBox[InsetBox["", 1040],
       "InsetString"], 
      TagBox[InsetBox["", 1041],
       "InsetString"], 
      TagBox[InsetBox["", 1042],
       "InsetString"], 
      TagBox[InsetBox["", 1043],
       "InsetString"], 
      TagBox[InsetBox["", 1044],
       "InsetString"], 
      TagBox[InsetBox["", 1045],
       "InsetString"], 
      TagBox[InsetBox["", 1046],
       "InsetString"], 
      TagBox[InsetBox["", 1047],
       "InsetString"], 
      TagBox[InsetBox["", 1048],
       "InsetString"], 
      TagBox[InsetBox["", 1049],
       "InsetString"], 
      TagBox[InsetBox["", 1050],
       "InsetString"], 
      TagBox[InsetBox["", 1051],
       "InsetString"], 
      TagBox[InsetBox["", 1052],
       "InsetString"], 
      TagBox[InsetBox["", 1053],
       "InsetString"], 
      TagBox[InsetBox["", 1054],
       "InsetString"], 
      TagBox[InsetBox["", 1055],
       "InsetString"], 
      TagBox[InsetBox["", 1056],
       "InsetString"], 
      TagBox[InsetBox["", 1057],
       "InsetString"], 
      TagBox[InsetBox["", 1058],
       "InsetString"], 
      TagBox[InsetBox["", 1059],
       "InsetString"], 
      TagBox[InsetBox["", 1060],
       "InsetString"], 
      TagBox[InsetBox["", 1061],
       "InsetString"], 
      TagBox[InsetBox["", 1062],
       "InsetString"], 
      TagBox[InsetBox["", 1063],
       "InsetString"], 
      TagBox[InsetBox["", 1064],
       "InsetString"], 
      TagBox[InsetBox["", 1065],
       "InsetString"], 
      TagBox[InsetBox["", 1066],
       "InsetString"], 
      TagBox[InsetBox["", 1067],
       "InsetString"], 
      TagBox[InsetBox["", 1068],
       "InsetString"], 
      TagBox[InsetBox["", 1069],
       "InsetString"], 
      TagBox[InsetBox["", 1070],
       "InsetString"], 
      TagBox[InsetBox["", 1071],
       "InsetString"], 
      TagBox[InsetBox["", 1072],
       "InsetString"], 
      TagBox[InsetBox["", 1073],
       "InsetString"], 
      TagBox[InsetBox["", 1074],
       "InsetString"], 
      TagBox[InsetBox["", 1075],
       "InsetString"], 
      TagBox[InsetBox["", 1076],
       "InsetString"], 
      TagBox[InsetBox["", 1077],
       "InsetString"], 
      TagBox[InsetBox["", 1078],
       "InsetString"], 
      TagBox[InsetBox["", 1079],
       "InsetString"], 
      TagBox[InsetBox["", 1080],
       "InsetString"], 
      TagBox[InsetBox["", 1081],
       "InsetString"], 
      TagBox[InsetBox["", 1082],
       "InsetString"], 
      TagBox[InsetBox["", 1083],
       "InsetString"], 
      TagBox[InsetBox["", 1084],
       "InsetString"], 
      TagBox[InsetBox["", 1085],
       "InsetString"], 
      TagBox[InsetBox["", 1086],
       "InsetString"], 
      TagBox[InsetBox["", 1087],
       "InsetString"], 
      TagBox[InsetBox["", 1088],
       "InsetString"], 
      TagBox[InsetBox["", 1089],
       "InsetString"], 
      TagBox[InsetBox["", 1090],
       "InsetString"], 
      TagBox[InsetBox["", 1091],
       "InsetString"], 
      TagBox[InsetBox["", 1092],
       "InsetString"], 
      TagBox[InsetBox["", 1093],
       "InsetString"], 
      TagBox[InsetBox["", 1094],
       "InsetString"], 
      TagBox[InsetBox["", 1095],
       "InsetString"], 
      TagBox[InsetBox["", 1096],
       "InsetString"], 
      TagBox[InsetBox["", 1097],
       "InsetString"], 
      TagBox[InsetBox["", 1098],
       "InsetString"], 
      TagBox[InsetBox["", 1099],
       "InsetString"], 
      TagBox[InsetBox["", 1100],
       "InsetString"], 
      TagBox[InsetBox["", 1101],
       "InsetString"], 
      TagBox[InsetBox["", 1102],
       "InsetString"], 
      TagBox[InsetBox["", 1103],
       "InsetString"], 
      TagBox[InsetBox["", 1104],
       "InsetString"], 
      TagBox[InsetBox["", 1105],
       "InsetString"], 
      TagBox[InsetBox["", 1106],
       "InsetString"], 
      TagBox[InsetBox["", 1107],
       "InsetString"], 
      TagBox[InsetBox["", 1108],
       "InsetString"], 
      TagBox[InsetBox["", 1109],
       "InsetString"], 
      TagBox[InsetBox["", 1110],
       "InsetString"], 
      TagBox[InsetBox["", 1111],
       "InsetString"], 
      TagBox[InsetBox["", 1112],
       "InsetString"], 
      TagBox[InsetBox["", 1113],
       "InsetString"], 
      TagBox[InsetBox["", 1114],
       "InsetString"], 
      TagBox[InsetBox["", 1115],
       "InsetString"], 
      TagBox[InsetBox["", 1116],
       "InsetString"], 
      TagBox[InsetBox["", 1117],
       "InsetString"], 
      TagBox[InsetBox["", 1118],
       "InsetString"], 
      TagBox[InsetBox["", 1119],
       "InsetString"], 
      TagBox[InsetBox["", 1120],
       "InsetString"], 
      TagBox[InsetBox["", 1121],
       "InsetString"], 
      TagBox[InsetBox["", 1122],
       "InsetString"], 
      TagBox[InsetBox["", 1123],
       "InsetString"], 
      TagBox[InsetBox["", 1124],
       "InsetString"], 
      TagBox[InsetBox["", 1125],
       "InsetString"], 
      TagBox[InsetBox["", 1126],
       "InsetString"], 
      TagBox[InsetBox["", 1127],
       "InsetString"], 
      TagBox[InsetBox["", 1128],
       "InsetString"], 
      TagBox[InsetBox["", 1129],
       "InsetString"], 
      TagBox[InsetBox["", 1130],
       "InsetString"], 
      TagBox[InsetBox["", 1131],
       "InsetString"], 
      TagBox[InsetBox["", 1132],
       "InsetString"], 
      TagBox[InsetBox["", 1133],
       "InsetString"], 
      TagBox[InsetBox["", 1134],
       "InsetString"], 
      TagBox[InsetBox["", 1135],
       "InsetString"], 
      TagBox[InsetBox["", 1136],
       "InsetString"], 
      TagBox[InsetBox["", 1137],
       "InsetString"], 
      TagBox[InsetBox["", 1138],
       "InsetString"], 
      TagBox[InsetBox["", 1139],
       "InsetString"], 
      TagBox[InsetBox["", 1140],
       "InsetString"], 
      TagBox[InsetBox["", 1141],
       "InsetString"], 
      TagBox[InsetBox["", 1142],
       "InsetString"], 
      TagBox[InsetBox["", 1143],
       "InsetString"], 
      TagBox[InsetBox["", 1144],
       "InsetString"], 
      TagBox[InsetBox["", 1145],
       "InsetString"], 
      TagBox[InsetBox["", 1146],
       "InsetString"], 
      TagBox[InsetBox["", 1147],
       "InsetString"], 
      TagBox[InsetBox["", 1148],
       "InsetString"], 
      TagBox[InsetBox["", 1149],
       "InsetString"], 
      TagBox[InsetBox["", 1150],
       "InsetString"], 
      TagBox[InsetBox["", 1151],
       "InsetString"], 
      TagBox[InsetBox["", 1152],
       "InsetString"], 
      TagBox[InsetBox["", 1153],
       "InsetString"], 
      TagBox[InsetBox["", 1154],
       "InsetString"], 
      TagBox[InsetBox["", 1155],
       "InsetString"], 
      TagBox[InsetBox["", 1156],
       "InsetString"], 
      TagBox[InsetBox["", 1157],
       "InsetString"], 
      TagBox[InsetBox["", 1158],
       "InsetString"], 
      TagBox[InsetBox["", 1159],
       "InsetString"], 
      TagBox[InsetBox["", 1160],
       "InsetString"], 
      TagBox[InsetBox["", 1161],
       "InsetString"], 
      TagBox[InsetBox["", 1162],
       "InsetString"], 
      TagBox[InsetBox["", 1163],
       "InsetString"], 
      TagBox[InsetBox["", 1164],
       "InsetString"], 
      TagBox[InsetBox["", 1165],
       "InsetString"], 
      TagBox[InsetBox["", 1166],
       "InsetString"], 
      TagBox[InsetBox["", 1167],
       "InsetString"], 
      TagBox[InsetBox["", 1168],
       "InsetString"], 
      TagBox[InsetBox["", 1169],
       "InsetString"], 
      TagBox[InsetBox["", 1170],
       "InsetString"], 
      TagBox[InsetBox["", 1171],
       "InsetString"], 
      TagBox[InsetBox["", 1172],
       "InsetString"], 
      TagBox[InsetBox["", 1173],
       "InsetString"], 
      TagBox[InsetBox["", 1174],
       "InsetString"], 
      TagBox[InsetBox["", 1175],
       "InsetString"], 
      TagBox[InsetBox["", 1176],
       "InsetString"], 
      TagBox[InsetBox["", 1177],
       "InsetString"], 
      TagBox[InsetBox["", 1178],
       "InsetString"], 
      TagBox[InsetBox["", 1179],
       "InsetString"], 
      TagBox[InsetBox["", 1180],
       "InsetString"], 
      TagBox[InsetBox["", 1181],
       "InsetString"], 
      TagBox[InsetBox["", 1182],
       "InsetString"], 
      TagBox[InsetBox["", 1183],
       "InsetString"], 
      TagBox[InsetBox["", 1184],
       "InsetString"], 
      TagBox[InsetBox["", 1185],
       "InsetString"], 
      TagBox[InsetBox["", 1186],
       "InsetString"], 
      TagBox[InsetBox["", 1187],
       "InsetString"], 
      TagBox[InsetBox["", 1188],
       "InsetString"], 
      TagBox[InsetBox["", 1189],
       "InsetString"], 
      TagBox[InsetBox["", 1190],
       "InsetString"], 
      TagBox[InsetBox["", 1191],
       "InsetString"], 
      TagBox[InsetBox["", 1192],
       "InsetString"], 
      TagBox[InsetBox["", 1193],
       "InsetString"], 
      TagBox[InsetBox["", 1194],
       "InsetString"], 
      TagBox[InsetBox["", 1195],
       "InsetString"], 
      TagBox[InsetBox["", 1196],
       "InsetString"], 
      TagBox[InsetBox["", 1197],
       "InsetString"], 
      TagBox[InsetBox["", 1198],
       "InsetString"], 
      TagBox[InsetBox["", 1199],
       "InsetString"], 
      TagBox[InsetBox["", 1200],
       "InsetString"], 
      TagBox[InsetBox["", 1201],
       "InsetString"], 
      TagBox[InsetBox["", 1202],
       "InsetString"], 
      TagBox[InsetBox["", 1203],
       "InsetString"], 
      TagBox[InsetBox["", 1204],
       "InsetString"], 
      TagBox[InsetBox["", 1205],
       "InsetString"], 
      TagBox[InsetBox["", 1206],
       "InsetString"], 
      TagBox[InsetBox["", 1207],
       "InsetString"], 
      TagBox[InsetBox["", 1208],
       "InsetString"], 
      TagBox[InsetBox["", 1209],
       "InsetString"], 
      TagBox[InsetBox["", 1210],
       "InsetString"], 
      TagBox[InsetBox["", 1211],
       "InsetString"], 
      TagBox[InsetBox["", 1212],
       "InsetString"], 
      TagBox[InsetBox["", 1213],
       "InsetString"], 
      TagBox[InsetBox["", 1214],
       "InsetString"], 
      TagBox[InsetBox["", 1215],
       "InsetString"], 
      TagBox[InsetBox["", 1216],
       "InsetString"], 
      TagBox[InsetBox["", 1217],
       "InsetString"], 
      TagBox[InsetBox["", 1218],
       "InsetString"], 
      TagBox[InsetBox["", 1219],
       "InsetString"], 
      TagBox[InsetBox["", 1220],
       "InsetString"], 
      TagBox[InsetBox["", 1221],
       "InsetString"], 
      TagBox[InsetBox["", 1222],
       "InsetString"], 
      TagBox[InsetBox["", 1223],
       "InsetString"], 
      TagBox[InsetBox["", 1224],
       "InsetString"], 
      TagBox[InsetBox["", 1225],
       "InsetString"], 
      TagBox[InsetBox["", 1226],
       "InsetString"], 
      TagBox[InsetBox["", 1227],
       "InsetString"], 
      TagBox[InsetBox["", 1228],
       "InsetString"], 
      TagBox[InsetBox["", 1229],
       "InsetString"], 
      TagBox[InsetBox["", 1230],
       "InsetString"], 
      TagBox[InsetBox["", 1231],
       "InsetString"], 
      TagBox[InsetBox["", 1232],
       "InsetString"], 
      TagBox[InsetBox["", 1233],
       "InsetString"], 
      TagBox[InsetBox["", 1234],
       "InsetString"], 
      TagBox[InsetBox["", 1235],
       "InsetString"], 
      TagBox[InsetBox["", 1236],
       "InsetString"], 
      TagBox[InsetBox["", 1237],
       "InsetString"], 
      TagBox[InsetBox["", 1238],
       "InsetString"], 
      TagBox[InsetBox["", 1239],
       "InsetString"]}, {}}}],
  AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
  Axes->True,
  AxesOrigin->NCache[{3431635200, 0}, {3.4316352*^9, 0}],
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"MACD\"", Bold, 
            RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{None, None}, {{{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"Jan\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"Apr\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"Jul\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}, {{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3447532800, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3463344000, 
       GrayLevel[0.8]}}, None}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4475328*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.463344*^9, 
       GrayLevel[0.8]}}, None}],
  GridLinesStyle->Directive[
    GrayLevel[0.85], 
    Dashing[{Small, Small}], 
    AbsoluteThickness[0.5]],
  ImagePadding->{{50, 10}, {1, 10}},
  ImageSize->600,
  PlotRange->{{3.4316352*^9, 3.4625664*^9}, {-24.490125434790798`, 
   14.446116074288113`}},
  PlotRangeClipping->True,
  PlotRangePadding->{200000, 1},
  Ticks->{{{
      NCache[3431808000, 3.431808*^9], 
      FormBox["\"Oct\"", TraditionalForm]}, {
      NCache[3439756800, 3.4397568*^9], 
      FormBox["\"Jan\"", TraditionalForm]}, {
      NCache[3447532800, 3.4475328*^9], 
      FormBox["\"Apr\"", TraditionalForm]}, {
      NCache[3455395200, 3.4553952*^9], 
      FormBox["\"Jul\"", TraditionalForm]}, {
      NCache[3463344000, 3.463344*^9], 
      FormBox["\"Oct\"", TraditionalForm]}, {
      NCache[3434486400, 3.4344864*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3437078400, 3.4370784*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3442435200, 3.4424352*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3444854400, 3.4448544*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3450124800, 3.4501248*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3452803200, 3.4528032*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3458073600, 3.4580736*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3460752000, 3.460752*^9], 
      FormBox["\"\"", TraditionalForm]}}, Automatic}]], "Output",
 CellChangeTimes->{3.462723822833778*^9, 3.4627286337644176`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "RSIPlot"}]], "Input",
 CellChangeTimes->{{3.462723225934141*^9, 3.462723248999812*^9}, 
   3.462723589597933*^9}],

Cell[BoxData[
 StyleBox["\<\"RSIPlot[data, num, range] will generate an RSI plot of the \
data. The data is a list of stock data with the form {dates, open, high, low, \
close, volume}, num is the number of days to use for an exponential moving \
average, range is the range between 0 and 100 to highlight on the plot, the \
default value is {20, 80}.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728637387081*^9},
 CellTags->"Info3462710636-9579012"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RSIPlot", "[", 
  RowBox[{"goog1", ",", "27", ",", 
   RowBox[{"{", 
    RowBox[{"20", ",", "80"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.46272383440407*^9, 3.462723845276463*^9}, {
  3.462723919741647*^9, 3.462723923731969*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.6, 0.24, 0.4429], AbsoluteThickness[1.5], 
    LineBox[CompressedData["
1:eJwl1glYTPsbB/Bpo2iZv6WSpcHlStuE5MqtOdFqaVqGVDRUWmnutddtkcSl
NHS1IE2ylMJIaa+hVJQaES4tY4n/qWSIIsp9e3+ep8fzec45c97f+3vP95zZ
W8Nc/RUZDEYn/I39D/9kVCpNifXCe8afXMcBM0PBf9z1N9SatHrMnBRwgGe4
tCzHccwCCXin1c3zGpfQDOs0mlLxn6VqpOIwZnYQuKXuzo72qfZj5ieDM2+c
mTPvyqoxCyvBsZ5XHz8rweOSd+CtGc23vuxBc7el0xRn+ERrqoXdmGOE4Lom
fWqezHbM4jLwow/25dcaVo5Z9gasEqlV4pJBYf1ap2lqvFWrwda5NmMWJYI1
pXt5fafxuLQYXF4QHZE+jL/HeAUupR31HBJIvUvP0NTilbmlEdfR8kJwWlEn
3WKC9bO6wIvk4y1khXg9V+0sTfl4uKu9CSX1LgZPyX3QfrqU1LsJ7BBttHl/
CP4epx1cpcRvmbzaCfs5LoOmlvkXh37qQIvY4MY0byvJWuy/1BPs1dB+uaiU
9PsQWM6LlWuYrMF+Kp2jqVqjKy8zzPF8iTG45beC6Z+2rcX6N4BTYsxUs//E
46xYsL6bw+XsWLwfNx8cpEklTI9EyxZmQn1HvFL0RrBeJg/8ZCRpd78h7i8n
GtyybFFSPBvrET0CWzatiOrKJvvvKqKpGXF6L9QZaP5f4DXKKSee8PB84SXw
hKXaeXl8vJ98GPx113DN/t+w3pj9WTS1YH3jm7f7cB7F2WCd2W2bXv7tjPU9
AEsUdCe7L8X1M7+C8+uqbl96hubMOU9TN5U/n6GX4+9LReDbnxfVz7lA+tcI
zrW2fFuqhsfZX8APB+pmcmpx//j62TT165XMhlQXtNARvPes6ff6ODT3E9j+
9KhPdwfOV8yMCzQliNPVdVqF6xXbgTf3Ll37Qx3vJxOAH9zLzPW5g/UJPoBV
qjNmKSeiRdMu0lTNOPnVou+4fulK8L2DTlE9vlysdwd4ns2SNg8urp+dBs4f
zqpoVsb+SLQv0dTMgxb54/3Qcg54rseUwg5PNCsErBiYkNfZieaeAhv/eJcc
vRN/P6Ya7HzCcsuTJhfsp9Vlmiob6nEJdHTFfgaCTYqtnW4dx+sFJ8FWCzrs
CnagRRXgCEbP+ghnrE/6FtzZ2+hSVYvrESbl0JSpcDXbuQ/7ISkFO12Pf26p
juuXvwb7ve36a/QMHmdp5sL1DCe3kiDcH3ECOLRNMLC6B/svuwW2k41u/WUm
Hme+BBf/7sub6oLXcyZeoakdj3dr9/NxPwTm4NpJGuf0hWhGIThDY0PCQwp/
j90JXlbLdXlK4/PKV82jKVHOwIzmZJJXi8B8X55ZtQHut8QbrCLeq9c4DvOH
+wKc6eVZ8M2Ig/1UyYd5iKYGkxrwerEpeObg5tQWJXyeZBvBYcYK91tWYD3M
OPD28o2qW8PQIsWrkL+xr5L8A/F8qRH4uVmbplsXrpexAaxz0If3ZgOZ3wNg
QVX+yMR/cP38PPCEiLxlWVloucE1mgpk9w3uFOF+sNzBKopiDYXlaG4UWDNp
U6YtD/cjJgdc6hzC/7KSPH+t4Lk/fiqcTSfPl8t1mL/1Fv1eCzBPBBHgb4Fl
ezWCyTxfBFv8f9W6yjtknlvAQ1pfU5OjcD4Yw+CYYM/6XTtw/oT7xDQ1MKWm
vWYBWnIevGeF9tHF2iS/msDdx1Ra1jwj9Q+BlcsNN4Wp4PmyzBs0tVBqmaCf
iGbeBzPvBoiNh/F8zmdwvUGzU4YaHhfMKqApG6tj6pynaJED2DzTyKSPhfPP
rgfvnrD0a58arp//ETytUs9Rdz9aOP0mzJ/pgOLTQfL82YJbzx7/QP2C65OH
geWuog+J2uT56gcrbj7py24j/dQthPfdIx2nR6Q+mQ2YHW730Pk5Xs/cDh6Z
eL+Gvx2v56SCSzql8W1vyPM1tYimHNdfzI6WkXzggLuO0od//YFmB4OZnnE3
xk3A9fD/Ab8wjld91kHqrwI/T/EJNK3C81m/34Lnpbe0ipaiuQHgGbFLRieV
kDw+AeaZ2DKWW5PnsRyc7vO+ojeR5Fs3eKIGU/kwyXOBXzFNuR8YMNWJJfNw
HKx0rf6QykwyDyXgEbXBj1x/PM54DQ7JWhj8URfN1iihKet3mnuG7pD3QxE4
1FCHNvEneSYDx5a45rZLSJ5NKIX8OhNmeGQq6fcSMG+6bVL3MJmHm2B3888K
rkokzzrAXNnC/I+L0ILxZZAf1Uk1nfFokRlYWu24TeTshvV6gV2/OZ3flY3m
PwebnEzY0q+L5wuVy2mqQT+05uUq7LfEBPx3GaPILxLvL/cAB0Tmfo80wv1j
HQTb+HwO3mqK/RArVNDURu+0uCoJ6acheKFdlNP4P9HM9eBNE1q9e3VIvsWA
JSv7k36eJfl2BVxUYO2axcbnkWFQSVPHuhPaqrRJvrmBxQEPRp/FkXyLBItG
5h4cmo35IrwMvm9fEPKpgeTzQ/CBPoPJ2UySD9wqeB+4L3B+m0DyIRwc4TY6
mBOC9xNfAI/rDLvAsyffD81g2/mbt9y2xvuL9lbTVN6A8rqyMrQ0C2zjGJr1
Q2yD9TaBvxrLJVYmWB97EGxU9DW30oB8P7Ak8D3iadvbdYzk2TnwPp6t7lAq
1sO6B+5KX5/eUIjmDoDrw42UTumSeZh5G/obrBkfdAottgcL1bOn9UaQPKsD
M23T5lftIu9nOfjO6yNRU0W4PpHeHZrSsuhyTT5GvsdWgf3yTjaMeuP8McLA
f4Tf8vdyJXn2fuw4y+f2/Me43xKdGuh3eRNrNnkfyimw/hIH+RQ3Un8o+Lz/
A7awh8xzCph7wpq35QXJhym1kE+t8hhXHTLP1uDz4b7eIbYkH4LA9SMZuu/4
eD9BMjjrkE90kzJaVAmuUP63NDuU5MOKuzQlex9aPnctmr8NPLLWpqd5OZ4v
FIKvORgu61tD8rgMvM1JNN2P5IP8DTgis3oyy4T017cOvh97vhu8j8b9ESeC
VeUtj/LI95asGBwefY0SzCDz/ApcernynJoQ18tRr6epSVzzSUdaSb4dBS/1
Sg/65kHyrQj8OL4l3SOf1N8FtjTLyC8qxPP5ag3wfCrW88NrSb4tBudoj+rG
zyJ5UQDuujH7kFIamY92cFPbVf+YV1hPzLh7NPWzsjHS4Bb5fmODacfyncOn
yDx7gq2O6ha1apA8U7oP77NIkVbBXfJ9aQzOLGbOO/QvyTMPsF7kaJ2ZOfle
iwWH3j7QLUoh88BohP69H3APWEX6uRDc5ZkRdfQCyWMeOLjQ3Sv5f5gv3Gjw
zmMNs57tRMfkgiu+dIpObUYzFzTR1OFq5ep9IyTfXMFTck5JNc3dOP8Bd22C
mw==
     "]]}},
  AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
  Axes->True,
  AxesOrigin->NCache[{3431635200, 50}, {3.4316352*^9, 50}],
  AxesStyle->GrayLevel[0.5],
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"RSI\"", Bold, 
            RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{{{20, 
       FormBox["20", TraditionalForm]}, {50, 
       FormBox["50", TraditionalForm]}, {80, 
       FormBox["80", TraditionalForm]}}, None}, {None, None}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3447532800, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3463344000, 
       GrayLevel[0.8]}}, {20, 80}}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4475328*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.463344*^9, 
       GrayLevel[0.8]}}, {20, 80}}],
  GridLinesStyle->{
    Directive[
     GrayLevel[0.85], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]], 
    Directive[
     GrayLevel[0.45], 
     Dashing[{Small, Small}], 
     AbsoluteThickness[0.5]]},
  ImagePadding->{{50, 10}, {1, 1}},
  ImageSize->600,
  PlotRange->
   NCache[{{3431635200, 3462566400}, {0, 100}}, {{3.4316352*^9, 
     3.4625664*^9}, {0, 100}}],
  PlotRangeClipping->True,
  PlotRangePadding->{200000, 0},
  Ticks->{{{
      NCache[3431808000, 3.431808*^9], 
      FormBox["\"Oct\"", TraditionalForm]}, {
      NCache[3439756800, 3.4397568*^9], 
      FormBox["\"Jan\"", TraditionalForm]}, {
      NCache[3447532800, 3.4475328*^9], 
      FormBox["\"Apr\"", TraditionalForm]}, {
      NCache[3455395200, 3.4553952*^9], 
      FormBox["\"Jul\"", TraditionalForm]}, {
      NCache[3463344000, 3.463344*^9], 
      FormBox["\"Oct\"", TraditionalForm]}, {
      NCache[3434486400, 3.4344864*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3437078400, 3.4370784*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3442435200, 3.4424352*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3444854400, 3.4448544*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3450124800, 3.4501248*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3452803200, 3.4528032*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3458073600, 3.4580736*^9], 
      FormBox["\"\"", TraditionalForm]}, {
      NCache[3460752000, 3.460752*^9], 
      FormBox["\"\"", TraditionalForm]}}, Automatic}]], "Output",
 CellChangeTimes->{3.462723925653633*^9, 3.462728639412819*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ADIPlot"}]], "Input",
 CellChangeTimes->{{3.462723239115*^9, 3.462723239459683*^9}}],

Cell[BoxData[
 StyleBox["\<\"ADIPlot[data, range] will generate an ADI plot. The data is a \
list of stock data with the form {dates, open, high, low, close, volume}, \
range is an option for PlotRange, the default value is Automatic.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728643102425*^9},
 CellTags->"Info3462710642-5540596"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ADIPlot", "[", "goog1", "]"}]], "Input",
 CellChangeTimes->{3.4627238577807407`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.24, 0.6, 0.34], AbsoluteThickness[1.5], LineBox[CompressedData["
1:eJwl1glYTOsfB/BpJeuURFSmRa6ljMreMocUV5hW7Y0WWjSmooXSqNSNy51o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     "]]}},
  AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
  AxesOrigin->NCache[{3431635200, 0}, {3.4316352*^9, 0}],
  Epilog->InsetBox[
    BoxData[
     FormBox[
      PanelBox[
       TagBox[
        GridBox[{{
           StyleBox["\"ADI\"", Bold, 
            RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
    Offset[{0, 0}, 
     Scaled[{0.05, 0.95}]], {Left, Top}],
  Frame->True,
  FrameTicks->{{None, None}, {{{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"Jan\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"Apr\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"Jul\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"Oct\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}, {{
       NCache[3431808000, 3.431808*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3439756800, 3.4397568*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3447532800, 3.4475328*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3455395200, 3.4553952*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3463344000, 3.463344*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3434486400, 3.4344864*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3437078400, 3.4370784*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3442435200, 3.4424352*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3444854400, 3.4448544*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3450124800, 3.4501248*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3452803200, 3.4528032*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3458073600, 3.4580736*^9], 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[3460752000, 3.460752*^9], 
       FormBox["\"\"", TraditionalForm]}}}},
  FrameTicksStyle->Directive[
    GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
  GridLines->NCache[{{{3431808000, 
       GrayLevel[0.8]}, {3439756800, 
       GrayLevel[0.8]}, {3447532800, 
       GrayLevel[0.8]}, {3455395200, 
       GrayLevel[0.8]}, {3463344000, 
       GrayLevel[0.8]}}, None}, {{{3.431808*^9, 
       GrayLevel[0.8]}, {3.4397568*^9, 
       GrayLevel[0.8]}, {3.4475328*^9, 
       GrayLevel[0.8]}, {3.4553952*^9, 
       GrayLevel[0.8]}, {3.463344*^9, 
       GrayLevel[0.8]}}, None}],
  GridLinesStyle->Directive[
    GrayLevel[0.85], 
    Dashing[{Small, Small}], 
    AbsoluteThickness[0.5]],
  ImagePadding->{{50, 10}, {15, 1}},
  ImageSize->600,
  PlotRange->{{3.4316352*^9, 3.4625664*^9}, {-4.933504850544699*^7, 
   2.8986750542983193`*^7}},
  PlotRangeClipping->True,
  PlotRangePadding->{200000, 0},
  Ticks->None]], "Output",
 CellChangeTimes->{3.462724004609084*^9, 3.462728648447104*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["StockPlot", "MB"],
 " is the function that brings the other plots together."
}], "Text",
 CellChangeTimes->{{3.462728656643497*^9, 3.4627286714661827`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "StockPlot"}]], "Input",
 CellChangeTimes->{{3.462723307871024*^9, 3.4627233106529427`*^9}}],

Cell[BoxData[
 StyleBox["\<\"StockPlot[data, {label, label color}] will generate a plot of \
price with a plot of violume underneath. The data is a list of stock data \
with the form {dates, open, high, low, close, volume}. PlotType possibilities \
are \\\"Price\\\", \\\"OHLC\\\", and \\\"Candlestick\\\". Options specific \
for the price and volume plots can be entered as options for PriceStyling and \
VolumeStyling respectively.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.462728679328806*^9},
 CellTags->"Info3462710678-2388317"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{"goog1", ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Red"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.462724028984227*^9}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[{{}, {}, 
      {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJxVl3lwU1UUxp+iKIsatYqi0qDjWFwgUFR0RFNRijKOoYisjhnAAgUhqAUt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        "]]}},
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.435*^9, 300.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[1, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3431808000, 
          GrayLevel[0.8]}, {3439756800, 
          GrayLevel[0.8]}, {3447532800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3463344000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
          GrayLevel[0.8]}, {3.4397568*^9, 
          GrayLevel[0.8]}, {3.4475328*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.463344*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {257.44, 499.06}},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 5}},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmflfjekbx8/QMGSmyIRshywhlG00Y+kpW2Emsg8zaRgSOvaxTQfVWFKH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      
      "], {{{}, {}, {}, {}, {}, 
        {RGBColor[0.75, 0., 0.], Thickness[Large], LineBox[{372, 1}], 
         LineBox[{373, 2}], LineBox[{374, 3}], LineBox[{375, 4}], 
         LineBox[{376, 5}], LineBox[{377, 6}], LineBox[{378, 7}], 
         LineBox[{379, 8}], LineBox[{380, 9}], LineBox[{381, 10}], 
         LineBox[{382, 11}], LineBox[{383, 12}], LineBox[{384, 13}], 
         LineBox[{385, 14}], LineBox[{386, 15}], LineBox[{387, 16}], 
         LineBox[{388, 17}], LineBox[{389, 18}], LineBox[{390, 19}], 
         LineBox[{391, 20}], LineBox[{392, 21}], LineBox[{393, 22}], 
         LineBox[{394, 23}], LineBox[{395, 24}], LineBox[{396, 25}], 
         LineBox[{397, 26}], LineBox[{398, 27}], LineBox[{399, 28}], 
         LineBox[{400, 29}], LineBox[{401, 30}], LineBox[{402, 31}], 
         LineBox[{403, 32}], LineBox[{404, 33}], LineBox[{405, 34}], 
         LineBox[{406, 35}], LineBox[{407, 36}], LineBox[{408, 37}], 
         LineBox[{409, 38}], LineBox[{410, 39}], LineBox[{411, 40}], 
         LineBox[{412, 41}], LineBox[{413, 42}], LineBox[{414, 43}], 
         LineBox[{415, 44}], LineBox[{416, 45}], LineBox[{417, 46}], 
         LineBox[{418, 47}], LineBox[{419, 48}], LineBox[{420, 49}], 
         LineBox[{421, 50}], LineBox[{422, 51}], LineBox[{423, 52}], 
         LineBox[{424, 53}], LineBox[{425, 54}], LineBox[{426, 55}], 
         LineBox[{427, 56}], LineBox[{428, 57}], LineBox[{429, 58}], 
         LineBox[{430, 59}], LineBox[{431, 60}], LineBox[{432, 61}], 
         LineBox[{433, 62}], LineBox[{434, 63}], LineBox[{435, 64}], 
         LineBox[{436, 65}], LineBox[{437, 66}], LineBox[{438, 67}], 
         LineBox[{439, 68}], LineBox[{440, 69}], LineBox[{441, 70}], 
         LineBox[{442, 71}], LineBox[{443, 72}], LineBox[{444, 73}], 
         LineBox[{445, 74}], LineBox[{446, 75}], LineBox[{447, 76}], 
         LineBox[{448, 77}], LineBox[{449, 78}], LineBox[{450, 79}], 
         LineBox[{451, 80}], LineBox[{452, 81}], LineBox[{453, 82}], 
         LineBox[{454, 83}], LineBox[{455, 84}], LineBox[{456, 85}], 
         LineBox[{457, 86}], LineBox[{458, 87}], LineBox[{459, 88}], 
         LineBox[{460, 89}], LineBox[{461, 90}], LineBox[{462, 91}], 
         LineBox[{463, 92}], LineBox[{464, 93}], LineBox[{465, 94}], 
         LineBox[{466, 95}], LineBox[{467, 96}], LineBox[{468, 97}], 
         LineBox[{469, 98}], LineBox[{470, 99}], LineBox[{471, 100}], 
         LineBox[{472, 101}], LineBox[{473, 102}], LineBox[{474, 103}], 
         LineBox[{475, 104}], LineBox[{476, 105}], LineBox[{477, 106}], 
         LineBox[{478, 107}], LineBox[{479, 108}], LineBox[{480, 109}], 
         LineBox[{481, 110}], LineBox[{482, 111}], LineBox[{483, 112}], 
         LineBox[{484, 113}], LineBox[{485, 114}], LineBox[{486, 115}], 
         LineBox[{487, 116}], LineBox[{488, 117}], LineBox[{489, 118}], 
         LineBox[{490, 119}], LineBox[{491, 120}], LineBox[{492, 121}], 
         LineBox[{493, 122}], LineBox[{494, 123}], LineBox[{495, 124}], 
         LineBox[{496, 125}]}, {}, 
        {RGBColor[0, 0.5019607843137255, 0], Thickness[Large], 
         LineBox[{249, 126}], LineBox[{250, 127}], LineBox[{251, 128}], 
         LineBox[{252, 129}], LineBox[{253, 130}], LineBox[{254, 131}], 
         LineBox[{255, 132}], LineBox[{256, 133}], LineBox[{257, 134}], 
         LineBox[{258, 135}], LineBox[{259, 136}], LineBox[{260, 137}], 
         LineBox[{261, 138}], LineBox[{262, 139}], LineBox[{263, 140}], 
         LineBox[{264, 141}], LineBox[{265, 142}], LineBox[{266, 143}], 
         LineBox[{267, 144}], LineBox[{268, 145}], LineBox[{269, 146}], 
         LineBox[{270, 147}], LineBox[{271, 148}], LineBox[{272, 149}], 
         LineBox[{273, 150}], LineBox[{274, 151}], LineBox[{275, 152}], 
         LineBox[{276, 153}], LineBox[{277, 154}], LineBox[{278, 155}], 
         LineBox[{279, 156}], LineBox[{280, 157}], LineBox[{281, 158}], 
         LineBox[{282, 159}], LineBox[{283, 160}], LineBox[{284, 161}], 
         LineBox[{285, 162}], LineBox[{286, 163}], LineBox[{287, 164}], 
         LineBox[{288, 165}], LineBox[{289, 166}], LineBox[{290, 167}], 
         LineBox[{291, 168}], LineBox[{292, 169}], LineBox[{293, 170}], 
         LineBox[{294, 171}], LineBox[{295, 172}], LineBox[{296, 173}], 
         LineBox[{297, 174}], LineBox[{298, 175}], LineBox[{299, 176}], 
         LineBox[{300, 177}], LineBox[{301, 178}], LineBox[{302, 179}], 
         LineBox[{303, 180}], LineBox[{304, 181}], LineBox[{305, 182}], 
         LineBox[{306, 183}], LineBox[{307, 184}], LineBox[{308, 185}], 
         LineBox[{309, 186}], LineBox[{310, 187}], LineBox[{311, 188}], 
         LineBox[{312, 189}], LineBox[{313, 190}], LineBox[{314, 191}], 
         LineBox[{315, 192}], LineBox[{316, 193}], LineBox[{317, 194}], 
         LineBox[{318, 195}], LineBox[{319, 196}], LineBox[{320, 197}], 
         LineBox[{321, 198}], LineBox[{322, 199}], LineBox[{323, 200}], 
         LineBox[{324, 201}], LineBox[{325, 202}], LineBox[{326, 203}], 
         LineBox[{327, 204}], LineBox[{328, 205}], LineBox[{329, 206}], 
         LineBox[{330, 207}], LineBox[{331, 208}], LineBox[{332, 209}], 
         LineBox[{333, 210}], LineBox[{334, 211}], LineBox[{335, 212}], 
         LineBox[{336, 213}], LineBox[{337, 214}], LineBox[{338, 215}], 
         LineBox[{339, 216}], LineBox[{340, 217}], LineBox[{341, 218}], 
         LineBox[{342, 219}], LineBox[{343, 220}], LineBox[{344, 221}], 
         LineBox[{345, 222}], LineBox[{346, 223}], LineBox[{347, 224}], 
         LineBox[{348, 225}], LineBox[{349, 226}], LineBox[{350, 227}], 
         LineBox[{351, 228}], LineBox[{352, 229}], LineBox[{353, 230}], 
         LineBox[{354, 231}], LineBox[{355, 232}], LineBox[{356, 233}], 
         LineBox[{357, 234}], LineBox[{358, 235}], LineBox[{359, 236}], 
         LineBox[{360, 237}], LineBox[{361, 238}], LineBox[{362, 239}], 
         LineBox[{363, 240}], LineBox[{364, 241}], LineBox[{365, 242}], 
         LineBox[{366, 243}], LineBox[{367, 244}], LineBox[{368, 245}], 
         LineBox[{369, 246}], LineBox[{370, 247}], LineBox[{371, 248}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"], 
         TagBox[InsetBox["", 601],
          "InsetString"], 
         TagBox[InsetBox["", 602],
          "InsetString"], 
         TagBox[InsetBox["", 603],
          "InsetString"], 
         TagBox[InsetBox["", 604],
          "InsetString"], 
         TagBox[InsetBox["", 605],
          "InsetString"], 
         TagBox[InsetBox["", 606],
          "InsetString"], 
         TagBox[InsetBox["", 607],
          "InsetString"], 
         TagBox[InsetBox["", 608],
          "InsetString"], 
         TagBox[InsetBox["", 609],
          "InsetString"], 
         TagBox[InsetBox["", 610],
          "InsetString"], 
         TagBox[InsetBox["", 611],
          "InsetString"], 
         TagBox[InsetBox["", 612],
          "InsetString"], 
         TagBox[InsetBox["", 613],
          "InsetString"], 
         TagBox[InsetBox["", 614],
          "InsetString"], 
         TagBox[InsetBox["", 615],
          "InsetString"], 
         TagBox[InsetBox["", 616],
          "InsetString"], 
         TagBox[InsetBox["", 617],
          "InsetString"], 
         TagBox[InsetBox["", 618],
          "InsetString"], 
         TagBox[InsetBox["", 619],
          "InsetString"], 
         TagBox[InsetBox["", 620],
          "InsetString"], 
         TagBox[InsetBox["", 621],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 622],
          "InsetString"], 
         TagBox[InsetBox["", 623],
          "InsetString"], 
         TagBox[InsetBox["", 624],
          "InsetString"], 
         TagBox[InsetBox["", 625],
          "InsetString"], 
         TagBox[InsetBox["", 626],
          "InsetString"], 
         TagBox[InsetBox["", 627],
          "InsetString"], 
         TagBox[InsetBox["", 628],
          "InsetString"], 
         TagBox[InsetBox["", 629],
          "InsetString"], 
         TagBox[InsetBox["", 630],
          "InsetString"], 
         TagBox[InsetBox["", 631],
          "InsetString"], 
         TagBox[InsetBox["", 632],
          "InsetString"], 
         TagBox[InsetBox["", 633],
          "InsetString"], 
         TagBox[InsetBox["", 634],
          "InsetString"], 
         TagBox[InsetBox["", 635],
          "InsetString"], 
         TagBox[InsetBox["", 636],
          "InsetString"], 
         TagBox[InsetBox["", 637],
          "InsetString"], 
         TagBox[InsetBox["", 638],
          "InsetString"], 
         TagBox[InsetBox["", 639],
          "InsetString"], 
         TagBox[InsetBox["", 640],
          "InsetString"], 
         TagBox[InsetBox["", 641],
          "InsetString"], 
         TagBox[InsetBox["", 642],
          "InsetString"], 
         TagBox[InsetBox["", 643],
          "InsetString"], 
         TagBox[InsetBox["", 644],
          "InsetString"], 
         TagBox[InsetBox["", 645],
          "InsetString"], 
         TagBox[InsetBox["", 646],
          "InsetString"], 
         TagBox[InsetBox["", 647],
          "InsetString"], 
         TagBox[InsetBox["", 648],
          "InsetString"], 
         TagBox[InsetBox["", 649],
          "InsetString"], 
         TagBox[InsetBox["", 650],
          "InsetString"], 
         TagBox[InsetBox["", 651],
          "InsetString"], 
         TagBox[InsetBox["", 652],
          "InsetString"], 
         TagBox[InsetBox["", 653],
          "InsetString"], 
         TagBox[InsetBox["", 654],
          "InsetString"], 
         TagBox[InsetBox["", 655],
          "InsetString"], 
         TagBox[InsetBox["", 656],
          "InsetString"], 
         TagBox[InsetBox["", 657],
          "InsetString"], 
         TagBox[InsetBox["", 658],
          "InsetString"], 
         TagBox[InsetBox["", 659],
          "InsetString"], 
         TagBox[InsetBox["", 660],
          "InsetString"], 
         TagBox[InsetBox["", 661],
          "InsetString"], 
         TagBox[InsetBox["", 662],
          "InsetString"], 
         TagBox[InsetBox["", 663],
          "InsetString"], 
         TagBox[InsetBox["", 664],
          "InsetString"], 
         TagBox[InsetBox["", 665],
          "InsetString"], 
         TagBox[InsetBox["", 666],
          "InsetString"], 
         TagBox[InsetBox["", 667],
          "InsetString"], 
         TagBox[InsetBox["", 668],
          "InsetString"], 
         TagBox[InsetBox["", 669],
          "InsetString"], 
         TagBox[InsetBox["", 670],
          "InsetString"], 
         TagBox[InsetBox["", 671],
          "InsetString"], 
         TagBox[InsetBox["", 672],
          "InsetString"], 
         TagBox[InsetBox["", 673],
          "InsetString"], 
         TagBox[InsetBox["", 674],
          "InsetString"], 
         TagBox[InsetBox["", 675],
          "InsetString"], 
         TagBox[InsetBox["", 676],
          "InsetString"], 
         TagBox[InsetBox["", 677],
          "InsetString"], 
         TagBox[InsetBox["", 678],
          "InsetString"], 
         TagBox[InsetBox["", 679],
          "InsetString"], 
         TagBox[InsetBox["", 680],
          "InsetString"], 
         TagBox[InsetBox["", 681],
          "InsetString"], 
         TagBox[InsetBox["", 682],
          "InsetString"], 
         TagBox[InsetBox["", 683],
          "InsetString"], 
         TagBox[InsetBox["", 684],
          "InsetString"], 
         TagBox[InsetBox["", 685],
          "InsetString"], 
         TagBox[InsetBox["", 686],
          "InsetString"], 
         TagBox[InsetBox["", 687],
          "InsetString"], 
         TagBox[InsetBox["", 688],
          "InsetString"], 
         TagBox[InsetBox["", 689],
          "InsetString"], 
         TagBox[InsetBox["", 690],
          "InsetString"], 
         TagBox[InsetBox["", 691],
          "InsetString"], 
         TagBox[InsetBox["", 692],
          "InsetString"], 
         TagBox[InsetBox["", 693],
          "InsetString"], 
         TagBox[InsetBox["", 694],
          "InsetString"], 
         TagBox[InsetBox["", 695],
          "InsetString"], 
         TagBox[InsetBox["", 696],
          "InsetString"], 
         TagBox[InsetBox["", 697],
          "InsetString"], 
         TagBox[InsetBox["", 698],
          "InsetString"], 
         TagBox[InsetBox["", 699],
          "InsetString"], 
         TagBox[InsetBox["", 700],
          "InsetString"], 
         TagBox[InsetBox["", 701],
          "InsetString"], 
         TagBox[InsetBox["", 702],
          "InsetString"], 
         TagBox[InsetBox["", 703],
          "InsetString"], 
         TagBox[InsetBox["", 704],
          "InsetString"], 
         TagBox[InsetBox["", 705],
          "InsetString"], 
         TagBox[InsetBox["", 706],
          "InsetString"], 
         TagBox[InsetBox["", 707],
          "InsetString"], 
         TagBox[InsetBox["", 708],
          "InsetString"], 
         TagBox[InsetBox["", 709],
          "InsetString"], 
         TagBox[InsetBox["", 710],
          "InsetString"], 
         TagBox[InsetBox["", 711],
          "InsetString"], 
         TagBox[InsetBox["", 712],
          "InsetString"], 
         TagBox[InsetBox["", 713],
          "InsetString"], 
         TagBox[InsetBox["", 714],
          "InsetString"], 
         TagBox[InsetBox["", 715],
          "InsetString"], 
         TagBox[InsetBox["", 716],
          "InsetString"], 
         TagBox[InsetBox["", 717],
          "InsetString"], 
         TagBox[InsetBox["", 718],
          "InsetString"], 
         TagBox[InsetBox["", 719],
          "InsetString"], 
         TagBox[InsetBox["", 720],
          "InsetString"], 
         TagBox[InsetBox["", 721],
          "InsetString"], 
         TagBox[InsetBox["", 722],
          "InsetString"], 
         TagBox[InsetBox["", 723],
          "InsetString"], 
         TagBox[InsetBox["", 724],
          "InsetString"], 
         TagBox[InsetBox["", 725],
          "InsetString"], 
         TagBox[InsetBox["", 726],
          "InsetString"], 
         TagBox[InsetBox["", 727],
          "InsetString"], 
         TagBox[InsetBox["", 728],
          "InsetString"], 
         TagBox[InsetBox["", 729],
          "InsetString"], 
         TagBox[InsetBox["", 730],
          "InsetString"], 
         TagBox[InsetBox["", 731],
          "InsetString"], 
         TagBox[InsetBox["", 732],
          "InsetString"], 
         TagBox[InsetBox["", 733],
          "InsetString"], 
         TagBox[InsetBox["", 734],
          "InsetString"], 
         TagBox[InsetBox["", 735],
          "InsetString"], 
         TagBox[InsetBox["", 736],
          "InsetString"], 
         TagBox[InsetBox["", 737],
          "InsetString"], 
         TagBox[InsetBox["", 738],
          "InsetString"], 
         TagBox[InsetBox["", 739],
          "InsetString"], 
         TagBox[InsetBox["", 740],
          "InsetString"], 
         TagBox[InsetBox["", 741],
          "InsetString"], 
         TagBox[InsetBox["", 742],
          "InsetString"], 
         TagBox[InsetBox["", 743],
          "InsetString"], 
         TagBox[InsetBox["", 744],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.435*^9, 0.2},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-7\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3431808000, 
          GrayLevel[0.8]}, {3439756800, 
          GrayLevel[0.8]}, {3447532800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3463344000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
          GrayLevel[0.8]}, {3.4397568*^9, 
          GrayLevel[0.8]}, {3.4475328*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.463344*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{3.462724030375904*^9, 3.462728688129739*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Options", "Section",
 CellChangeTimes->{{3.462718971679179*^9, 3.462718972852662*^9}}],

Cell[CellGroupData[{

Cell["PriceStyling", "Subsection",
 CellChangeTimes->{{3.462719105161021*^9, 3.462719108687684*^9}}],

Cell[TextData[{
 "Styling options found in ",
 StyleBox["DateListPlot", "MB"],
 " can be added to the ",
 StyleBox["PricePlot", "MB"],
 " embedded within the ",
 StyleBox["StockPlot", "MB"],
 " using ",
 StyleBox["PriceStyling", "MB"],
 "."
}], "Text",
 CellChangeTimes->{{3.462719342524539*^9, 3.462719385092956*^9}, {
  3.4627195951242657`*^9, 3.462719600442484*^9}, {3.462789118342565*^9, 
  3.4627891198266563`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{"goog1", ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Black"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PriceStyling", "\[Rule]", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Filling", "\[Rule]", "0"}], ",", 
      RowBox[{"FillingStyle", "\[Rule]", 
       RowBox[{"Lighter", "[", 
        RowBox[{
         RowBox[{"ColorData", "[", 
          RowBox[{"35", ",", "6"}], "]"}], ",", "0.7"}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Thick", ",", 
         RowBox[{"ColorData", "[", 
          RowBox[{"35", ",", "6"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
      "}"}]}]}], "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.462719261248073*^9, 3.462719332975121*^9}, {
  3.46271939453642*^9, 3.462719429403324*^9}, {3.4627287300093927`*^9, 
  3.46272873877744*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxVlweMVFUUhp8gCog4yiKKwA4aIqDAUFQ0IrO0BYlxaFKNE0BAQBjQFV3a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      "], {{{}, 
        {RGBColor[0.7929411764705883, 0.8411764705882352, 0.8035294117647058],
          EdgeForm[None], 
         GraphicsGroupBox[
          PolygonBox[{{1, 249, 250, 248, 247, 246, 245, 244, 243, 242, 241, 
           240, 239, 238, 237, 236, 235, 234, 233, 232, 231, 230, 229, 228, 
           227, 226, 225, 224, 223, 222, 221, 220, 219, 218, 217, 216, 215, 
           214, 213, 212, 211, 210, 209, 208, 207, 206, 205, 204, 203, 202, 
           201, 200, 199, 198, 197, 196, 195, 194, 193, 192, 191, 190, 189, 
           188, 187, 186, 185, 184, 183, 182, 181, 180, 179, 178, 177, 176, 
           175, 174, 173, 172, 171, 170, 169, 168, 167, 166, 165, 164, 163, 
           162, 161, 160, 159, 158, 157, 156, 155, 154, 153, 152, 151, 150, 
           149, 148, 147, 146, 145, 144, 143, 142, 141, 140, 139, 138, 137, 
           136, 135, 134, 133, 132, 131, 130, 129, 128, 127, 126, 125, 124, 
           123, 122, 121, 120, 119, 118, 117, 116, 115, 114, 113, 112, 111, 
           110, 109, 108, 107, 106, 105, 104, 103, 102, 101, 100, 99, 98, 97, 
           96, 95, 94, 93, 92, 91, 90, 89, 88, 87, 86, 85, 84, 83, 82, 81, 80,
            79, 78, 77, 76, 75, 74, 73, 72, 71, 70, 69, 68, 67, 66, 65, 64, 
           63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52, 51, 50, 49, 48, 47,
            46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 
           30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14,
            13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2}}]]}, {}, {}}, {{}, {}, 
        {RGBColor[
         0.30980392156862746`, 0.47058823529411764`, 0.34509803921568627`], 
         Thickness[Large], 
         LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
          18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 
          35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 
          69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 
          86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 
          102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 
          115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 
          128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 
          141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 
          154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 
          167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 
          180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 
          193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 
          206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 
          219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 
          232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 
          245, 246, 247, 248}]}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.435*^9, 300.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3431808000, 
          GrayLevel[0.8]}, {3439756800, 
          GrayLevel[0.8]}, {3447532800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3463344000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
          GrayLevel[0.8]}, {3.4397568*^9, 
          GrayLevel[0.8]}, {3.4475328*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.463344*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {257.44, 499.06}},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 5}},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmflfjekbx8/QMGSmyIRshywhlG00Y+kpW2Emsg8zaRgSOvaxTQfVWFKH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      
      "], {{{}, {}, {}, {}, {}, 
        {RGBColor[0.75, 0., 0.], Thickness[Large], LineBox[{372, 1}], 
         LineBox[{373, 2}], LineBox[{374, 3}], LineBox[{375, 4}], 
         LineBox[{376, 5}], LineBox[{377, 6}], LineBox[{378, 7}], 
         LineBox[{379, 8}], LineBox[{380, 9}], LineBox[{381, 10}], 
         LineBox[{382, 11}], LineBox[{383, 12}], LineBox[{384, 13}], 
         LineBox[{385, 14}], LineBox[{386, 15}], LineBox[{387, 16}], 
         LineBox[{388, 17}], LineBox[{389, 18}], LineBox[{390, 19}], 
         LineBox[{391, 20}], LineBox[{392, 21}], LineBox[{393, 22}], 
         LineBox[{394, 23}], LineBox[{395, 24}], LineBox[{396, 25}], 
         LineBox[{397, 26}], LineBox[{398, 27}], LineBox[{399, 28}], 
         LineBox[{400, 29}], LineBox[{401, 30}], LineBox[{402, 31}], 
         LineBox[{403, 32}], LineBox[{404, 33}], LineBox[{405, 34}], 
         LineBox[{406, 35}], LineBox[{407, 36}], LineBox[{408, 37}], 
         LineBox[{409, 38}], LineBox[{410, 39}], LineBox[{411, 40}], 
         LineBox[{412, 41}], LineBox[{413, 42}], LineBox[{414, 43}], 
         LineBox[{415, 44}], LineBox[{416, 45}], LineBox[{417, 46}], 
         LineBox[{418, 47}], LineBox[{419, 48}], LineBox[{420, 49}], 
         LineBox[{421, 50}], LineBox[{422, 51}], LineBox[{423, 52}], 
         LineBox[{424, 53}], LineBox[{425, 54}], LineBox[{426, 55}], 
         LineBox[{427, 56}], LineBox[{428, 57}], LineBox[{429, 58}], 
         LineBox[{430, 59}], LineBox[{431, 60}], LineBox[{432, 61}], 
         LineBox[{433, 62}], LineBox[{434, 63}], LineBox[{435, 64}], 
         LineBox[{436, 65}], LineBox[{437, 66}], LineBox[{438, 67}], 
         LineBox[{439, 68}], LineBox[{440, 69}], LineBox[{441, 70}], 
         LineBox[{442, 71}], LineBox[{443, 72}], LineBox[{444, 73}], 
         LineBox[{445, 74}], LineBox[{446, 75}], LineBox[{447, 76}], 
         LineBox[{448, 77}], LineBox[{449, 78}], LineBox[{450, 79}], 
         LineBox[{451, 80}], LineBox[{452, 81}], LineBox[{453, 82}], 
         LineBox[{454, 83}], LineBox[{455, 84}], LineBox[{456, 85}], 
         LineBox[{457, 86}], LineBox[{458, 87}], LineBox[{459, 88}], 
         LineBox[{460, 89}], LineBox[{461, 90}], LineBox[{462, 91}], 
         LineBox[{463, 92}], LineBox[{464, 93}], LineBox[{465, 94}], 
         LineBox[{466, 95}], LineBox[{467, 96}], LineBox[{468, 97}], 
         LineBox[{469, 98}], LineBox[{470, 99}], LineBox[{471, 100}], 
         LineBox[{472, 101}], LineBox[{473, 102}], LineBox[{474, 103}], 
         LineBox[{475, 104}], LineBox[{476, 105}], LineBox[{477, 106}], 
         LineBox[{478, 107}], LineBox[{479, 108}], LineBox[{480, 109}], 
         LineBox[{481, 110}], LineBox[{482, 111}], LineBox[{483, 112}], 
         LineBox[{484, 113}], LineBox[{485, 114}], LineBox[{486, 115}], 
         LineBox[{487, 116}], LineBox[{488, 117}], LineBox[{489, 118}], 
         LineBox[{490, 119}], LineBox[{491, 120}], LineBox[{492, 121}], 
         LineBox[{493, 122}], LineBox[{494, 123}], LineBox[{495, 124}], 
         LineBox[{496, 125}]}, {}, 
        {RGBColor[0, 0.5019607843137255, 0], Thickness[Large], 
         LineBox[{249, 126}], LineBox[{250, 127}], LineBox[{251, 128}], 
         LineBox[{252, 129}], LineBox[{253, 130}], LineBox[{254, 131}], 
         LineBox[{255, 132}], LineBox[{256, 133}], LineBox[{257, 134}], 
         LineBox[{258, 135}], LineBox[{259, 136}], LineBox[{260, 137}], 
         LineBox[{261, 138}], LineBox[{262, 139}], LineBox[{263, 140}], 
         LineBox[{264, 141}], LineBox[{265, 142}], LineBox[{266, 143}], 
         LineBox[{267, 144}], LineBox[{268, 145}], LineBox[{269, 146}], 
         LineBox[{270, 147}], LineBox[{271, 148}], LineBox[{272, 149}], 
         LineBox[{273, 150}], LineBox[{274, 151}], LineBox[{275, 152}], 
         LineBox[{276, 153}], LineBox[{277, 154}], LineBox[{278, 155}], 
         LineBox[{279, 156}], LineBox[{280, 157}], LineBox[{281, 158}], 
         LineBox[{282, 159}], LineBox[{283, 160}], LineBox[{284, 161}], 
         LineBox[{285, 162}], LineBox[{286, 163}], LineBox[{287, 164}], 
         LineBox[{288, 165}], LineBox[{289, 166}], LineBox[{290, 167}], 
         LineBox[{291, 168}], LineBox[{292, 169}], LineBox[{293, 170}], 
         LineBox[{294, 171}], LineBox[{295, 172}], LineBox[{296, 173}], 
         LineBox[{297, 174}], LineBox[{298, 175}], LineBox[{299, 176}], 
         LineBox[{300, 177}], LineBox[{301, 178}], LineBox[{302, 179}], 
         LineBox[{303, 180}], LineBox[{304, 181}], LineBox[{305, 182}], 
         LineBox[{306, 183}], LineBox[{307, 184}], LineBox[{308, 185}], 
         LineBox[{309, 186}], LineBox[{310, 187}], LineBox[{311, 188}], 
         LineBox[{312, 189}], LineBox[{313, 190}], LineBox[{314, 191}], 
         LineBox[{315, 192}], LineBox[{316, 193}], LineBox[{317, 194}], 
         LineBox[{318, 195}], LineBox[{319, 196}], LineBox[{320, 197}], 
         LineBox[{321, 198}], LineBox[{322, 199}], LineBox[{323, 200}], 
         LineBox[{324, 201}], LineBox[{325, 202}], LineBox[{326, 203}], 
         LineBox[{327, 204}], LineBox[{328, 205}], LineBox[{329, 206}], 
         LineBox[{330, 207}], LineBox[{331, 208}], LineBox[{332, 209}], 
         LineBox[{333, 210}], LineBox[{334, 211}], LineBox[{335, 212}], 
         LineBox[{336, 213}], LineBox[{337, 214}], LineBox[{338, 215}], 
         LineBox[{339, 216}], LineBox[{340, 217}], LineBox[{341, 218}], 
         LineBox[{342, 219}], LineBox[{343, 220}], LineBox[{344, 221}], 
         LineBox[{345, 222}], LineBox[{346, 223}], LineBox[{347, 224}], 
         LineBox[{348, 225}], LineBox[{349, 226}], LineBox[{350, 227}], 
         LineBox[{351, 228}], LineBox[{352, 229}], LineBox[{353, 230}], 
         LineBox[{354, 231}], LineBox[{355, 232}], LineBox[{356, 233}], 
         LineBox[{357, 234}], LineBox[{358, 235}], LineBox[{359, 236}], 
         LineBox[{360, 237}], LineBox[{361, 238}], LineBox[{362, 239}], 
         LineBox[{363, 240}], LineBox[{364, 241}], LineBox[{365, 242}], 
         LineBox[{366, 243}], LineBox[{367, 244}], LineBox[{368, 245}], 
         LineBox[{369, 246}], LineBox[{370, 247}], LineBox[{371, 248}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"], 
         TagBox[InsetBox["", 601],
          "InsetString"], 
         TagBox[InsetBox["", 602],
          "InsetString"], 
         TagBox[InsetBox["", 603],
          "InsetString"], 
         TagBox[InsetBox["", 604],
          "InsetString"], 
         TagBox[InsetBox["", 605],
          "InsetString"], 
         TagBox[InsetBox["", 606],
          "InsetString"], 
         TagBox[InsetBox["", 607],
          "InsetString"], 
         TagBox[InsetBox["", 608],
          "InsetString"], 
         TagBox[InsetBox["", 609],
          "InsetString"], 
         TagBox[InsetBox["", 610],
          "InsetString"], 
         TagBox[InsetBox["", 611],
          "InsetString"], 
         TagBox[InsetBox["", 612],
          "InsetString"], 
         TagBox[InsetBox["", 613],
          "InsetString"], 
         TagBox[InsetBox["", 614],
          "InsetString"], 
         TagBox[InsetBox["", 615],
          "InsetString"], 
         TagBox[InsetBox["", 616],
          "InsetString"], 
         TagBox[InsetBox["", 617],
          "InsetString"], 
         TagBox[InsetBox["", 618],
          "InsetString"], 
         TagBox[InsetBox["", 619],
          "InsetString"], 
         TagBox[InsetBox["", 620],
          "InsetString"], 
         TagBox[InsetBox["", 621],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 622],
          "InsetString"], 
         TagBox[InsetBox["", 623],
          "InsetString"], 
         TagBox[InsetBox["", 624],
          "InsetString"], 
         TagBox[InsetBox["", 625],
          "InsetString"], 
         TagBox[InsetBox["", 626],
          "InsetString"], 
         TagBox[InsetBox["", 627],
          "InsetString"], 
         TagBox[InsetBox["", 628],
          "InsetString"], 
         TagBox[InsetBox["", 629],
          "InsetString"], 
         TagBox[InsetBox["", 630],
          "InsetString"], 
         TagBox[InsetBox["", 631],
          "InsetString"], 
         TagBox[InsetBox["", 632],
          "InsetString"], 
         TagBox[InsetBox["", 633],
          "InsetString"], 
         TagBox[InsetBox["", 634],
          "InsetString"], 
         TagBox[InsetBox["", 635],
          "InsetString"], 
         TagBox[InsetBox["", 636],
          "InsetString"], 
         TagBox[InsetBox["", 637],
          "InsetString"], 
         TagBox[InsetBox["", 638],
          "InsetString"], 
         TagBox[InsetBox["", 639],
          "InsetString"], 
         TagBox[InsetBox["", 640],
          "InsetString"], 
         TagBox[InsetBox["", 641],
          "InsetString"], 
         TagBox[InsetBox["", 642],
          "InsetString"], 
         TagBox[InsetBox["", 643],
          "InsetString"], 
         TagBox[InsetBox["", 644],
          "InsetString"], 
         TagBox[InsetBox["", 645],
          "InsetString"], 
         TagBox[InsetBox["", 646],
          "InsetString"], 
         TagBox[InsetBox["", 647],
          "InsetString"], 
         TagBox[InsetBox["", 648],
          "InsetString"], 
         TagBox[InsetBox["", 649],
          "InsetString"], 
         TagBox[InsetBox["", 650],
          "InsetString"], 
         TagBox[InsetBox["", 651],
          "InsetString"], 
         TagBox[InsetBox["", 652],
          "InsetString"], 
         TagBox[InsetBox["", 653],
          "InsetString"], 
         TagBox[InsetBox["", 654],
          "InsetString"], 
         TagBox[InsetBox["", 655],
          "InsetString"], 
         TagBox[InsetBox["", 656],
          "InsetString"], 
         TagBox[InsetBox["", 657],
          "InsetString"], 
         TagBox[InsetBox["", 658],
          "InsetString"], 
         TagBox[InsetBox["", 659],
          "InsetString"], 
         TagBox[InsetBox["", 660],
          "InsetString"], 
         TagBox[InsetBox["", 661],
          "InsetString"], 
         TagBox[InsetBox["", 662],
          "InsetString"], 
         TagBox[InsetBox["", 663],
          "InsetString"], 
         TagBox[InsetBox["", 664],
          "InsetString"], 
         TagBox[InsetBox["", 665],
          "InsetString"], 
         TagBox[InsetBox["", 666],
          "InsetString"], 
         TagBox[InsetBox["", 667],
          "InsetString"], 
         TagBox[InsetBox["", 668],
          "InsetString"], 
         TagBox[InsetBox["", 669],
          "InsetString"], 
         TagBox[InsetBox["", 670],
          "InsetString"], 
         TagBox[InsetBox["", 671],
          "InsetString"], 
         TagBox[InsetBox["", 672],
          "InsetString"], 
         TagBox[InsetBox["", 673],
          "InsetString"], 
         TagBox[InsetBox["", 674],
          "InsetString"], 
         TagBox[InsetBox["", 675],
          "InsetString"], 
         TagBox[InsetBox["", 676],
          "InsetString"], 
         TagBox[InsetBox["", 677],
          "InsetString"], 
         TagBox[InsetBox["", 678],
          "InsetString"], 
         TagBox[InsetBox["", 679],
          "InsetString"], 
         TagBox[InsetBox["", 680],
          "InsetString"], 
         TagBox[InsetBox["", 681],
          "InsetString"], 
         TagBox[InsetBox["", 682],
          "InsetString"], 
         TagBox[InsetBox["", 683],
          "InsetString"], 
         TagBox[InsetBox["", 684],
          "InsetString"], 
         TagBox[InsetBox["", 685],
          "InsetString"], 
         TagBox[InsetBox["", 686],
          "InsetString"], 
         TagBox[InsetBox["", 687],
          "InsetString"], 
         TagBox[InsetBox["", 688],
          "InsetString"], 
         TagBox[InsetBox["", 689],
          "InsetString"], 
         TagBox[InsetBox["", 690],
          "InsetString"], 
         TagBox[InsetBox["", 691],
          "InsetString"], 
         TagBox[InsetBox["", 692],
          "InsetString"], 
         TagBox[InsetBox["", 693],
          "InsetString"], 
         TagBox[InsetBox["", 694],
          "InsetString"], 
         TagBox[InsetBox["", 695],
          "InsetString"], 
         TagBox[InsetBox["", 696],
          "InsetString"], 
         TagBox[InsetBox["", 697],
          "InsetString"], 
         TagBox[InsetBox["", 698],
          "InsetString"], 
         TagBox[InsetBox["", 699],
          "InsetString"], 
         TagBox[InsetBox["", 700],
          "InsetString"], 
         TagBox[InsetBox["", 701],
          "InsetString"], 
         TagBox[InsetBox["", 702],
          "InsetString"], 
         TagBox[InsetBox["", 703],
          "InsetString"], 
         TagBox[InsetBox["", 704],
          "InsetString"], 
         TagBox[InsetBox["", 705],
          "InsetString"], 
         TagBox[InsetBox["", 706],
          "InsetString"], 
         TagBox[InsetBox["", 707],
          "InsetString"], 
         TagBox[InsetBox["", 708],
          "InsetString"], 
         TagBox[InsetBox["", 709],
          "InsetString"], 
         TagBox[InsetBox["", 710],
          "InsetString"], 
         TagBox[InsetBox["", 711],
          "InsetString"], 
         TagBox[InsetBox["", 712],
          "InsetString"], 
         TagBox[InsetBox["", 713],
          "InsetString"], 
         TagBox[InsetBox["", 714],
          "InsetString"], 
         TagBox[InsetBox["", 715],
          "InsetString"], 
         TagBox[InsetBox["", 716],
          "InsetString"], 
         TagBox[InsetBox["", 717],
          "InsetString"], 
         TagBox[InsetBox["", 718],
          "InsetString"], 
         TagBox[InsetBox["", 719],
          "InsetString"], 
         TagBox[InsetBox["", 720],
          "InsetString"], 
         TagBox[InsetBox["", 721],
          "InsetString"], 
         TagBox[InsetBox["", 722],
          "InsetString"], 
         TagBox[InsetBox["", 723],
          "InsetString"], 
         TagBox[InsetBox["", 724],
          "InsetString"], 
         TagBox[InsetBox["", 725],
          "InsetString"], 
         TagBox[InsetBox["", 726],
          "InsetString"], 
         TagBox[InsetBox["", 727],
          "InsetString"], 
         TagBox[InsetBox["", 728],
          "InsetString"], 
         TagBox[InsetBox["", 729],
          "InsetString"], 
         TagBox[InsetBox["", 730],
          "InsetString"], 
         TagBox[InsetBox["", 731],
          "InsetString"], 
         TagBox[InsetBox["", 732],
          "InsetString"], 
         TagBox[InsetBox["", 733],
          "InsetString"], 
         TagBox[InsetBox["", 734],
          "InsetString"], 
         TagBox[InsetBox["", 735],
          "InsetString"], 
         TagBox[InsetBox["", 736],
          "InsetString"], 
         TagBox[InsetBox["", 737],
          "InsetString"], 
         TagBox[InsetBox["", 738],
          "InsetString"], 
         TagBox[InsetBox["", 739],
          "InsetString"], 
         TagBox[InsetBox["", 740],
          "InsetString"], 
         TagBox[InsetBox["", 741],
          "InsetString"], 
         TagBox[InsetBox["", 742],
          "InsetString"], 
         TagBox[InsetBox["", 743],
          "InsetString"], 
         TagBox[InsetBox["", 744],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.435*^9, 0.2},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-7\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"Oct\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3431808000, 3.431808*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3463344000, 3.463344*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3434486400, 3.4344864*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3437078400, 3.4370784*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3431808000, 
          GrayLevel[0.8]}, {3439756800, 
          GrayLevel[0.8]}, {3447532800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3463344000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.431808*^9, 
          GrayLevel[0.8]}, {3.4397568*^9, 
          GrayLevel[0.8]}, {3.4475328*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.463344*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{{3.462719256474231*^9, 3.462719303405216*^9}, {
  3.462719396224568*^9, 3.4627194210927687`*^9}, {3.462728722741156*^9, 
  3.4627287394585648`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["VolumeStyling", "Subsection",
 CellChangeTimes->{{3.462719113120367*^9, 3.462719115407721*^9}}],

Cell[TextData[{
 "Styling options found in ",
 StyleBox["DateListPlot", "MB"],
 " can be added to the ",
 StyleBox["VolumePlot", "MB"],
 " embedded within the ",
 StyleBox["StockPlot", "MB"],
 " using ",
 StyleBox["VolumeStyling", "MB"],
 "."
}], "Text",
 CellChangeTimes->{{3.462719342524539*^9, 3.462719385092956*^9}, {
  3.4627195951242657`*^9, 3.462719600442484*^9}, {3.4627244269037113`*^9, 
  3.462724434181631*^9}, {3.46278913600303*^9, 3.462789137914892*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "200"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Blue"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PriceStyling", "\[Rule]", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Filling", "\[Rule]", "0"}], ",", 
      RowBox[{"FillingStyle", "\[Rule]", 
       RowBox[{"Lighter", "[", 
        RowBox[{
         RowBox[{"ColorData", "[", 
          RowBox[{"35", ",", "6"}], "]"}], ",", "0.6"}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Thick", ",", 
         RowBox[{"ColorData", "[", 
          RowBox[{"35", ",", "6"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
      "}"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"VolumeStyling", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"FillingStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Darker", "[", 
          RowBox[{
           RowBox[{"ColorData", "[", 
            RowBox[{"35", ",", "6"}], "]"}], ",", "0.3"}], "]"}], ",", 
         "Thick"}], "}"}], "}"}]}], "}"}]}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{{3.46272438909079*^9, 3.462724399080591*^9}, {
  3.462728756402582*^9, 3.462728804124654*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxFlwlsVFUUhp/gxiKpUlwA6WBiLCgwAkY0ojMgW4jJUMCWgnGCiFjQDmhl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      "], {{{}, 
        {RGBColor[0.7239215686274509, 0.788235294117647, 0.7380392156862745], 
         EdgeForm[None], 
         GraphicsGroupBox[
          PolygonBox[{{1, 201, 202, 200, 199, 198, 197, 196, 195, 194, 193, 
           192, 191, 190, 189, 188, 187, 186, 185, 184, 183, 182, 181, 180, 
           179, 178, 177, 176, 175, 174, 173, 172, 171, 170, 169, 168, 167, 
           166, 165, 164, 163, 162, 161, 160, 159, 158, 157, 156, 155, 154, 
           153, 152, 151, 150, 149, 148, 147, 146, 145, 144, 143, 142, 141, 
           140, 139, 138, 137, 136, 135, 134, 133, 132, 131, 130, 129, 128, 
           127, 126, 125, 124, 123, 122, 121, 120, 119, 118, 117, 116, 115, 
           114, 113, 112, 111, 110, 109, 108, 107, 106, 105, 104, 103, 102, 
           101, 100, 99, 98, 97, 96, 95, 94, 93, 92, 91, 90, 89, 88, 87, 86, 
           85, 84, 83, 82, 81, 80, 79, 78, 77, 76, 75, 74, 73, 72, 71, 70, 69,
            68, 67, 66, 65, 64, 63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 
           52, 51, 50, 49, 48, 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36,
            35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 
           19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 
           2}}]]}, {}, {}}, {{}, {}, 
        {RGBColor[
         0.30980392156862746`, 0.47058823529411764`, 0.34509803921568627`], 
         Thickness[Large], 
         LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
          18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 
          35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 
          69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 
          86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 
          102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 
          115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 
          128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 
          141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 
          154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 
          167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 
          180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 
          193, 194, 195, 196, 197, 198, 199, 200}]}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.44*^9, 300.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[0, 0, 1], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"Mar\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3439756800, 
          GrayLevel[0.8]}, {3444854400, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4397568*^9, 
          GrayLevel[0.8]}, {3.4448544*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {282.75, 499.06}},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 5}},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmftbTWkbx/fQMGQUkeOwHWbUKBIZZkKrnMoph5SGsZ2GnMp5wkxRuhya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      
      "], {{{}, {}, {}, {}, {}, 
        {RGBColor[
         0.2168627450980392, 0.32941176470588235`, 0.2415686274509804], 
         Thickness[Large], LineBox[{306, 1}], LineBox[{307, 2}], 
         LineBox[{308, 3}], LineBox[{309, 4}], LineBox[{310, 5}], 
         LineBox[{311, 6}], LineBox[{312, 7}], LineBox[{313, 8}], 
         LineBox[{314, 9}], LineBox[{315, 10}], LineBox[{316, 11}], 
         LineBox[{317, 12}], LineBox[{318, 13}], LineBox[{319, 14}], 
         LineBox[{320, 15}], LineBox[{321, 16}], LineBox[{322, 17}], 
         LineBox[{323, 18}], LineBox[{324, 19}], LineBox[{325, 20}], 
         LineBox[{326, 21}], LineBox[{327, 22}], LineBox[{328, 23}], 
         LineBox[{329, 24}], LineBox[{330, 25}], LineBox[{331, 26}], 
         LineBox[{332, 27}], LineBox[{333, 28}], LineBox[{334, 29}], 
         LineBox[{335, 30}], LineBox[{336, 31}], LineBox[{337, 32}], 
         LineBox[{338, 33}], LineBox[{339, 34}], LineBox[{340, 35}], 
         LineBox[{341, 36}], LineBox[{342, 37}], LineBox[{343, 38}], 
         LineBox[{344, 39}], LineBox[{345, 40}], LineBox[{346, 41}], 
         LineBox[{347, 42}], LineBox[{348, 43}], LineBox[{349, 44}], 
         LineBox[{350, 45}], LineBox[{351, 46}], LineBox[{352, 47}], 
         LineBox[{353, 48}], LineBox[{354, 49}], LineBox[{355, 50}], 
         LineBox[{356, 51}], LineBox[{357, 52}], LineBox[{358, 53}], 
         LineBox[{359, 54}], LineBox[{360, 55}], LineBox[{361, 56}], 
         LineBox[{362, 57}], LineBox[{363, 58}], LineBox[{364, 59}], 
         LineBox[{365, 60}], LineBox[{366, 61}], LineBox[{367, 62}], 
         LineBox[{368, 63}], LineBox[{369, 64}], LineBox[{370, 65}], 
         LineBox[{371, 66}], LineBox[{372, 67}], LineBox[{373, 68}], 
         LineBox[{374, 69}], LineBox[{375, 70}], LineBox[{376, 71}], 
         LineBox[{377, 72}], LineBox[{378, 73}], LineBox[{379, 74}], 
         LineBox[{380, 75}], LineBox[{381, 76}], LineBox[{382, 77}], 
         LineBox[{383, 78}], LineBox[{384, 79}], LineBox[{385, 80}], 
         LineBox[{386, 81}], LineBox[{387, 82}], LineBox[{388, 83}], 
         LineBox[{389, 84}], LineBox[{390, 85}], LineBox[{391, 86}], 
         LineBox[{392, 87}], LineBox[{393, 88}], LineBox[{394, 89}], 
         LineBox[{395, 90}], LineBox[{396, 91}], LineBox[{397, 92}], 
         LineBox[{398, 93}], LineBox[{399, 94}], LineBox[{400, 95}]}, {}, 
        {RGBColor[
         0.2168627450980392, 0.32941176470588235`, 0.2415686274509804], 
         Thickness[Large], LineBox[{201, 96}], LineBox[{202, 97}], 
         LineBox[{203, 98}], LineBox[{204, 99}], LineBox[{205, 100}], 
         LineBox[{206, 101}], LineBox[{207, 102}], LineBox[{208, 103}], 
         LineBox[{209, 104}], LineBox[{210, 105}], LineBox[{211, 106}], 
         LineBox[{212, 107}], LineBox[{213, 108}], LineBox[{214, 109}], 
         LineBox[{215, 110}], LineBox[{216, 111}], LineBox[{217, 112}], 
         LineBox[{218, 113}], LineBox[{219, 114}], LineBox[{220, 115}], 
         LineBox[{221, 116}], LineBox[{222, 117}], LineBox[{223, 118}], 
         LineBox[{224, 119}], LineBox[{225, 120}], LineBox[{226, 121}], 
         LineBox[{227, 122}], LineBox[{228, 123}], LineBox[{229, 124}], 
         LineBox[{230, 125}], LineBox[{231, 126}], LineBox[{232, 127}], 
         LineBox[{233, 128}], LineBox[{234, 129}], LineBox[{235, 130}], 
         LineBox[{236, 131}], LineBox[{237, 132}], LineBox[{238, 133}], 
         LineBox[{239, 134}], LineBox[{240, 135}], LineBox[{241, 136}], 
         LineBox[{242, 137}], LineBox[{243, 138}], LineBox[{244, 139}], 
         LineBox[{245, 140}], LineBox[{246, 141}], LineBox[{247, 142}], 
         LineBox[{248, 143}], LineBox[{249, 144}], LineBox[{250, 145}], 
         LineBox[{251, 146}], LineBox[{252, 147}], LineBox[{253, 148}], 
         LineBox[{254, 149}], LineBox[{255, 150}], LineBox[{256, 151}], 
         LineBox[{257, 152}], LineBox[{258, 153}], LineBox[{259, 154}], 
         LineBox[{260, 155}], LineBox[{261, 156}], LineBox[{262, 157}], 
         LineBox[{263, 158}], LineBox[{264, 159}], LineBox[{265, 160}], 
         LineBox[{266, 161}], LineBox[{267, 162}], LineBox[{268, 163}], 
         LineBox[{269, 164}], LineBox[{270, 165}], LineBox[{271, 166}], 
         LineBox[{272, 167}], LineBox[{273, 168}], LineBox[{274, 169}], 
         LineBox[{275, 170}], LineBox[{276, 171}], LineBox[{277, 172}], 
         LineBox[{278, 173}], LineBox[{279, 174}], LineBox[{280, 175}], 
         LineBox[{281, 176}], LineBox[{282, 177}], LineBox[{283, 178}], 
         LineBox[{284, 179}], LineBox[{285, 180}], LineBox[{286, 181}], 
         LineBox[{287, 182}], LineBox[{288, 183}], LineBox[{289, 184}], 
         LineBox[{290, 185}], LineBox[{291, 186}], LineBox[{292, 187}], 
         LineBox[{293, 188}], LineBox[{294, 189}], LineBox[{295, 190}], 
         LineBox[{296, 191}], LineBox[{297, 192}], LineBox[{298, 193}], 
         LineBox[{299, 194}], LineBox[{300, 195}], LineBox[{301, 196}], 
         LineBox[{302, 197}], LineBox[{303, 198}], LineBox[{304, 199}], 
         LineBox[{305, 200}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 401],
          "InsetString"], 
         TagBox[InsetBox["", 402],
          "InsetString"], 
         TagBox[InsetBox["", 403],
          "InsetString"], 
         TagBox[InsetBox["", 404],
          "InsetString"], 
         TagBox[InsetBox["", 405],
          "InsetString"], 
         TagBox[InsetBox["", 406],
          "InsetString"], 
         TagBox[InsetBox["", 407],
          "InsetString"], 
         TagBox[InsetBox["", 408],
          "InsetString"], 
         TagBox[InsetBox["", 409],
          "InsetString"], 
         TagBox[InsetBox["", 410],
          "InsetString"], 
         TagBox[InsetBox["", 411],
          "InsetString"], 
         TagBox[InsetBox["", 412],
          "InsetString"], 
         TagBox[InsetBox["", 413],
          "InsetString"], 
         TagBox[InsetBox["", 414],
          "InsetString"], 
         TagBox[InsetBox["", 415],
          "InsetString"], 
         TagBox[InsetBox["", 416],
          "InsetString"], 
         TagBox[InsetBox["", 417],
          "InsetString"], 
         TagBox[InsetBox["", 418],
          "InsetString"], 
         TagBox[InsetBox["", 419],
          "InsetString"], 
         TagBox[InsetBox["", 420],
          "InsetString"], 
         TagBox[InsetBox["", 421],
          "InsetString"], 
         TagBox[InsetBox["", 422],
          "InsetString"], 
         TagBox[InsetBox["", 423],
          "InsetString"], 
         TagBox[InsetBox["", 424],
          "InsetString"], 
         TagBox[InsetBox["", 425],
          "InsetString"], 
         TagBox[InsetBox["", 426],
          "InsetString"], 
         TagBox[InsetBox["", 427],
          "InsetString"], 
         TagBox[InsetBox["", 428],
          "InsetString"], 
         TagBox[InsetBox["", 429],
          "InsetString"], 
         TagBox[InsetBox["", 430],
          "InsetString"], 
         TagBox[InsetBox["", 431],
          "InsetString"], 
         TagBox[InsetBox["", 432],
          "InsetString"], 
         TagBox[InsetBox["", 433],
          "InsetString"], 
         TagBox[InsetBox["", 434],
          "InsetString"], 
         TagBox[InsetBox["", 435],
          "InsetString"], 
         TagBox[InsetBox["", 436],
          "InsetString"], 
         TagBox[InsetBox["", 437],
          "InsetString"], 
         TagBox[InsetBox["", 438],
          "InsetString"], 
         TagBox[InsetBox["", 439],
          "InsetString"], 
         TagBox[InsetBox["", 440],
          "InsetString"], 
         TagBox[InsetBox["", 441],
          "InsetString"], 
         TagBox[InsetBox["", 442],
          "InsetString"], 
         TagBox[InsetBox["", 443],
          "InsetString"], 
         TagBox[InsetBox["", 444],
          "InsetString"], 
         TagBox[InsetBox["", 445],
          "InsetString"], 
         TagBox[InsetBox["", 446],
          "InsetString"], 
         TagBox[InsetBox["", 447],
          "InsetString"], 
         TagBox[InsetBox["", 448],
          "InsetString"], 
         TagBox[InsetBox["", 449],
          "InsetString"], 
         TagBox[InsetBox["", 450],
          "InsetString"], 
         TagBox[InsetBox["", 451],
          "InsetString"], 
         TagBox[InsetBox["", 452],
          "InsetString"], 
         TagBox[InsetBox["", 453],
          "InsetString"], 
         TagBox[InsetBox["", 454],
          "InsetString"], 
         TagBox[InsetBox["", 455],
          "InsetString"], 
         TagBox[InsetBox["", 456],
          "InsetString"], 
         TagBox[InsetBox["", 457],
          "InsetString"], 
         TagBox[InsetBox["", 458],
          "InsetString"], 
         TagBox[InsetBox["", 459],
          "InsetString"], 
         TagBox[InsetBox["", 460],
          "InsetString"], 
         TagBox[InsetBox["", 461],
          "InsetString"], 
         TagBox[InsetBox["", 462],
          "InsetString"], 
         TagBox[InsetBox["", 463],
          "InsetString"], 
         TagBox[InsetBox["", 464],
          "InsetString"], 
         TagBox[InsetBox["", 465],
          "InsetString"], 
         TagBox[InsetBox["", 466],
          "InsetString"], 
         TagBox[InsetBox["", 467],
          "InsetString"], 
         TagBox[InsetBox["", 468],
          "InsetString"], 
         TagBox[InsetBox["", 469],
          "InsetString"], 
         TagBox[InsetBox["", 470],
          "InsetString"], 
         TagBox[InsetBox["", 471],
          "InsetString"], 
         TagBox[InsetBox["", 472],
          "InsetString"], 
         TagBox[InsetBox["", 473],
          "InsetString"], 
         TagBox[InsetBox["", 474],
          "InsetString"], 
         TagBox[InsetBox["", 475],
          "InsetString"], 
         TagBox[InsetBox["", 476],
          "InsetString"], 
         TagBox[InsetBox["", 477],
          "InsetString"], 
         TagBox[InsetBox["", 478],
          "InsetString"], 
         TagBox[InsetBox["", 479],
          "InsetString"], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 481],
          "InsetString"], 
         TagBox[InsetBox["", 482],
          "InsetString"], 
         TagBox[InsetBox["", 483],
          "InsetString"], 
         TagBox[InsetBox["", 484],
          "InsetString"], 
         TagBox[InsetBox["", 485],
          "InsetString"], 
         TagBox[InsetBox["", 486],
          "InsetString"], 
         TagBox[InsetBox["", 487],
          "InsetString"], 
         TagBox[InsetBox["", 488],
          "InsetString"], 
         TagBox[InsetBox["", 489],
          "InsetString"], 
         TagBox[InsetBox["", 490],
          "InsetString"], 
         TagBox[InsetBox["", 491],
          "InsetString"], 
         TagBox[InsetBox["", 492],
          "InsetString"], 
         TagBox[InsetBox["", 493],
          "InsetString"], 
         TagBox[InsetBox["", 494],
          "InsetString"], 
         TagBox[InsetBox["", 495],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 496],
          "InsetString"], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.44*^9, 0.2},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-7\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"Jan\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"Mar\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}}, {{
          NCache[3439756800, 3.4397568*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3444854400, 3.4448544*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3442435200, 3.4424352*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm]}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm]}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3439756800, 
          GrayLevel[0.8]}, {3444854400, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4397568*^9, 
          GrayLevel[0.8]}, {3.4448544*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{
  3.462724380775853*^9, {3.462728765921575*^9, 3.462728804606176*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PlotType", "Subsection",
 CellChangeTimes->{{3.4627189942296667`*^9, 3.4627189957099648`*^9}}],

Cell[TextData[{
 "The default ",
 StyleBox["PlotType", "MB"],
 " for ",
 StyleBox["StockPlot", "MB"],
 " is ",
 StyleBox["PricePlot", "MB"],
 ". Other options are ",
 StyleBox["OHLCPlot", "MB"],
 " and ",
 StyleBox["CandlestickPlot", "MB"],
 "."
}], "Text",
 CellChangeTimes->{{3.462724465495496*^9, 3.46272453994378*^9}, {
  3.462728823045862*^9, 3.462728828183817*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "100"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Orange"}], "}"}], ",", 
   RowBox[{"PlotType", "\[Rule]", "\"\<OHLC\>\""}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.462724522107327*^9, {3.462724568189015*^9, 3.4627245694897137`*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmX1s1dUZx69icLwYmwBzYRuUuUmFoZ2wsBeZ94a5juG0wlwqdFJFEQmx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      
      "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1], LineBox[{201, 401}], 
         LineBox[{202, 402}], LineBox[{203, 403}], LineBox[{204, 404}], 
         LineBox[{205, 405}], LineBox[{206, 406}], LineBox[{207, 407}], 
         LineBox[{208, 408}], LineBox[{209, 409}], LineBox[{210, 410}], 
         LineBox[{211, 411}], LineBox[{212, 412}], LineBox[{213, 413}], 
         LineBox[{214, 414}], LineBox[{215, 415}], LineBox[{216, 416}], 
         LineBox[{217, 417}], LineBox[{218, 418}], LineBox[{219, 419}], 
         LineBox[{220, 420}], LineBox[{221, 421}], LineBox[{222, 422}], 
         LineBox[{223, 423}], LineBox[{224, 424}], LineBox[{225, 425}], 
         LineBox[{226, 426}], LineBox[{227, 427}], LineBox[{228, 428}], 
         LineBox[{229, 429}], LineBox[{230, 430}], LineBox[{231, 431}], 
         LineBox[{232, 432}], LineBox[{233, 433}], LineBox[{234, 434}], 
         LineBox[{235, 435}], LineBox[{236, 436}], LineBox[{237, 437}], 
         LineBox[{238, 438}], LineBox[{239, 439}], LineBox[{240, 440}], 
         LineBox[{241, 441}], LineBox[{242, 442}], LineBox[{243, 443}], 
         LineBox[{244, 444}], LineBox[{245, 445}], LineBox[{246, 446}], 
         LineBox[{247, 447}], LineBox[{248, 448}], LineBox[{249, 449}], 
         LineBox[{250, 450}], LineBox[{251, 451}], LineBox[{252, 452}], 
         LineBox[{253, 453}], LineBox[{254, 454}], LineBox[{255, 455}], 
         LineBox[{256, 456}], LineBox[{257, 457}], LineBox[{258, 458}], 
         LineBox[{259, 459}], LineBox[{260, 460}], LineBox[{261, 461}], 
         LineBox[{262, 462}], LineBox[{263, 463}], LineBox[{264, 464}], 
         LineBox[{265, 465}], LineBox[{266, 466}], LineBox[{267, 467}], 
         LineBox[{268, 468}], LineBox[{269, 469}], LineBox[{270, 470}], 
         LineBox[{271, 471}], LineBox[{272, 472}], LineBox[{273, 473}], 
         LineBox[{274, 474}], LineBox[{275, 475}], LineBox[{276, 476}], 
         LineBox[{277, 477}], LineBox[{278, 478}], LineBox[{279, 479}], 
         LineBox[{280, 480}], LineBox[{281, 481}], LineBox[{282, 482}], 
         LineBox[{283, 483}], LineBox[{284, 484}], LineBox[{285, 485}], 
         LineBox[{286, 486}], LineBox[{287, 487}], LineBox[{288, 488}], 
         LineBox[{289, 489}], LineBox[{290, 490}], LineBox[{291, 491}], 
         LineBox[{292, 492}], LineBox[{293, 493}], LineBox[{294, 494}], 
         LineBox[{295, 495}], LineBox[{296, 496}], LineBox[{297, 497}], 
         LineBox[{298, 498}], LineBox[{299, 499}], LineBox[{300, 500}]}}, {
        {GrayLevel[0], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 501], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 502], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 503], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 504], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 505], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 506], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 507], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 508], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 509], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 510], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 511], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 512], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 513], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 514], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 515], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 516], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 517], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 518], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 519], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 520], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 521], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 522], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 523], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 524], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 525], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 526], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 527], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 528], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 529], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 530], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 531], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 532], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 533], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 534], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 535], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 536], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 537], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 538], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 539], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 540], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 541], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 542], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 543], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 544], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 545], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 546], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 547], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 548], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 549], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 550], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 551], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 552], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 553], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 554], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 555], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 556], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 557], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 558], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 559], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 560], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 561], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 562], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 563], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 564], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 565], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 566], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 567], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 568], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 569], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 570], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 571], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 572], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 573], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 574], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 575], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 576], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 577], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 578], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 579], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 580], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 581], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 582], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 583], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 584], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 585], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 586], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 587], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 588], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 589], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 590], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 591], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 592], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 593], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 594], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 595], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 596], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 597], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 598], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 599], InsetBox[
          StyleBox["\<\"\[SkeletonIndicator] \"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 600]}, 
        {GrayLevel[0], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 601], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 602], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 603], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 604], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 605], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 606], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 607], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 608], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 609], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 610], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 611], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 612], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 613], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 614], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 615], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 616], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 617], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 618], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 619], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 620], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 621], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 622], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 623], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 624], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 625], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 626], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 627], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 628], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 629], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 630], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 631], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 632], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 633], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 634], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 635], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 636], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 637], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 638], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 639], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 640], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 641], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 642], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 643], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 644], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 645], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 646], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 647], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 648], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 649], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 650], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 651], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 652], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 653], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 654], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 655], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 656], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 657], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 658], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 659], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 660], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 661], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 662], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 663], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 664], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 665], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 666], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 667], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 668], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 669], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 670], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 671], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 672], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 673], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 674], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 675], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 676], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 677], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 678], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 679], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 680], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 681], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 682], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 683], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 684], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 685], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 686], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 687], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 688], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 689], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 690], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 691], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 692], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 693], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 694], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 695], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 696], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 697], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 698], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 699], InsetBox[
          StyleBox["\<\" \[SkeletonIndicator]\"\>",
           StripOnInput->False,
           FrontFaceColor->GrayLevel[0],
           BackFaceColor->GrayLevel[0],
           GraphicsColor->GrayLevel[0],
           FontColor->GrayLevel[0]], 700]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 701],
          "InsetString"], 
         TagBox[InsetBox["", 702],
          "InsetString"], 
         TagBox[InsetBox["", 703],
          "InsetString"], 
         TagBox[InsetBox["", 704],
          "InsetString"], 
         TagBox[InsetBox["", 705],
          "InsetString"], 
         TagBox[InsetBox["", 706],
          "InsetString"], 
         TagBox[InsetBox["", 707],
          "InsetString"], 
         TagBox[InsetBox["", 708],
          "InsetString"], 
         TagBox[InsetBox["", 709],
          "InsetString"], 
         TagBox[InsetBox["", 710],
          "InsetString"], 
         TagBox[InsetBox["", 711],
          "InsetString"], 
         TagBox[InsetBox["", 712],
          "InsetString"], 
         TagBox[InsetBox["", 713],
          "InsetString"], 
         TagBox[InsetBox["", 714],
          "InsetString"], 
         TagBox[InsetBox["", 715],
          "InsetString"], 
         TagBox[InsetBox["", 716],
          "InsetString"], 
         TagBox[InsetBox["", 717],
          "InsetString"], 
         TagBox[InsetBox["", 718],
          "InsetString"], 
         TagBox[InsetBox["", 719],
          "InsetString"], 
         TagBox[InsetBox["", 720],
          "InsetString"], 
         TagBox[InsetBox["", 721],
          "InsetString"], 
         TagBox[InsetBox["", 722],
          "InsetString"], 
         TagBox[InsetBox["", 723],
          "InsetString"], 
         TagBox[InsetBox["", 724],
          "InsetString"], 
         TagBox[InsetBox["", 725],
          "InsetString"], 
         TagBox[InsetBox["", 726],
          "InsetString"], 
         TagBox[InsetBox["", 727],
          "InsetString"], 
         TagBox[InsetBox["", 728],
          "InsetString"], 
         TagBox[InsetBox["", 729],
          "InsetString"], 
         TagBox[InsetBox["", 730],
          "InsetString"], 
         TagBox[InsetBox["", 731],
          "InsetString"], 
         TagBox[InsetBox["", 732],
          "InsetString"], 
         TagBox[InsetBox["", 733],
          "InsetString"], 
         TagBox[InsetBox["", 734],
          "InsetString"], 
         TagBox[InsetBox["", 735],
          "InsetString"], 
         TagBox[InsetBox["", 736],
          "InsetString"], 
         TagBox[InsetBox["", 737],
          "InsetString"], 
         TagBox[InsetBox["", 738],
          "InsetString"], 
         TagBox[InsetBox["", 739],
          "InsetString"], 
         TagBox[InsetBox["", 740],
          "InsetString"], 
         TagBox[InsetBox["", 741],
          "InsetString"], 
         TagBox[InsetBox["", 742],
          "InsetString"], 
         TagBox[InsetBox["", 743],
          "InsetString"], 
         TagBox[InsetBox["", 744],
          "InsetString"], 
         TagBox[InsetBox["", 745],
          "InsetString"], 
         TagBox[InsetBox["", 746],
          "InsetString"], 
         TagBox[InsetBox["", 747],
          "InsetString"], 
         TagBox[InsetBox["", 748],
          "InsetString"], 
         TagBox[InsetBox["", 749],
          "InsetString"], 
         TagBox[InsetBox["", 750],
          "InsetString"], 
         TagBox[InsetBox["", 751],
          "InsetString"], 
         TagBox[InsetBox["", 752],
          "InsetString"], 
         TagBox[InsetBox["", 753],
          "InsetString"], 
         TagBox[InsetBox["", 754],
          "InsetString"], 
         TagBox[InsetBox["", 755],
          "InsetString"], 
         TagBox[InsetBox["", 756],
          "InsetString"], 
         TagBox[InsetBox["", 757],
          "InsetString"], 
         TagBox[InsetBox["", 758],
          "InsetString"], 
         TagBox[InsetBox["", 759],
          "InsetString"], 
         TagBox[InsetBox["", 760],
          "InsetString"], 
         TagBox[InsetBox["", 761],
          "InsetString"], 
         TagBox[InsetBox["", 762],
          "InsetString"], 
         TagBox[InsetBox["", 763],
          "InsetString"], 
         TagBox[InsetBox["", 764],
          "InsetString"], 
         TagBox[InsetBox["", 765],
          "InsetString"], 
         TagBox[InsetBox["", 766],
          "InsetString"], 
         TagBox[InsetBox["", 767],
          "InsetString"], 
         TagBox[InsetBox["", 768],
          "InsetString"], 
         TagBox[InsetBox["", 769],
          "InsetString"], 
         TagBox[InsetBox["", 770],
          "InsetString"], 
         TagBox[InsetBox["", 771],
          "InsetString"], 
         TagBox[InsetBox["", 772],
          "InsetString"], 
         TagBox[InsetBox["", 773],
          "InsetString"], 
         TagBox[InsetBox["", 774],
          "InsetString"], 
         TagBox[InsetBox["", 775],
          "InsetString"], 
         TagBox[InsetBox["", 776],
          "InsetString"], 
         TagBox[InsetBox["", 777],
          "InsetString"], 
         TagBox[InsetBox["", 778],
          "InsetString"], 
         TagBox[InsetBox["", 779],
          "InsetString"], 
         TagBox[InsetBox["", 780],
          "InsetString"], 
         TagBox[InsetBox["", 781],
          "InsetString"], 
         TagBox[InsetBox["", 782],
          "InsetString"], 
         TagBox[InsetBox["", 783],
          "InsetString"], 
         TagBox[InsetBox["", 784],
          "InsetString"], 
         TagBox[InsetBox["", 785],
          "InsetString"], 
         TagBox[InsetBox["", 786],
          "InsetString"], 
         TagBox[InsetBox["", 787],
          "InsetString"], 
         TagBox[InsetBox["", 788],
          "InsetString"], 
         TagBox[InsetBox["", 789],
          "InsetString"], 
         TagBox[InsetBox["", 790],
          "InsetString"], 
         TagBox[InsetBox["", 791],
          "InsetString"], 
         TagBox[InsetBox["", 792],
          "InsetString"], 
         TagBox[InsetBox["", 793],
          "InsetString"], 
         TagBox[InsetBox["", 794],
          "InsetString"], 
         TagBox[InsetBox["", 795],
          "InsetString"], 
         TagBox[InsetBox["", 796],
          "InsetString"], 
         TagBox[InsetBox["", 797],
          "InsetString"], 
         TagBox[InsetBox["", 798],
          "InsetString"], 
         TagBox[InsetBox["", 799],
          "InsetString"], 
         TagBox[InsetBox["", 800],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 801],
          "InsetString"], 
         TagBox[InsetBox["", 802],
          "InsetString"], 
         TagBox[InsetBox["", 803],
          "InsetString"], 
         TagBox[InsetBox["", 804],
          "InsetString"], 
         TagBox[InsetBox["", 805],
          "InsetString"], 
         TagBox[InsetBox["", 806],
          "InsetString"], 
         TagBox[InsetBox["", 807],
          "InsetString"], 
         TagBox[InsetBox["", 808],
          "InsetString"], 
         TagBox[InsetBox["", 809],
          "InsetString"], 
         TagBox[InsetBox["", 810],
          "InsetString"], 
         TagBox[InsetBox["", 811],
          "InsetString"], 
         TagBox[InsetBox["", 812],
          "InsetString"], 
         TagBox[InsetBox["", 813],
          "InsetString"], 
         TagBox[InsetBox["", 814],
          "InsetString"], 
         TagBox[InsetBox["", 815],
          "InsetString"], 
         TagBox[InsetBox["", 816],
          "InsetString"], 
         TagBox[InsetBox["", 817],
          "InsetString"], 
         TagBox[InsetBox["", 818],
          "InsetString"], 
         TagBox[InsetBox["", 819],
          "InsetString"], 
         TagBox[InsetBox["", 820],
          "InsetString"], 
         TagBox[InsetBox["", 821],
          "InsetString"], 
         TagBox[InsetBox["", 822],
          "InsetString"], 
         TagBox[InsetBox["", 823],
          "InsetString"], 
         TagBox[InsetBox["", 824],
          "InsetString"], 
         TagBox[InsetBox["", 825],
          "InsetString"], 
         TagBox[InsetBox["", 826],
          "InsetString"], 
         TagBox[InsetBox["", 827],
          "InsetString"], 
         TagBox[InsetBox["", 828],
          "InsetString"], 
         TagBox[InsetBox["", 829],
          "InsetString"], 
         TagBox[InsetBox["", 830],
          "InsetString"], 
         TagBox[InsetBox["", 831],
          "InsetString"], 
         TagBox[InsetBox["", 832],
          "InsetString"], 
         TagBox[InsetBox["", 833],
          "InsetString"], 
         TagBox[InsetBox["", 834],
          "InsetString"], 
         TagBox[InsetBox["", 835],
          "InsetString"], 
         TagBox[InsetBox["", 836],
          "InsetString"], 
         TagBox[InsetBox["", 837],
          "InsetString"], 
         TagBox[InsetBox["", 838],
          "InsetString"], 
         TagBox[InsetBox["", 839],
          "InsetString"], 
         TagBox[InsetBox["", 840],
          "InsetString"], 
         TagBox[InsetBox["", 841],
          "InsetString"], 
         TagBox[InsetBox["", 842],
          "InsetString"], 
         TagBox[InsetBox["", 843],
          "InsetString"], 
         TagBox[InsetBox["", 844],
          "InsetString"], 
         TagBox[InsetBox["", 845],
          "InsetString"], 
         TagBox[InsetBox["", 846],
          "InsetString"], 
         TagBox[InsetBox["", 847],
          "InsetString"], 
         TagBox[InsetBox["", 848],
          "InsetString"], 
         TagBox[InsetBox["", 849],
          "InsetString"], 
         TagBox[InsetBox["", 850],
          "InsetString"], 
         TagBox[InsetBox["", 851],
          "InsetString"], 
         TagBox[InsetBox["", 852],
          "InsetString"], 
         TagBox[InsetBox["", 853],
          "InsetString"], 
         TagBox[InsetBox["", 854],
          "InsetString"], 
         TagBox[InsetBox["", 855],
          "InsetString"], 
         TagBox[InsetBox["", 856],
          "InsetString"], 
         TagBox[InsetBox["", 857],
          "InsetString"], 
         TagBox[InsetBox["", 858],
          "InsetString"], 
         TagBox[InsetBox["", 859],
          "InsetString"], 
         TagBox[InsetBox["", 860],
          "InsetString"], 
         TagBox[InsetBox["", 861],
          "InsetString"], 
         TagBox[InsetBox["", 862],
          "InsetString"], 
         TagBox[InsetBox["", 863],
          "InsetString"], 
         TagBox[InsetBox["", 864],
          "InsetString"], 
         TagBox[InsetBox["", 865],
          "InsetString"], 
         TagBox[InsetBox["", 866],
          "InsetString"], 
         TagBox[InsetBox["", 867],
          "InsetString"], 
         TagBox[InsetBox["", 868],
          "InsetString"], 
         TagBox[InsetBox["", 869],
          "InsetString"], 
         TagBox[InsetBox["", 870],
          "InsetString"], 
         TagBox[InsetBox["", 871],
          "InsetString"], 
         TagBox[InsetBox["", 872],
          "InsetString"], 
         TagBox[InsetBox["", 873],
          "InsetString"], 
         TagBox[InsetBox["", 874],
          "InsetString"], 
         TagBox[InsetBox["", 875],
          "InsetString"], 
         TagBox[InsetBox["", 876],
          "InsetString"], 
         TagBox[InsetBox["", 877],
          "InsetString"], 
         TagBox[InsetBox["", 878],
          "InsetString"], 
         TagBox[InsetBox["", 879],
          "InsetString"], 
         TagBox[InsetBox["", 880],
          "InsetString"], 
         TagBox[InsetBox["", 881],
          "InsetString"], 
         TagBox[InsetBox["", 882],
          "InsetString"], 
         TagBox[InsetBox["", 883],
          "InsetString"], 
         TagBox[InsetBox["", 884],
          "InsetString"], 
         TagBox[InsetBox["", 885],
          "InsetString"], 
         TagBox[InsetBox["", 886],
          "InsetString"], 
         TagBox[InsetBox["", 887],
          "InsetString"], 
         TagBox[InsetBox["", 888],
          "InsetString"], 
         TagBox[InsetBox["", 889],
          "InsetString"], 
         TagBox[InsetBox["", 890],
          "InsetString"], 
         TagBox[InsetBox["", 891],
          "InsetString"], 
         TagBox[InsetBox["", 892],
          "InsetString"], 
         TagBox[InsetBox["", 893],
          "InsetString"], 
         TagBox[InsetBox["", 894],
          "InsetString"], 
         TagBox[InsetBox["", 895],
          "InsetString"], 
         TagBox[InsetBox["", 896],
          "InsetString"], 
         TagBox[InsetBox["", 897],
          "InsetString"], 
         TagBox[InsetBox["", 898],
          "InsetString"], 
         TagBox[InsetBox["", 899],
          "InsetString"], 
         TagBox[InsetBox["", 900],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.45*^9, 380.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[1, 0.5, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {384.69, 501.98999999999995`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 5},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztl2tQVGUYxw+CGOjWei01cw1ryPWCt7zHHi9oNSpKOhtmoiiOSEFFXlFQ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      "], {{{}, {}, {}, {}, {}, 
        {RGBColor[0.75, 0., 0.], Thickness[Large], LineBox[{154, 1}], 
         LineBox[{155, 2}], LineBox[{156, 3}], LineBox[{157, 4}], 
         LineBox[{158, 5}], LineBox[{159, 6}], LineBox[{160, 7}], 
         LineBox[{161, 8}], LineBox[{162, 9}], LineBox[{163, 10}], 
         LineBox[{164, 11}], LineBox[{165, 12}], LineBox[{166, 13}], 
         LineBox[{167, 14}], LineBox[{168, 15}], LineBox[{169, 16}], 
         LineBox[{170, 17}], LineBox[{171, 18}], LineBox[{172, 19}], 
         LineBox[{173, 20}], LineBox[{174, 21}], LineBox[{175, 22}], 
         LineBox[{176, 23}], LineBox[{177, 24}], LineBox[{178, 25}], 
         LineBox[{179, 26}], LineBox[{180, 27}], LineBox[{181, 28}], 
         LineBox[{182, 29}], LineBox[{183, 30}], LineBox[{184, 31}], 
         LineBox[{185, 32}], LineBox[{186, 33}], LineBox[{187, 34}], 
         LineBox[{188, 35}], LineBox[{189, 36}], LineBox[{190, 37}], 
         LineBox[{191, 38}], LineBox[{192, 39}], LineBox[{193, 40}], 
         LineBox[{194, 41}], LineBox[{195, 42}], LineBox[{196, 43}], 
         LineBox[{197, 44}], LineBox[{198, 45}], LineBox[{199, 46}], 
         LineBox[{200, 47}]}, {}, 
        {RGBColor[0, 0.5019607843137255, 0], Thickness[Large], 
         LineBox[{101, 48}], LineBox[{102, 49}], LineBox[{103, 50}], 
         LineBox[{104, 51}], LineBox[{105, 52}], LineBox[{106, 53}], 
         LineBox[{107, 54}], LineBox[{108, 55}], LineBox[{109, 56}], 
         LineBox[{110, 57}], LineBox[{111, 58}], LineBox[{112, 59}], 
         LineBox[{113, 60}], LineBox[{114, 61}], LineBox[{115, 62}], 
         LineBox[{116, 63}], LineBox[{117, 64}], LineBox[{118, 65}], 
         LineBox[{119, 66}], LineBox[{120, 67}], LineBox[{121, 68}], 
         LineBox[{122, 69}], LineBox[{123, 70}], LineBox[{124, 71}], 
         LineBox[{125, 72}], LineBox[{126, 73}], LineBox[{127, 74}], 
         LineBox[{128, 75}], LineBox[{129, 76}], LineBox[{130, 77}], 
         LineBox[{131, 78}], LineBox[{132, 79}], LineBox[{133, 80}], 
         LineBox[{134, 81}], LineBox[{135, 82}], LineBox[{136, 83}], 
         LineBox[{137, 84}], LineBox[{138, 85}], LineBox[{139, 86}], 
         LineBox[{140, 87}], LineBox[{141, 88}], LineBox[{142, 89}], 
         LineBox[{143, 90}], LineBox[{144, 91}], LineBox[{145, 92}], 
         LineBox[{146, 93}], LineBox[{147, 94}], LineBox[{148, 95}], 
         LineBox[{149, 96}], LineBox[{150, 97}], LineBox[{151, 98}], 
         LineBox[{152, 99}], LineBox[{153, 100}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 202],
          "InsetString"], 
         TagBox[InsetBox["", 203],
          "InsetString"], 
         TagBox[InsetBox["", 204],
          "InsetString"], 
         TagBox[InsetBox["", 205],
          "InsetString"], 
         TagBox[InsetBox["", 206],
          "InsetString"], 
         TagBox[InsetBox["", 207],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 211],
          "InsetString"], 
         TagBox[InsetBox["", 212],
          "InsetString"], 
         TagBox[InsetBox["", 213],
          "InsetString"], 
         TagBox[InsetBox["", 214],
          "InsetString"], 
         TagBox[InsetBox["", 215],
          "InsetString"], 
         TagBox[InsetBox["", 216],
          "InsetString"], 
         TagBox[InsetBox["", 217],
          "InsetString"], 
         TagBox[InsetBox["", 218],
          "InsetString"], 
         TagBox[InsetBox["", 219],
          "InsetString"], 
         TagBox[InsetBox["", 220],
          "InsetString"], 
         TagBox[InsetBox["", 221],
          "InsetString"], 
         TagBox[InsetBox["", 222],
          "InsetString"], 
         TagBox[InsetBox["", 223],
          "InsetString"], 
         TagBox[InsetBox["", 224],
          "InsetString"], 
         TagBox[InsetBox["", 225],
          "InsetString"], 
         TagBox[InsetBox["", 226],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 229],
          "InsetString"], 
         TagBox[InsetBox["", 230],
          "InsetString"], 
         TagBox[InsetBox["", 231],
          "InsetString"], 
         TagBox[InsetBox["", 232],
          "InsetString"], 
         TagBox[InsetBox["", 233],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 235],
          "InsetString"], 
         TagBox[InsetBox["", 236],
          "InsetString"], 
         TagBox[InsetBox["", 237],
          "InsetString"], 
         TagBox[InsetBox["", 238],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 240],
          "InsetString"], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 242],
          "InsetString"], 
         TagBox[InsetBox["", 243],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 249],
          "InsetString"], 
         TagBox[InsetBox["", 250],
          "InsetString"], 
         TagBox[InsetBox["", 251],
          "InsetString"], 
         TagBox[InsetBox["", 252],
          "InsetString"], 
         TagBox[InsetBox["", 253],
          "InsetString"], 
         TagBox[InsetBox["", 254],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 258],
          "InsetString"], 
         TagBox[InsetBox["", 259],
          "InsetString"], 
         TagBox[InsetBox["", 260],
          "InsetString"], 
         TagBox[InsetBox["", 261],
          "InsetString"], 
         TagBox[InsetBox["", 262],
          "InsetString"], 
         TagBox[InsetBox["", 263],
          "InsetString"], 
         TagBox[InsetBox["", 264],
          "InsetString"], 
         TagBox[InsetBox["", 265],
          "InsetString"], 
         TagBox[InsetBox["", 266],
          "InsetString"], 
         TagBox[InsetBox["", 267],
          "InsetString"], 
         TagBox[InsetBox["", 268],
          "InsetString"], 
         TagBox[InsetBox["", 269],
          "InsetString"], 
         TagBox[InsetBox["", 270],
          "InsetString"], 
         TagBox[InsetBox["", 271],
          "InsetString"], 
         TagBox[InsetBox["", 272],
          "InsetString"], 
         TagBox[InsetBox["", 273],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 276],
          "InsetString"], 
         TagBox[InsetBox["", 277],
          "InsetString"], 
         TagBox[InsetBox["", 278],
          "InsetString"], 
         TagBox[InsetBox["", 279],
          "InsetString"], 
         TagBox[InsetBox["", 280],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 282],
          "InsetString"], 
         TagBox[InsetBox["", 283],
          "InsetString"], 
         TagBox[InsetBox["", 284],
          "InsetString"], 
         TagBox[InsetBox["", 285],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 287],
          "InsetString"], 
         TagBox[InsetBox["", 288],
          "InsetString"], 
         TagBox[InsetBox["", 289],
          "InsetString"], 
         TagBox[InsetBox["", 290],
          "InsetString"], 
         TagBox[InsetBox["", 291],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 296],
          "InsetString"], 
         TagBox[InsetBox["", 297],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.45*^9, 2.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-6\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{3.462718956778543*^9, 3.46272452604527*^9, 
  3.4627245710174713`*^9, 3.462728832999649*^9}]
}, Open  ]],

Cell[TextData[{
 "By default the ",
 StyleBox["VolumePlot", "MB"],
 " reverts to black for ",
 StyleBox["PlotType\[ThinSpace]\[Rule]\[ThinSpace]\"Candlestick\"", "MB"],
 " because the winning and losing days are color coded into the ",
 StyleBox["CandlestickPlot", "MB"],
 "."
}], "Text",
 CellChangeTimes->{{3.462789164435521*^9, 3.462789238355665*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "100"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Red"}], "}"}], ",", 
   RowBox[{"PlotType", "\[Rule]", "\"\<Candlestick\>\""}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4627245516209927`*^9, 3.462724575153892*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmn1sllcZxt+NhclX1gRwBhVaP0YHstWBwY/h+gYnQ6a24EwHOLqxdYyQ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      
      "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1], LineBox[{1, 482}], 
         LineBox[{2, 483}], LineBox[{3, 484}], LineBox[{4, 485}], 
         LineBox[{5, 486}], LineBox[{6, 487}], LineBox[{7, 488}], 
         LineBox[{8, 489}], LineBox[{9, 490}], LineBox[{10, 491}], 
         LineBox[{11, 492}], LineBox[{12, 493}], LineBox[{13, 494}], 
         LineBox[{14, 495}], LineBox[{15, 496}], LineBox[{16, 497}], 
         LineBox[{17, 498}], LineBox[{18, 499}], LineBox[{19, 500}], 
         LineBox[{20, 501}], LineBox[{21, 502}], LineBox[{22, 503}], 
         LineBox[{23, 504}], LineBox[{24, 505}], LineBox[{25, 506}], 
         LineBox[{26, 507}], LineBox[{27, 508}], LineBox[{28, 509}], 
         LineBox[{29, 510}], LineBox[{30, 511}], LineBox[{31, 512}], 
         LineBox[{32, 513}], LineBox[{33, 514}], LineBox[{34, 515}], 
         LineBox[{35, 516}], LineBox[{36, 517}], LineBox[{37, 518}], 
         LineBox[{38, 519}], LineBox[{39, 520}], LineBox[{40, 521}], 
         LineBox[{41, 522}], LineBox[{42, 523}], LineBox[{43, 524}], 
         LineBox[{44, 525}], LineBox[{45, 526}], LineBox[{46, 527}], 
         LineBox[{47, 528}], LineBox[{48, 529}], LineBox[{49, 530}], 
         LineBox[{50, 531}], LineBox[{51, 532}], LineBox[{52, 533}], 
         LineBox[{53, 534}], LineBox[{54, 535}], LineBox[{55, 536}], 
         LineBox[{56, 537}], LineBox[{57, 538}], LineBox[{58, 539}], 
         LineBox[{59, 540}], LineBox[{60, 541}], LineBox[{61, 542}], 
         LineBox[{62, 543}], LineBox[{63, 544}], LineBox[{64, 545}], 
         LineBox[{65, 546}], LineBox[{66, 547}], LineBox[{67, 548}], 
         LineBox[{68, 549}], LineBox[{69, 550}], LineBox[{70, 551}], 
         LineBox[{71, 552}], LineBox[{72, 553}], LineBox[{73, 554}], 
         LineBox[{74, 555}], LineBox[{75, 556}], LineBox[{76, 557}], 
         LineBox[{77, 558}], LineBox[{78, 559}], LineBox[{79, 560}], 
         LineBox[{80, 561}], LineBox[{81, 562}], LineBox[{82, 563}], 
         LineBox[{83, 564}], LineBox[{84, 565}], LineBox[{85, 566}], 
         LineBox[{86, 567}], LineBox[{87, 568}], LineBox[{88, 569}], 
         LineBox[{89, 570}], LineBox[{90, 571}], LineBox[{91, 572}], 
         LineBox[{92, 573}], LineBox[{93, 574}], LineBox[{94, 575}], 
         LineBox[{95, 576}], LineBox[{96, 577}], LineBox[{97, 578}], 
         LineBox[{98, 579}], LineBox[{99, 580}], 
         LineBox[{100, 
          581}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[2.5], LineBox[{202, 446}], 
         LineBox[{203, 447}], LineBox[{204, 448}], LineBox[{205, 449}], 
         LineBox[{206, 450}], LineBox[{207, 451}], LineBox[{211, 452}], 
         LineBox[{212, 453}], LineBox[{213, 454}], LineBox[{214, 455}], 
         LineBox[{215, 456}], LineBox[{216, 457}], LineBox[{217, 458}], 
         LineBox[{218, 459}], LineBox[{219, 460}], LineBox[{220, 461}], 
         LineBox[{221, 462}], LineBox[{222, 463}], LineBox[{223, 464}], 
         LineBox[{224, 465}], LineBox[{225, 466}], LineBox[{226, 467}], 
         LineBox[{229, 468}], LineBox[{230, 469}], LineBox[{231, 470}], 
         LineBox[{232, 471}], LineBox[{233, 472}], LineBox[{235, 473}], 
         LineBox[{236, 474}], LineBox[{237, 475}], LineBox[{238, 476}], 
         LineBox[{240, 477}], LineBox[{241, 478}], LineBox[{242, 479}], 
         LineBox[{243, 480}], 
         LineBox[{244, 481}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[0., 0.8, 0.], AbsoluteThickness[2.5], LineBox[{295, 401}], 
         LineBox[{349, 402}], LineBox[{297, 403}], LineBox[{351, 404}], 
         LineBox[{299, 405}], LineBox[{353, 406}], LineBox[{303, 407}], 
         LineBox[{357, 408}], LineBox[{358, 409}], LineBox[{359, 410}], 
         LineBox[{360, 411}], LineBox[{361, 412}], LineBox[{310, 413}], 
         LineBox[{364, 414}], LineBox[{365, 415}], LineBox[{314, 416}], 
         LineBox[{368, 417}], LineBox[{369, 418}], LineBox[{370, 419}], 
         LineBox[{371, 420}], LineBox[{372, 421}], LineBox[{373, 422}], 
         LineBox[{322, 423}], LineBox[{376, 424}], LineBox[{325, 425}], 
         LineBox[{379, 426}], LineBox[{327, 427}], LineBox[{381, 428}], 
         LineBox[{382, 429}], LineBox[{383, 430}], LineBox[{384, 431}], 
         LineBox[{332, 432}], LineBox[{386, 433}], LineBox[{387, 434}], 
         LineBox[{335, 435}], LineBox[{389, 436}], LineBox[{338, 437}], 
         LineBox[{392, 438}], LineBox[{340, 439}], LineBox[{394, 440}], 
         LineBox[{395, 441}], LineBox[{396, 442}], LineBox[{397, 443}], 
         LineBox[{398, 444}], LineBox[{399, 445}]}}, {
        {GrayLevel[0], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 301],
          "InsetString"], 
         TagBox[InsetBox["", 302],
          "InsetString"], 
         TagBox[InsetBox["", 309],
          "InsetString"], 
         TagBox[InsetBox["", 313],
          "InsetString"], 
         TagBox[InsetBox["", 321],
          "InsetString"], 
         TagBox[InsetBox["", 324],
          "InsetString"], 
         TagBox[InsetBox["", 337],
          "InsetString"], 
         TagBox[InsetBox["", 347],
          "InsetString"], 
         TagBox[InsetBox["", 354],
          "InsetString"], 
         TagBox[InsetBox["", 355],
          "InsetString"], 
         TagBox[InsetBox["", 362],
          "InsetString"], 
         TagBox[InsetBox["", 366],
          "InsetString"], 
         TagBox[InsetBox["", 374],
          "InsetString"], 
         TagBox[InsetBox["", 377],
          "InsetString"], 
         TagBox[InsetBox["", 390],
          "InsetString"], 
         TagBox[InsetBox["", 400],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"], 
         TagBox[InsetBox["", 601],
          "InsetString"], 
         TagBox[InsetBox["", 602],
          "InsetString"], 
         TagBox[InsetBox["", 603],
          "InsetString"], 
         TagBox[InsetBox["", 604],
          "InsetString"], 
         TagBox[InsetBox["", 605],
          "InsetString"], 
         TagBox[InsetBox["", 606],
          "InsetString"], 
         TagBox[InsetBox["", 607],
          "InsetString"], 
         TagBox[InsetBox["", 608],
          "InsetString"], 
         TagBox[InsetBox["", 609],
          "InsetString"], 
         TagBox[InsetBox["", 610],
          "InsetString"], 
         TagBox[InsetBox["", 611],
          "InsetString"], 
         TagBox[InsetBox["", 612],
          "InsetString"], 
         TagBox[InsetBox["", 613],
          "InsetString"], 
         TagBox[InsetBox["", 614],
          "InsetString"], 
         TagBox[InsetBox["", 615],
          "InsetString"], 
         TagBox[InsetBox["", 616],
          "InsetString"], 
         TagBox[InsetBox["", 617],
          "InsetString"], 
         TagBox[InsetBox["", 618],
          "InsetString"], 
         TagBox[InsetBox["", 619],
          "InsetString"], 
         TagBox[InsetBox["", 620],
          "InsetString"], 
         TagBox[InsetBox["", 621],
          "InsetString"], 
         TagBox[InsetBox["", 622],
          "InsetString"], 
         TagBox[InsetBox["", 623],
          "InsetString"], 
         TagBox[InsetBox["", 624],
          "InsetString"], 
         TagBox[InsetBox["", 625],
          "InsetString"], 
         TagBox[InsetBox["", 626],
          "InsetString"], 
         TagBox[InsetBox["", 627],
          "InsetString"], 
         TagBox[InsetBox["", 628],
          "InsetString"], 
         TagBox[InsetBox["", 629],
          "InsetString"], 
         TagBox[InsetBox["", 630],
          "InsetString"], 
         TagBox[InsetBox["", 631],
          "InsetString"], 
         TagBox[InsetBox["", 632],
          "InsetString"], 
         TagBox[InsetBox["", 633],
          "InsetString"], 
         TagBox[InsetBox["", 634],
          "InsetString"], 
         TagBox[InsetBox["", 635],
          "InsetString"], 
         TagBox[InsetBox["", 636],
          "InsetString"], 
         TagBox[InsetBox["", 637],
          "InsetString"], 
         TagBox[InsetBox["", 638],
          "InsetString"], 
         TagBox[InsetBox["", 639],
          "InsetString"], 
         TagBox[InsetBox["", 640],
          "InsetString"], 
         TagBox[InsetBox["", 641],
          "InsetString"], 
         TagBox[InsetBox["", 642],
          "InsetString"], 
         TagBox[InsetBox["", 643],
          "InsetString"], 
         TagBox[InsetBox["", 644],
          "InsetString"], 
         TagBox[InsetBox["", 645],
          "InsetString"], 
         TagBox[InsetBox["", 646],
          "InsetString"], 
         TagBox[InsetBox["", 647],
          "InsetString"], 
         TagBox[InsetBox["", 648],
          "InsetString"], 
         TagBox[InsetBox["", 649],
          "InsetString"], 
         TagBox[InsetBox["", 650],
          "InsetString"], 
         TagBox[InsetBox["", 651],
          "InsetString"], 
         TagBox[InsetBox["", 652],
          "InsetString"], 
         TagBox[InsetBox["", 653],
          "InsetString"], 
         TagBox[InsetBox["", 654],
          "InsetString"], 
         TagBox[InsetBox["", 655],
          "InsetString"], 
         TagBox[InsetBox["", 656],
          "InsetString"], 
         TagBox[InsetBox["", 657],
          "InsetString"], 
         TagBox[InsetBox["", 658],
          "InsetString"], 
         TagBox[InsetBox["", 659],
          "InsetString"], 
         TagBox[InsetBox["", 660],
          "InsetString"], 
         TagBox[InsetBox["", 661],
          "InsetString"], 
         TagBox[InsetBox["", 662],
          "InsetString"], 
         TagBox[InsetBox["", 663],
          "InsetString"], 
         TagBox[InsetBox["", 664],
          "InsetString"], 
         TagBox[InsetBox["", 665],
          "InsetString"], 
         TagBox[InsetBox["", 666],
          "InsetString"], 
         TagBox[InsetBox["", 667],
          "InsetString"], 
         TagBox[InsetBox["", 668],
          "InsetString"], 
         TagBox[InsetBox["", 669],
          "InsetString"], 
         TagBox[InsetBox["", 670],
          "InsetString"], 
         TagBox[InsetBox["", 671],
          "InsetString"], 
         TagBox[InsetBox["", 672],
          "InsetString"], 
         TagBox[InsetBox["", 673],
          "InsetString"], 
         TagBox[InsetBox["", 674],
          "InsetString"], 
         TagBox[InsetBox["", 675],
          "InsetString"], 
         TagBox[InsetBox["", 676],
          "InsetString"], 
         TagBox[InsetBox["", 677],
          "InsetString"], 
         TagBox[InsetBox["", 678],
          "InsetString"], 
         TagBox[InsetBox["", 679],
          "InsetString"], 
         TagBox[InsetBox["", 680],
          "InsetString"], 
         TagBox[InsetBox["", 681],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 682],
          "InsetString"], 
         TagBox[InsetBox["", 683],
          "InsetString"], 
         TagBox[InsetBox["", 684],
          "InsetString"], 
         TagBox[InsetBox["", 685],
          "InsetString"], 
         TagBox[InsetBox["", 686],
          "InsetString"], 
         TagBox[InsetBox["", 687],
          "InsetString"], 
         TagBox[InsetBox["", 688],
          "InsetString"], 
         TagBox[InsetBox["", 689],
          "InsetString"], 
         TagBox[InsetBox["", 690],
          "InsetString"], 
         TagBox[InsetBox["", 691],
          "InsetString"], 
         TagBox[InsetBox["", 692],
          "InsetString"], 
         TagBox[InsetBox["", 693],
          "InsetString"], 
         TagBox[InsetBox["", 694],
          "InsetString"], 
         TagBox[InsetBox["", 695],
          "InsetString"], 
         TagBox[InsetBox["", 696],
          "InsetString"], 
         TagBox[InsetBox["", 697],
          "InsetString"], 
         TagBox[InsetBox["", 698],
          "InsetString"], 
         TagBox[InsetBox["", 699],
          "InsetString"], 
         TagBox[InsetBox["", 700],
          "InsetString"], 
         TagBox[InsetBox["", 701],
          "InsetString"], 
         TagBox[InsetBox["", 702],
          "InsetString"], 
         TagBox[InsetBox["", 703],
          "InsetString"], 
         TagBox[InsetBox["", 704],
          "InsetString"], 
         TagBox[InsetBox["", 705],
          "InsetString"], 
         TagBox[InsetBox["", 706],
          "InsetString"], 
         TagBox[InsetBox["", 707],
          "InsetString"], 
         TagBox[InsetBox["", 708],
          "InsetString"], 
         TagBox[InsetBox["", 709],
          "InsetString"], 
         TagBox[InsetBox["", 710],
          "InsetString"], 
         TagBox[InsetBox["", 711],
          "InsetString"], 
         TagBox[InsetBox["", 712],
          "InsetString"], 
         TagBox[InsetBox["", 713],
          "InsetString"], 
         TagBox[InsetBox["", 714],
          "InsetString"], 
         TagBox[InsetBox["", 715],
          "InsetString"], 
         TagBox[InsetBox["", 716],
          "InsetString"], 
         TagBox[InsetBox["", 717],
          "InsetString"], 
         TagBox[InsetBox["", 718],
          "InsetString"], 
         TagBox[InsetBox["", 719],
          "InsetString"], 
         TagBox[InsetBox["", 720],
          "InsetString"], 
         TagBox[InsetBox["", 721],
          "InsetString"], 
         TagBox[InsetBox["", 722],
          "InsetString"], 
         TagBox[InsetBox["", 723],
          "InsetString"], 
         TagBox[InsetBox["", 724],
          "InsetString"], 
         TagBox[InsetBox["", 725],
          "InsetString"], 
         TagBox[InsetBox["", 726],
          "InsetString"], 
         TagBox[InsetBox["", 727],
          "InsetString"], 
         TagBox[InsetBox["", 728],
          "InsetString"], 
         TagBox[InsetBox["", 729],
          "InsetString"], 
         TagBox[InsetBox["", 730],
          "InsetString"], 
         TagBox[InsetBox["", 731],
          "InsetString"], 
         TagBox[InsetBox["", 732],
          "InsetString"], 
         TagBox[InsetBox["", 733],
          "InsetString"], 
         TagBox[InsetBox["", 734],
          "InsetString"], 
         TagBox[InsetBox["", 735],
          "InsetString"], 
         TagBox[InsetBox["", 736],
          "InsetString"], 
         TagBox[InsetBox["", 737],
          "InsetString"], 
         TagBox[InsetBox["", 738],
          "InsetString"], 
         TagBox[InsetBox["", 739],
          "InsetString"], 
         TagBox[InsetBox["", 740],
          "InsetString"], 
         TagBox[InsetBox["", 741],
          "InsetString"], 
         TagBox[InsetBox["", 742],
          "InsetString"], 
         TagBox[InsetBox["", 743],
          "InsetString"], 
         TagBox[InsetBox["", 744],
          "InsetString"], 
         TagBox[InsetBox["", 745],
          "InsetString"], 
         TagBox[InsetBox["", 746],
          "InsetString"], 
         TagBox[InsetBox["", 747],
          "InsetString"], 
         TagBox[InsetBox["", 748],
          "InsetString"], 
         TagBox[InsetBox["", 749],
          "InsetString"], 
         TagBox[InsetBox["", 750],
          "InsetString"], 
         TagBox[InsetBox["", 751],
          "InsetString"], 
         TagBox[InsetBox["", 752],
          "InsetString"], 
         TagBox[InsetBox["", 753],
          "InsetString"], 
         TagBox[InsetBox["", 754],
          "InsetString"], 
         TagBox[InsetBox["", 755],
          "InsetString"], 
         TagBox[InsetBox["", 756],
          "InsetString"], 
         TagBox[InsetBox["", 757],
          "InsetString"], 
         TagBox[InsetBox["", 758],
          "InsetString"], 
         TagBox[InsetBox["", 759],
          "InsetString"], 
         TagBox[InsetBox["", 760],
          "InsetString"], 
         TagBox[InsetBox["", 761],
          "InsetString"], 
         TagBox[InsetBox["", 762],
          "InsetString"], 
         TagBox[InsetBox["", 763],
          "InsetString"], 
         TagBox[InsetBox["", 764],
          "InsetString"], 
         TagBox[InsetBox["", 765],
          "InsetString"], 
         TagBox[InsetBox["", 766],
          "InsetString"], 
         TagBox[InsetBox["", 767],
          "InsetString"], 
         TagBox[InsetBox["", 768],
          "InsetString"], 
         TagBox[InsetBox["", 769],
          "InsetString"], 
         TagBox[InsetBox["", 770],
          "InsetString"], 
         TagBox[InsetBox["", 771],
          "InsetString"], 
         TagBox[InsetBox["", 772],
          "InsetString"], 
         TagBox[InsetBox["", 773],
          "InsetString"], 
         TagBox[InsetBox["", 774],
          "InsetString"], 
         TagBox[InsetBox["", 775],
          "InsetString"], 
         TagBox[InsetBox["", 776],
          "InsetString"], 
         TagBox[InsetBox["", 777],
          "InsetString"], 
         TagBox[InsetBox["", 778],
          "InsetString"], 
         TagBox[InsetBox["", 779],
          "InsetString"], 
         TagBox[InsetBox["", 780],
          "InsetString"], 
         TagBox[InsetBox["", 781],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 782],
          "InsetString"], 
         TagBox[InsetBox["", 783],
          "InsetString"], 
         TagBox[InsetBox["", 784],
          "InsetString"], 
         TagBox[InsetBox["", 785],
          "InsetString"], 
         TagBox[InsetBox["", 786],
          "InsetString"], 
         TagBox[InsetBox["", 787],
          "InsetString"], 
         TagBox[InsetBox["", 788],
          "InsetString"], 
         TagBox[InsetBox["", 789],
          "InsetString"], 
         TagBox[InsetBox["", 790],
          "InsetString"], 
         TagBox[InsetBox["", 791],
          "InsetString"], 
         TagBox[InsetBox["", 792],
          "InsetString"], 
         TagBox[InsetBox["", 793],
          "InsetString"], 
         TagBox[InsetBox["", 794],
          "InsetString"], 
         TagBox[InsetBox["", 795],
          "InsetString"], 
         TagBox[InsetBox["", 796],
          "InsetString"], 
         TagBox[InsetBox["", 797],
          "InsetString"], 
         TagBox[InsetBox["", 798],
          "InsetString"], 
         TagBox[InsetBox["", 799],
          "InsetString"], 
         TagBox[InsetBox["", 800],
          "InsetString"], 
         TagBox[InsetBox["", 801],
          "InsetString"], 
         TagBox[InsetBox["", 802],
          "InsetString"], 
         TagBox[InsetBox["", 803],
          "InsetString"], 
         TagBox[InsetBox["", 804],
          "InsetString"], 
         TagBox[InsetBox["", 805],
          "InsetString"], 
         TagBox[InsetBox["", 806],
          "InsetString"], 
         TagBox[InsetBox["", 807],
          "InsetString"], 
         TagBox[InsetBox["", 808],
          "InsetString"], 
         TagBox[InsetBox["", 809],
          "InsetString"], 
         TagBox[InsetBox["", 810],
          "InsetString"], 
         TagBox[InsetBox["", 811],
          "InsetString"], 
         TagBox[InsetBox["", 812],
          "InsetString"], 
         TagBox[InsetBox["", 813],
          "InsetString"], 
         TagBox[InsetBox["", 814],
          "InsetString"], 
         TagBox[InsetBox["", 815],
          "InsetString"], 
         TagBox[InsetBox["", 816],
          "InsetString"], 
         TagBox[InsetBox["", 817],
          "InsetString"], 
         TagBox[InsetBox["", 818],
          "InsetString"], 
         TagBox[InsetBox["", 819],
          "InsetString"], 
         TagBox[InsetBox["", 820],
          "InsetString"], 
         TagBox[InsetBox["", 821],
          "InsetString"], 
         TagBox[InsetBox["", 822],
          "InsetString"], 
         TagBox[InsetBox["", 823],
          "InsetString"], 
         TagBox[InsetBox["", 824],
          "InsetString"], 
         TagBox[InsetBox["", 825],
          "InsetString"], 
         TagBox[InsetBox["", 826],
          "InsetString"], 
         TagBox[InsetBox["", 827],
          "InsetString"], 
         TagBox[InsetBox["", 828],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 829],
          "InsetString"], 
         TagBox[InsetBox["", 830],
          "InsetString"], 
         TagBox[InsetBox["", 831],
          "InsetString"], 
         TagBox[InsetBox["", 832],
          "InsetString"], 
         TagBox[InsetBox["", 833],
          "InsetString"], 
         TagBox[InsetBox["", 834],
          "InsetString"], 
         TagBox[InsetBox["", 835],
          "InsetString"], 
         TagBox[InsetBox["", 836],
          "InsetString"], 
         TagBox[InsetBox["", 837],
          "InsetString"], 
         TagBox[InsetBox["", 838],
          "InsetString"], 
         TagBox[InsetBox["", 839],
          "InsetString"], 
         TagBox[InsetBox["", 840],
          "InsetString"], 
         TagBox[InsetBox["", 841],
          "InsetString"], 
         TagBox[InsetBox["", 842],
          "InsetString"], 
         TagBox[InsetBox["", 843],
          "InsetString"], 
         TagBox[InsetBox["", 844],
          "InsetString"], 
         TagBox[InsetBox["", 845],
          "InsetString"], 
         TagBox[InsetBox["", 846],
          "InsetString"], 
         TagBox[InsetBox["", 847],
          "InsetString"], 
         TagBox[InsetBox["", 848],
          "InsetString"], 
         TagBox[InsetBox["", 849],
          "InsetString"], 
         TagBox[InsetBox["", 850],
          "InsetString"], 
         TagBox[InsetBox["", 851],
          "InsetString"], 
         TagBox[InsetBox["", 852],
          "InsetString"], 
         TagBox[InsetBox["", 853],
          "InsetString"], 
         TagBox[InsetBox["", 854],
          "InsetString"], 
         TagBox[InsetBox["", 855],
          "InsetString"], 
         TagBox[InsetBox["", 856],
          "InsetString"], 
         TagBox[InsetBox["", 857],
          "InsetString"], 
         TagBox[InsetBox["", 858],
          "InsetString"], 
         TagBox[InsetBox["", 859],
          "InsetString"], 
         TagBox[InsetBox["", 860],
          "InsetString"], 
         TagBox[InsetBox["", 861],
          "InsetString"], 
         TagBox[InsetBox["", 862],
          "InsetString"], 
         TagBox[InsetBox["", 863],
          "InsetString"], 
         TagBox[InsetBox["", 864],
          "InsetString"], 
         TagBox[InsetBox["", 865],
          "InsetString"], 
         TagBox[InsetBox["", 866],
          "InsetString"], 
         TagBox[InsetBox["", 867],
          "InsetString"], 
         TagBox[InsetBox["", 868],
          "InsetString"], 
         TagBox[InsetBox["", 869],
          "InsetString"], 
         TagBox[InsetBox["", 870],
          "InsetString"], 
         TagBox[InsetBox["", 871],
          "InsetString"], 
         TagBox[InsetBox["", 872],
          "InsetString"], 
         TagBox[InsetBox["", 873],
          "InsetString"], 
         TagBox[InsetBox["", 874],
          "InsetString"], 
         TagBox[InsetBox["", 875],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 876],
          "InsetString"], 
         TagBox[InsetBox["", 877],
          "InsetString"], 
         TagBox[InsetBox["", 878],
          "InsetString"], 
         TagBox[InsetBox["", 879],
          "InsetString"], 
         TagBox[InsetBox["", 880],
          "InsetString"], 
         TagBox[InsetBox["", 881],
          "InsetString"], 
         TagBox[InsetBox["", 882],
          "InsetString"], 
         TagBox[InsetBox["", 883],
          "InsetString"], 
         TagBox[InsetBox["", 884],
          "InsetString"], 
         TagBox[InsetBox["", 885],
          "InsetString"], 
         TagBox[InsetBox["", 886],
          "InsetString"], 
         TagBox[InsetBox["", 887],
          "InsetString"], 
         TagBox[InsetBox["", 888],
          "InsetString"], 
         TagBox[InsetBox["", 889],
          "InsetString"], 
         TagBox[InsetBox["", 890],
          "InsetString"], 
         TagBox[InsetBox["", 891],
          "InsetString"], 
         TagBox[InsetBox["", 892],
          "InsetString"], 
         TagBox[InsetBox["", 893],
          "InsetString"], 
         TagBox[InsetBox["", 894],
          "InsetString"], 
         TagBox[InsetBox["", 895],
          "InsetString"], 
         TagBox[InsetBox["", 896],
          "InsetString"], 
         TagBox[InsetBox["", 897],
          "InsetString"], 
         TagBox[InsetBox["", 898],
          "InsetString"], 
         TagBox[InsetBox["", 899],
          "InsetString"], 
         TagBox[InsetBox["", 900],
          "InsetString"], 
         TagBox[InsetBox["", 901],
          "InsetString"], 
         TagBox[InsetBox["", 902],
          "InsetString"], 
         TagBox[InsetBox["", 903],
          "InsetString"], 
         TagBox[InsetBox["", 904],
          "InsetString"], 
         TagBox[InsetBox["", 905],
          "InsetString"], 
         TagBox[InsetBox["", 906],
          "InsetString"], 
         TagBox[InsetBox["", 907],
          "InsetString"], 
         TagBox[InsetBox["", 908],
          "InsetString"], 
         TagBox[InsetBox["", 909],
          "InsetString"], 
         TagBox[InsetBox["", 910],
          "InsetString"], 
         TagBox[InsetBox["", 911],
          "InsetString"], 
         TagBox[InsetBox["", 912],
          "InsetString"], 
         TagBox[InsetBox["", 913],
          "InsetString"], 
         TagBox[InsetBox["", 914],
          "InsetString"], 
         TagBox[InsetBox["", 915],
          "InsetString"], 
         TagBox[InsetBox["", 916],
          "InsetString"], 
         TagBox[InsetBox["", 917],
          "InsetString"], 
         TagBox[InsetBox["", 918],
          "InsetString"], 
         TagBox[InsetBox["", 919],
          "InsetString"], 
         TagBox[InsetBox["", 920],
          "InsetString"], 
         TagBox[InsetBox["", 921],
          "InsetString"], 
         TagBox[InsetBox["", 922],
          "InsetString"], 
         TagBox[InsetBox["", 923],
          "InsetString"], 
         TagBox[InsetBox["", 924],
          "InsetString"], 
         TagBox[InsetBox["", 925],
          "InsetString"], 
         TagBox[InsetBox["", 926],
          "InsetString"], 
         TagBox[InsetBox["", 927],
          "InsetString"], 
         TagBox[InsetBox["", 928],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 929],
          "InsetString"], 
         TagBox[InsetBox["", 930],
          "InsetString"], 
         TagBox[InsetBox["", 931],
          "InsetString"], 
         TagBox[InsetBox["", 932],
          "InsetString"], 
         TagBox[InsetBox["", 933],
          "InsetString"], 
         TagBox[InsetBox["", 934],
          "InsetString"], 
         TagBox[InsetBox["", 935],
          "InsetString"], 
         TagBox[InsetBox["", 936],
          "InsetString"], 
         TagBox[InsetBox["", 937],
          "InsetString"], 
         TagBox[InsetBox["", 938],
          "InsetString"], 
         TagBox[InsetBox["", 939],
          "InsetString"], 
         TagBox[InsetBox["", 940],
          "InsetString"], 
         TagBox[InsetBox["", 941],
          "InsetString"], 
         TagBox[InsetBox["", 942],
          "InsetString"], 
         TagBox[InsetBox["", 943],
          "InsetString"], 
         TagBox[InsetBox["", 944],
          "InsetString"], 
         TagBox[InsetBox["", 945],
          "InsetString"], 
         TagBox[InsetBox["", 946],
          "InsetString"], 
         TagBox[InsetBox["", 947],
          "InsetString"], 
         TagBox[InsetBox["", 948],
          "InsetString"], 
         TagBox[InsetBox["", 949],
          "InsetString"], 
         TagBox[InsetBox["", 950],
          "InsetString"], 
         TagBox[InsetBox["", 951],
          "InsetString"], 
         TagBox[InsetBox["", 952],
          "InsetString"], 
         TagBox[InsetBox["", 953],
          "InsetString"], 
         TagBox[InsetBox["", 954],
          "InsetString"], 
         TagBox[InsetBox["", 955],
          "InsetString"], 
         TagBox[InsetBox["", 956],
          "InsetString"], 
         TagBox[InsetBox["", 957],
          "InsetString"], 
         TagBox[InsetBox["", 958],
          "InsetString"], 
         TagBox[InsetBox["", 959],
          "InsetString"], 
         TagBox[InsetBox["", 960],
          "InsetString"], 
         TagBox[InsetBox["", 961],
          "InsetString"], 
         TagBox[InsetBox["", 962],
          "InsetString"], 
         TagBox[InsetBox["", 963],
          "InsetString"], 
         TagBox[InsetBox["", 964],
          "InsetString"], 
         TagBox[InsetBox["", 965],
          "InsetString"], 
         TagBox[InsetBox["", 966],
          "InsetString"], 
         TagBox[InsetBox["", 967],
          "InsetString"], 
         TagBox[InsetBox["", 968],
          "InsetString"], 
         TagBox[InsetBox["", 969],
          "InsetString"], 
         TagBox[InsetBox["", 970],
          "InsetString"], 
         TagBox[InsetBox["", 971],
          "InsetString"], 
         TagBox[InsetBox["", 972],
          "InsetString"], 
         TagBox[InsetBox["", 973],
          "InsetString"], 
         TagBox[InsetBox["", 974],
          "InsetString"], 
         TagBox[InsetBox["", 975],
          "InsetString"], 
         TagBox[InsetBox["", 976],
          "InsetString"], 
         TagBox[InsetBox["", 977],
          "InsetString"], 
         TagBox[InsetBox["", 978],
          "InsetString"], 
         TagBox[InsetBox["", 979],
          "InsetString"], 
         TagBox[InsetBox["", 980],
          "InsetString"], 
         TagBox[InsetBox["", 981],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.45*^9, 380.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[1, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {384.69, 501.98999999999995`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 5},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztl2tQVGUYxw+CGOjWei01cw1ryPWCt7zHHi9oNSpKOhtmoiiOSEFFXlFQ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      "], {{{}, {}, {}, {}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{154, 1}], LineBox[{155, 2}],
          LineBox[{156, 3}], LineBox[{157, 4}], LineBox[{158, 5}], 
         LineBox[{159, 6}], LineBox[{160, 7}], LineBox[{161, 8}], 
         LineBox[{162, 9}], LineBox[{163, 10}], LineBox[{164, 11}], 
         LineBox[{165, 12}], LineBox[{166, 13}], LineBox[{167, 14}], 
         LineBox[{168, 15}], LineBox[{169, 16}], LineBox[{170, 17}], 
         LineBox[{171, 18}], LineBox[{172, 19}], LineBox[{173, 20}], 
         LineBox[{174, 21}], LineBox[{175, 22}], LineBox[{176, 23}], 
         LineBox[{177, 24}], LineBox[{178, 25}], LineBox[{179, 26}], 
         LineBox[{180, 27}], LineBox[{181, 28}], LineBox[{182, 29}], 
         LineBox[{183, 30}], LineBox[{184, 31}], LineBox[{185, 32}], 
         LineBox[{186, 33}], LineBox[{187, 34}], LineBox[{188, 35}], 
         LineBox[{189, 36}], LineBox[{190, 37}], LineBox[{191, 38}], 
         LineBox[{192, 39}], LineBox[{193, 40}], LineBox[{194, 41}], 
         LineBox[{195, 42}], LineBox[{196, 43}], LineBox[{197, 44}], 
         LineBox[{198, 45}], LineBox[{199, 46}], LineBox[{200, 47}]}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{101, 48}], 
         LineBox[{102, 49}], LineBox[{103, 50}], LineBox[{104, 51}], 
         LineBox[{105, 52}], LineBox[{106, 53}], LineBox[{107, 54}], 
         LineBox[{108, 55}], LineBox[{109, 56}], LineBox[{110, 57}], 
         LineBox[{111, 58}], LineBox[{112, 59}], LineBox[{113, 60}], 
         LineBox[{114, 61}], LineBox[{115, 62}], LineBox[{116, 63}], 
         LineBox[{117, 64}], LineBox[{118, 65}], LineBox[{119, 66}], 
         LineBox[{120, 67}], LineBox[{121, 68}], LineBox[{122, 69}], 
         LineBox[{123, 70}], LineBox[{124, 71}], LineBox[{125, 72}], 
         LineBox[{126, 73}], LineBox[{127, 74}], LineBox[{128, 75}], 
         LineBox[{129, 76}], LineBox[{130, 77}], LineBox[{131, 78}], 
         LineBox[{132, 79}], LineBox[{133, 80}], LineBox[{134, 81}], 
         LineBox[{135, 82}], LineBox[{136, 83}], LineBox[{137, 84}], 
         LineBox[{138, 85}], LineBox[{139, 86}], LineBox[{140, 87}], 
         LineBox[{141, 88}], LineBox[{142, 89}], LineBox[{143, 90}], 
         LineBox[{144, 91}], LineBox[{145, 92}], LineBox[{146, 93}], 
         LineBox[{147, 94}], LineBox[{148, 95}], LineBox[{149, 96}], 
         LineBox[{150, 97}], LineBox[{151, 98}], LineBox[{152, 99}], 
         LineBox[{153, 100}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 202],
          "InsetString"], 
         TagBox[InsetBox["", 203],
          "InsetString"], 
         TagBox[InsetBox["", 204],
          "InsetString"], 
         TagBox[InsetBox["", 205],
          "InsetString"], 
         TagBox[InsetBox["", 206],
          "InsetString"], 
         TagBox[InsetBox["", 207],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 211],
          "InsetString"], 
         TagBox[InsetBox["", 212],
          "InsetString"], 
         TagBox[InsetBox["", 213],
          "InsetString"], 
         TagBox[InsetBox["", 214],
          "InsetString"], 
         TagBox[InsetBox["", 215],
          "InsetString"], 
         TagBox[InsetBox["", 216],
          "InsetString"], 
         TagBox[InsetBox["", 217],
          "InsetString"], 
         TagBox[InsetBox["", 218],
          "InsetString"], 
         TagBox[InsetBox["", 219],
          "InsetString"], 
         TagBox[InsetBox["", 220],
          "InsetString"], 
         TagBox[InsetBox["", 221],
          "InsetString"], 
         TagBox[InsetBox["", 222],
          "InsetString"], 
         TagBox[InsetBox["", 223],
          "InsetString"], 
         TagBox[InsetBox["", 224],
          "InsetString"], 
         TagBox[InsetBox["", 225],
          "InsetString"], 
         TagBox[InsetBox["", 226],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 229],
          "InsetString"], 
         TagBox[InsetBox["", 230],
          "InsetString"], 
         TagBox[InsetBox["", 231],
          "InsetString"], 
         TagBox[InsetBox["", 232],
          "InsetString"], 
         TagBox[InsetBox["", 233],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 235],
          "InsetString"], 
         TagBox[InsetBox["", 236],
          "InsetString"], 
         TagBox[InsetBox["", 237],
          "InsetString"], 
         TagBox[InsetBox["", 238],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 240],
          "InsetString"], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 242],
          "InsetString"], 
         TagBox[InsetBox["", 243],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 249],
          "InsetString"], 
         TagBox[InsetBox["", 250],
          "InsetString"], 
         TagBox[InsetBox["", 251],
          "InsetString"], 
         TagBox[InsetBox["", 252],
          "InsetString"], 
         TagBox[InsetBox["", 253],
          "InsetString"], 
         TagBox[InsetBox["", 254],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 258],
          "InsetString"], 
         TagBox[InsetBox["", 259],
          "InsetString"], 
         TagBox[InsetBox["", 260],
          "InsetString"], 
         TagBox[InsetBox["", 261],
          "InsetString"], 
         TagBox[InsetBox["", 262],
          "InsetString"], 
         TagBox[InsetBox["", 263],
          "InsetString"], 
         TagBox[InsetBox["", 264],
          "InsetString"], 
         TagBox[InsetBox["", 265],
          "InsetString"], 
         TagBox[InsetBox["", 266],
          "InsetString"], 
         TagBox[InsetBox["", 267],
          "InsetString"], 
         TagBox[InsetBox["", 268],
          "InsetString"], 
         TagBox[InsetBox["", 269],
          "InsetString"], 
         TagBox[InsetBox["", 270],
          "InsetString"], 
         TagBox[InsetBox["", 271],
          "InsetString"], 
         TagBox[InsetBox["", 272],
          "InsetString"], 
         TagBox[InsetBox["", 273],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 276],
          "InsetString"], 
         TagBox[InsetBox["", 277],
          "InsetString"], 
         TagBox[InsetBox["", 278],
          "InsetString"], 
         TagBox[InsetBox["", 279],
          "InsetString"], 
         TagBox[InsetBox["", 280],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 282],
          "InsetString"], 
         TagBox[InsetBox["", 283],
          "InsetString"], 
         TagBox[InsetBox["", 284],
          "InsetString"], 
         TagBox[InsetBox["", 285],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 287],
          "InsetString"], 
         TagBox[InsetBox["", 288],
          "InsetString"], 
         TagBox[InsetBox["", 289],
          "InsetString"], 
         TagBox[InsetBox["", 290],
          "InsetString"], 
         TagBox[InsetBox["", 291],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 296],
          "InsetString"], 
         TagBox[InsetBox["", 297],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.45*^9, 2.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-6\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{{3.462724554176334*^9, 3.462724576027646*^9}, 
   3.462728841441455*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Technicals", "Subsection",
 CellChangeTimes->{{3.462719134617889*^9, 3.4627191362643757`*^9}}],

Cell[TextData[{
 "A block of 3 technical charts can be added by setting ",
 StyleBox["Technicals\[ThinSpace]\[Rule]\[ThinSpace]\"True\"", "MB"],
 ". Note that the RSI is based on a moving average so points at the beginning \
are lost."
}], "Text",
 CellChangeTimes->{{3.462724590970192*^9, 3.4627246240895643`*^9}, {
  3.462724730885206*^9, 3.462724748531962*^9}, {3.462728869072446*^9, 
  3.462728869518326*^9}, {3.462789269639284*^9, 3.46278929775672*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "120"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Brown"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotType", "\[Rule]", "\"\<Candlestick\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Technicals", "\[Rule]", "\"\<True\>\""}]}], "\[IndentingNewLine]",
   "]"}]], "Input",
 CellChangeTimes->{{3.462724648469636*^9, 3.4627246523316393`*^9}, {
   3.462728927610223*^9, 3.462728930514024*^9}, 3.462728983334936*^9}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmntwVdUVxq9CsbxqZoDq+IBgbUEomgqOtoWaW2qBYtsAtUVINYogONRE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      
      "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1], LineBox[{1, 577}], 
         LineBox[{2, 578}], LineBox[{3, 579}], LineBox[{4, 580}], 
         LineBox[{5, 581}], LineBox[{6, 582}], LineBox[{7, 583}], 
         LineBox[{8, 584}], LineBox[{9, 585}], LineBox[{10, 586}], 
         LineBox[{11, 587}], LineBox[{12, 588}], LineBox[{13, 589}], 
         LineBox[{14, 590}], LineBox[{15, 591}], LineBox[{16, 592}], 
         LineBox[{17, 593}], LineBox[{18, 594}], LineBox[{19, 595}], 
         LineBox[{20, 596}], LineBox[{21, 597}], LineBox[{22, 598}], 
         LineBox[{23, 599}], LineBox[{24, 600}], LineBox[{25, 601}], 
         LineBox[{26, 602}], LineBox[{27, 603}], LineBox[{28, 604}], 
         LineBox[{29, 605}], LineBox[{30, 606}], LineBox[{31, 607}], 
         LineBox[{32, 608}], LineBox[{33, 609}], LineBox[{34, 610}], 
         LineBox[{35, 611}], LineBox[{36, 612}], LineBox[{37, 613}], 
         LineBox[{38, 614}], LineBox[{39, 615}], LineBox[{40, 616}], 
         LineBox[{41, 617}], LineBox[{42, 618}], LineBox[{43, 619}], 
         LineBox[{44, 620}], LineBox[{45, 621}], LineBox[{46, 622}], 
         LineBox[{47, 623}], LineBox[{48, 624}], LineBox[{49, 625}], 
         LineBox[{50, 626}], LineBox[{51, 627}], LineBox[{52, 628}], 
         LineBox[{53, 629}], LineBox[{54, 630}], LineBox[{55, 631}], 
         LineBox[{56, 632}], LineBox[{57, 633}], LineBox[{58, 634}], 
         LineBox[{59, 635}], LineBox[{60, 636}], LineBox[{61, 637}], 
         LineBox[{62, 638}], LineBox[{63, 639}], LineBox[{64, 640}], 
         LineBox[{65, 641}], LineBox[{66, 642}], LineBox[{67, 643}], 
         LineBox[{68, 644}], LineBox[{69, 645}], LineBox[{70, 646}], 
         LineBox[{71, 647}], LineBox[{72, 648}], LineBox[{73, 649}], 
         LineBox[{74, 650}], LineBox[{75, 651}], LineBox[{76, 652}], 
         LineBox[{77, 653}], LineBox[{78, 654}], LineBox[{79, 655}], 
         LineBox[{80, 656}], LineBox[{81, 657}], LineBox[{82, 658}], 
         LineBox[{83, 659}], LineBox[{84, 660}], LineBox[{85, 661}], 
         LineBox[{86, 662}], LineBox[{87, 663}], LineBox[{88, 664}], 
         LineBox[{89, 665}], LineBox[{90, 666}], LineBox[{91, 667}], 
         LineBox[{92, 668}], LineBox[{93, 669}], LineBox[{94, 670}], 
         LineBox[{95, 671}], LineBox[{96, 672}], LineBox[{97, 673}], 
         LineBox[{98, 674}], LineBox[{99, 675}], LineBox[{100, 676}], 
         LineBox[{101, 677}], LineBox[{102, 678}], LineBox[{103, 679}], 
         LineBox[{104, 680}], LineBox[{105, 681}], LineBox[{106, 682}], 
         LineBox[{107, 683}], LineBox[{108, 684}], LineBox[{109, 685}], 
         LineBox[{110, 686}], LineBox[{111, 687}], LineBox[{112, 688}], 
         LineBox[{113, 689}], LineBox[{114, 690}], LineBox[{115, 691}], 
         LineBox[{116, 692}], LineBox[{117, 693}], LineBox[{118, 694}], 
         LineBox[{119, 695}], 
         LineBox[{120, 
          696}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[2.5], LineBox[{242, 539}], 
         LineBox[{243, 540}], LineBox[{249, 541}], LineBox[{250, 542}], 
         LineBox[{251, 543}], LineBox[{252, 544}], LineBox[{253, 545}], 
         LineBox[{254, 546}], LineBox[{258, 547}], LineBox[{259, 548}], 
         LineBox[{260, 549}], LineBox[{261, 550}], LineBox[{262, 551}], 
         LineBox[{263, 552}], LineBox[{264, 553}], LineBox[{265, 554}], 
         LineBox[{266, 555}], LineBox[{267, 556}], LineBox[{268, 557}], 
         LineBox[{269, 558}], LineBox[{270, 559}], LineBox[{271, 560}], 
         LineBox[{272, 561}], LineBox[{273, 562}], LineBox[{276, 563}], 
         LineBox[{277, 564}], LineBox[{278, 565}], LineBox[{279, 566}], 
         LineBox[{280, 567}], LineBox[{282, 568}], LineBox[{283, 569}], 
         LineBox[{284, 570}], LineBox[{285, 571}], LineBox[{287, 572}], 
         LineBox[{288, 573}], LineBox[{289, 574}], LineBox[{290, 575}], 
         LineBox[{291, 576}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[0., 0.8, 0.], AbsoluteThickness[2.5], LineBox[{349, 481}], 
         LineBox[{416, 482}], LineBox[{351, 483}], LineBox[{418, 484}], 
         LineBox[{353, 485}], LineBox[{420, 486}], LineBox[{421, 487}], 
         LineBox[{356, 488}], LineBox[{423, 489}], LineBox[{358, 490}], 
         LineBox[{425, 491}], LineBox[{360, 492}], LineBox[{427, 493}], 
         LineBox[{362, 494}], LineBox[{429, 495}], LineBox[{364, 496}], 
         LineBox[{431, 497}], LineBox[{366, 498}], LineBox[{433, 499}], 
         LineBox[{370, 500}], LineBox[{437, 501}], LineBox[{438, 502}], 
         LineBox[{439, 503}], LineBox[{440, 504}], LineBox[{441, 505}], 
         LineBox[{377, 506}], LineBox[{444, 507}], LineBox[{445, 508}], 
         LineBox[{381, 509}], LineBox[{448, 510}], LineBox[{449, 511}], 
         LineBox[{450, 512}], LineBox[{451, 513}], LineBox[{452, 514}], 
         LineBox[{453, 515}], LineBox[{389, 516}], LineBox[{456, 517}], 
         LineBox[{392, 518}], LineBox[{459, 519}], LineBox[{394, 520}], 
         LineBox[{461, 521}], LineBox[{462, 522}], LineBox[{463, 523}], 
         LineBox[{464, 524}], LineBox[{399, 525}], LineBox[{466, 526}], 
         LineBox[{467, 527}], LineBox[{402, 528}], LineBox[{469, 529}], 
         LineBox[{405, 530}], LineBox[{472, 531}], LineBox[{407, 532}], 
         LineBox[{474, 533}], LineBox[{475, 534}], LineBox[{476, 535}], 
         LineBox[{477, 536}], LineBox[{478, 537}], LineBox[{479, 538}]}}, {
        {GrayLevel[0], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"], 
         TagBox[InsetBox["", 301],
          "InsetString"], 
         TagBox[InsetBox["", 302],
          "InsetString"], 
         TagBox[InsetBox["", 309],
          "InsetString"], 
         TagBox[InsetBox["", 310],
          "InsetString"], 
         TagBox[InsetBox["", 311],
          "InsetString"], 
         TagBox[InsetBox["", 328],
          "InsetString"], 
         TagBox[InsetBox["", 329],
          "InsetString"], 
         TagBox[InsetBox["", 335],
          "InsetString"], 
         TagBox[InsetBox["", 340],
          "InsetString"], 
         TagBox[InsetBox["", 346],
          "InsetString"], 
         TagBox[InsetBox["", 347],
          "InsetString"], 
         TagBox[InsetBox["", 348],
          "InsetString"], 
         TagBox[InsetBox["", 368],
          "InsetString"], 
         TagBox[InsetBox["", 369],
          "InsetString"], 
         TagBox[InsetBox["", 376],
          "InsetString"], 
         TagBox[InsetBox["", 380],
          "InsetString"], 
         TagBox[InsetBox["", 388],
          "InsetString"], 
         TagBox[InsetBox["", 391],
          "InsetString"], 
         TagBox[InsetBox["", 404],
          "InsetString"], 
         TagBox[InsetBox["", 414],
          "InsetString"], 
         TagBox[InsetBox["", 434],
          "InsetString"], 
         TagBox[InsetBox["", 435],
          "InsetString"], 
         TagBox[InsetBox["", 442],
          "InsetString"], 
         TagBox[InsetBox["", 446],
          "InsetString"], 
         TagBox[InsetBox["", 454],
          "InsetString"], 
         TagBox[InsetBox["", 457],
          "InsetString"], 
         TagBox[InsetBox["", 470],
          "InsetString"], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 697],
          "InsetString"], 
         TagBox[InsetBox["", 698],
          "InsetString"], 
         TagBox[InsetBox["", 699],
          "InsetString"], 
         TagBox[InsetBox["", 700],
          "InsetString"], 
         TagBox[InsetBox["", 701],
          "InsetString"], 
         TagBox[InsetBox["", 702],
          "InsetString"], 
         TagBox[InsetBox["", 703],
          "InsetString"], 
         TagBox[InsetBox["", 704],
          "InsetString"], 
         TagBox[InsetBox["", 705],
          "InsetString"], 
         TagBox[InsetBox["", 706],
          "InsetString"], 
         TagBox[InsetBox["", 707],
          "InsetString"], 
         TagBox[InsetBox["", 708],
          "InsetString"], 
         TagBox[InsetBox["", 709],
          "InsetString"], 
         TagBox[InsetBox["", 710],
          "InsetString"], 
         TagBox[InsetBox["", 711],
          "InsetString"], 
         TagBox[InsetBox["", 712],
          "InsetString"], 
         TagBox[InsetBox["", 713],
          "InsetString"], 
         TagBox[InsetBox["", 714],
          "InsetString"], 
         TagBox[InsetBox["", 715],
          "InsetString"], 
         TagBox[InsetBox["", 716],
          "InsetString"], 
         TagBox[InsetBox["", 717],
          "InsetString"], 
         TagBox[InsetBox["", 718],
          "InsetString"], 
         TagBox[InsetBox["", 719],
          "InsetString"], 
         TagBox[InsetBox["", 720],
          "InsetString"], 
         TagBox[InsetBox["", 721],
          "InsetString"], 
         TagBox[InsetBox["", 722],
          "InsetString"], 
         TagBox[InsetBox["", 723],
          "InsetString"], 
         TagBox[InsetBox["", 724],
          "InsetString"], 
         TagBox[InsetBox["", 725],
          "InsetString"], 
         TagBox[InsetBox["", 726],
          "InsetString"], 
         TagBox[InsetBox["", 727],
          "InsetString"], 
         TagBox[InsetBox["", 728],
          "InsetString"], 
         TagBox[InsetBox["", 729],
          "InsetString"], 
         TagBox[InsetBox["", 730],
          "InsetString"], 
         TagBox[InsetBox["", 731],
          "InsetString"], 
         TagBox[InsetBox["", 732],
          "InsetString"], 
         TagBox[InsetBox["", 733],
          "InsetString"], 
         TagBox[InsetBox["", 734],
          "InsetString"], 
         TagBox[InsetBox["", 735],
          "InsetString"], 
         TagBox[InsetBox["", 736],
          "InsetString"], 
         TagBox[InsetBox["", 737],
          "InsetString"], 
         TagBox[InsetBox["", 738],
          "InsetString"], 
         TagBox[InsetBox["", 739],
          "InsetString"], 
         TagBox[InsetBox["", 740],
          "InsetString"], 
         TagBox[InsetBox["", 741],
          "InsetString"], 
         TagBox[InsetBox["", 742],
          "InsetString"], 
         TagBox[InsetBox["", 743],
          "InsetString"], 
         TagBox[InsetBox["", 744],
          "InsetString"], 
         TagBox[InsetBox["", 745],
          "InsetString"], 
         TagBox[InsetBox["", 746],
          "InsetString"], 
         TagBox[InsetBox["", 747],
          "InsetString"], 
         TagBox[InsetBox["", 748],
          "InsetString"], 
         TagBox[InsetBox["", 749],
          "InsetString"], 
         TagBox[InsetBox["", 750],
          "InsetString"], 
         TagBox[InsetBox["", 751],
          "InsetString"], 
         TagBox[InsetBox["", 752],
          "InsetString"], 
         TagBox[InsetBox["", 753],
          "InsetString"], 
         TagBox[InsetBox["", 754],
          "InsetString"], 
         TagBox[InsetBox["", 755],
          "InsetString"], 
         TagBox[InsetBox["", 756],
          "InsetString"], 
         TagBox[InsetBox["", 757],
          "InsetString"], 
         TagBox[InsetBox["", 758],
          "InsetString"], 
         TagBox[InsetBox["", 759],
          "InsetString"], 
         TagBox[InsetBox["", 760],
          "InsetString"], 
         TagBox[InsetBox["", 761],
          "InsetString"], 
         TagBox[InsetBox["", 762],
          "InsetString"], 
         TagBox[InsetBox["", 763],
          "InsetString"], 
         TagBox[InsetBox["", 764],
          "InsetString"], 
         TagBox[InsetBox["", 765],
          "InsetString"], 
         TagBox[InsetBox["", 766],
          "InsetString"], 
         TagBox[InsetBox["", 767],
          "InsetString"], 
         TagBox[InsetBox["", 768],
          "InsetString"], 
         TagBox[InsetBox["", 769],
          "InsetString"], 
         TagBox[InsetBox["", 770],
          "InsetString"], 
         TagBox[InsetBox["", 771],
          "InsetString"], 
         TagBox[InsetBox["", 772],
          "InsetString"], 
         TagBox[InsetBox["", 773],
          "InsetString"], 
         TagBox[InsetBox["", 774],
          "InsetString"], 
         TagBox[InsetBox["", 775],
          "InsetString"], 
         TagBox[InsetBox["", 776],
          "InsetString"], 
         TagBox[InsetBox["", 777],
          "InsetString"], 
         TagBox[InsetBox["", 778],
          "InsetString"], 
         TagBox[InsetBox["", 779],
          "InsetString"], 
         TagBox[InsetBox["", 780],
          "InsetString"], 
         TagBox[InsetBox["", 781],
          "InsetString"], 
         TagBox[InsetBox["", 782],
          "InsetString"], 
         TagBox[InsetBox["", 783],
          "InsetString"], 
         TagBox[InsetBox["", 784],
          "InsetString"], 
         TagBox[InsetBox["", 785],
          "InsetString"], 
         TagBox[InsetBox["", 786],
          "InsetString"], 
         TagBox[InsetBox["", 787],
          "InsetString"], 
         TagBox[InsetBox["", 788],
          "InsetString"], 
         TagBox[InsetBox["", 789],
          "InsetString"], 
         TagBox[InsetBox["", 790],
          "InsetString"], 
         TagBox[InsetBox["", 791],
          "InsetString"], 
         TagBox[InsetBox["", 792],
          "InsetString"], 
         TagBox[InsetBox["", 793],
          "InsetString"], 
         TagBox[InsetBox["", 794],
          "InsetString"], 
         TagBox[InsetBox["", 795],
          "InsetString"], 
         TagBox[InsetBox["", 796],
          "InsetString"], 
         TagBox[InsetBox["", 797],
          "InsetString"], 
         TagBox[InsetBox["", 798],
          "InsetString"], 
         TagBox[InsetBox["", 799],
          "InsetString"], 
         TagBox[InsetBox["", 800],
          "InsetString"], 
         TagBox[InsetBox["", 801],
          "InsetString"], 
         TagBox[InsetBox["", 802],
          "InsetString"], 
         TagBox[InsetBox["", 803],
          "InsetString"], 
         TagBox[InsetBox["", 804],
          "InsetString"], 
         TagBox[InsetBox["", 805],
          "InsetString"], 
         TagBox[InsetBox["", 806],
          "InsetString"], 
         TagBox[InsetBox["", 807],
          "InsetString"], 
         TagBox[InsetBox["", 808],
          "InsetString"], 
         TagBox[InsetBox["", 809],
          "InsetString"], 
         TagBox[InsetBox["", 810],
          "InsetString"], 
         TagBox[InsetBox["", 811],
          "InsetString"], 
         TagBox[InsetBox["", 812],
          "InsetString"], 
         TagBox[InsetBox["", 813],
          "InsetString"], 
         TagBox[InsetBox["", 814],
          "InsetString"], 
         TagBox[InsetBox["", 815],
          "InsetString"], 
         TagBox[InsetBox["", 816],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 817],
          "InsetString"], 
         TagBox[InsetBox["", 818],
          "InsetString"], 
         TagBox[InsetBox["", 819],
          "InsetString"], 
         TagBox[InsetBox["", 820],
          "InsetString"], 
         TagBox[InsetBox["", 821],
          "InsetString"], 
         TagBox[InsetBox["", 822],
          "InsetString"], 
         TagBox[InsetBox["", 823],
          "InsetString"], 
         TagBox[InsetBox["", 824],
          "InsetString"], 
         TagBox[InsetBox["", 825],
          "InsetString"], 
         TagBox[InsetBox["", 826],
          "InsetString"], 
         TagBox[InsetBox["", 827],
          "InsetString"], 
         TagBox[InsetBox["", 828],
          "InsetString"], 
         TagBox[InsetBox["", 829],
          "InsetString"], 
         TagBox[InsetBox["", 830],
          "InsetString"], 
         TagBox[InsetBox["", 831],
          "InsetString"], 
         TagBox[InsetBox["", 832],
          "InsetString"], 
         TagBox[InsetBox["", 833],
          "InsetString"], 
         TagBox[InsetBox["", 834],
          "InsetString"], 
         TagBox[InsetBox["", 835],
          "InsetString"], 
         TagBox[InsetBox["", 836],
          "InsetString"], 
         TagBox[InsetBox["", 837],
          "InsetString"], 
         TagBox[InsetBox["", 838],
          "InsetString"], 
         TagBox[InsetBox["", 839],
          "InsetString"], 
         TagBox[InsetBox["", 840],
          "InsetString"], 
         TagBox[InsetBox["", 841],
          "InsetString"], 
         TagBox[InsetBox["", 842],
          "InsetString"], 
         TagBox[InsetBox["", 843],
          "InsetString"], 
         TagBox[InsetBox["", 844],
          "InsetString"], 
         TagBox[InsetBox["", 845],
          "InsetString"], 
         TagBox[InsetBox["", 846],
          "InsetString"], 
         TagBox[InsetBox["", 847],
          "InsetString"], 
         TagBox[InsetBox["", 848],
          "InsetString"], 
         TagBox[InsetBox["", 849],
          "InsetString"], 
         TagBox[InsetBox["", 850],
          "InsetString"], 
         TagBox[InsetBox["", 851],
          "InsetString"], 
         TagBox[InsetBox["", 852],
          "InsetString"], 
         TagBox[InsetBox["", 853],
          "InsetString"], 
         TagBox[InsetBox["", 854],
          "InsetString"], 
         TagBox[InsetBox["", 855],
          "InsetString"], 
         TagBox[InsetBox["", 856],
          "InsetString"], 
         TagBox[InsetBox["", 857],
          "InsetString"], 
         TagBox[InsetBox["", 858],
          "InsetString"], 
         TagBox[InsetBox["", 859],
          "InsetString"], 
         TagBox[InsetBox["", 860],
          "InsetString"], 
         TagBox[InsetBox["", 861],
          "InsetString"], 
         TagBox[InsetBox["", 862],
          "InsetString"], 
         TagBox[InsetBox["", 863],
          "InsetString"], 
         TagBox[InsetBox["", 864],
          "InsetString"], 
         TagBox[InsetBox["", 865],
          "InsetString"], 
         TagBox[InsetBox["", 866],
          "InsetString"], 
         TagBox[InsetBox["", 867],
          "InsetString"], 
         TagBox[InsetBox["", 868],
          "InsetString"], 
         TagBox[InsetBox["", 869],
          "InsetString"], 
         TagBox[InsetBox["", 870],
          "InsetString"], 
         TagBox[InsetBox["", 871],
          "InsetString"], 
         TagBox[InsetBox["", 872],
          "InsetString"], 
         TagBox[InsetBox["", 873],
          "InsetString"], 
         TagBox[InsetBox["", 874],
          "InsetString"], 
         TagBox[InsetBox["", 875],
          "InsetString"], 
         TagBox[InsetBox["", 876],
          "InsetString"], 
         TagBox[InsetBox["", 877],
          "InsetString"], 
         TagBox[InsetBox["", 878],
          "InsetString"], 
         TagBox[InsetBox["", 879],
          "InsetString"], 
         TagBox[InsetBox["", 880],
          "InsetString"], 
         TagBox[InsetBox["", 881],
          "InsetString"], 
         TagBox[InsetBox["", 882],
          "InsetString"], 
         TagBox[InsetBox["", 883],
          "InsetString"], 
         TagBox[InsetBox["", 884],
          "InsetString"], 
         TagBox[InsetBox["", 885],
          "InsetString"], 
         TagBox[InsetBox["", 886],
          "InsetString"], 
         TagBox[InsetBox["", 887],
          "InsetString"], 
         TagBox[InsetBox["", 888],
          "InsetString"], 
         TagBox[InsetBox["", 889],
          "InsetString"], 
         TagBox[InsetBox["", 890],
          "InsetString"], 
         TagBox[InsetBox["", 891],
          "InsetString"], 
         TagBox[InsetBox["", 892],
          "InsetString"], 
         TagBox[InsetBox["", 893],
          "InsetString"], 
         TagBox[InsetBox["", 894],
          "InsetString"], 
         TagBox[InsetBox["", 895],
          "InsetString"], 
         TagBox[InsetBox["", 896],
          "InsetString"], 
         TagBox[InsetBox["", 897],
          "InsetString"], 
         TagBox[InsetBox["", 898],
          "InsetString"], 
         TagBox[InsetBox["", 899],
          "InsetString"], 
         TagBox[InsetBox["", 900],
          "InsetString"], 
         TagBox[InsetBox["", 901],
          "InsetString"], 
         TagBox[InsetBox["", 902],
          "InsetString"], 
         TagBox[InsetBox["", 903],
          "InsetString"], 
         TagBox[InsetBox["", 904],
          "InsetString"], 
         TagBox[InsetBox["", 905],
          "InsetString"], 
         TagBox[InsetBox["", 906],
          "InsetString"], 
         TagBox[InsetBox["", 907],
          "InsetString"], 
         TagBox[InsetBox["", 908],
          "InsetString"], 
         TagBox[InsetBox["", 909],
          "InsetString"], 
         TagBox[InsetBox["", 910],
          "InsetString"], 
         TagBox[InsetBox["", 911],
          "InsetString"], 
         TagBox[InsetBox["", 912],
          "InsetString"], 
         TagBox[InsetBox["", 913],
          "InsetString"], 
         TagBox[InsetBox["", 914],
          "InsetString"], 
         TagBox[InsetBox["", 915],
          "InsetString"], 
         TagBox[InsetBox["", 916],
          "InsetString"], 
         TagBox[InsetBox["", 917],
          "InsetString"], 
         TagBox[InsetBox["", 918],
          "InsetString"], 
         TagBox[InsetBox["", 919],
          "InsetString"], 
         TagBox[InsetBox["", 920],
          "InsetString"], 
         TagBox[InsetBox["", 921],
          "InsetString"], 
         TagBox[InsetBox["", 922],
          "InsetString"], 
         TagBox[InsetBox["", 923],
          "InsetString"], 
         TagBox[InsetBox["", 924],
          "InsetString"], 
         TagBox[InsetBox["", 925],
          "InsetString"], 
         TagBox[InsetBox["", 926],
          "InsetString"], 
         TagBox[InsetBox["", 927],
          "InsetString"], 
         TagBox[InsetBox["", 928],
          "InsetString"], 
         TagBox[InsetBox["", 929],
          "InsetString"], 
         TagBox[InsetBox["", 930],
          "InsetString"], 
         TagBox[InsetBox["", 931],
          "InsetString"], 
         TagBox[InsetBox["", 932],
          "InsetString"], 
         TagBox[InsetBox["", 933],
          "InsetString"], 
         TagBox[InsetBox["", 934],
          "InsetString"], 
         TagBox[InsetBox["", 935],
          "InsetString"], 
         TagBox[InsetBox["", 936],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 937],
          "InsetString"], 
         TagBox[InsetBox["", 938],
          "InsetString"], 
         TagBox[InsetBox["", 939],
          "InsetString"], 
         TagBox[InsetBox["", 940],
          "InsetString"], 
         TagBox[InsetBox["", 941],
          "InsetString"], 
         TagBox[InsetBox["", 942],
          "InsetString"], 
         TagBox[InsetBox["", 943],
          "InsetString"], 
         TagBox[InsetBox["", 944],
          "InsetString"], 
         TagBox[InsetBox["", 945],
          "InsetString"], 
         TagBox[InsetBox["", 946],
          "InsetString"], 
         TagBox[InsetBox["", 947],
          "InsetString"], 
         TagBox[InsetBox["", 948],
          "InsetString"], 
         TagBox[InsetBox["", 949],
          "InsetString"], 
         TagBox[InsetBox["", 950],
          "InsetString"], 
         TagBox[InsetBox["", 951],
          "InsetString"], 
         TagBox[InsetBox["", 952],
          "InsetString"], 
         TagBox[InsetBox["", 953],
          "InsetString"], 
         TagBox[InsetBox["", 954],
          "InsetString"], 
         TagBox[InsetBox["", 955],
          "InsetString"], 
         TagBox[InsetBox["", 956],
          "InsetString"], 
         TagBox[InsetBox["", 957],
          "InsetString"], 
         TagBox[InsetBox["", 958],
          "InsetString"], 
         TagBox[InsetBox["", 959],
          "InsetString"], 
         TagBox[InsetBox["", 960],
          "InsetString"], 
         TagBox[InsetBox["", 961],
          "InsetString"], 
         TagBox[InsetBox["", 962],
          "InsetString"], 
         TagBox[InsetBox["", 963],
          "InsetString"], 
         TagBox[InsetBox["", 964],
          "InsetString"], 
         TagBox[InsetBox["", 965],
          "InsetString"], 
         TagBox[InsetBox["", 966],
          "InsetString"], 
         TagBox[InsetBox["", 967],
          "InsetString"], 
         TagBox[InsetBox["", 968],
          "InsetString"], 
         TagBox[InsetBox["", 969],
          "InsetString"], 
         TagBox[InsetBox["", 970],
          "InsetString"], 
         TagBox[InsetBox["", 971],
          "InsetString"], 
         TagBox[InsetBox["", 972],
          "InsetString"], 
         TagBox[InsetBox["", 973],
          "InsetString"], 
         TagBox[InsetBox["", 974],
          "InsetString"], 
         TagBox[InsetBox["", 975],
          "InsetString"], 
         TagBox[InsetBox["", 976],
          "InsetString"], 
         TagBox[InsetBox["", 977],
          "InsetString"], 
         TagBox[InsetBox["", 978],
          "InsetString"], 
         TagBox[InsetBox["", 979],
          "InsetString"], 
         TagBox[InsetBox["", 980],
          "InsetString"], 
         TagBox[InsetBox["", 981],
          "InsetString"], 
         TagBox[InsetBox["", 982],
          "InsetString"], 
         TagBox[InsetBox["", 983],
          "InsetString"], 
         TagBox[InsetBox["", 984],
          "InsetString"], 
         TagBox[InsetBox["", 985],
          "InsetString"], 
         TagBox[InsetBox["", 986],
          "InsetString"], 
         TagBox[InsetBox["", 987],
          "InsetString"], 
         TagBox[InsetBox["", 988],
          "InsetString"], 
         TagBox[InsetBox["", 989],
          "InsetString"], 
         TagBox[InsetBox["", 990],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 991],
          "InsetString"], 
         TagBox[InsetBox["", 992],
          "InsetString"], 
         TagBox[InsetBox["", 993],
          "InsetString"], 
         TagBox[InsetBox["", 994],
          "InsetString"], 
         TagBox[InsetBox["", 995],
          "InsetString"], 
         TagBox[InsetBox["", 996],
          "InsetString"], 
         TagBox[InsetBox["", 997],
          "InsetString"], 
         TagBox[InsetBox["", 998],
          "InsetString"], 
         TagBox[InsetBox["", 999],
          "InsetString"], 
         TagBox[InsetBox["", 1000],
          "InsetString"], 
         TagBox[InsetBox["", 1001],
          "InsetString"], 
         TagBox[InsetBox["", 1002],
          "InsetString"], 
         TagBox[InsetBox["", 1003],
          "InsetString"], 
         TagBox[InsetBox["", 1004],
          "InsetString"], 
         TagBox[InsetBox["", 1005],
          "InsetString"], 
         TagBox[InsetBox["", 1006],
          "InsetString"], 
         TagBox[InsetBox["", 1007],
          "InsetString"], 
         TagBox[InsetBox["", 1008],
          "InsetString"], 
         TagBox[InsetBox["", 1009],
          "InsetString"], 
         TagBox[InsetBox["", 1010],
          "InsetString"], 
         TagBox[InsetBox["", 1011],
          "InsetString"], 
         TagBox[InsetBox["", 1012],
          "InsetString"], 
         TagBox[InsetBox["", 1013],
          "InsetString"], 
         TagBox[InsetBox["", 1014],
          "InsetString"], 
         TagBox[InsetBox["", 1015],
          "InsetString"], 
         TagBox[InsetBox["", 1016],
          "InsetString"], 
         TagBox[InsetBox["", 1017],
          "InsetString"], 
         TagBox[InsetBox["", 1018],
          "InsetString"], 
         TagBox[InsetBox["", 1019],
          "InsetString"], 
         TagBox[InsetBox["", 1020],
          "InsetString"], 
         TagBox[InsetBox["", 1021],
          "InsetString"], 
         TagBox[InsetBox["", 1022],
          "InsetString"], 
         TagBox[InsetBox["", 1023],
          "InsetString"], 
         TagBox[InsetBox["", 1024],
          "InsetString"], 
         TagBox[InsetBox["", 1025],
          "InsetString"], 
         TagBox[InsetBox["", 1026],
          "InsetString"], 
         TagBox[InsetBox["", 1027],
          "InsetString"], 
         TagBox[InsetBox["", 1028],
          "InsetString"], 
         TagBox[InsetBox["", 1029],
          "InsetString"], 
         TagBox[InsetBox["", 1030],
          "InsetString"], 
         TagBox[InsetBox["", 1031],
          "InsetString"], 
         TagBox[InsetBox["", 1032],
          "InsetString"], 
         TagBox[InsetBox["", 1033],
          "InsetString"], 
         TagBox[InsetBox["", 1034],
          "InsetString"], 
         TagBox[InsetBox["", 1035],
          "InsetString"], 
         TagBox[InsetBox["", 1036],
          "InsetString"], 
         TagBox[InsetBox["", 1037],
          "InsetString"], 
         TagBox[InsetBox["", 1038],
          "InsetString"], 
         TagBox[InsetBox["", 1039],
          "InsetString"], 
         TagBox[InsetBox["", 1040],
          "InsetString"], 
         TagBox[InsetBox["", 1041],
          "InsetString"], 
         TagBox[InsetBox["", 1042],
          "InsetString"], 
         TagBox[InsetBox["", 1043],
          "InsetString"], 
         TagBox[InsetBox["", 1044],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1045],
          "InsetString"], 
         TagBox[InsetBox["", 1046],
          "InsetString"], 
         TagBox[InsetBox["", 1047],
          "InsetString"], 
         TagBox[InsetBox["", 1048],
          "InsetString"], 
         TagBox[InsetBox["", 1049],
          "InsetString"], 
         TagBox[InsetBox["", 1050],
          "InsetString"], 
         TagBox[InsetBox["", 1051],
          "InsetString"], 
         TagBox[InsetBox["", 1052],
          "InsetString"], 
         TagBox[InsetBox["", 1053],
          "InsetString"], 
         TagBox[InsetBox["", 1054],
          "InsetString"], 
         TagBox[InsetBox["", 1055],
          "InsetString"], 
         TagBox[InsetBox["", 1056],
          "InsetString"], 
         TagBox[InsetBox["", 1057],
          "InsetString"], 
         TagBox[InsetBox["", 1058],
          "InsetString"], 
         TagBox[InsetBox["", 1059],
          "InsetString"], 
         TagBox[InsetBox["", 1060],
          "InsetString"], 
         TagBox[InsetBox["", 1061],
          "InsetString"], 
         TagBox[InsetBox["", 1062],
          "InsetString"], 
         TagBox[InsetBox["", 1063],
          "InsetString"], 
         TagBox[InsetBox["", 1064],
          "InsetString"], 
         TagBox[InsetBox["", 1065],
          "InsetString"], 
         TagBox[InsetBox["", 1066],
          "InsetString"], 
         TagBox[InsetBox["", 1067],
          "InsetString"], 
         TagBox[InsetBox["", 1068],
          "InsetString"], 
         TagBox[InsetBox["", 1069],
          "InsetString"], 
         TagBox[InsetBox["", 1070],
          "InsetString"], 
         TagBox[InsetBox["", 1071],
          "InsetString"], 
         TagBox[InsetBox["", 1072],
          "InsetString"], 
         TagBox[InsetBox["", 1073],
          "InsetString"], 
         TagBox[InsetBox["", 1074],
          "InsetString"], 
         TagBox[InsetBox["", 1075],
          "InsetString"], 
         TagBox[InsetBox["", 1076],
          "InsetString"], 
         TagBox[InsetBox["", 1077],
          "InsetString"], 
         TagBox[InsetBox["", 1078],
          "InsetString"], 
         TagBox[InsetBox["", 1079],
          "InsetString"], 
         TagBox[InsetBox["", 1080],
          "InsetString"], 
         TagBox[InsetBox["", 1081],
          "InsetString"], 
         TagBox[InsetBox["", 1082],
          "InsetString"], 
         TagBox[InsetBox["", 1083],
          "InsetString"], 
         TagBox[InsetBox["", 1084],
          "InsetString"], 
         TagBox[InsetBox["", 1085],
          "InsetString"], 
         TagBox[InsetBox["", 1086],
          "InsetString"], 
         TagBox[InsetBox["", 1087],
          "InsetString"], 
         TagBox[InsetBox["", 1088],
          "InsetString"], 
         TagBox[InsetBox["", 1089],
          "InsetString"], 
         TagBox[InsetBox["", 1090],
          "InsetString"], 
         TagBox[InsetBox["", 1091],
          "InsetString"], 
         TagBox[InsetBox["", 1092],
          "InsetString"], 
         TagBox[InsetBox["", 1093],
          "InsetString"], 
         TagBox[InsetBox["", 1094],
          "InsetString"], 
         TagBox[InsetBox["", 1095],
          "InsetString"], 
         TagBox[InsetBox["", 1096],
          "InsetString"], 
         TagBox[InsetBox["", 1097],
          "InsetString"], 
         TagBox[InsetBox["", 1098],
          "InsetString"], 
         TagBox[InsetBox["", 1099],
          "InsetString"], 
         TagBox[InsetBox["", 1100],
          "InsetString"], 
         TagBox[InsetBox["", 1101],
          "InsetString"], 
         TagBox[InsetBox["", 1102],
          "InsetString"], 
         TagBox[InsetBox["", 1103],
          "InsetString"], 
         TagBox[InsetBox["", 1104],
          "InsetString"], 
         TagBox[InsetBox["", 1105],
          "InsetString"], 
         TagBox[InsetBox["", 1106],
          "InsetString"], 
         TagBox[InsetBox["", 1107],
          "InsetString"], 
         TagBox[InsetBox["", 1108],
          "InsetString"], 
         TagBox[InsetBox["", 1109],
          "InsetString"], 
         TagBox[InsetBox["", 1110],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1111],
          "InsetString"], 
         TagBox[InsetBox["", 1112],
          "InsetString"], 
         TagBox[InsetBox["", 1113],
          "InsetString"], 
         TagBox[InsetBox["", 1114],
          "InsetString"], 
         TagBox[InsetBox["", 1115],
          "InsetString"], 
         TagBox[InsetBox["", 1116],
          "InsetString"], 
         TagBox[InsetBox["", 1117],
          "InsetString"], 
         TagBox[InsetBox["", 1118],
          "InsetString"], 
         TagBox[InsetBox["", 1119],
          "InsetString"], 
         TagBox[InsetBox["", 1120],
          "InsetString"], 
         TagBox[InsetBox["", 1121],
          "InsetString"], 
         TagBox[InsetBox["", 1122],
          "InsetString"], 
         TagBox[InsetBox["", 1123],
          "InsetString"], 
         TagBox[InsetBox["", 1124],
          "InsetString"], 
         TagBox[InsetBox["", 1125],
          "InsetString"], 
         TagBox[InsetBox["", 1126],
          "InsetString"], 
         TagBox[InsetBox["", 1127],
          "InsetString"], 
         TagBox[InsetBox["", 1128],
          "InsetString"], 
         TagBox[InsetBox["", 1129],
          "InsetString"], 
         TagBox[InsetBox["", 1130],
          "InsetString"], 
         TagBox[InsetBox["", 1131],
          "InsetString"], 
         TagBox[InsetBox["", 1132],
          "InsetString"], 
         TagBox[InsetBox["", 1133],
          "InsetString"], 
         TagBox[InsetBox["", 1134],
          "InsetString"], 
         TagBox[InsetBox["", 1135],
          "InsetString"], 
         TagBox[InsetBox["", 1136],
          "InsetString"], 
         TagBox[InsetBox["", 1137],
          "InsetString"], 
         TagBox[InsetBox["", 1138],
          "InsetString"], 
         TagBox[InsetBox["", 1139],
          "InsetString"], 
         TagBox[InsetBox["", 1140],
          "InsetString"], 
         TagBox[InsetBox["", 1141],
          "InsetString"], 
         TagBox[InsetBox["", 1142],
          "InsetString"], 
         TagBox[InsetBox["", 1143],
          "InsetString"], 
         TagBox[InsetBox["", 1144],
          "InsetString"], 
         TagBox[InsetBox["", 1145],
          "InsetString"], 
         TagBox[InsetBox["", 1146],
          "InsetString"], 
         TagBox[InsetBox["", 1147],
          "InsetString"], 
         TagBox[InsetBox["", 1148],
          "InsetString"], 
         TagBox[InsetBox["", 1149],
          "InsetString"], 
         TagBox[InsetBox["", 1150],
          "InsetString"], 
         TagBox[InsetBox["", 1151],
          "InsetString"], 
         TagBox[InsetBox["", 1152],
          "InsetString"], 
         TagBox[InsetBox["", 1153],
          "InsetString"], 
         TagBox[InsetBox["", 1154],
          "InsetString"], 
         TagBox[InsetBox["", 1155],
          "InsetString"], 
         TagBox[InsetBox["", 1156],
          "InsetString"], 
         TagBox[InsetBox["", 1157],
          "InsetString"], 
         TagBox[InsetBox["", 1158],
          "InsetString"], 
         TagBox[InsetBox["", 1159],
          "InsetString"], 
         TagBox[InsetBox["", 1160],
          "InsetString"], 
         TagBox[InsetBox["", 1161],
          "InsetString"], 
         TagBox[InsetBox["", 1162],
          "InsetString"], 
         TagBox[InsetBox["", 1163],
          "InsetString"], 
         TagBox[InsetBox["", 1164],
          "InsetString"], 
         TagBox[InsetBox["", 1165],
          "InsetString"], 
         TagBox[InsetBox["", 1166],
          "InsetString"], 
         TagBox[InsetBox["", 1167],
          "InsetString"], 
         TagBox[InsetBox["", 1168],
          "InsetString"], 
         TagBox[InsetBox["", 1169],
          "InsetString"], 
         TagBox[InsetBox["", 1170],
          "InsetString"], 
         TagBox[InsetBox["", 1171],
          "InsetString"], 
         TagBox[InsetBox["", 1172],
          "InsetString"], 
         TagBox[InsetBox["", 1173],
          "InsetString"], 
         TagBox[InsetBox["", 1174],
          "InsetString"], 
         TagBox[InsetBox["", 1175],
          "InsetString"], 
         TagBox[InsetBox["", 1176],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.448*^9, 360.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[0.6, 0.4, 0.2], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3447532800, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4475328*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {355.31, 501.98999999999995`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 5},
     Ticks->None], ImageScaled[{0.5, 0.765}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztl/tfjncYxx/0YgxlaEVLZocskU5y7i5GaRYWGV7LWWLlfNyKyouQp0Wb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      "], {{{}, {}, {}, {}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{187, 1}], LineBox[{188, 2}],
          LineBox[{189, 3}], LineBox[{190, 4}], LineBox[{191, 5}], 
         LineBox[{192, 6}], LineBox[{193, 7}], LineBox[{194, 8}], 
         LineBox[{195, 9}], LineBox[{196, 10}], LineBox[{197, 11}], 
         LineBox[{198, 12}], LineBox[{199, 13}], LineBox[{200, 14}], 
         LineBox[{201, 15}], LineBox[{202, 16}], LineBox[{203, 17}], 
         LineBox[{204, 18}], LineBox[{205, 19}], LineBox[{206, 20}], 
         LineBox[{207, 21}], LineBox[{208, 22}], LineBox[{209, 23}], 
         LineBox[{210, 24}], LineBox[{211, 25}], LineBox[{212, 26}], 
         LineBox[{213, 27}], LineBox[{214, 28}], LineBox[{215, 29}], 
         LineBox[{216, 30}], LineBox[{217, 31}], LineBox[{218, 32}], 
         LineBox[{219, 33}], LineBox[{220, 34}], LineBox[{221, 35}], 
         LineBox[{222, 36}], LineBox[{223, 37}], LineBox[{224, 38}], 
         LineBox[{225, 39}], LineBox[{226, 40}], LineBox[{227, 41}], 
         LineBox[{228, 42}], LineBox[{229, 43}], LineBox[{230, 44}], 
         LineBox[{231, 45}], LineBox[{232, 46}], LineBox[{233, 47}], 
         LineBox[{234, 48}], LineBox[{235, 49}], LineBox[{236, 50}], 
         LineBox[{237, 51}], LineBox[{238, 52}], LineBox[{239, 53}], 
         LineBox[{240, 54}]}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{121, 55}], 
         LineBox[{122, 56}], LineBox[{123, 57}], LineBox[{124, 58}], 
         LineBox[{125, 59}], LineBox[{126, 60}], LineBox[{127, 61}], 
         LineBox[{128, 62}], LineBox[{129, 63}], LineBox[{130, 64}], 
         LineBox[{131, 65}], LineBox[{132, 66}], LineBox[{133, 67}], 
         LineBox[{134, 68}], LineBox[{135, 69}], LineBox[{136, 70}], 
         LineBox[{137, 71}], LineBox[{138, 72}], LineBox[{139, 73}], 
         LineBox[{140, 74}], LineBox[{141, 75}], LineBox[{142, 76}], 
         LineBox[{143, 77}], LineBox[{144, 78}], LineBox[{145, 79}], 
         LineBox[{146, 80}], LineBox[{147, 81}], LineBox[{148, 82}], 
         LineBox[{149, 83}], LineBox[{150, 84}], LineBox[{151, 85}], 
         LineBox[{152, 86}], LineBox[{153, 87}], LineBox[{154, 88}], 
         LineBox[{155, 89}], LineBox[{156, 90}], LineBox[{157, 91}], 
         LineBox[{158, 92}], LineBox[{159, 93}], LineBox[{160, 94}], 
         LineBox[{161, 95}], LineBox[{162, 96}], LineBox[{163, 97}], 
         LineBox[{164, 98}], LineBox[{165, 99}], LineBox[{166, 100}], 
         LineBox[{167, 101}], LineBox[{168, 102}], LineBox[{169, 103}], 
         LineBox[{170, 104}], LineBox[{171, 105}], LineBox[{172, 106}], 
         LineBox[{173, 107}], LineBox[{174, 108}], LineBox[{175, 109}], 
         LineBox[{176, 110}], LineBox[{177, 111}], LineBox[{178, 112}], 
         LineBox[{179, 113}], LineBox[{180, 114}], LineBox[{181, 115}], 
         LineBox[{182, 116}], LineBox[{183, 117}], LineBox[{184, 118}], 
         LineBox[{185, 119}], LineBox[{186, 120}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 242],
          "InsetString"], 
         TagBox[InsetBox["", 243],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 249],
          "InsetString"], 
         TagBox[InsetBox["", 250],
          "InsetString"], 
         TagBox[InsetBox["", 251],
          "InsetString"], 
         TagBox[InsetBox["", 252],
          "InsetString"], 
         TagBox[InsetBox["", 253],
          "InsetString"], 
         TagBox[InsetBox["", 254],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 258],
          "InsetString"], 
         TagBox[InsetBox["", 259],
          "InsetString"], 
         TagBox[InsetBox["", 260],
          "InsetString"], 
         TagBox[InsetBox["", 261],
          "InsetString"], 
         TagBox[InsetBox["", 262],
          "InsetString"], 
         TagBox[InsetBox["", 263],
          "InsetString"], 
         TagBox[InsetBox["", 264],
          "InsetString"], 
         TagBox[InsetBox["", 265],
          "InsetString"], 
         TagBox[InsetBox["", 266],
          "InsetString"], 
         TagBox[InsetBox["", 267],
          "InsetString"], 
         TagBox[InsetBox["", 268],
          "InsetString"], 
         TagBox[InsetBox["", 269],
          "InsetString"], 
         TagBox[InsetBox["", 270],
          "InsetString"], 
         TagBox[InsetBox["", 271],
          "InsetString"], 
         TagBox[InsetBox["", 272],
          "InsetString"], 
         TagBox[InsetBox["", 273],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 276],
          "InsetString"], 
         TagBox[InsetBox["", 277],
          "InsetString"], 
         TagBox[InsetBox["", 278],
          "InsetString"], 
         TagBox[InsetBox["", 279],
          "InsetString"], 
         TagBox[InsetBox["", 280],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 282],
          "InsetString"], 
         TagBox[InsetBox["", 283],
          "InsetString"], 
         TagBox[InsetBox["", 284],
          "InsetString"], 
         TagBox[InsetBox["", 285],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 287],
          "InsetString"], 
         TagBox[InsetBox["", 288],
          "InsetString"], 
         TagBox[InsetBox["", 289],
          "InsetString"], 
         TagBox[InsetBox["", 290],
          "InsetString"], 
         TagBox[InsetBox["", 291],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 296],
          "InsetString"], 
         TagBox[InsetBox["", 297],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"], 
         TagBox[InsetBox["", 301],
          "InsetString"], 
         TagBox[InsetBox["", 302],
          "InsetString"], 
         TagBox[InsetBox["", 303],
          "InsetString"], 
         TagBox[InsetBox["", 304],
          "InsetString"], 
         TagBox[InsetBox["", 305],
          "InsetString"], 
         TagBox[InsetBox["", 306],
          "InsetString"], 
         TagBox[InsetBox["", 307],
          "InsetString"], 
         TagBox[InsetBox["", 308],
          "InsetString"], 
         TagBox[InsetBox["", 309],
          "InsetString"], 
         TagBox[InsetBox["", 310],
          "InsetString"], 
         TagBox[InsetBox["", 311],
          "InsetString"], 
         TagBox[InsetBox["", 312],
          "InsetString"], 
         TagBox[InsetBox["", 313],
          "InsetString"], 
         TagBox[InsetBox["", 314],
          "InsetString"], 
         TagBox[InsetBox["", 315],
          "InsetString"], 
         TagBox[InsetBox["", 316],
          "InsetString"], 
         TagBox[InsetBox["", 317],
          "InsetString"], 
         TagBox[InsetBox["", 318],
          "InsetString"], 
         TagBox[InsetBox["", 319],
          "InsetString"], 
         TagBox[InsetBox["", 320],
          "InsetString"], 
         TagBox[InsetBox["", 321],
          "InsetString"], 
         TagBox[InsetBox["", 322],
          "InsetString"], 
         TagBox[InsetBox["", 323],
          "InsetString"], 
         TagBox[InsetBox["", 324],
          "InsetString"], 
         TagBox[InsetBox["", 325],
          "InsetString"], 
         TagBox[InsetBox["", 326],
          "InsetString"], 
         TagBox[InsetBox["", 327],
          "InsetString"], 
         TagBox[InsetBox["", 328],
          "InsetString"], 
         TagBox[InsetBox["", 329],
          "InsetString"], 
         TagBox[InsetBox["", 330],
          "InsetString"], 
         TagBox[InsetBox["", 331],
          "InsetString"], 
         TagBox[InsetBox["", 332],
          "InsetString"], 
         TagBox[InsetBox["", 333],
          "InsetString"], 
         TagBox[InsetBox["", 334],
          "InsetString"], 
         TagBox[InsetBox["", 335],
          "InsetString"], 
         TagBox[InsetBox["", 336],
          "InsetString"], 
         TagBox[InsetBox["", 337],
          "InsetString"], 
         TagBox[InsetBox["", 338],
          "InsetString"], 
         TagBox[InsetBox["", 339],
          "InsetString"], 
         TagBox[InsetBox["", 340],
          "InsetString"], 
         TagBox[InsetBox["", 341],
          "InsetString"], 
         TagBox[InsetBox["", 342],
          "InsetString"], 
         TagBox[InsetBox["", 343],
          "InsetString"], 
         TagBox[InsetBox["", 344],
          "InsetString"], 
         TagBox[InsetBox["", 345],
          "InsetString"], 
         TagBox[InsetBox["", 346],
          "InsetString"], 
         TagBox[InsetBox["", 347],
          "InsetString"], 
         TagBox[InsetBox["", 348],
          "InsetString"], 
         TagBox[InsetBox["", 349],
          "InsetString"], 
         TagBox[InsetBox["", 350],
          "InsetString"], 
         TagBox[InsetBox["", 351],
          "InsetString"], 
         TagBox[InsetBox["", 352],
          "InsetString"], 
         TagBox[InsetBox["", 353],
          "InsetString"], 
         TagBox[InsetBox["", 354],
          "InsetString"], 
         TagBox[InsetBox["", 355],
          "InsetString"], 
         TagBox[InsetBox["", 356],
          "InsetString"], 
         TagBox[InsetBox["", 357],
          "InsetString"], 
         TagBox[InsetBox["", 358],
          "InsetString"], 
         TagBox[InsetBox["", 359],
          "InsetString"], 
         TagBox[InsetBox["", 360],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.448*^9, 0.2},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-7\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3447532800, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4475328*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.465}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmGlYjdvfx7dKJemUMjWwOcZSNro40rDrRMmUJGU6DYRO00FC1K6O/nQ0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      "], {{{}, {}, 
        {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
         LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
          18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 
          35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 
          69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 
          86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 
          102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 
          115, 116, 117, 118, 119, 120}]}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[1.], 
         LineBox[{121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 
          133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 
          146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 
          159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 
          172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 
          185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 
          198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 
          211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 
          224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 
          237, 238, 239, 240}]}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1.], LineBox[{364, 245}], 
         LineBox[{388, 269}], LineBox[{389, 270}], LineBox[{390, 271}], 
         LineBox[{391, 272}], LineBox[{392, 273}], LineBox[{393, 274}], 
         LineBox[{394, 275}], LineBox[{395, 276}], LineBox[{396, 277}], 
         LineBox[{397, 278}], LineBox[{409, 290}], LineBox[{410, 291}], 
         LineBox[{411, 292}], LineBox[{412, 293}], LineBox[{413, 294}], 
         LineBox[{414, 295}], LineBox[{415, 296}], LineBox[{416, 297}], 
         LineBox[{417, 298}], LineBox[{418, 299}], LineBox[{419, 300}], 
         LineBox[{420, 301}], LineBox[{421, 302}], LineBox[{422, 303}], 
         LineBox[{423, 304}], LineBox[{424, 305}], LineBox[{425, 306}], 
         LineBox[{426, 307}], LineBox[{427, 308}], LineBox[{428, 309}], 
         LineBox[{429, 310}], LineBox[{454, 335}], LineBox[{455, 336}], 
         LineBox[{456, 337}], LineBox[{457, 338}], LineBox[{458, 339}], 
         LineBox[{464, 345}], LineBox[{465, 346}], LineBox[{466, 347}], 
         LineBox[{467, 348}], LineBox[{468, 349}], LineBox[{469, 350}], 
         LineBox[{470, 351}], LineBox[{471, 352}], LineBox[{472, 353}]}, 
        {GrayLevel[0], AbsoluteThickness[1.], LineBox[{361, 242}], 
         LineBox[{362, 243}], LineBox[{363, 244}], LineBox[{365, 246}], 
         LineBox[{366, 247}], LineBox[{367, 248}], LineBox[{368, 249}], 
         LineBox[{369, 250}], LineBox[{370, 251}], LineBox[{371, 252}], 
         LineBox[{372, 253}], LineBox[{373, 254}], LineBox[{374, 255}], 
         LineBox[{375, 256}], LineBox[{376, 257}], LineBox[{377, 258}], 
         LineBox[{378, 259}], LineBox[{379, 260}], LineBox[{380, 261}], 
         LineBox[{381, 262}], LineBox[{382, 263}], LineBox[{383, 264}], 
         LineBox[{384, 265}], LineBox[{385, 266}], LineBox[{386, 267}], 
         LineBox[{387, 268}], LineBox[{398, 279}], LineBox[{399, 280}], 
         LineBox[{400, 281}], LineBox[{401, 282}], LineBox[{402, 283}], 
         LineBox[{403, 284}], LineBox[{404, 285}], LineBox[{405, 286}], 
         LineBox[{406, 287}], LineBox[{407, 288}], LineBox[{408, 289}], 
         LineBox[{430, 311}], LineBox[{431, 312}], LineBox[{432, 313}], 
         LineBox[{433, 314}], LineBox[{434, 315}], LineBox[{435, 316}], 
         LineBox[{436, 317}], LineBox[{437, 318}], LineBox[{438, 319}], 
         LineBox[{439, 320}], LineBox[{440, 321}], LineBox[{441, 322}], 
         LineBox[{442, 323}], LineBox[{443, 324}], LineBox[{444, 325}], 
         LineBox[{445, 326}], LineBox[{446, 327}], LineBox[{447, 328}], 
         LineBox[{448, 329}], LineBox[{449, 330}], LineBox[{450, 331}], 
         LineBox[{451, 332}], LineBox[{452, 333}], LineBox[{453, 334}], 
         LineBox[{459, 340}], LineBox[{460, 341}], LineBox[{461, 342}], 
         LineBox[{462, 343}], LineBox[{463, 344}], LineBox[{473, 354}], 
         LineBox[{474, 355}], LineBox[{475, 356}], LineBox[{476, 357}], 
         LineBox[{477, 358}], LineBox[{478, 359}], LineBox[{479, 360}]}}, {
        {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
         TagBox[InsetBox["", 1],
          "InsetString"], 
         TagBox[InsetBox["", 2],
          "InsetString"], 
         TagBox[InsetBox["", 3],
          "InsetString"], 
         TagBox[InsetBox["", 4],
          "InsetString"], 
         TagBox[InsetBox["", 5],
          "InsetString"], 
         TagBox[InsetBox["", 6],
          "InsetString"], 
         TagBox[InsetBox["", 7],
          "InsetString"], 
         TagBox[InsetBox["", 8],
          "InsetString"], 
         TagBox[InsetBox["", 9],
          "InsetString"], 
         TagBox[InsetBox["", 10],
          "InsetString"], 
         TagBox[InsetBox["", 11],
          "InsetString"], 
         TagBox[InsetBox["", 12],
          "InsetString"], 
         TagBox[InsetBox["", 13],
          "InsetString"], 
         TagBox[InsetBox["", 14],
          "InsetString"], 
         TagBox[InsetBox["", 15],
          "InsetString"], 
         TagBox[InsetBox["", 16],
          "InsetString"], 
         TagBox[InsetBox["", 17],
          "InsetString"], 
         TagBox[InsetBox["", 18],
          "InsetString"], 
         TagBox[InsetBox["", 19],
          "InsetString"], 
         TagBox[InsetBox["", 20],
          "InsetString"], 
         TagBox[InsetBox["", 21],
          "InsetString"], 
         TagBox[InsetBox["", 22],
          "InsetString"], 
         TagBox[InsetBox["", 23],
          "InsetString"], 
         TagBox[InsetBox["", 24],
          "InsetString"], 
         TagBox[InsetBox["", 25],
          "InsetString"], 
         TagBox[InsetBox["", 26],
          "InsetString"], 
         TagBox[InsetBox["", 27],
          "InsetString"], 
         TagBox[InsetBox["", 28],
          "InsetString"], 
         TagBox[InsetBox["", 29],
          "InsetString"], 
         TagBox[InsetBox["", 30],
          "InsetString"], 
         TagBox[InsetBox["", 31],
          "InsetString"], 
         TagBox[InsetBox["", 32],
          "InsetString"], 
         TagBox[InsetBox["", 33],
          "InsetString"], 
         TagBox[InsetBox["", 34],
          "InsetString"], 
         TagBox[InsetBox["", 35],
          "InsetString"], 
         TagBox[InsetBox["", 36],
          "InsetString"], 
         TagBox[InsetBox["", 37],
          "InsetString"], 
         TagBox[InsetBox["", 38],
          "InsetString"], 
         TagBox[InsetBox["", 39],
          "InsetString"], 
         TagBox[InsetBox["", 40],
          "InsetString"], 
         TagBox[InsetBox["", 41],
          "InsetString"], 
         TagBox[InsetBox["", 42],
          "InsetString"], 
         TagBox[InsetBox["", 43],
          "InsetString"], 
         TagBox[InsetBox["", 44],
          "InsetString"], 
         TagBox[InsetBox["", 45],
          "InsetString"], 
         TagBox[InsetBox["", 46],
          "InsetString"], 
         TagBox[InsetBox["", 47],
          "InsetString"], 
         TagBox[InsetBox["", 48],
          "InsetString"], 
         TagBox[InsetBox["", 49],
          "InsetString"], 
         TagBox[InsetBox["", 50],
          "InsetString"], 
         TagBox[InsetBox["", 51],
          "InsetString"], 
         TagBox[InsetBox["", 52],
          "InsetString"], 
         TagBox[InsetBox["", 53],
          "InsetString"], 
         TagBox[InsetBox["", 54],
          "InsetString"], 
         TagBox[InsetBox["", 55],
          "InsetString"], 
         TagBox[InsetBox["", 56],
          "InsetString"], 
         TagBox[InsetBox["", 57],
          "InsetString"], 
         TagBox[InsetBox["", 58],
          "InsetString"], 
         TagBox[InsetBox["", 59],
          "InsetString"], 
         TagBox[InsetBox["", 60],
          "InsetString"], 
         TagBox[InsetBox["", 61],
          "InsetString"], 
         TagBox[InsetBox["", 62],
          "InsetString"], 
         TagBox[InsetBox["", 63],
          "InsetString"], 
         TagBox[InsetBox["", 64],
          "InsetString"], 
         TagBox[InsetBox["", 65],
          "InsetString"], 
         TagBox[InsetBox["", 66],
          "InsetString"], 
         TagBox[InsetBox["", 67],
          "InsetString"], 
         TagBox[InsetBox["", 68],
          "InsetString"], 
         TagBox[InsetBox["", 69],
          "InsetString"], 
         TagBox[InsetBox["", 70],
          "InsetString"], 
         TagBox[InsetBox["", 71],
          "InsetString"], 
         TagBox[InsetBox["", 72],
          "InsetString"], 
         TagBox[InsetBox["", 73],
          "InsetString"], 
         TagBox[InsetBox["", 74],
          "InsetString"], 
         TagBox[InsetBox["", 75],
          "InsetString"], 
         TagBox[InsetBox["", 76],
          "InsetString"], 
         TagBox[InsetBox["", 77],
          "InsetString"], 
         TagBox[InsetBox["", 78],
          "InsetString"], 
         TagBox[InsetBox["", 79],
          "InsetString"], 
         TagBox[InsetBox["", 80],
          "InsetString"], 
         TagBox[InsetBox["", 81],
          "InsetString"], 
         TagBox[InsetBox["", 82],
          "InsetString"], 
         TagBox[InsetBox["", 83],
          "InsetString"], 
         TagBox[InsetBox["", 84],
          "InsetString"], 
         TagBox[InsetBox["", 85],
          "InsetString"], 
         TagBox[InsetBox["", 86],
          "InsetString"], 
         TagBox[InsetBox["", 87],
          "InsetString"], 
         TagBox[InsetBox["", 88],
          "InsetString"], 
         TagBox[InsetBox["", 89],
          "InsetString"], 
         TagBox[InsetBox["", 90],
          "InsetString"], 
         TagBox[InsetBox["", 91],
          "InsetString"], 
         TagBox[InsetBox["", 92],
          "InsetString"], 
         TagBox[InsetBox["", 93],
          "InsetString"], 
         TagBox[InsetBox["", 94],
          "InsetString"], 
         TagBox[InsetBox["", 95],
          "InsetString"], 
         TagBox[InsetBox["", 96],
          "InsetString"], 
         TagBox[InsetBox["", 97],
          "InsetString"], 
         TagBox[InsetBox["", 98],
          "InsetString"], 
         TagBox[InsetBox["", 99],
          "InsetString"], 
         TagBox[InsetBox["", 100],
          "InsetString"], 
         TagBox[InsetBox["", 101],
          "InsetString"], 
         TagBox[InsetBox["", 102],
          "InsetString"], 
         TagBox[InsetBox["", 103],
          "InsetString"], 
         TagBox[InsetBox["", 104],
          "InsetString"], 
         TagBox[InsetBox["", 105],
          "InsetString"], 
         TagBox[InsetBox["", 106],
          "InsetString"], 
         TagBox[InsetBox["", 107],
          "InsetString"], 
         TagBox[InsetBox["", 108],
          "InsetString"], 
         TagBox[InsetBox["", 109],
          "InsetString"], 
         TagBox[InsetBox["", 110],
          "InsetString"], 
         TagBox[InsetBox["", 111],
          "InsetString"], 
         TagBox[InsetBox["", 112],
          "InsetString"], 
         TagBox[InsetBox["", 113],
          "InsetString"], 
         TagBox[InsetBox["", 114],
          "InsetString"], 
         TagBox[InsetBox["", 115],
          "InsetString"], 
         TagBox[InsetBox["", 116],
          "InsetString"], 
         TagBox[InsetBox["", 117],
          "InsetString"], 
         TagBox[InsetBox["", 118],
          "InsetString"], 
         TagBox[InsetBox["", 119],
          "InsetString"], 
         TagBox[InsetBox["", 120],
          "InsetString"]}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[1.], 
         TagBox[InsetBox["", 121],
          "InsetString"], 
         TagBox[InsetBox["", 122],
          "InsetString"], 
         TagBox[InsetBox["", 123],
          "InsetString"], 
         TagBox[InsetBox["", 124],
          "InsetString"], 
         TagBox[InsetBox["", 125],
          "InsetString"], 
         TagBox[InsetBox["", 126],
          "InsetString"], 
         TagBox[InsetBox["", 127],
          "InsetString"], 
         TagBox[InsetBox["", 128],
          "InsetString"], 
         TagBox[InsetBox["", 129],
          "InsetString"], 
         TagBox[InsetBox["", 130],
          "InsetString"], 
         TagBox[InsetBox["", 131],
          "InsetString"], 
         TagBox[InsetBox["", 132],
          "InsetString"], 
         TagBox[InsetBox["", 133],
          "InsetString"], 
         TagBox[InsetBox["", 134],
          "InsetString"], 
         TagBox[InsetBox["", 135],
          "InsetString"], 
         TagBox[InsetBox["", 136],
          "InsetString"], 
         TagBox[InsetBox["", 137],
          "InsetString"], 
         TagBox[InsetBox["", 138],
          "InsetString"], 
         TagBox[InsetBox["", 139],
          "InsetString"], 
         TagBox[InsetBox["", 140],
          "InsetString"], 
         TagBox[InsetBox["", 141],
          "InsetString"], 
         TagBox[InsetBox["", 142],
          "InsetString"], 
         TagBox[InsetBox["", 143],
          "InsetString"], 
         TagBox[InsetBox["", 144],
          "InsetString"], 
         TagBox[InsetBox["", 145],
          "InsetString"], 
         TagBox[InsetBox["", 146],
          "InsetString"], 
         TagBox[InsetBox["", 147],
          "InsetString"], 
         TagBox[InsetBox["", 148],
          "InsetString"], 
         TagBox[InsetBox["", 149],
          "InsetString"], 
         TagBox[InsetBox["", 150],
          "InsetString"], 
         TagBox[InsetBox["", 151],
          "InsetString"], 
         TagBox[InsetBox["", 152],
          "InsetString"], 
         TagBox[InsetBox["", 153],
          "InsetString"], 
         TagBox[InsetBox["", 154],
          "InsetString"], 
         TagBox[InsetBox["", 155],
          "InsetString"], 
         TagBox[InsetBox["", 156],
          "InsetString"], 
         TagBox[InsetBox["", 157],
          "InsetString"], 
         TagBox[InsetBox["", 158],
          "InsetString"], 
         TagBox[InsetBox["", 159],
          "InsetString"], 
         TagBox[InsetBox["", 160],
          "InsetString"], 
         TagBox[InsetBox["", 161],
          "InsetString"], 
         TagBox[InsetBox["", 162],
          "InsetString"], 
         TagBox[InsetBox["", 163],
          "InsetString"], 
         TagBox[InsetBox["", 164],
          "InsetString"], 
         TagBox[InsetBox["", 165],
          "InsetString"], 
         TagBox[InsetBox["", 166],
          "InsetString"], 
         TagBox[InsetBox["", 167],
          "InsetString"], 
         TagBox[InsetBox["", 168],
          "InsetString"], 
         TagBox[InsetBox["", 169],
          "InsetString"], 
         TagBox[InsetBox["", 170],
          "InsetString"], 
         TagBox[InsetBox["", 171],
          "InsetString"], 
         TagBox[InsetBox["", 172],
          "InsetString"], 
         TagBox[InsetBox["", 173],
          "InsetString"], 
         TagBox[InsetBox["", 174],
          "InsetString"], 
         TagBox[InsetBox["", 175],
          "InsetString"], 
         TagBox[InsetBox["", 176],
          "InsetString"], 
         TagBox[InsetBox["", 177],
          "InsetString"], 
         TagBox[InsetBox["", 178],
          "InsetString"], 
         TagBox[InsetBox["", 179],
          "InsetString"], 
         TagBox[InsetBox["", 180],
          "InsetString"], 
         TagBox[InsetBox["", 181],
          "InsetString"], 
         TagBox[InsetBox["", 182],
          "InsetString"], 
         TagBox[InsetBox["", 183],
          "InsetString"], 
         TagBox[InsetBox["", 184],
          "InsetString"], 
         TagBox[InsetBox["", 185],
          "InsetString"], 
         TagBox[InsetBox["", 186],
          "InsetString"], 
         TagBox[InsetBox["", 187],
          "InsetString"], 
         TagBox[InsetBox["", 188],
          "InsetString"], 
         TagBox[InsetBox["", 189],
          "InsetString"], 
         TagBox[InsetBox["", 190],
          "InsetString"], 
         TagBox[InsetBox["", 191],
          "InsetString"], 
         TagBox[InsetBox["", 192],
          "InsetString"], 
         TagBox[InsetBox["", 193],
          "InsetString"], 
         TagBox[InsetBox["", 194],
          "InsetString"], 
         TagBox[InsetBox["", 195],
          "InsetString"], 
         TagBox[InsetBox["", 196],
          "InsetString"], 
         TagBox[InsetBox["", 197],
          "InsetString"], 
         TagBox[InsetBox["", 198],
          "InsetString"], 
         TagBox[InsetBox["", 199],
          "InsetString"], 
         TagBox[InsetBox["", 200],
          "InsetString"], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 202],
          "InsetString"], 
         TagBox[InsetBox["", 203],
          "InsetString"], 
         TagBox[InsetBox["", 204],
          "InsetString"], 
         TagBox[InsetBox["", 205],
          "InsetString"], 
         TagBox[InsetBox["", 206],
          "InsetString"], 
         TagBox[InsetBox["", 207],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 211],
          "InsetString"], 
         TagBox[InsetBox["", 212],
          "InsetString"], 
         TagBox[InsetBox["", 213],
          "InsetString"], 
         TagBox[InsetBox["", 214],
          "InsetString"], 
         TagBox[InsetBox["", 215],
          "InsetString"], 
         TagBox[InsetBox["", 216],
          "InsetString"], 
         TagBox[InsetBox["", 217],
          "InsetString"], 
         TagBox[InsetBox["", 218],
          "InsetString"], 
         TagBox[InsetBox["", 219],
          "InsetString"], 
         TagBox[InsetBox["", 220],
          "InsetString"], 
         TagBox[InsetBox["", 221],
          "InsetString"], 
         TagBox[InsetBox["", 222],
          "InsetString"], 
         TagBox[InsetBox["", 223],
          "InsetString"], 
         TagBox[InsetBox["", 224],
          "InsetString"], 
         TagBox[InsetBox["", 225],
          "InsetString"], 
         TagBox[InsetBox["", 226],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 229],
          "InsetString"], 
         TagBox[InsetBox["", 230],
          "InsetString"], 
         TagBox[InsetBox["", 231],
          "InsetString"], 
         TagBox[InsetBox["", 232],
          "InsetString"], 
         TagBox[InsetBox["", 233],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 235],
          "InsetString"], 
         TagBox[InsetBox["", 236],
          "InsetString"], 
         TagBox[InsetBox["", 237],
          "InsetString"], 
         TagBox[InsetBox["", 238],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 240],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 481],
          "InsetString"], 
         TagBox[InsetBox["", 482],
          "InsetString"], 
         TagBox[InsetBox["", 483],
          "InsetString"], 
         TagBox[InsetBox["", 484],
          "InsetString"], 
         TagBox[InsetBox["", 485],
          "InsetString"], 
         TagBox[InsetBox["", 486],
          "InsetString"], 
         TagBox[InsetBox["", 487],
          "InsetString"], 
         TagBox[InsetBox["", 488],
          "InsetString"], 
         TagBox[InsetBox["", 489],
          "InsetString"], 
         TagBox[InsetBox["", 490],
          "InsetString"], 
         TagBox[InsetBox["", 491],
          "InsetString"], 
         TagBox[InsetBox["", 492],
          "InsetString"], 
         TagBox[InsetBox["", 493],
          "InsetString"], 
         TagBox[InsetBox["", 494],
          "InsetString"], 
         TagBox[InsetBox["", 495],
          "InsetString"], 
         TagBox[InsetBox["", 496],
          "InsetString"], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     Axes->True,
     AxesOrigin->NCache[{3447619200, 0}, {3.4476192*^9, 0}],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"MACD\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{None, None}, {{{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
     GridLines->NCache[{{{3447532800, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, None}, {{{3.4475328*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, None}],
     GridLinesStyle->Directive[
       GrayLevel[0.85], 
       Dashing[{Small, Small}], 
       AbsoluteThickness[0.5]],
     ImagePadding->{{50, 10}, {1, 10}},
     ImageSize->600,
     PlotRange->{{3.4476192*^9, 3.4625664*^9}, {-3.885225344475044, 
      13.205713130015283`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 1},
     Ticks->{{{
         NCache[3447532800, 3.4475328*^9], 
         FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3450124800, 3.4501248*^9], 
         FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3452803200, 3.4528032*^9], 
         FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3455395200, 3.4553952*^9], 
         FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3458073600, 3.4580736*^9], 
         FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3460752000, 3.460752*^9], 
         FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3448742400, 3.4487424*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3451334400, 3.4513344*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3454012800, 3.4540128*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3456604800, 3.4566048*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3459283200, 3.4592832*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3461961600, 3.4619616*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, 
       Automatic}], ImageScaled[{0.5, 0.3237}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[{{}, {}, 
      {RGBColor[0.6, 0.24, 0.4429], AbsoluteThickness[1.5], 
       LineBox[CompressedData["
1:eJwl1AtMU1cYB/CiiwqbrDAWhYgp4IbysiDyGK9e5F2F0grqEK2CK5uMXhyb
UYJWIJjAkKIE2GaQggSo6CoizyIdQqmC2MkGIirIY+5oiAQYzAmyr99tctP8
cs4953/O+e6xOSoVHlvFYrEOwWP4hx/Hr5FQiz3zumylkAcWSMCz+evCHDIF
BssKDXY6X5LrssdgVRs45FlvaPY3uw0emwL7FTmpv7fDdjqxiVD6AKH0+aoo
g8svgBU1v0ZY5qH1zWCer2hluSvSYNYEuKq04ouyE2ju+mZCXcw/bVRnhOPN
3AaPjybGn7DDPJwxsOM72kq9Gy0waSHU9NTgwN9vmLzuYNMHLQfr70QbzL4F
lh3x5gddx/XxnoGl57pNqMdoem0roXyue/HWbBFhXldwwTpb1wPDaH0cWO8Q
e2jjZ3sNFj8Bz8f/dGtrIb4v/6CNUL7pm8QrTTifxgWsiKQTlK8xz8x+cKVZ
S3PrMq6fkwV2KFOxzeaZ/TRSE6r+ssT+nDQC99MRnG6jsnu4Gc2OBW9JTApM
6QzH/DKwS1iTPCkITSvB17RO9lFaPu7ntnZC9YlZdiWFIbifIvCGRyG1NUvB
mD8DfGoicLryYijmrwbHiT58+58zzqf5HRzxPCMvRYL5BII7hHLS+wz4j+N5
y06Dqc052/XtOJ/qKrhhOHzCv5jJ3w+uUORNW2bh/OUnOwh19pOYxsdLaL0C
/NBjYvD+y0DM2wceNfWo+k6M+bgL4H3hh83PxIZhXo6GUJYSUa91I653pgys
rIvwalFjHs49sDNxddo3jBbMge3z3HLv2jP1YP0bob4l3bbqSrQqFOxdadXf
UYT9eVqwjvN67sd8ND0DXowMMtnZjOsrt+oklNCu+I2iEq0PAutautpepTL1
KwU/8U/qLojD8eXT4Bvszw/Y/IXnrdlwF86vYMRm9Z9MPVPg5pxJsWsCkz8Z
vPbSq4W6f3E8QTFY0fOixphg/zGLLkKd+XR9d4w1js8OAFcc5dORfDTva/Ce
zJOuYck4H30J3HWlhGfKZr6/dnD6oPvbhlTsz/XtJlRgLT3pKESLvwKn/TCQ
cSoI+8vl4OwjtFFiNLZrWsFDS2YuxX8w9TwJjkhbDHZyY/Y3QUuoF1xZRmIO
no8qHxyd7Pd+4zX0WBPYO5O1qLZFs8fBeo/hqy+LcL28j3oItc3IU2o+xNwP
uWDJZWWbDbOfrNtgL0ln6vEbTP5RsO3HxinKZuwvNtYRauXKzWWhDseT7wDv
8NjlzrdFc+rBJv2TWT//wtTHU3DCoyht1xTmka25Ryj6+FxfeAvWm4oLrpWb
tT4tZer5S/A/ttU3s9n4fvnq+/C9Vr7bHqLDdr0zuFbyQDsygu2s/eCYgCHu
Lk/Mx80E0xZ+btWlTD2wegklPpbScDCY2U8HsEuyxWJaFXO/xYBbQzRNe82Z
+/gsONhzIeNCGlpWC3YITPbjH0azt/YRijR2zG56z9xvQnC8j/ds3U4R73/W
aFd1
        "]]}},
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     Axes->True,
     AxesOrigin->NCache[{3447619200, 50}, {3.4476192*^9, 50}],
     AxesStyle->GrayLevel[0.5],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"RSI\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{{{20, 
          FormBox["20", TraditionalForm]}, {50, 
          FormBox["50", TraditionalForm]}, {80, 
          FormBox["80", TraditionalForm]}}, None}, {None, None}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
     GridLines->NCache[{{{3447532800, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, {20, 80}}, {{{3.4475328*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, {20, 80}}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.45], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 1}},
     ImageSize->600,
     PlotRange->
      NCache[{{3447619200, 3462566400}, {0, 100}}, {{3.4476192*^9, 
        3.4625664*^9}, {0, 100}}],
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 0},
     Ticks->{{{
         NCache[3447532800, 3.4475328*^9], 
         FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3450124800, 3.4501248*^9], 
         FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3452803200, 3.4528032*^9], 
         FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3455395200, 3.4553952*^9], 
         FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3458073600, 3.4580736*^9], 
         FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3460752000, 3.460752*^9], 
         FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3448742400, 3.4487424*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3451334400, 3.4513344*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3454012800, 3.4540128*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3456604800, 3.4566048*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3459283200, 3.4592832*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3461961600, 3.4619616*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, 
       Automatic}], ImageScaled[{0.5, 0.203}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[{{}, {}, 
      {RGBColor[0.24, 0.6, 0.34], AbsoluteThickness[1.5], 
       LineBox[CompressedData["
1:eJwl1A1Q0mccwHFS5ktqOXNNLTtqyys1o5etsFg8GrNsFeZ7llEkaenU1Eyz
xErXtBqt5qozpVZz2RupoGkasLI3S0hPM6lYzqu/qYmiy7TYjx/ccdyH5/88
/+//dxzTtySui7Gg0Wi74W36hJe66RpFPuyO57ok+ClNX3wA64Llt5RZTgQo
3i2FdbvRsWE9MVlxDqwu8VAt8llscn8jeNeLnqLI+34mM/4DX0hzZjtt45ms
K7lOEUmas3v2jkiTHR+A564WSwL1wSZzDOCFRk1C3R+bTU6aVk4Rena0pdWu
bSZLVoD9NM4pFwmaeRc8MBbTFybfYjJfDw7N49D9u7Zi75QKisw6vdntl6lo
BRfcPm++a7ifAHsTwVPtZrC/H4oxWdQHPhTKL3h2Ey11qaSINqA6YXY0Xq/z
A9PS6R4vP+B5jglgnbYxY5l0I/b/Dv7Eip+14+N6k9VfyKBvaKVyfB/20Tjg
X+NcRtkz8HzmdnB026DGMhfP558A53dNeEzE+PzierDYa7rq40IhzpMtpwin
9aF3ZCo+P28bOEgxVJrfgPtFx8AZ7xpL5d2bsL8WnOX1ZoPdZOzTdYElJwR+
rndCcb5bqyjyyCtWaLjDx/keBdvUrDJya7BXXQ02qDaU3fLAdVon+H3ujZD0
iRuw36GaIisC7YUrdkXgPGXgrMwom78uYD9DB7Zr2kQ76YPz4I2/QZF9oqDU
1lg8T7QQXAh+nYbzcKwAF1dVhXkFxeI8n4MZt3On6fu3Y691DUV4YoGsaV4S
9s4D62bOLTNe3om9UeBJ69J6Wval4DyfgR02H7w0fCIV50mvpUhp/og/azFa
4QP2t59mMArR/RHgU4kcfc9aPI9xABxU0lScdh4tHXeTIvWTAwpt5ck4Ty/w
d2M27DMPf8T+MPBiMMMSzRGB1RXXJwwFmfvLwGvzYivfybCfNruOIpftNbHy
iYk4z2CwZ51K89Ub3M/fC56pLnK/XosWl5rWX9S6HWFij0ID/qyVVXbSFft5
vHqK1LYPdgwk4/OLMsFTWNcmpzab+8+DbbS+3DlO5v7H4EOD4a7hi/D+kvRb
FLnxucW3EVxcV58FjyhKctrfxWNvI1h7OFWcX56AvcPgPtZhTr6zuZehoMjS
kconolY8r78YHFfx+FHSSexh3AcXTMr4iZlo7h0Eu/cOFo27ko697kqKpFoN
Wb4syMTeAHAwW9B+dVUGzrMB7MDd/1uIFp3UD04/ZUh5a4WWuKng9zLUs5J7
Aa1eDi7vkKx3CtmD/YlgL9GAIc9tL86zFywo1QomZuzDeX75N/wfvO4sfKZE
9xOw96pR7+d/ZmN/PDi62jlukhTXeYVgt4opMofALJyn822KZBaGzTHG4/mO
y8BRvbmB/+zB/Zw4sLAl4ciAFvcnHQcXvDKIz1qgJXXgzqmFY/O7cD9z6R2K
NHtYB7v44zpfCK5puus5fMXcLwa7vKGr3reiFTXgb5JawvVfm/v/BQuftvhu
ZaBFggb4fZDT1sYS7JUeAQ/ObrMuajf3V4E7Rptt7Xh4veMr8KOqjT9E7RRh
v/1disRsp49bsCYH55sPtjp+VEDvRtNk4E9FYxuXt6OZL8H3xKxeZSCab3uP
IpG+5UvczqHFC8DHNDkrlXZoRjlY4JsS4tuJ5mnBo1dIfW833l9kdZ8ieYfa
hB0+aCkT/OLaatufz6B168HF5xmhDRNwv8TyAUWcvNsq58bjunoO2MozLt2j
GU2LMLvqsM1+7N0PrrUIrHlwDK2gPaTIuSVP2d3jD+A8PcFv/Uf0AVfRjFDw
E6Ia5V46iL3Z4GCiZCUno0UXwWFEeUpKw+sdZzVSZE1Ls/yMDM1ZB37aY3wS
yzhA/geLjBXU
        "]]}},
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->NCache[{3447619200, 0}, {3.4476192*^9, 0}],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"ADI\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{None, None}, {{{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"Apr\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3447532800, 3.4475328*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3448742400, 3.4487424*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3447532800, 
          GrayLevel[0.8]}, {3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, None}, {{{3.4475328*^9, 
          GrayLevel[0.8]}, {3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, None}],
     GridLinesStyle->Directive[
       GrayLevel[0.85], 
       Dashing[{Small, Small}], 
       AbsoluteThickness[0.5]],
     ImagePadding->{{50, 10}, {15, 1}},
     ImageSize->600,
     PlotRange->{{3.4476192*^9, 3.4625664*^9}, {-2.41515254237282*^6, 
      4.3346111469852105`*^7}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 0},
     Ticks->None], ImageScaled[{0.5, 0.075}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 1.25}}]], "Output",
 CellChangeTimes->{{3.4627246425404654`*^9, 3.46272465381148*^9}, 
   3.462728984544121*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bollinger", "Subsection",
 CellChangeTimes->{{3.462719140433147*^9, 3.462719146704483*^9}}],

Cell[TextData[{
 "Traders use Bolinger bands on plots. These can be added by setting ",
 StyleBox["Bollinger\[ThinSpace]\[Rule]\[ThinSpace]\"True\"", "MB"],
 ". Note that Bollinger band use a moving average so points at the beginning \
are lost."
}], "Text",
 CellChangeTimes->{{3.4627246836997023`*^9, 3.462724720883353*^9}, {
  3.4627247553253736`*^9, 3.462724759836151*^9}, {3.462789320838138*^9, 
  3.462789339381132*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "100"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Blue"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotType", "\[Rule]", "\"\<Candlestick\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Bollinger", "\[Rule]", "\"\<True\>\""}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{
  3.462724765096089*^9, {3.4627289147961082`*^9, 3.462728917185766*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztW3lYVfXWJjVUnHBCr6UimeIIqTmkJifyKtkgaESOqEg4gg2KCIJDaXpV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      "], {{{}, {}, {}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], LineBox[CompressedData["
1:eJwNxmk2ggEAAMCvoziSjuAAOgsSkbRI2YkKERFSUUKoLN3E/Jj3ZmomEp4N
BUEwzZzMs0CURWIssUycFVZJsEaSdVKkyZBlgxyb5CmwxTY77LLHPgccckSR
Y04oUabCKWecU+WCS2pccU2dG25pcMc9DzR5pEWbDk8806XHC31eeeOdAR98
8sWQEWO++eGXPyb8A6NrPN8=
          "]]}, 
        {GrayLevel[0], LineBox[CompressedData["
1:eJwVzdk2gmEAQNHvLzJUZIoQypAGGd6ER/AA9c7InCSZSuwu9lrn7hQumxeN
KIRwzpW45oYWt9xxzwOPPPFMmxc6vNLljR7v9Pngky+++WHAkF9G/I3nsRAi
YsSZYJIEU0wzwyxJUqSZY54MCyyyxDIrZFlljRzrbLBJni222aFAkV322OeA
EoeUqVClxhF1jjnhlDP+AWrYI7M=
          "]]}, 
        {GrayLevel[0], LineBox[CompressedData["
1:eJwNw4c6QmEAANCfN+kZshJlRWTUVSlkZCTjXqRE2S/unO87qXYSxRMhhIrp
yRCmnHbGWefMOG/WBRfNmXfJZVdcdc2C625YdNMtS2674657lq0YuW/VmnUP
bNj00COPbXniqWee2/bCS6+8tuONXW+9894HYxMfffLZni/2Hfjq0DffHTn2
w0+//PbHX//8B8gpH3I=
          
          "]]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1], LineBox[{1, 725}], 
         LineBox[{2, 726}], LineBox[{3, 727}], LineBox[{4, 728}], 
         LineBox[{5, 729}], LineBox[{6, 730}], LineBox[{7, 731}], 
         LineBox[{8, 732}], LineBox[{9, 733}], LineBox[{10, 734}], 
         LineBox[{11, 735}], LineBox[{12, 736}], LineBox[{13, 737}], 
         LineBox[{14, 738}], LineBox[{15, 739}], LineBox[{16, 740}], 
         LineBox[{17, 741}], LineBox[{18, 742}], LineBox[{19, 743}], 
         LineBox[{20, 744}], LineBox[{21, 745}], LineBox[{22, 746}], 
         LineBox[{23, 747}], LineBox[{24, 748}], LineBox[{25, 749}], 
         LineBox[{26, 750}], LineBox[{27, 751}], LineBox[{28, 752}], 
         LineBox[{29, 753}], LineBox[{30, 754}], LineBox[{31, 755}], 
         LineBox[{32, 756}], LineBox[{33, 757}], LineBox[{34, 758}], 
         LineBox[{35, 759}], LineBox[{36, 760}], LineBox[{37, 761}], 
         LineBox[{38, 762}], LineBox[{39, 763}], LineBox[{40, 764}], 
         LineBox[{41, 765}], LineBox[{42, 766}], LineBox[{43, 767}], 
         LineBox[{44, 768}], LineBox[{45, 769}], LineBox[{46, 770}], 
         LineBox[{47, 771}], LineBox[{48, 772}], LineBox[{49, 773}], 
         LineBox[{50, 774}], LineBox[{51, 775}], LineBox[{52, 776}], 
         LineBox[{53, 777}], LineBox[{54, 778}], LineBox[{55, 779}], 
         LineBox[{56, 780}], LineBox[{57, 781}], LineBox[{58, 782}], 
         LineBox[{59, 783}], LineBox[{60, 784}], LineBox[{61, 785}], 
         LineBox[{62, 786}], LineBox[{63, 787}], LineBox[{64, 788}], 
         LineBox[{65, 789}], LineBox[{66, 790}], LineBox[{67, 791}], 
         LineBox[{68, 792}], LineBox[{69, 793}], LineBox[{70, 794}], 
         LineBox[{71, 795}], LineBox[{72, 796}], LineBox[{73, 797}], 
         LineBox[{74, 798}], LineBox[{75, 799}], LineBox[{76, 800}], 
         LineBox[{77, 801}], LineBox[{78, 802}], LineBox[{79, 803}], 
         LineBox[{80, 804}], LineBox[{81, 805}], LineBox[{82, 806}], 
         LineBox[{83, 807}], LineBox[{84, 808}], LineBox[{85, 809}], 
         LineBox[{86, 810}], LineBox[{87, 811}], LineBox[{88, 812}], 
         LineBox[{89, 813}], LineBox[{90, 814}], LineBox[{91, 815}], 
         LineBox[{92, 816}], LineBox[{93, 817}], LineBox[{94, 818}], 
         LineBox[{95, 819}], LineBox[{96, 820}], LineBox[{97, 821}], 
         LineBox[{98, 822}], LineBox[{99, 823}], 
         LineBox[{100, 
          824}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[2.5], LineBox[{202, 689}], 
         LineBox[{203, 690}], LineBox[{204, 691}], LineBox[{205, 692}], 
         LineBox[{206, 693}], LineBox[{207, 694}], LineBox[{211, 695}], 
         LineBox[{212, 696}], LineBox[{213, 697}], LineBox[{214, 698}], 
         LineBox[{215, 699}], LineBox[{216, 700}], LineBox[{217, 701}], 
         LineBox[{218, 702}], LineBox[{219, 703}], LineBox[{220, 704}], 
         LineBox[{221, 705}], LineBox[{222, 706}], LineBox[{223, 707}], 
         LineBox[{224, 708}], LineBox[{225, 709}], LineBox[{226, 710}], 
         LineBox[{229, 711}], LineBox[{230, 712}], LineBox[{231, 713}], 
         LineBox[{232, 714}], LineBox[{233, 715}], LineBox[{235, 716}], 
         LineBox[{236, 717}], LineBox[{237, 718}], LineBox[{238, 719}], 
         LineBox[{240, 720}], LineBox[{241, 721}], LineBox[{242, 722}], 
         LineBox[{243, 723}], 
         LineBox[{244, 
          724}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[0., 0.8, 0.], AbsoluteThickness[2.5], LineBox[{295, 644}], 
         LineBox[{349, 645}], LineBox[{297, 646}], LineBox[{351, 647}], 
         LineBox[{299, 648}], LineBox[{353, 649}], LineBox[{303, 650}], 
         LineBox[{357, 651}], LineBox[{358, 652}], LineBox[{359, 653}], 
         LineBox[{360, 654}], LineBox[{361, 655}], LineBox[{310, 656}], 
         LineBox[{364, 657}], LineBox[{365, 658}], LineBox[{314, 659}], 
         LineBox[{368, 660}], LineBox[{369, 661}], LineBox[{370, 662}], 
         LineBox[{371, 663}], LineBox[{372, 664}], LineBox[{373, 665}], 
         LineBox[{322, 666}], LineBox[{376, 667}], LineBox[{325, 668}], 
         LineBox[{379, 669}], LineBox[{327, 670}], LineBox[{381, 671}], 
         LineBox[{382, 672}], LineBox[{383, 673}], LineBox[{384, 674}], 
         LineBox[{332, 675}], LineBox[{386, 676}], LineBox[{387, 677}], 
         LineBox[{335, 678}], LineBox[{389, 679}], LineBox[{338, 680}], 
         LineBox[{392, 681}], LineBox[{340, 682}], LineBox[{394, 683}], 
         LineBox[{395, 684}], LineBox[{396, 685}], LineBox[{397, 686}], 
         LineBox[{398, 687}], LineBox[{399, 688}]}}, {
        {GrayLevel[0], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 301],
          "InsetString"], 
         TagBox[InsetBox["", 302],
          "InsetString"], 
         TagBox[InsetBox["", 309],
          "InsetString"], 
         TagBox[InsetBox["", 313],
          "InsetString"], 
         TagBox[InsetBox["", 321],
          "InsetString"], 
         TagBox[InsetBox["", 324],
          "InsetString"], 
         TagBox[InsetBox["", 337],
          "InsetString"], 
         TagBox[InsetBox["", 347],
          "InsetString"], 
         TagBox[InsetBox["", 354],
          "InsetString"], 
         TagBox[InsetBox["", 355],
          "InsetString"], 
         TagBox[InsetBox["", 362],
          "InsetString"], 
         TagBox[InsetBox["", 366],
          "InsetString"], 
         TagBox[InsetBox["", 374],
          "InsetString"], 
         TagBox[InsetBox["", 377],
          "InsetString"], 
         TagBox[InsetBox["", 390],
          "InsetString"], 
         TagBox[InsetBox["", 400],
          "InsetString"], 
         TagBox[InsetBox["", 825],
          "InsetString"], 
         TagBox[InsetBox["", 826],
          "InsetString"], 
         TagBox[InsetBox["", 827],
          "InsetString"], 
         TagBox[InsetBox["", 828],
          "InsetString"], 
         TagBox[InsetBox["", 829],
          "InsetString"], 
         TagBox[InsetBox["", 830],
          "InsetString"], 
         TagBox[InsetBox["", 831],
          "InsetString"], 
         TagBox[InsetBox["", 832],
          "InsetString"], 
         TagBox[InsetBox["", 833],
          "InsetString"], 
         TagBox[InsetBox["", 834],
          "InsetString"], 
         TagBox[InsetBox["", 835],
          "InsetString"], 
         TagBox[InsetBox["", 836],
          "InsetString"], 
         TagBox[InsetBox["", 837],
          "InsetString"], 
         TagBox[InsetBox["", 838],
          "InsetString"], 
         TagBox[InsetBox["", 839],
          "InsetString"], 
         TagBox[InsetBox["", 840],
          "InsetString"], 
         TagBox[InsetBox["", 841],
          "InsetString"], 
         TagBox[InsetBox["", 842],
          "InsetString"], 
         TagBox[InsetBox["", 843],
          "InsetString"], 
         TagBox[InsetBox["", 844],
          "InsetString"], 
         TagBox[InsetBox["", 845],
          "InsetString"], 
         TagBox[InsetBox["", 846],
          "InsetString"], 
         TagBox[InsetBox["", 847],
          "InsetString"], 
         TagBox[InsetBox["", 848],
          "InsetString"], 
         TagBox[InsetBox["", 849],
          "InsetString"], 
         TagBox[InsetBox["", 850],
          "InsetString"], 
         TagBox[InsetBox["", 851],
          "InsetString"], 
         TagBox[InsetBox["", 852],
          "InsetString"], 
         TagBox[InsetBox["", 853],
          "InsetString"], 
         TagBox[InsetBox["", 854],
          "InsetString"], 
         TagBox[InsetBox["", 855],
          "InsetString"], 
         TagBox[InsetBox["", 856],
          "InsetString"], 
         TagBox[InsetBox["", 857],
          "InsetString"], 
         TagBox[InsetBox["", 858],
          "InsetString"], 
         TagBox[InsetBox["", 859],
          "InsetString"], 
         TagBox[InsetBox["", 860],
          "InsetString"], 
         TagBox[InsetBox["", 861],
          "InsetString"], 
         TagBox[InsetBox["", 862],
          "InsetString"], 
         TagBox[InsetBox["", 863],
          "InsetString"], 
         TagBox[InsetBox["", 864],
          "InsetString"], 
         TagBox[InsetBox["", 865],
          "InsetString"], 
         TagBox[InsetBox["", 866],
          "InsetString"], 
         TagBox[InsetBox["", 867],
          "InsetString"], 
         TagBox[InsetBox["", 868],
          "InsetString"], 
         TagBox[InsetBox["", 869],
          "InsetString"], 
         TagBox[InsetBox["", 870],
          "InsetString"], 
         TagBox[InsetBox["", 871],
          "InsetString"], 
         TagBox[InsetBox["", 872],
          "InsetString"], 
         TagBox[InsetBox["", 873],
          "InsetString"], 
         TagBox[InsetBox["", 874],
          "InsetString"], 
         TagBox[InsetBox["", 875],
          "InsetString"], 
         TagBox[InsetBox["", 876],
          "InsetString"], 
         TagBox[InsetBox["", 877],
          "InsetString"], 
         TagBox[InsetBox["", 878],
          "InsetString"], 
         TagBox[InsetBox["", 879],
          "InsetString"], 
         TagBox[InsetBox["", 880],
          "InsetString"], 
         TagBox[InsetBox["", 881],
          "InsetString"], 
         TagBox[InsetBox["", 882],
          "InsetString"], 
         TagBox[InsetBox["", 883],
          "InsetString"], 
         TagBox[InsetBox["", 884],
          "InsetString"], 
         TagBox[InsetBox["", 885],
          "InsetString"], 
         TagBox[InsetBox["", 886],
          "InsetString"], 
         TagBox[InsetBox["", 887],
          "InsetString"], 
         TagBox[InsetBox["", 888],
          "InsetString"], 
         TagBox[InsetBox["", 889],
          "InsetString"], 
         TagBox[InsetBox["", 890],
          "InsetString"], 
         TagBox[InsetBox["", 891],
          "InsetString"], 
         TagBox[InsetBox["", 892],
          "InsetString"], 
         TagBox[InsetBox["", 893],
          "InsetString"], 
         TagBox[InsetBox["", 894],
          "InsetString"], 
         TagBox[InsetBox["", 895],
          "InsetString"], 
         TagBox[InsetBox["", 896],
          "InsetString"], 
         TagBox[InsetBox["", 897],
          "InsetString"], 
         TagBox[InsetBox["", 898],
          "InsetString"], 
         TagBox[InsetBox["", 899],
          "InsetString"], 
         TagBox[InsetBox["", 900],
          "InsetString"], 
         TagBox[InsetBox["", 901],
          "InsetString"], 
         TagBox[InsetBox["", 902],
          "InsetString"], 
         TagBox[InsetBox["", 903],
          "InsetString"], 
         TagBox[InsetBox["", 904],
          "InsetString"], 
         TagBox[InsetBox["", 905],
          "InsetString"], 
         TagBox[InsetBox["", 906],
          "InsetString"], 
         TagBox[InsetBox["", 907],
          "InsetString"], 
         TagBox[InsetBox["", 908],
          "InsetString"], 
         TagBox[InsetBox["", 909],
          "InsetString"], 
         TagBox[InsetBox["", 910],
          "InsetString"], 
         TagBox[InsetBox["", 911],
          "InsetString"], 
         TagBox[InsetBox["", 912],
          "InsetString"], 
         TagBox[InsetBox["", 913],
          "InsetString"], 
         TagBox[InsetBox["", 914],
          "InsetString"], 
         TagBox[InsetBox["", 915],
          "InsetString"], 
         TagBox[InsetBox["", 916],
          "InsetString"], 
         TagBox[InsetBox["", 917],
          "InsetString"], 
         TagBox[InsetBox["", 918],
          "InsetString"], 
         TagBox[InsetBox["", 919],
          "InsetString"], 
         TagBox[InsetBox["", 920],
          "InsetString"], 
         TagBox[InsetBox["", 921],
          "InsetString"], 
         TagBox[InsetBox["", 922],
          "InsetString"], 
         TagBox[InsetBox["", 923],
          "InsetString"], 
         TagBox[InsetBox["", 924],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 925],
          "InsetString"], 
         TagBox[InsetBox["", 926],
          "InsetString"], 
         TagBox[InsetBox["", 927],
          "InsetString"], 
         TagBox[InsetBox["", 928],
          "InsetString"], 
         TagBox[InsetBox["", 929],
          "InsetString"], 
         TagBox[InsetBox["", 930],
          "InsetString"], 
         TagBox[InsetBox["", 931],
          "InsetString"], 
         TagBox[InsetBox["", 932],
          "InsetString"], 
         TagBox[InsetBox["", 933],
          "InsetString"], 
         TagBox[InsetBox["", 934],
          "InsetString"], 
         TagBox[InsetBox["", 935],
          "InsetString"], 
         TagBox[InsetBox["", 936],
          "InsetString"], 
         TagBox[InsetBox["", 937],
          "InsetString"], 
         TagBox[InsetBox["", 938],
          "InsetString"], 
         TagBox[InsetBox["", 939],
          "InsetString"], 
         TagBox[InsetBox["", 940],
          "InsetString"], 
         TagBox[InsetBox["", 941],
          "InsetString"], 
         TagBox[InsetBox["", 942],
          "InsetString"], 
         TagBox[InsetBox["", 943],
          "InsetString"], 
         TagBox[InsetBox["", 944],
          "InsetString"], 
         TagBox[InsetBox["", 945],
          "InsetString"], 
         TagBox[InsetBox["", 946],
          "InsetString"], 
         TagBox[InsetBox["", 947],
          "InsetString"], 
         TagBox[InsetBox["", 948],
          "InsetString"], 
         TagBox[InsetBox["", 949],
          "InsetString"], 
         TagBox[InsetBox["", 950],
          "InsetString"], 
         TagBox[InsetBox["", 951],
          "InsetString"], 
         TagBox[InsetBox["", 952],
          "InsetString"], 
         TagBox[InsetBox["", 953],
          "InsetString"], 
         TagBox[InsetBox["", 954],
          "InsetString"], 
         TagBox[InsetBox["", 955],
          "InsetString"], 
         TagBox[InsetBox["", 956],
          "InsetString"], 
         TagBox[InsetBox["", 957],
          "InsetString"], 
         TagBox[InsetBox["", 958],
          "InsetString"], 
         TagBox[InsetBox["", 959],
          "InsetString"], 
         TagBox[InsetBox["", 960],
          "InsetString"], 
         TagBox[InsetBox["", 961],
          "InsetString"], 
         TagBox[InsetBox["", 962],
          "InsetString"], 
         TagBox[InsetBox["", 963],
          "InsetString"], 
         TagBox[InsetBox["", 964],
          "InsetString"], 
         TagBox[InsetBox["", 965],
          "InsetString"], 
         TagBox[InsetBox["", 966],
          "InsetString"], 
         TagBox[InsetBox["", 967],
          "InsetString"], 
         TagBox[InsetBox["", 968],
          "InsetString"], 
         TagBox[InsetBox["", 969],
          "InsetString"], 
         TagBox[InsetBox["", 970],
          "InsetString"], 
         TagBox[InsetBox["", 971],
          "InsetString"], 
         TagBox[InsetBox["", 972],
          "InsetString"], 
         TagBox[InsetBox["", 973],
          "InsetString"], 
         TagBox[InsetBox["", 974],
          "InsetString"], 
         TagBox[InsetBox["", 975],
          "InsetString"], 
         TagBox[InsetBox["", 976],
          "InsetString"], 
         TagBox[InsetBox["", 977],
          "InsetString"], 
         TagBox[InsetBox["", 978],
          "InsetString"], 
         TagBox[InsetBox["", 979],
          "InsetString"], 
         TagBox[InsetBox["", 980],
          "InsetString"], 
         TagBox[InsetBox["", 981],
          "InsetString"], 
         TagBox[InsetBox["", 982],
          "InsetString"], 
         TagBox[InsetBox["", 983],
          "InsetString"], 
         TagBox[InsetBox["", 984],
          "InsetString"], 
         TagBox[InsetBox["", 985],
          "InsetString"], 
         TagBox[InsetBox["", 986],
          "InsetString"], 
         TagBox[InsetBox["", 987],
          "InsetString"], 
         TagBox[InsetBox["", 988],
          "InsetString"], 
         TagBox[InsetBox["", 989],
          "InsetString"], 
         TagBox[InsetBox["", 990],
          "InsetString"], 
         TagBox[InsetBox["", 991],
          "InsetString"], 
         TagBox[InsetBox["", 992],
          "InsetString"], 
         TagBox[InsetBox["", 993],
          "InsetString"], 
         TagBox[InsetBox["", 994],
          "InsetString"], 
         TagBox[InsetBox["", 995],
          "InsetString"], 
         TagBox[InsetBox["", 996],
          "InsetString"], 
         TagBox[InsetBox["", 997],
          "InsetString"], 
         TagBox[InsetBox["", 998],
          "InsetString"], 
         TagBox[InsetBox["", 999],
          "InsetString"], 
         TagBox[InsetBox["", 1000],
          "InsetString"], 
         TagBox[InsetBox["", 1001],
          "InsetString"], 
         TagBox[InsetBox["", 1002],
          "InsetString"], 
         TagBox[InsetBox["", 1003],
          "InsetString"], 
         TagBox[InsetBox["", 1004],
          "InsetString"], 
         TagBox[InsetBox["", 1005],
          "InsetString"], 
         TagBox[InsetBox["", 1006],
          "InsetString"], 
         TagBox[InsetBox["", 1007],
          "InsetString"], 
         TagBox[InsetBox["", 1008],
          "InsetString"], 
         TagBox[InsetBox["", 1009],
          "InsetString"], 
         TagBox[InsetBox["", 1010],
          "InsetString"], 
         TagBox[InsetBox["", 1011],
          "InsetString"], 
         TagBox[InsetBox["", 1012],
          "InsetString"], 
         TagBox[InsetBox["", 1013],
          "InsetString"], 
         TagBox[InsetBox["", 1014],
          "InsetString"], 
         TagBox[InsetBox["", 1015],
          "InsetString"], 
         TagBox[InsetBox["", 1016],
          "InsetString"], 
         TagBox[InsetBox["", 1017],
          "InsetString"], 
         TagBox[InsetBox["", 1018],
          "InsetString"], 
         TagBox[InsetBox["", 1019],
          "InsetString"], 
         TagBox[InsetBox["", 1020],
          "InsetString"], 
         TagBox[InsetBox["", 1021],
          "InsetString"], 
         TagBox[InsetBox["", 1022],
          "InsetString"], 
         TagBox[InsetBox["", 1023],
          "InsetString"], 
         TagBox[InsetBox["", 1024],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1025],
          "InsetString"], 
         TagBox[InsetBox["", 1026],
          "InsetString"], 
         TagBox[InsetBox["", 1027],
          "InsetString"], 
         TagBox[InsetBox["", 1028],
          "InsetString"], 
         TagBox[InsetBox["", 1029],
          "InsetString"], 
         TagBox[InsetBox["", 1030],
          "InsetString"], 
         TagBox[InsetBox["", 1031],
          "InsetString"], 
         TagBox[InsetBox["", 1032],
          "InsetString"], 
         TagBox[InsetBox["", 1033],
          "InsetString"], 
         TagBox[InsetBox["", 1034],
          "InsetString"], 
         TagBox[InsetBox["", 1035],
          "InsetString"], 
         TagBox[InsetBox["", 1036],
          "InsetString"], 
         TagBox[InsetBox["", 1037],
          "InsetString"], 
         TagBox[InsetBox["", 1038],
          "InsetString"], 
         TagBox[InsetBox["", 1039],
          "InsetString"], 
         TagBox[InsetBox["", 1040],
          "InsetString"], 
         TagBox[InsetBox["", 1041],
          "InsetString"], 
         TagBox[InsetBox["", 1042],
          "InsetString"], 
         TagBox[InsetBox["", 1043],
          "InsetString"], 
         TagBox[InsetBox["", 1044],
          "InsetString"], 
         TagBox[InsetBox["", 1045],
          "InsetString"], 
         TagBox[InsetBox["", 1046],
          "InsetString"], 
         TagBox[InsetBox["", 1047],
          "InsetString"], 
         TagBox[InsetBox["", 1048],
          "InsetString"], 
         TagBox[InsetBox["", 1049],
          "InsetString"], 
         TagBox[InsetBox["", 1050],
          "InsetString"], 
         TagBox[InsetBox["", 1051],
          "InsetString"], 
         TagBox[InsetBox["", 1052],
          "InsetString"], 
         TagBox[InsetBox["", 1053],
          "InsetString"], 
         TagBox[InsetBox["", 1054],
          "InsetString"], 
         TagBox[InsetBox["", 1055],
          "InsetString"], 
         TagBox[InsetBox["", 1056],
          "InsetString"], 
         TagBox[InsetBox["", 1057],
          "InsetString"], 
         TagBox[InsetBox["", 1058],
          "InsetString"], 
         TagBox[InsetBox["", 1059],
          "InsetString"], 
         TagBox[InsetBox["", 1060],
          "InsetString"], 
         TagBox[InsetBox["", 1061],
          "InsetString"], 
         TagBox[InsetBox["", 1062],
          "InsetString"], 
         TagBox[InsetBox["", 1063],
          "InsetString"], 
         TagBox[InsetBox["", 1064],
          "InsetString"], 
         TagBox[InsetBox["", 1065],
          "InsetString"], 
         TagBox[InsetBox["", 1066],
          "InsetString"], 
         TagBox[InsetBox["", 1067],
          "InsetString"], 
         TagBox[InsetBox["", 1068],
          "InsetString"], 
         TagBox[InsetBox["", 1069],
          "InsetString"], 
         TagBox[InsetBox["", 1070],
          "InsetString"], 
         TagBox[InsetBox["", 1071],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1072],
          "InsetString"], 
         TagBox[InsetBox["", 1073],
          "InsetString"], 
         TagBox[InsetBox["", 1074],
          "InsetString"], 
         TagBox[InsetBox["", 1075],
          "InsetString"], 
         TagBox[InsetBox["", 1076],
          "InsetString"], 
         TagBox[InsetBox["", 1077],
          "InsetString"], 
         TagBox[InsetBox["", 1078],
          "InsetString"], 
         TagBox[InsetBox["", 1079],
          "InsetString"], 
         TagBox[InsetBox["", 1080],
          "InsetString"], 
         TagBox[InsetBox["", 1081],
          "InsetString"], 
         TagBox[InsetBox["", 1082],
          "InsetString"], 
         TagBox[InsetBox["", 1083],
          "InsetString"], 
         TagBox[InsetBox["", 1084],
          "InsetString"], 
         TagBox[InsetBox["", 1085],
          "InsetString"], 
         TagBox[InsetBox["", 1086],
          "InsetString"], 
         TagBox[InsetBox["", 1087],
          "InsetString"], 
         TagBox[InsetBox["", 1088],
          "InsetString"], 
         TagBox[InsetBox["", 1089],
          "InsetString"], 
         TagBox[InsetBox["", 1090],
          "InsetString"], 
         TagBox[InsetBox["", 1091],
          "InsetString"], 
         TagBox[InsetBox["", 1092],
          "InsetString"], 
         TagBox[InsetBox["", 1093],
          "InsetString"], 
         TagBox[InsetBox["", 1094],
          "InsetString"], 
         TagBox[InsetBox["", 1095],
          "InsetString"], 
         TagBox[InsetBox["", 1096],
          "InsetString"], 
         TagBox[InsetBox["", 1097],
          "InsetString"], 
         TagBox[InsetBox["", 1098],
          "InsetString"], 
         TagBox[InsetBox["", 1099],
          "InsetString"], 
         TagBox[InsetBox["", 1100],
          "InsetString"], 
         TagBox[InsetBox["", 1101],
          "InsetString"], 
         TagBox[InsetBox["", 1102],
          "InsetString"], 
         TagBox[InsetBox["", 1103],
          "InsetString"], 
         TagBox[InsetBox["", 1104],
          "InsetString"], 
         TagBox[InsetBox["", 1105],
          "InsetString"], 
         TagBox[InsetBox["", 1106],
          "InsetString"], 
         TagBox[InsetBox["", 1107],
          "InsetString"], 
         TagBox[InsetBox["", 1108],
          "InsetString"], 
         TagBox[InsetBox["", 1109],
          "InsetString"], 
         TagBox[InsetBox["", 1110],
          "InsetString"], 
         TagBox[InsetBox["", 1111],
          "InsetString"], 
         TagBox[InsetBox["", 1112],
          "InsetString"], 
         TagBox[InsetBox["", 1113],
          "InsetString"], 
         TagBox[InsetBox["", 1114],
          "InsetString"], 
         TagBox[InsetBox["", 1115],
          "InsetString"], 
         TagBox[InsetBox["", 1116],
          "InsetString"], 
         TagBox[InsetBox["", 1117],
          "InsetString"], 
         TagBox[InsetBox["", 1118],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1119],
          "InsetString"], 
         TagBox[InsetBox["", 1120],
          "InsetString"], 
         TagBox[InsetBox["", 1121],
          "InsetString"], 
         TagBox[InsetBox["", 1122],
          "InsetString"], 
         TagBox[InsetBox["", 1123],
          "InsetString"], 
         TagBox[InsetBox["", 1124],
          "InsetString"], 
         TagBox[InsetBox["", 1125],
          "InsetString"], 
         TagBox[InsetBox["", 1126],
          "InsetString"], 
         TagBox[InsetBox["", 1127],
          "InsetString"], 
         TagBox[InsetBox["", 1128],
          "InsetString"], 
         TagBox[InsetBox["", 1129],
          "InsetString"], 
         TagBox[InsetBox["", 1130],
          "InsetString"], 
         TagBox[InsetBox["", 1131],
          "InsetString"], 
         TagBox[InsetBox["", 1132],
          "InsetString"], 
         TagBox[InsetBox["", 1133],
          "InsetString"], 
         TagBox[InsetBox["", 1134],
          "InsetString"], 
         TagBox[InsetBox["", 1135],
          "InsetString"], 
         TagBox[InsetBox["", 1136],
          "InsetString"], 
         TagBox[InsetBox["", 1137],
          "InsetString"], 
         TagBox[InsetBox["", 1138],
          "InsetString"], 
         TagBox[InsetBox["", 1139],
          "InsetString"], 
         TagBox[InsetBox["", 1140],
          "InsetString"], 
         TagBox[InsetBox["", 1141],
          "InsetString"], 
         TagBox[InsetBox["", 1142],
          "InsetString"], 
         TagBox[InsetBox["", 1143],
          "InsetString"], 
         TagBox[InsetBox["", 1144],
          "InsetString"], 
         TagBox[InsetBox["", 1145],
          "InsetString"], 
         TagBox[InsetBox["", 1146],
          "InsetString"], 
         TagBox[InsetBox["", 1147],
          "InsetString"], 
         TagBox[InsetBox["", 1148],
          "InsetString"], 
         TagBox[InsetBox["", 1149],
          "InsetString"], 
         TagBox[InsetBox["", 1150],
          "InsetString"], 
         TagBox[InsetBox["", 1151],
          "InsetString"], 
         TagBox[InsetBox["", 1152],
          "InsetString"], 
         TagBox[InsetBox["", 1153],
          "InsetString"], 
         TagBox[InsetBox["", 1154],
          "InsetString"], 
         TagBox[InsetBox["", 1155],
          "InsetString"], 
         TagBox[InsetBox["", 1156],
          "InsetString"], 
         TagBox[InsetBox["", 1157],
          "InsetString"], 
         TagBox[InsetBox["", 1158],
          "InsetString"], 
         TagBox[InsetBox["", 1159],
          "InsetString"], 
         TagBox[InsetBox["", 1160],
          "InsetString"], 
         TagBox[InsetBox["", 1161],
          "InsetString"], 
         TagBox[InsetBox["", 1162],
          "InsetString"], 
         TagBox[InsetBox["", 1163],
          "InsetString"], 
         TagBox[InsetBox["", 1164],
          "InsetString"], 
         TagBox[InsetBox["", 1165],
          "InsetString"], 
         TagBox[InsetBox["", 1166],
          "InsetString"], 
         TagBox[InsetBox["", 1167],
          "InsetString"], 
         TagBox[InsetBox["", 1168],
          "InsetString"], 
         TagBox[InsetBox["", 1169],
          "InsetString"], 
         TagBox[InsetBox["", 1170],
          "InsetString"], 
         TagBox[InsetBox["", 1171],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 1172],
          "InsetString"], 
         TagBox[InsetBox["", 1173],
          "InsetString"], 
         TagBox[InsetBox["", 1174],
          "InsetString"], 
         TagBox[InsetBox["", 1175],
          "InsetString"], 
         TagBox[InsetBox["", 1176],
          "InsetString"], 
         TagBox[InsetBox["", 1177],
          "InsetString"], 
         TagBox[InsetBox["", 1178],
          "InsetString"], 
         TagBox[InsetBox["", 1179],
          "InsetString"], 
         TagBox[InsetBox["", 1180],
          "InsetString"], 
         TagBox[InsetBox["", 1181],
          "InsetString"], 
         TagBox[InsetBox["", 1182],
          "InsetString"], 
         TagBox[InsetBox["", 1183],
          "InsetString"], 
         TagBox[InsetBox["", 1184],
          "InsetString"], 
         TagBox[InsetBox["", 1185],
          "InsetString"], 
         TagBox[InsetBox["", 1186],
          "InsetString"], 
         TagBox[InsetBox["", 1187],
          "InsetString"], 
         TagBox[InsetBox["", 1188],
          "InsetString"], 
         TagBox[InsetBox["", 1189],
          "InsetString"], 
         TagBox[InsetBox["", 1190],
          "InsetString"], 
         TagBox[InsetBox["", 1191],
          "InsetString"], 
         TagBox[InsetBox["", 1192],
          "InsetString"], 
         TagBox[InsetBox["", 1193],
          "InsetString"], 
         TagBox[InsetBox["", 1194],
          "InsetString"], 
         TagBox[InsetBox["", 1195],
          "InsetString"], 
         TagBox[InsetBox["", 1196],
          "InsetString"], 
         TagBox[InsetBox["", 1197],
          "InsetString"], 
         TagBox[InsetBox["", 1198],
          "InsetString"], 
         TagBox[InsetBox["", 1199],
          "InsetString"], 
         TagBox[InsetBox["", 1200],
          "InsetString"], 
         TagBox[InsetBox["", 1201],
          "InsetString"], 
         TagBox[InsetBox["", 1202],
          "InsetString"], 
         TagBox[InsetBox["", 1203],
          "InsetString"], 
         TagBox[InsetBox["", 1204],
          "InsetString"], 
         TagBox[InsetBox["", 1205],
          "InsetString"], 
         TagBox[InsetBox["", 1206],
          "InsetString"], 
         TagBox[InsetBox["", 1207],
          "InsetString"], 
         TagBox[InsetBox["", 1208],
          "InsetString"], 
         TagBox[InsetBox["", 1209],
          "InsetString"], 
         TagBox[InsetBox["", 1210],
          "InsetString"], 
         TagBox[InsetBox["", 1211],
          "InsetString"], 
         TagBox[InsetBox["", 1212],
          "InsetString"], 
         TagBox[InsetBox["", 1213],
          "InsetString"], 
         TagBox[InsetBox["", 1214],
          "InsetString"], 
         TagBox[InsetBox["", 1215],
          "InsetString"], 
         TagBox[InsetBox["", 1216],
          "InsetString"], 
         TagBox[InsetBox["", 1217],
          "InsetString"], 
         TagBox[InsetBox["", 1218],
          "InsetString"], 
         TagBox[InsetBox["", 1219],
          "InsetString"], 
         TagBox[InsetBox["", 1220],
          "InsetString"], 
         TagBox[InsetBox["", 1221],
          "InsetString"], 
         TagBox[InsetBox["", 1222],
          "InsetString"], 
         TagBox[InsetBox["", 1223],
          "InsetString"], 
         TagBox[InsetBox["", 1224],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 401],
          "InsetString"], 
         TagBox[InsetBox["", 402],
          "InsetString"], 
         TagBox[InsetBox["", 403],
          "InsetString"], 
         TagBox[InsetBox["", 404],
          "InsetString"], 
         TagBox[InsetBox["", 405],
          "InsetString"], 
         TagBox[InsetBox["", 406],
          "InsetString"], 
         TagBox[InsetBox["", 407],
          "InsetString"], 
         TagBox[InsetBox["", 408],
          "InsetString"], 
         TagBox[InsetBox["", 409],
          "InsetString"], 
         TagBox[InsetBox["", 410],
          "InsetString"], 
         TagBox[InsetBox["", 411],
          "InsetString"], 
         TagBox[InsetBox["", 412],
          "InsetString"], 
         TagBox[InsetBox["", 413],
          "InsetString"], 
         TagBox[InsetBox["", 414],
          "InsetString"], 
         TagBox[InsetBox["", 415],
          "InsetString"], 
         TagBox[InsetBox["", 416],
          "InsetString"], 
         TagBox[InsetBox["", 417],
          "InsetString"], 
         TagBox[InsetBox["", 418],
          "InsetString"], 
         TagBox[InsetBox["", 419],
          "InsetString"], 
         TagBox[InsetBox["", 420],
          "InsetString"], 
         TagBox[InsetBox["", 421],
          "InsetString"], 
         TagBox[InsetBox["", 422],
          "InsetString"], 
         TagBox[InsetBox["", 423],
          "InsetString"], 
         TagBox[InsetBox["", 424],
          "InsetString"], 
         TagBox[InsetBox["", 425],
          "InsetString"], 
         TagBox[InsetBox["", 426],
          "InsetString"], 
         TagBox[InsetBox["", 427],
          "InsetString"], 
         TagBox[InsetBox["", 428],
          "InsetString"], 
         TagBox[InsetBox["", 429],
          "InsetString"], 
         TagBox[InsetBox["", 430],
          "InsetString"], 
         TagBox[InsetBox["", 431],
          "InsetString"], 
         TagBox[InsetBox["", 432],
          "InsetString"], 
         TagBox[InsetBox["", 433],
          "InsetString"], 
         TagBox[InsetBox["", 434],
          "InsetString"], 
         TagBox[InsetBox["", 435],
          "InsetString"], 
         TagBox[InsetBox["", 436],
          "InsetString"], 
         TagBox[InsetBox["", 437],
          "InsetString"], 
         TagBox[InsetBox["", 438],
          "InsetString"], 
         TagBox[InsetBox["", 439],
          "InsetString"], 
         TagBox[InsetBox["", 440],
          "InsetString"], 
         TagBox[InsetBox["", 441],
          "InsetString"], 
         TagBox[InsetBox["", 442],
          "InsetString"], 
         TagBox[InsetBox["", 443],
          "InsetString"], 
         TagBox[InsetBox["", 444],
          "InsetString"], 
         TagBox[InsetBox["", 445],
          "InsetString"], 
         TagBox[InsetBox["", 446],
          "InsetString"], 
         TagBox[InsetBox["", 447],
          "InsetString"], 
         TagBox[InsetBox["", 448],
          "InsetString"], 
         TagBox[InsetBox["", 449],
          "InsetString"], 
         TagBox[InsetBox["", 450],
          "InsetString"], 
         TagBox[InsetBox["", 451],
          "InsetString"], 
         TagBox[InsetBox["", 452],
          "InsetString"], 
         TagBox[InsetBox["", 453],
          "InsetString"], 
         TagBox[InsetBox["", 454],
          "InsetString"], 
         TagBox[InsetBox["", 455],
          "InsetString"], 
         TagBox[InsetBox["", 456],
          "InsetString"], 
         TagBox[InsetBox["", 457],
          "InsetString"], 
         TagBox[InsetBox["", 458],
          "InsetString"], 
         TagBox[InsetBox["", 459],
          "InsetString"], 
         TagBox[InsetBox["", 460],
          "InsetString"], 
         TagBox[InsetBox["", 461],
          "InsetString"], 
         TagBox[InsetBox["", 462],
          "InsetString"], 
         TagBox[InsetBox["", 463],
          "InsetString"], 
         TagBox[InsetBox["", 464],
          "InsetString"], 
         TagBox[InsetBox["", 465],
          "InsetString"], 
         TagBox[InsetBox["", 466],
          "InsetString"], 
         TagBox[InsetBox["", 467],
          "InsetString"], 
         TagBox[InsetBox["", 468],
          "InsetString"], 
         TagBox[InsetBox["", 469],
          "InsetString"], 
         TagBox[InsetBox["", 470],
          "InsetString"], 
         TagBox[InsetBox["", 471],
          "InsetString"], 
         TagBox[InsetBox["", 472],
          "InsetString"], 
         TagBox[InsetBox["", 473],
          "InsetString"], 
         TagBox[InsetBox["", 474],
          "InsetString"], 
         TagBox[InsetBox["", 475],
          "InsetString"], 
         TagBox[InsetBox["", 476],
          "InsetString"], 
         TagBox[InsetBox["", 477],
          "InsetString"], 
         TagBox[InsetBox["", 478],
          "InsetString"], 
         TagBox[InsetBox["", 479],
          "InsetString"], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 481],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 482],
          "InsetString"], 
         TagBox[InsetBox["", 483],
          "InsetString"], 
         TagBox[InsetBox["", 484],
          "InsetString"], 
         TagBox[InsetBox["", 485],
          "InsetString"], 
         TagBox[InsetBox["", 486],
          "InsetString"], 
         TagBox[InsetBox["", 487],
          "InsetString"], 
         TagBox[InsetBox["", 488],
          "InsetString"], 
         TagBox[InsetBox["", 489],
          "InsetString"], 
         TagBox[InsetBox["", 490],
          "InsetString"], 
         TagBox[InsetBox["", 491],
          "InsetString"], 
         TagBox[InsetBox["", 492],
          "InsetString"], 
         TagBox[InsetBox["", 493],
          "InsetString"], 
         TagBox[InsetBox["", 494],
          "InsetString"], 
         TagBox[InsetBox["", 495],
          "InsetString"], 
         TagBox[InsetBox["", 496],
          "InsetString"], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"], 
         TagBox[InsetBox["", 601],
          "InsetString"], 
         TagBox[InsetBox["", 602],
          "InsetString"], 
         TagBox[InsetBox["", 603],
          "InsetString"], 
         TagBox[InsetBox["", 604],
          "InsetString"], 
         TagBox[InsetBox["", 605],
          "InsetString"], 
         TagBox[InsetBox["", 606],
          "InsetString"], 
         TagBox[InsetBox["", 607],
          "InsetString"], 
         TagBox[InsetBox["", 608],
          "InsetString"], 
         TagBox[InsetBox["", 609],
          "InsetString"], 
         TagBox[InsetBox["", 610],
          "InsetString"], 
         TagBox[InsetBox["", 611],
          "InsetString"], 
         TagBox[InsetBox["", 612],
          "InsetString"], 
         TagBox[InsetBox["", 613],
          "InsetString"], 
         TagBox[InsetBox["", 614],
          "InsetString"], 
         TagBox[InsetBox["", 615],
          "InsetString"], 
         TagBox[InsetBox["", 616],
          "InsetString"], 
         TagBox[InsetBox["", 617],
          "InsetString"], 
         TagBox[InsetBox["", 618],
          "InsetString"], 
         TagBox[InsetBox["", 619],
          "InsetString"], 
         TagBox[InsetBox["", 620],
          "InsetString"], 
         TagBox[InsetBox["", 621],
          "InsetString"], 
         TagBox[InsetBox["", 622],
          "InsetString"], 
         TagBox[InsetBox["", 623],
          "InsetString"], 
         TagBox[InsetBox["", 624],
          "InsetString"], 
         TagBox[InsetBox["", 625],
          "InsetString"], 
         TagBox[InsetBox["", 626],
          "InsetString"], 
         TagBox[InsetBox["", 627],
          "InsetString"], 
         TagBox[InsetBox["", 628],
          "InsetString"], 
         TagBox[InsetBox["", 629],
          "InsetString"], 
         TagBox[InsetBox["", 630],
          "InsetString"], 
         TagBox[InsetBox["", 631],
          "InsetString"], 
         TagBox[InsetBox["", 632],
          "InsetString"], 
         TagBox[InsetBox["", 633],
          "InsetString"], 
         TagBox[InsetBox["", 634],
          "InsetString"], 
         TagBox[InsetBox["", 635],
          "InsetString"], 
         TagBox[InsetBox["", 636],
          "InsetString"], 
         TagBox[InsetBox["", 637],
          "InsetString"], 
         TagBox[InsetBox["", 638],
          "InsetString"], 
         TagBox[InsetBox["", 639],
          "InsetString"], 
         TagBox[InsetBox["", 640],
          "InsetString"], 
         TagBox[InsetBox["", 641],
          "InsetString"], 
         TagBox[InsetBox["", 642],
          "InsetString"], 
         TagBox[InsetBox["", 643],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.45*^9, 380.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[0, 0, 1], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {373.01368955185296`, 501.98999999999995`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 5},
     Ticks->None], ImageScaled[{0.5, 0.61}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztl2tQVGUYxw+CGOjWei01cw1ryPWCt7zHHi9oNSpKOhtmoiiOSEFFXlFQ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      "], {{{}, {}, {}, {}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{154, 1}], LineBox[{155, 2}],
          LineBox[{156, 3}], LineBox[{157, 4}], LineBox[{158, 5}], 
         LineBox[{159, 6}], LineBox[{160, 7}], LineBox[{161, 8}], 
         LineBox[{162, 9}], LineBox[{163, 10}], LineBox[{164, 11}], 
         LineBox[{165, 12}], LineBox[{166, 13}], LineBox[{167, 14}], 
         LineBox[{168, 15}], LineBox[{169, 16}], LineBox[{170, 17}], 
         LineBox[{171, 18}], LineBox[{172, 19}], LineBox[{173, 20}], 
         LineBox[{174, 21}], LineBox[{175, 22}], LineBox[{176, 23}], 
         LineBox[{177, 24}], LineBox[{178, 25}], LineBox[{179, 26}], 
         LineBox[{180, 27}], LineBox[{181, 28}], LineBox[{182, 29}], 
         LineBox[{183, 30}], LineBox[{184, 31}], LineBox[{185, 32}], 
         LineBox[{186, 33}], LineBox[{187, 34}], LineBox[{188, 35}], 
         LineBox[{189, 36}], LineBox[{190, 37}], LineBox[{191, 38}], 
         LineBox[{192, 39}], LineBox[{193, 40}], LineBox[{194, 41}], 
         LineBox[{195, 42}], LineBox[{196, 43}], LineBox[{197, 44}], 
         LineBox[{198, 45}], LineBox[{199, 46}], LineBox[{200, 47}]}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{101, 48}], 
         LineBox[{102, 49}], LineBox[{103, 50}], LineBox[{104, 51}], 
         LineBox[{105, 52}], LineBox[{106, 53}], LineBox[{107, 54}], 
         LineBox[{108, 55}], LineBox[{109, 56}], LineBox[{110, 57}], 
         LineBox[{111, 58}], LineBox[{112, 59}], LineBox[{113, 60}], 
         LineBox[{114, 61}], LineBox[{115, 62}], LineBox[{116, 63}], 
         LineBox[{117, 64}], LineBox[{118, 65}], LineBox[{119, 66}], 
         LineBox[{120, 67}], LineBox[{121, 68}], LineBox[{122, 69}], 
         LineBox[{123, 70}], LineBox[{124, 71}], LineBox[{125, 72}], 
         LineBox[{126, 73}], LineBox[{127, 74}], LineBox[{128, 75}], 
         LineBox[{129, 76}], LineBox[{130, 77}], LineBox[{131, 78}], 
         LineBox[{132, 79}], LineBox[{133, 80}], LineBox[{134, 81}], 
         LineBox[{135, 82}], LineBox[{136, 83}], LineBox[{137, 84}], 
         LineBox[{138, 85}], LineBox[{139, 86}], LineBox[{140, 87}], 
         LineBox[{141, 88}], LineBox[{142, 89}], LineBox[{143, 90}], 
         LineBox[{144, 91}], LineBox[{145, 92}], LineBox[{146, 93}], 
         LineBox[{147, 94}], LineBox[{148, 95}], LineBox[{149, 96}], 
         LineBox[{150, 97}], LineBox[{151, 98}], LineBox[{152, 99}], 
         LineBox[{153, 100}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 202],
          "InsetString"], 
         TagBox[InsetBox["", 203],
          "InsetString"], 
         TagBox[InsetBox["", 204],
          "InsetString"], 
         TagBox[InsetBox["", 205],
          "InsetString"], 
         TagBox[InsetBox["", 206],
          "InsetString"], 
         TagBox[InsetBox["", 207],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 211],
          "InsetString"], 
         TagBox[InsetBox["", 212],
          "InsetString"], 
         TagBox[InsetBox["", 213],
          "InsetString"], 
         TagBox[InsetBox["", 214],
          "InsetString"], 
         TagBox[InsetBox["", 215],
          "InsetString"], 
         TagBox[InsetBox["", 216],
          "InsetString"], 
         TagBox[InsetBox["", 217],
          "InsetString"], 
         TagBox[InsetBox["", 218],
          "InsetString"], 
         TagBox[InsetBox["", 219],
          "InsetString"], 
         TagBox[InsetBox["", 220],
          "InsetString"], 
         TagBox[InsetBox["", 221],
          "InsetString"], 
         TagBox[InsetBox["", 222],
          "InsetString"], 
         TagBox[InsetBox["", 223],
          "InsetString"], 
         TagBox[InsetBox["", 224],
          "InsetString"], 
         TagBox[InsetBox["", 225],
          "InsetString"], 
         TagBox[InsetBox["", 226],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 229],
          "InsetString"], 
         TagBox[InsetBox["", 230],
          "InsetString"], 
         TagBox[InsetBox["", 231],
          "InsetString"], 
         TagBox[InsetBox["", 232],
          "InsetString"], 
         TagBox[InsetBox["", 233],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 235],
          "InsetString"], 
         TagBox[InsetBox["", 236],
          "InsetString"], 
         TagBox[InsetBox["", 237],
          "InsetString"], 
         TagBox[InsetBox["", 238],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 240],
          "InsetString"], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 242],
          "InsetString"], 
         TagBox[InsetBox["", 243],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 249],
          "InsetString"], 
         TagBox[InsetBox["", 250],
          "InsetString"], 
         TagBox[InsetBox["", 251],
          "InsetString"], 
         TagBox[InsetBox["", 252],
          "InsetString"], 
         TagBox[InsetBox["", 253],
          "InsetString"], 
         TagBox[InsetBox["", 254],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 258],
          "InsetString"], 
         TagBox[InsetBox["", 259],
          "InsetString"], 
         TagBox[InsetBox["", 260],
          "InsetString"], 
         TagBox[InsetBox["", 261],
          "InsetString"], 
         TagBox[InsetBox["", 262],
          "InsetString"], 
         TagBox[InsetBox["", 263],
          "InsetString"], 
         TagBox[InsetBox["", 264],
          "InsetString"], 
         TagBox[InsetBox["", 265],
          "InsetString"], 
         TagBox[InsetBox["", 266],
          "InsetString"], 
         TagBox[InsetBox["", 267],
          "InsetString"], 
         TagBox[InsetBox["", 268],
          "InsetString"], 
         TagBox[InsetBox["", 269],
          "InsetString"], 
         TagBox[InsetBox["", 270],
          "InsetString"], 
         TagBox[InsetBox["", 271],
          "InsetString"], 
         TagBox[InsetBox["", 272],
          "InsetString"], 
         TagBox[InsetBox["", 273],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 276],
          "InsetString"], 
         TagBox[InsetBox["", 277],
          "InsetString"], 
         TagBox[InsetBox["", 278],
          "InsetString"], 
         TagBox[InsetBox["", 279],
          "InsetString"], 
         TagBox[InsetBox["", 280],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 282],
          "InsetString"], 
         TagBox[InsetBox["", 283],
          "InsetString"], 
         TagBox[InsetBox["", 284],
          "InsetString"], 
         TagBox[InsetBox["", 285],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 287],
          "InsetString"], 
         TagBox[InsetBox["", 288],
          "InsetString"], 
         TagBox[InsetBox["", 289],
          "InsetString"], 
         TagBox[InsetBox["", 290],
          "InsetString"], 
         TagBox[InsetBox["", 291],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 296],
          "InsetString"], 
         TagBox[InsetBox["", 297],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.45*^9, 2.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-6\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.11}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 0.74}}]], "Output",
 CellChangeTimes->{3.462724766423937*^9}]
}, Open  ]],

Cell["Styling can be added to the bands.", "Text",
 CellChangeTimes->{{3.4627288940706882`*^9, 3.462728899046636*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StockPlot", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"goog1", ",", 
     RowBox[{"-", "100"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Google\>\"", ",", "Brown"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotType", "\[Rule]", "\"\<Candlestick\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Bollinger", "\[Rule]", " ", "\"\<True\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Technicals", "\[Rule]", "\"\<True\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PriceStyling", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "Green", ",", "Blue"}], "}"}]}], "}"}]}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.462724902835133*^9, 3.4627249857158003`*^9}, {
  3.462728902986371*^9, 3.462728909668625*^9}, {3.46272901415034*^9, 
  3.462729032557625*^9}}],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztW3lYVfXWJjVUnHBCr6UimeIIqTmkJifyKtkgaESOqEg4gg2KCIJDaXpV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      "], {{{}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwNxmk2ggEAAMCvoziSjuAAOgsSkbRI2YkKERFSUUKoLN3E/Jj3ZmomEp4N
BUEwzZzMs0CURWIssUycFVZJsEaSdVKkyZBlgxyb5CmwxTY77LLHPgccckSR
Y04oUabCKWecU+WCS2pccU2dG25pcMc9DzR5pEWbDk8806XHC31eeeOdAR98
8sWQEWO++eGXPyb8A6NrPN8=
          "]]}, 
        {RGBColor[0, 1, 0], LineBox[CompressedData["
1:eJwVzdk2gmEAQNHvLzJUZIoQypAGGd6ER/AA9c7InCSZSuwu9lrn7hQumxeN
KIRwzpW45oYWt9xxzwOPPPFMmxc6vNLljR7v9Pngky+++WHAkF9G/I3nsRAi
YsSZYJIEU0wzwyxJUqSZY54MCyyyxDIrZFlljRzrbLBJni222aFAkV322OeA
EoeUqVClxhF1jjnhlDP+AWrYI7M=
          "]]}, 
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwNw4c6QmEAANCfN+kZshJlRWTUVSlkZCTjXqRE2S/unO87qXYSxRMhhIrp
yRCmnHbGWefMOG/WBRfNmXfJZVdcdc2C625YdNMtS2674657lq0YuW/VmnUP
bNj00COPbXniqWee2/bCS6+8tuONXW+9894HYxMfffLZni/2Hfjq0DffHTn2
w0+//PbHX//8B8gpH3I=
          
          "]]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1], LineBox[{1, 725}], 
         LineBox[{2, 726}], LineBox[{3, 727}], LineBox[{4, 728}], 
         LineBox[{5, 729}], LineBox[{6, 730}], LineBox[{7, 731}], 
         LineBox[{8, 732}], LineBox[{9, 733}], LineBox[{10, 734}], 
         LineBox[{11, 735}], LineBox[{12, 736}], LineBox[{13, 737}], 
         LineBox[{14, 738}], LineBox[{15, 739}], LineBox[{16, 740}], 
         LineBox[{17, 741}], LineBox[{18, 742}], LineBox[{19, 743}], 
         LineBox[{20, 744}], LineBox[{21, 745}], LineBox[{22, 746}], 
         LineBox[{23, 747}], LineBox[{24, 748}], LineBox[{25, 749}], 
         LineBox[{26, 750}], LineBox[{27, 751}], LineBox[{28, 752}], 
         LineBox[{29, 753}], LineBox[{30, 754}], LineBox[{31, 755}], 
         LineBox[{32, 756}], LineBox[{33, 757}], LineBox[{34, 758}], 
         LineBox[{35, 759}], LineBox[{36, 760}], LineBox[{37, 761}], 
         LineBox[{38, 762}], LineBox[{39, 763}], LineBox[{40, 764}], 
         LineBox[{41, 765}], LineBox[{42, 766}], LineBox[{43, 767}], 
         LineBox[{44, 768}], LineBox[{45, 769}], LineBox[{46, 770}], 
         LineBox[{47, 771}], LineBox[{48, 772}], LineBox[{49, 773}], 
         LineBox[{50, 774}], LineBox[{51, 775}], LineBox[{52, 776}], 
         LineBox[{53, 777}], LineBox[{54, 778}], LineBox[{55, 779}], 
         LineBox[{56, 780}], LineBox[{57, 781}], LineBox[{58, 782}], 
         LineBox[{59, 783}], LineBox[{60, 784}], LineBox[{61, 785}], 
         LineBox[{62, 786}], LineBox[{63, 787}], LineBox[{64, 788}], 
         LineBox[{65, 789}], LineBox[{66, 790}], LineBox[{67, 791}], 
         LineBox[{68, 792}], LineBox[{69, 793}], LineBox[{70, 794}], 
         LineBox[{71, 795}], LineBox[{72, 796}], LineBox[{73, 797}], 
         LineBox[{74, 798}], LineBox[{75, 799}], LineBox[{76, 800}], 
         LineBox[{77, 801}], LineBox[{78, 802}], LineBox[{79, 803}], 
         LineBox[{80, 804}], LineBox[{81, 805}], LineBox[{82, 806}], 
         LineBox[{83, 807}], LineBox[{84, 808}], LineBox[{85, 809}], 
         LineBox[{86, 810}], LineBox[{87, 811}], LineBox[{88, 812}], 
         LineBox[{89, 813}], LineBox[{90, 814}], LineBox[{91, 815}], 
         LineBox[{92, 816}], LineBox[{93, 817}], LineBox[{94, 818}], 
         LineBox[{95, 819}], LineBox[{96, 820}], LineBox[{97, 821}], 
         LineBox[{98, 822}], LineBox[{99, 823}], 
         LineBox[{100, 
          824}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[2.5], LineBox[{202, 689}], 
         LineBox[{203, 690}], LineBox[{204, 691}], LineBox[{205, 692}], 
         LineBox[{206, 693}], LineBox[{207, 694}], LineBox[{211, 695}], 
         LineBox[{212, 696}], LineBox[{213, 697}], LineBox[{214, 698}], 
         LineBox[{215, 699}], LineBox[{216, 700}], LineBox[{217, 701}], 
         LineBox[{218, 702}], LineBox[{219, 703}], LineBox[{220, 704}], 
         LineBox[{221, 705}], LineBox[{222, 706}], LineBox[{223, 707}], 
         LineBox[{224, 708}], LineBox[{225, 709}], LineBox[{226, 710}], 
         LineBox[{229, 711}], LineBox[{230, 712}], LineBox[{231, 713}], 
         LineBox[{232, 714}], LineBox[{233, 715}], LineBox[{235, 716}], 
         LineBox[{236, 717}], LineBox[{237, 718}], LineBox[{238, 719}], 
         LineBox[{240, 720}], LineBox[{241, 721}], LineBox[{242, 722}], 
         LineBox[{243, 723}], 
         LineBox[{244, 
          724}]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, 
        {RGBColor[0., 0.8, 0.], AbsoluteThickness[2.5], LineBox[{295, 644}], 
         LineBox[{349, 645}], LineBox[{297, 646}], LineBox[{351, 647}], 
         LineBox[{299, 648}], LineBox[{353, 649}], LineBox[{303, 650}], 
         LineBox[{357, 651}], LineBox[{358, 652}], LineBox[{359, 653}], 
         LineBox[{360, 654}], LineBox[{361, 655}], LineBox[{310, 656}], 
         LineBox[{364, 657}], LineBox[{365, 658}], LineBox[{314, 659}], 
         LineBox[{368, 660}], LineBox[{369, 661}], LineBox[{370, 662}], 
         LineBox[{371, 663}], LineBox[{372, 664}], LineBox[{373, 665}], 
         LineBox[{322, 666}], LineBox[{376, 667}], LineBox[{325, 668}], 
         LineBox[{379, 669}], LineBox[{327, 670}], LineBox[{381, 671}], 
         LineBox[{382, 672}], LineBox[{383, 673}], LineBox[{384, 674}], 
         LineBox[{332, 675}], LineBox[{386, 676}], LineBox[{387, 677}], 
         LineBox[{335, 678}], LineBox[{389, 679}], LineBox[{338, 680}], 
         LineBox[{392, 681}], LineBox[{340, 682}], LineBox[{394, 683}], 
         LineBox[{395, 684}], LineBox[{396, 685}], LineBox[{397, 686}], 
         LineBox[{398, 687}], LineBox[{399, 688}]}}, {
        {RGBColor[1, 0, 0], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 301],
          "InsetString"], 
         TagBox[InsetBox["", 302],
          "InsetString"], 
         TagBox[InsetBox["", 309],
          "InsetString"], 
         TagBox[InsetBox["", 313],
          "InsetString"], 
         TagBox[InsetBox["", 321],
          "InsetString"], 
         TagBox[InsetBox["", 324],
          "InsetString"], 
         TagBox[InsetBox["", 337],
          "InsetString"], 
         TagBox[InsetBox["", 347],
          "InsetString"], 
         TagBox[InsetBox["", 354],
          "InsetString"], 
         TagBox[InsetBox["", 355],
          "InsetString"], 
         TagBox[InsetBox["", 362],
          "InsetString"], 
         TagBox[InsetBox["", 366],
          "InsetString"], 
         TagBox[InsetBox["", 374],
          "InsetString"], 
         TagBox[InsetBox["", 377],
          "InsetString"], 
         TagBox[InsetBox["", 390],
          "InsetString"], 
         TagBox[InsetBox["", 400],
          "InsetString"], 
         TagBox[InsetBox["", 825],
          "InsetString"], 
         TagBox[InsetBox["", 826],
          "InsetString"], 
         TagBox[InsetBox["", 827],
          "InsetString"], 
         TagBox[InsetBox["", 828],
          "InsetString"], 
         TagBox[InsetBox["", 829],
          "InsetString"], 
         TagBox[InsetBox["", 830],
          "InsetString"], 
         TagBox[InsetBox["", 831],
          "InsetString"], 
         TagBox[InsetBox["", 832],
          "InsetString"], 
         TagBox[InsetBox["", 833],
          "InsetString"], 
         TagBox[InsetBox["", 834],
          "InsetString"], 
         TagBox[InsetBox["", 835],
          "InsetString"], 
         TagBox[InsetBox["", 836],
          "InsetString"], 
         TagBox[InsetBox["", 837],
          "InsetString"], 
         TagBox[InsetBox["", 838],
          "InsetString"], 
         TagBox[InsetBox["", 839],
          "InsetString"], 
         TagBox[InsetBox["", 840],
          "InsetString"], 
         TagBox[InsetBox["", 841],
          "InsetString"], 
         TagBox[InsetBox["", 842],
          "InsetString"], 
         TagBox[InsetBox["", 843],
          "InsetString"], 
         TagBox[InsetBox["", 844],
          "InsetString"], 
         TagBox[InsetBox["", 845],
          "InsetString"], 
         TagBox[InsetBox["", 846],
          "InsetString"], 
         TagBox[InsetBox["", 847],
          "InsetString"], 
         TagBox[InsetBox["", 848],
          "InsetString"], 
         TagBox[InsetBox["", 849],
          "InsetString"], 
         TagBox[InsetBox["", 850],
          "InsetString"], 
         TagBox[InsetBox["", 851],
          "InsetString"], 
         TagBox[InsetBox["", 852],
          "InsetString"], 
         TagBox[InsetBox["", 853],
          "InsetString"], 
         TagBox[InsetBox["", 854],
          "InsetString"], 
         TagBox[InsetBox["", 855],
          "InsetString"], 
         TagBox[InsetBox["", 856],
          "InsetString"], 
         TagBox[InsetBox["", 857],
          "InsetString"], 
         TagBox[InsetBox["", 858],
          "InsetString"], 
         TagBox[InsetBox["", 859],
          "InsetString"], 
         TagBox[InsetBox["", 860],
          "InsetString"], 
         TagBox[InsetBox["", 861],
          "InsetString"], 
         TagBox[InsetBox["", 862],
          "InsetString"], 
         TagBox[InsetBox["", 863],
          "InsetString"], 
         TagBox[InsetBox["", 864],
          "InsetString"], 
         TagBox[InsetBox["", 865],
          "InsetString"], 
         TagBox[InsetBox["", 866],
          "InsetString"], 
         TagBox[InsetBox["", 867],
          "InsetString"], 
         TagBox[InsetBox["", 868],
          "InsetString"], 
         TagBox[InsetBox["", 869],
          "InsetString"], 
         TagBox[InsetBox["", 870],
          "InsetString"], 
         TagBox[InsetBox["", 871],
          "InsetString"], 
         TagBox[InsetBox["", 872],
          "InsetString"], 
         TagBox[InsetBox["", 873],
          "InsetString"], 
         TagBox[InsetBox["", 874],
          "InsetString"], 
         TagBox[InsetBox["", 875],
          "InsetString"], 
         TagBox[InsetBox["", 876],
          "InsetString"], 
         TagBox[InsetBox["", 877],
          "InsetString"], 
         TagBox[InsetBox["", 878],
          "InsetString"], 
         TagBox[InsetBox["", 879],
          "InsetString"], 
         TagBox[InsetBox["", 880],
          "InsetString"], 
         TagBox[InsetBox["", 881],
          "InsetString"], 
         TagBox[InsetBox["", 882],
          "InsetString"], 
         TagBox[InsetBox["", 883],
          "InsetString"], 
         TagBox[InsetBox["", 884],
          "InsetString"], 
         TagBox[InsetBox["", 885],
          "InsetString"], 
         TagBox[InsetBox["", 886],
          "InsetString"], 
         TagBox[InsetBox["", 887],
          "InsetString"], 
         TagBox[InsetBox["", 888],
          "InsetString"], 
         TagBox[InsetBox["", 889],
          "InsetString"], 
         TagBox[InsetBox["", 890],
          "InsetString"], 
         TagBox[InsetBox["", 891],
          "InsetString"], 
         TagBox[InsetBox["", 892],
          "InsetString"], 
         TagBox[InsetBox["", 893],
          "InsetString"], 
         TagBox[InsetBox["", 894],
          "InsetString"], 
         TagBox[InsetBox["", 895],
          "InsetString"], 
         TagBox[InsetBox["", 896],
          "InsetString"], 
         TagBox[InsetBox["", 897],
          "InsetString"], 
         TagBox[InsetBox["", 898],
          "InsetString"], 
         TagBox[InsetBox["", 899],
          "InsetString"], 
         TagBox[InsetBox["", 900],
          "InsetString"], 
         TagBox[InsetBox["", 901],
          "InsetString"], 
         TagBox[InsetBox["", 902],
          "InsetString"], 
         TagBox[InsetBox["", 903],
          "InsetString"], 
         TagBox[InsetBox["", 904],
          "InsetString"], 
         TagBox[InsetBox["", 905],
          "InsetString"], 
         TagBox[InsetBox["", 906],
          "InsetString"], 
         TagBox[InsetBox["", 907],
          "InsetString"], 
         TagBox[InsetBox["", 908],
          "InsetString"], 
         TagBox[InsetBox["", 909],
          "InsetString"], 
         TagBox[InsetBox["", 910],
          "InsetString"], 
         TagBox[InsetBox["", 911],
          "InsetString"], 
         TagBox[InsetBox["", 912],
          "InsetString"], 
         TagBox[InsetBox["", 913],
          "InsetString"], 
         TagBox[InsetBox["", 914],
          "InsetString"], 
         TagBox[InsetBox["", 915],
          "InsetString"], 
         TagBox[InsetBox["", 916],
          "InsetString"], 
         TagBox[InsetBox["", 917],
          "InsetString"], 
         TagBox[InsetBox["", 918],
          "InsetString"], 
         TagBox[InsetBox["", 919],
          "InsetString"], 
         TagBox[InsetBox["", 920],
          "InsetString"], 
         TagBox[InsetBox["", 921],
          "InsetString"], 
         TagBox[InsetBox["", 922],
          "InsetString"], 
         TagBox[InsetBox["", 923],
          "InsetString"], 
         TagBox[InsetBox["", 924],
          "InsetString"]}, 
        {RGBColor[0, 1, 0], 
         TagBox[InsetBox["", 925],
          "InsetString"], 
         TagBox[InsetBox["", 926],
          "InsetString"], 
         TagBox[InsetBox["", 927],
          "InsetString"], 
         TagBox[InsetBox["", 928],
          "InsetString"], 
         TagBox[InsetBox["", 929],
          "InsetString"], 
         TagBox[InsetBox["", 930],
          "InsetString"], 
         TagBox[InsetBox["", 931],
          "InsetString"], 
         TagBox[InsetBox["", 932],
          "InsetString"], 
         TagBox[InsetBox["", 933],
          "InsetString"], 
         TagBox[InsetBox["", 934],
          "InsetString"], 
         TagBox[InsetBox["", 935],
          "InsetString"], 
         TagBox[InsetBox["", 936],
          "InsetString"], 
         TagBox[InsetBox["", 937],
          "InsetString"], 
         TagBox[InsetBox["", 938],
          "InsetString"], 
         TagBox[InsetBox["", 939],
          "InsetString"], 
         TagBox[InsetBox["", 940],
          "InsetString"], 
         TagBox[InsetBox["", 941],
          "InsetString"], 
         TagBox[InsetBox["", 942],
          "InsetString"], 
         TagBox[InsetBox["", 943],
          "InsetString"], 
         TagBox[InsetBox["", 944],
          "InsetString"], 
         TagBox[InsetBox["", 945],
          "InsetString"], 
         TagBox[InsetBox["", 946],
          "InsetString"], 
         TagBox[InsetBox["", 947],
          "InsetString"], 
         TagBox[InsetBox["", 948],
          "InsetString"], 
         TagBox[InsetBox["", 949],
          "InsetString"], 
         TagBox[InsetBox["", 950],
          "InsetString"], 
         TagBox[InsetBox["", 951],
          "InsetString"], 
         TagBox[InsetBox["", 952],
          "InsetString"], 
         TagBox[InsetBox["", 953],
          "InsetString"], 
         TagBox[InsetBox["", 954],
          "InsetString"], 
         TagBox[InsetBox["", 955],
          "InsetString"], 
         TagBox[InsetBox["", 956],
          "InsetString"], 
         TagBox[InsetBox["", 957],
          "InsetString"], 
         TagBox[InsetBox["", 958],
          "InsetString"], 
         TagBox[InsetBox["", 959],
          "InsetString"], 
         TagBox[InsetBox["", 960],
          "InsetString"], 
         TagBox[InsetBox["", 961],
          "InsetString"], 
         TagBox[InsetBox["", 962],
          "InsetString"], 
         TagBox[InsetBox["", 963],
          "InsetString"], 
         TagBox[InsetBox["", 964],
          "InsetString"], 
         TagBox[InsetBox["", 965],
          "InsetString"], 
         TagBox[InsetBox["", 966],
          "InsetString"], 
         TagBox[InsetBox["", 967],
          "InsetString"], 
         TagBox[InsetBox["", 968],
          "InsetString"], 
         TagBox[InsetBox["", 969],
          "InsetString"], 
         TagBox[InsetBox["", 970],
          "InsetString"], 
         TagBox[InsetBox["", 971],
          "InsetString"], 
         TagBox[InsetBox["", 972],
          "InsetString"], 
         TagBox[InsetBox["", 973],
          "InsetString"], 
         TagBox[InsetBox["", 974],
          "InsetString"], 
         TagBox[InsetBox["", 975],
          "InsetString"], 
         TagBox[InsetBox["", 976],
          "InsetString"], 
         TagBox[InsetBox["", 977],
          "InsetString"], 
         TagBox[InsetBox["", 978],
          "InsetString"], 
         TagBox[InsetBox["", 979],
          "InsetString"], 
         TagBox[InsetBox["", 980],
          "InsetString"], 
         TagBox[InsetBox["", 981],
          "InsetString"], 
         TagBox[InsetBox["", 982],
          "InsetString"], 
         TagBox[InsetBox["", 983],
          "InsetString"], 
         TagBox[InsetBox["", 984],
          "InsetString"], 
         TagBox[InsetBox["", 985],
          "InsetString"], 
         TagBox[InsetBox["", 986],
          "InsetString"], 
         TagBox[InsetBox["", 987],
          "InsetString"], 
         TagBox[InsetBox["", 988],
          "InsetString"], 
         TagBox[InsetBox["", 989],
          "InsetString"], 
         TagBox[InsetBox["", 990],
          "InsetString"], 
         TagBox[InsetBox["", 991],
          "InsetString"], 
         TagBox[InsetBox["", 992],
          "InsetString"], 
         TagBox[InsetBox["", 993],
          "InsetString"], 
         TagBox[InsetBox["", 994],
          "InsetString"], 
         TagBox[InsetBox["", 995],
          "InsetString"], 
         TagBox[InsetBox["", 996],
          "InsetString"], 
         TagBox[InsetBox["", 997],
          "InsetString"], 
         TagBox[InsetBox["", 998],
          "InsetString"], 
         TagBox[InsetBox["", 999],
          "InsetString"], 
         TagBox[InsetBox["", 1000],
          "InsetString"], 
         TagBox[InsetBox["", 1001],
          "InsetString"], 
         TagBox[InsetBox["", 1002],
          "InsetString"], 
         TagBox[InsetBox["", 1003],
          "InsetString"], 
         TagBox[InsetBox["", 1004],
          "InsetString"], 
         TagBox[InsetBox["", 1005],
          "InsetString"], 
         TagBox[InsetBox["", 1006],
          "InsetString"], 
         TagBox[InsetBox["", 1007],
          "InsetString"], 
         TagBox[InsetBox["", 1008],
          "InsetString"], 
         TagBox[InsetBox["", 1009],
          "InsetString"], 
         TagBox[InsetBox["", 1010],
          "InsetString"], 
         TagBox[InsetBox["", 1011],
          "InsetString"], 
         TagBox[InsetBox["", 1012],
          "InsetString"], 
         TagBox[InsetBox["", 1013],
          "InsetString"], 
         TagBox[InsetBox["", 1014],
          "InsetString"], 
         TagBox[InsetBox["", 1015],
          "InsetString"], 
         TagBox[InsetBox["", 1016],
          "InsetString"], 
         TagBox[InsetBox["", 1017],
          "InsetString"], 
         TagBox[InsetBox["", 1018],
          "InsetString"], 
         TagBox[InsetBox["", 1019],
          "InsetString"], 
         TagBox[InsetBox["", 1020],
          "InsetString"], 
         TagBox[InsetBox["", 1021],
          "InsetString"], 
         TagBox[InsetBox["", 1022],
          "InsetString"], 
         TagBox[InsetBox["", 1023],
          "InsetString"], 
         TagBox[InsetBox["", 1024],
          "InsetString"]}, 
        {RGBColor[0, 0, 1], 
         TagBox[InsetBox["", 1025],
          "InsetString"], 
         TagBox[InsetBox["", 1026],
          "InsetString"], 
         TagBox[InsetBox["", 1027],
          "InsetString"], 
         TagBox[InsetBox["", 1028],
          "InsetString"], 
         TagBox[InsetBox["", 1029],
          "InsetString"], 
         TagBox[InsetBox["", 1030],
          "InsetString"], 
         TagBox[InsetBox["", 1031],
          "InsetString"], 
         TagBox[InsetBox["", 1032],
          "InsetString"], 
         TagBox[InsetBox["", 1033],
          "InsetString"], 
         TagBox[InsetBox["", 1034],
          "InsetString"], 
         TagBox[InsetBox["", 1035],
          "InsetString"], 
         TagBox[InsetBox["", 1036],
          "InsetString"], 
         TagBox[InsetBox["", 1037],
          "InsetString"], 
         TagBox[InsetBox["", 1038],
          "InsetString"], 
         TagBox[InsetBox["", 1039],
          "InsetString"], 
         TagBox[InsetBox["", 1040],
          "InsetString"], 
         TagBox[InsetBox["", 1041],
          "InsetString"], 
         TagBox[InsetBox["", 1042],
          "InsetString"], 
         TagBox[InsetBox["", 1043],
          "InsetString"], 
         TagBox[InsetBox["", 1044],
          "InsetString"], 
         TagBox[InsetBox["", 1045],
          "InsetString"], 
         TagBox[InsetBox["", 1046],
          "InsetString"], 
         TagBox[InsetBox["", 1047],
          "InsetString"], 
         TagBox[InsetBox["", 1048],
          "InsetString"], 
         TagBox[InsetBox["", 1049],
          "InsetString"], 
         TagBox[InsetBox["", 1050],
          "InsetString"], 
         TagBox[InsetBox["", 1051],
          "InsetString"], 
         TagBox[InsetBox["", 1052],
          "InsetString"], 
         TagBox[InsetBox["", 1053],
          "InsetString"], 
         TagBox[InsetBox["", 1054],
          "InsetString"], 
         TagBox[InsetBox["", 1055],
          "InsetString"], 
         TagBox[InsetBox["", 1056],
          "InsetString"], 
         TagBox[InsetBox["", 1057],
          "InsetString"], 
         TagBox[InsetBox["", 1058],
          "InsetString"], 
         TagBox[InsetBox["", 1059],
          "InsetString"], 
         TagBox[InsetBox["", 1060],
          "InsetString"], 
         TagBox[InsetBox["", 1061],
          "InsetString"], 
         TagBox[InsetBox["", 1062],
          "InsetString"], 
         TagBox[InsetBox["", 1063],
          "InsetString"], 
         TagBox[InsetBox["", 1064],
          "InsetString"], 
         TagBox[InsetBox["", 1065],
          "InsetString"], 
         TagBox[InsetBox["", 1066],
          "InsetString"], 
         TagBox[InsetBox["", 1067],
          "InsetString"], 
         TagBox[InsetBox["", 1068],
          "InsetString"], 
         TagBox[InsetBox["", 1069],
          "InsetString"], 
         TagBox[InsetBox["", 1070],
          "InsetString"], 
         TagBox[InsetBox["", 1071],
          "InsetString"]}, 
        {RGBColor[1, 0, 0], 
         TagBox[InsetBox["", 1072],
          "InsetString"], 
         TagBox[InsetBox["", 1073],
          "InsetString"], 
         TagBox[InsetBox["", 1074],
          "InsetString"], 
         TagBox[InsetBox["", 1075],
          "InsetString"], 
         TagBox[InsetBox["", 1076],
          "InsetString"], 
         TagBox[InsetBox["", 1077],
          "InsetString"], 
         TagBox[InsetBox["", 1078],
          "InsetString"], 
         TagBox[InsetBox["", 1079],
          "InsetString"], 
         TagBox[InsetBox["", 1080],
          "InsetString"], 
         TagBox[InsetBox["", 1081],
          "InsetString"], 
         TagBox[InsetBox["", 1082],
          "InsetString"], 
         TagBox[InsetBox["", 1083],
          "InsetString"], 
         TagBox[InsetBox["", 1084],
          "InsetString"], 
         TagBox[InsetBox["", 1085],
          "InsetString"], 
         TagBox[InsetBox["", 1086],
          "InsetString"], 
         TagBox[InsetBox["", 1087],
          "InsetString"], 
         TagBox[InsetBox["", 1088],
          "InsetString"], 
         TagBox[InsetBox["", 1089],
          "InsetString"], 
         TagBox[InsetBox["", 1090],
          "InsetString"], 
         TagBox[InsetBox["", 1091],
          "InsetString"], 
         TagBox[InsetBox["", 1092],
          "InsetString"], 
         TagBox[InsetBox["", 1093],
          "InsetString"], 
         TagBox[InsetBox["", 1094],
          "InsetString"], 
         TagBox[InsetBox["", 1095],
          "InsetString"], 
         TagBox[InsetBox["", 1096],
          "InsetString"], 
         TagBox[InsetBox["", 1097],
          "InsetString"], 
         TagBox[InsetBox["", 1098],
          "InsetString"], 
         TagBox[InsetBox["", 1099],
          "InsetString"], 
         TagBox[InsetBox["", 1100],
          "InsetString"], 
         TagBox[InsetBox["", 1101],
          "InsetString"], 
         TagBox[InsetBox["", 1102],
          "InsetString"], 
         TagBox[InsetBox["", 1103],
          "InsetString"], 
         TagBox[InsetBox["", 1104],
          "InsetString"], 
         TagBox[InsetBox["", 1105],
          "InsetString"], 
         TagBox[InsetBox["", 1106],
          "InsetString"], 
         TagBox[InsetBox["", 1107],
          "InsetString"], 
         TagBox[InsetBox["", 1108],
          "InsetString"], 
         TagBox[InsetBox["", 1109],
          "InsetString"], 
         TagBox[InsetBox["", 1110],
          "InsetString"], 
         TagBox[InsetBox["", 1111],
          "InsetString"], 
         TagBox[InsetBox["", 1112],
          "InsetString"], 
         TagBox[InsetBox["", 1113],
          "InsetString"], 
         TagBox[InsetBox["", 1114],
          "InsetString"], 
         TagBox[InsetBox["", 1115],
          "InsetString"], 
         TagBox[InsetBox["", 1116],
          "InsetString"], 
         TagBox[InsetBox["", 1117],
          "InsetString"], 
         TagBox[InsetBox["", 1118],
          "InsetString"]}, 
        {RGBColor[0, 1, 0], 
         TagBox[InsetBox["", 1119],
          "InsetString"], 
         TagBox[InsetBox["", 1120],
          "InsetString"], 
         TagBox[InsetBox["", 1121],
          "InsetString"], 
         TagBox[InsetBox["", 1122],
          "InsetString"], 
         TagBox[InsetBox["", 1123],
          "InsetString"], 
         TagBox[InsetBox["", 1124],
          "InsetString"], 
         TagBox[InsetBox["", 1125],
          "InsetString"], 
         TagBox[InsetBox["", 1126],
          "InsetString"], 
         TagBox[InsetBox["", 1127],
          "InsetString"], 
         TagBox[InsetBox["", 1128],
          "InsetString"], 
         TagBox[InsetBox["", 1129],
          "InsetString"], 
         TagBox[InsetBox["", 1130],
          "InsetString"], 
         TagBox[InsetBox["", 1131],
          "InsetString"], 
         TagBox[InsetBox["", 1132],
          "InsetString"], 
         TagBox[InsetBox["", 1133],
          "InsetString"], 
         TagBox[InsetBox["", 1134],
          "InsetString"], 
         TagBox[InsetBox["", 1135],
          "InsetString"], 
         TagBox[InsetBox["", 1136],
          "InsetString"], 
         TagBox[InsetBox["", 1137],
          "InsetString"], 
         TagBox[InsetBox["", 1138],
          "InsetString"], 
         TagBox[InsetBox["", 1139],
          "InsetString"], 
         TagBox[InsetBox["", 1140],
          "InsetString"], 
         TagBox[InsetBox["", 1141],
          "InsetString"], 
         TagBox[InsetBox["", 1142],
          "InsetString"], 
         TagBox[InsetBox["", 1143],
          "InsetString"], 
         TagBox[InsetBox["", 1144],
          "InsetString"], 
         TagBox[InsetBox["", 1145],
          "InsetString"], 
         TagBox[InsetBox["", 1146],
          "InsetString"], 
         TagBox[InsetBox["", 1147],
          "InsetString"], 
         TagBox[InsetBox["", 1148],
          "InsetString"], 
         TagBox[InsetBox["", 1149],
          "InsetString"], 
         TagBox[InsetBox["", 1150],
          "InsetString"], 
         TagBox[InsetBox["", 1151],
          "InsetString"], 
         TagBox[InsetBox["", 1152],
          "InsetString"], 
         TagBox[InsetBox["", 1153],
          "InsetString"], 
         TagBox[InsetBox["", 1154],
          "InsetString"], 
         TagBox[InsetBox["", 1155],
          "InsetString"], 
         TagBox[InsetBox["", 1156],
          "InsetString"], 
         TagBox[InsetBox["", 1157],
          "InsetString"], 
         TagBox[InsetBox["", 1158],
          "InsetString"], 
         TagBox[InsetBox["", 1159],
          "InsetString"], 
         TagBox[InsetBox["", 1160],
          "InsetString"], 
         TagBox[InsetBox["", 1161],
          "InsetString"], 
         TagBox[InsetBox["", 1162],
          "InsetString"], 
         TagBox[InsetBox["", 1163],
          "InsetString"], 
         TagBox[InsetBox["", 1164],
          "InsetString"], 
         TagBox[InsetBox["", 1165],
          "InsetString"], 
         TagBox[InsetBox["", 1166],
          "InsetString"], 
         TagBox[InsetBox["", 1167],
          "InsetString"], 
         TagBox[InsetBox["", 1168],
          "InsetString"], 
         TagBox[InsetBox["", 1169],
          "InsetString"], 
         TagBox[InsetBox["", 1170],
          "InsetString"], 
         TagBox[InsetBox["", 1171],
          "InsetString"]}, 
        {RGBColor[0, 0, 1], 
         TagBox[InsetBox["", 1172],
          "InsetString"], 
         TagBox[InsetBox["", 1173],
          "InsetString"], 
         TagBox[InsetBox["", 1174],
          "InsetString"], 
         TagBox[InsetBox["", 1175],
          "InsetString"], 
         TagBox[InsetBox["", 1176],
          "InsetString"], 
         TagBox[InsetBox["", 1177],
          "InsetString"], 
         TagBox[InsetBox["", 1178],
          "InsetString"], 
         TagBox[InsetBox["", 1179],
          "InsetString"], 
         TagBox[InsetBox["", 1180],
          "InsetString"], 
         TagBox[InsetBox["", 1181],
          "InsetString"], 
         TagBox[InsetBox["", 1182],
          "InsetString"], 
         TagBox[InsetBox["", 1183],
          "InsetString"], 
         TagBox[InsetBox["", 1184],
          "InsetString"], 
         TagBox[InsetBox["", 1185],
          "InsetString"], 
         TagBox[InsetBox["", 1186],
          "InsetString"], 
         TagBox[InsetBox["", 1187],
          "InsetString"], 
         TagBox[InsetBox["", 1188],
          "InsetString"], 
         TagBox[InsetBox["", 1189],
          "InsetString"], 
         TagBox[InsetBox["", 1190],
          "InsetString"], 
         TagBox[InsetBox["", 1191],
          "InsetString"], 
         TagBox[InsetBox["", 1192],
          "InsetString"], 
         TagBox[InsetBox["", 1193],
          "InsetString"], 
         TagBox[InsetBox["", 1194],
          "InsetString"], 
         TagBox[InsetBox["", 1195],
          "InsetString"], 
         TagBox[InsetBox["", 1196],
          "InsetString"], 
         TagBox[InsetBox["", 1197],
          "InsetString"], 
         TagBox[InsetBox["", 1198],
          "InsetString"], 
         TagBox[InsetBox["", 1199],
          "InsetString"], 
         TagBox[InsetBox["", 1200],
          "InsetString"], 
         TagBox[InsetBox["", 1201],
          "InsetString"], 
         TagBox[InsetBox["", 1202],
          "InsetString"], 
         TagBox[InsetBox["", 1203],
          "InsetString"], 
         TagBox[InsetBox["", 1204],
          "InsetString"], 
         TagBox[InsetBox["", 1205],
          "InsetString"], 
         TagBox[InsetBox["", 1206],
          "InsetString"], 
         TagBox[InsetBox["", 1207],
          "InsetString"], 
         TagBox[InsetBox["", 1208],
          "InsetString"], 
         TagBox[InsetBox["", 1209],
          "InsetString"], 
         TagBox[InsetBox["", 1210],
          "InsetString"], 
         TagBox[InsetBox["", 1211],
          "InsetString"], 
         TagBox[InsetBox["", 1212],
          "InsetString"], 
         TagBox[InsetBox["", 1213],
          "InsetString"], 
         TagBox[InsetBox["", 1214],
          "InsetString"], 
         TagBox[InsetBox["", 1215],
          "InsetString"], 
         TagBox[InsetBox["", 1216],
          "InsetString"], 
         TagBox[InsetBox["", 1217],
          "InsetString"], 
         TagBox[InsetBox["", 1218],
          "InsetString"], 
         TagBox[InsetBox["", 1219],
          "InsetString"], 
         TagBox[InsetBox["", 1220],
          "InsetString"], 
         TagBox[InsetBox["", 1221],
          "InsetString"], 
         TagBox[InsetBox["", 1222],
          "InsetString"], 
         TagBox[InsetBox["", 1223],
          "InsetString"], 
         TagBox[InsetBox["", 1224],
          "InsetString"]}, 
        {RGBColor[1, 0, 0], 
         TagBox[InsetBox["", 401],
          "InsetString"], 
         TagBox[InsetBox["", 402],
          "InsetString"], 
         TagBox[InsetBox["", 403],
          "InsetString"], 
         TagBox[InsetBox["", 404],
          "InsetString"], 
         TagBox[InsetBox["", 405],
          "InsetString"], 
         TagBox[InsetBox["", 406],
          "InsetString"], 
         TagBox[InsetBox["", 407],
          "InsetString"], 
         TagBox[InsetBox["", 408],
          "InsetString"], 
         TagBox[InsetBox["", 409],
          "InsetString"], 
         TagBox[InsetBox["", 410],
          "InsetString"], 
         TagBox[InsetBox["", 411],
          "InsetString"], 
         TagBox[InsetBox["", 412],
          "InsetString"], 
         TagBox[InsetBox["", 413],
          "InsetString"], 
         TagBox[InsetBox["", 414],
          "InsetString"], 
         TagBox[InsetBox["", 415],
          "InsetString"], 
         TagBox[InsetBox["", 416],
          "InsetString"], 
         TagBox[InsetBox["", 417],
          "InsetString"], 
         TagBox[InsetBox["", 418],
          "InsetString"], 
         TagBox[InsetBox["", 419],
          "InsetString"], 
         TagBox[InsetBox["", 420],
          "InsetString"], 
         TagBox[InsetBox["", 421],
          "InsetString"], 
         TagBox[InsetBox["", 422],
          "InsetString"], 
         TagBox[InsetBox["", 423],
          "InsetString"], 
         TagBox[InsetBox["", 424],
          "InsetString"], 
         TagBox[InsetBox["", 425],
          "InsetString"], 
         TagBox[InsetBox["", 426],
          "InsetString"], 
         TagBox[InsetBox["", 427],
          "InsetString"], 
         TagBox[InsetBox["", 428],
          "InsetString"], 
         TagBox[InsetBox["", 429],
          "InsetString"], 
         TagBox[InsetBox["", 430],
          "InsetString"], 
         TagBox[InsetBox["", 431],
          "InsetString"], 
         TagBox[InsetBox["", 432],
          "InsetString"], 
         TagBox[InsetBox["", 433],
          "InsetString"], 
         TagBox[InsetBox["", 434],
          "InsetString"], 
         TagBox[InsetBox["", 435],
          "InsetString"], 
         TagBox[InsetBox["", 436],
          "InsetString"], 
         TagBox[InsetBox["", 437],
          "InsetString"], 
         TagBox[InsetBox["", 438],
          "InsetString"], 
         TagBox[InsetBox["", 439],
          "InsetString"], 
         TagBox[InsetBox["", 440],
          "InsetString"], 
         TagBox[InsetBox["", 441],
          "InsetString"], 
         TagBox[InsetBox["", 442],
          "InsetString"], 
         TagBox[InsetBox["", 443],
          "InsetString"], 
         TagBox[InsetBox["", 444],
          "InsetString"], 
         TagBox[InsetBox["", 445],
          "InsetString"], 
         TagBox[InsetBox["", 446],
          "InsetString"], 
         TagBox[InsetBox["", 447],
          "InsetString"], 
         TagBox[InsetBox["", 448],
          "InsetString"], 
         TagBox[InsetBox["", 449],
          "InsetString"], 
         TagBox[InsetBox["", 450],
          "InsetString"], 
         TagBox[InsetBox["", 451],
          "InsetString"], 
         TagBox[InsetBox["", 452],
          "InsetString"], 
         TagBox[InsetBox["", 453],
          "InsetString"], 
         TagBox[InsetBox["", 454],
          "InsetString"], 
         TagBox[InsetBox["", 455],
          "InsetString"], 
         TagBox[InsetBox["", 456],
          "InsetString"], 
         TagBox[InsetBox["", 457],
          "InsetString"], 
         TagBox[InsetBox["", 458],
          "InsetString"], 
         TagBox[InsetBox["", 459],
          "InsetString"], 
         TagBox[InsetBox["", 460],
          "InsetString"], 
         TagBox[InsetBox["", 461],
          "InsetString"], 
         TagBox[InsetBox["", 462],
          "InsetString"], 
         TagBox[InsetBox["", 463],
          "InsetString"], 
         TagBox[InsetBox["", 464],
          "InsetString"], 
         TagBox[InsetBox["", 465],
          "InsetString"], 
         TagBox[InsetBox["", 466],
          "InsetString"], 
         TagBox[InsetBox["", 467],
          "InsetString"], 
         TagBox[InsetBox["", 468],
          "InsetString"], 
         TagBox[InsetBox["", 469],
          "InsetString"], 
         TagBox[InsetBox["", 470],
          "InsetString"], 
         TagBox[InsetBox["", 471],
          "InsetString"], 
         TagBox[InsetBox["", 472],
          "InsetString"], 
         TagBox[InsetBox["", 473],
          "InsetString"], 
         TagBox[InsetBox["", 474],
          "InsetString"], 
         TagBox[InsetBox["", 475],
          "InsetString"], 
         TagBox[InsetBox["", 476],
          "InsetString"], 
         TagBox[InsetBox["", 477],
          "InsetString"], 
         TagBox[InsetBox["", 478],
          "InsetString"], 
         TagBox[InsetBox["", 479],
          "InsetString"], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 481],
          "InsetString"]}, 
        {RGBColor[0, 1, 0], 
         TagBox[InsetBox["", 482],
          "InsetString"], 
         TagBox[InsetBox["", 483],
          "InsetString"], 
         TagBox[InsetBox["", 484],
          "InsetString"], 
         TagBox[InsetBox["", 485],
          "InsetString"], 
         TagBox[InsetBox["", 486],
          "InsetString"], 
         TagBox[InsetBox["", 487],
          "InsetString"], 
         TagBox[InsetBox["", 488],
          "InsetString"], 
         TagBox[InsetBox["", 489],
          "InsetString"], 
         TagBox[InsetBox["", 490],
          "InsetString"], 
         TagBox[InsetBox["", 491],
          "InsetString"], 
         TagBox[InsetBox["", 492],
          "InsetString"], 
         TagBox[InsetBox["", 493],
          "InsetString"], 
         TagBox[InsetBox["", 494],
          "InsetString"], 
         TagBox[InsetBox["", 495],
          "InsetString"], 
         TagBox[InsetBox["", 496],
          "InsetString"], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"], 
         TagBox[InsetBox["", 500],
          "InsetString"], 
         TagBox[InsetBox["", 501],
          "InsetString"], 
         TagBox[InsetBox["", 502],
          "InsetString"], 
         TagBox[InsetBox["", 503],
          "InsetString"], 
         TagBox[InsetBox["", 504],
          "InsetString"], 
         TagBox[InsetBox["", 505],
          "InsetString"], 
         TagBox[InsetBox["", 506],
          "InsetString"], 
         TagBox[InsetBox["", 507],
          "InsetString"], 
         TagBox[InsetBox["", 508],
          "InsetString"], 
         TagBox[InsetBox["", 509],
          "InsetString"], 
         TagBox[InsetBox["", 510],
          "InsetString"], 
         TagBox[InsetBox["", 511],
          "InsetString"], 
         TagBox[InsetBox["", 512],
          "InsetString"], 
         TagBox[InsetBox["", 513],
          "InsetString"], 
         TagBox[InsetBox["", 514],
          "InsetString"], 
         TagBox[InsetBox["", 515],
          "InsetString"], 
         TagBox[InsetBox["", 516],
          "InsetString"], 
         TagBox[InsetBox["", 517],
          "InsetString"], 
         TagBox[InsetBox["", 518],
          "InsetString"], 
         TagBox[InsetBox["", 519],
          "InsetString"], 
         TagBox[InsetBox["", 520],
          "InsetString"], 
         TagBox[InsetBox["", 521],
          "InsetString"], 
         TagBox[InsetBox["", 522],
          "InsetString"], 
         TagBox[InsetBox["", 523],
          "InsetString"], 
         TagBox[InsetBox["", 524],
          "InsetString"], 
         TagBox[InsetBox["", 525],
          "InsetString"], 
         TagBox[InsetBox["", 526],
          "InsetString"], 
         TagBox[InsetBox["", 527],
          "InsetString"], 
         TagBox[InsetBox["", 528],
          "InsetString"], 
         TagBox[InsetBox["", 529],
          "InsetString"], 
         TagBox[InsetBox["", 530],
          "InsetString"], 
         TagBox[InsetBox["", 531],
          "InsetString"], 
         TagBox[InsetBox["", 532],
          "InsetString"], 
         TagBox[InsetBox["", 533],
          "InsetString"], 
         TagBox[InsetBox["", 534],
          "InsetString"], 
         TagBox[InsetBox["", 535],
          "InsetString"], 
         TagBox[InsetBox["", 536],
          "InsetString"], 
         TagBox[InsetBox["", 537],
          "InsetString"], 
         TagBox[InsetBox["", 538],
          "InsetString"], 
         TagBox[InsetBox["", 539],
          "InsetString"], 
         TagBox[InsetBox["", 540],
          "InsetString"], 
         TagBox[InsetBox["", 541],
          "InsetString"], 
         TagBox[InsetBox["", 542],
          "InsetString"], 
         TagBox[InsetBox["", 543],
          "InsetString"], 
         TagBox[InsetBox["", 544],
          "InsetString"], 
         TagBox[InsetBox["", 545],
          "InsetString"], 
         TagBox[InsetBox["", 546],
          "InsetString"], 
         TagBox[InsetBox["", 547],
          "InsetString"], 
         TagBox[InsetBox["", 548],
          "InsetString"], 
         TagBox[InsetBox["", 549],
          "InsetString"], 
         TagBox[InsetBox["", 550],
          "InsetString"], 
         TagBox[InsetBox["", 551],
          "InsetString"], 
         TagBox[InsetBox["", 552],
          "InsetString"], 
         TagBox[InsetBox["", 553],
          "InsetString"], 
         TagBox[InsetBox["", 554],
          "InsetString"], 
         TagBox[InsetBox["", 555],
          "InsetString"], 
         TagBox[InsetBox["", 556],
          "InsetString"], 
         TagBox[InsetBox["", 557],
          "InsetString"], 
         TagBox[InsetBox["", 558],
          "InsetString"], 
         TagBox[InsetBox["", 559],
          "InsetString"], 
         TagBox[InsetBox["", 560],
          "InsetString"], 
         TagBox[InsetBox["", 561],
          "InsetString"], 
         TagBox[InsetBox["", 562],
          "InsetString"]}, 
        {RGBColor[0, 0, 1], 
         TagBox[InsetBox["", 563],
          "InsetString"], 
         TagBox[InsetBox["", 564],
          "InsetString"], 
         TagBox[InsetBox["", 565],
          "InsetString"], 
         TagBox[InsetBox["", 566],
          "InsetString"], 
         TagBox[InsetBox["", 567],
          "InsetString"], 
         TagBox[InsetBox["", 568],
          "InsetString"], 
         TagBox[InsetBox["", 569],
          "InsetString"], 
         TagBox[InsetBox["", 570],
          "InsetString"], 
         TagBox[InsetBox["", 571],
          "InsetString"], 
         TagBox[InsetBox["", 572],
          "InsetString"], 
         TagBox[InsetBox["", 573],
          "InsetString"], 
         TagBox[InsetBox["", 574],
          "InsetString"], 
         TagBox[InsetBox["", 575],
          "InsetString"], 
         TagBox[InsetBox["", 576],
          "InsetString"], 
         TagBox[InsetBox["", 577],
          "InsetString"], 
         TagBox[InsetBox["", 578],
          "InsetString"], 
         TagBox[InsetBox["", 579],
          "InsetString"], 
         TagBox[InsetBox["", 580],
          "InsetString"], 
         TagBox[InsetBox["", 581],
          "InsetString"], 
         TagBox[InsetBox["", 582],
          "InsetString"], 
         TagBox[InsetBox["", 583],
          "InsetString"], 
         TagBox[InsetBox["", 584],
          "InsetString"], 
         TagBox[InsetBox["", 585],
          "InsetString"], 
         TagBox[InsetBox["", 586],
          "InsetString"], 
         TagBox[InsetBox["", 587],
          "InsetString"], 
         TagBox[InsetBox["", 588],
          "InsetString"], 
         TagBox[InsetBox["", 589],
          "InsetString"], 
         TagBox[InsetBox["", 590],
          "InsetString"], 
         TagBox[InsetBox["", 591],
          "InsetString"], 
         TagBox[InsetBox["", 592],
          "InsetString"], 
         TagBox[InsetBox["", 593],
          "InsetString"], 
         TagBox[InsetBox["", 594],
          "InsetString"], 
         TagBox[InsetBox["", 595],
          "InsetString"], 
         TagBox[InsetBox["", 596],
          "InsetString"], 
         TagBox[InsetBox["", 597],
          "InsetString"], 
         TagBox[InsetBox["", 598],
          "InsetString"], 
         TagBox[InsetBox["", 599],
          "InsetString"], 
         TagBox[InsetBox["", 600],
          "InsetString"], 
         TagBox[InsetBox["", 601],
          "InsetString"], 
         TagBox[InsetBox["", 602],
          "InsetString"], 
         TagBox[InsetBox["", 603],
          "InsetString"], 
         TagBox[InsetBox["", 604],
          "InsetString"], 
         TagBox[InsetBox["", 605],
          "InsetString"], 
         TagBox[InsetBox["", 606],
          "InsetString"], 
         TagBox[InsetBox["", 607],
          "InsetString"], 
         TagBox[InsetBox["", 608],
          "InsetString"], 
         TagBox[InsetBox["", 609],
          "InsetString"], 
         TagBox[InsetBox["", 610],
          "InsetString"], 
         TagBox[InsetBox["", 611],
          "InsetString"], 
         TagBox[InsetBox["", 612],
          "InsetString"], 
         TagBox[InsetBox["", 613],
          "InsetString"], 
         TagBox[InsetBox["", 614],
          "InsetString"], 
         TagBox[InsetBox["", 615],
          "InsetString"], 
         TagBox[InsetBox["", 616],
          "InsetString"], 
         TagBox[InsetBox["", 617],
          "InsetString"], 
         TagBox[InsetBox["", 618],
          "InsetString"], 
         TagBox[InsetBox["", 619],
          "InsetString"], 
         TagBox[InsetBox["", 620],
          "InsetString"], 
         TagBox[InsetBox["", 621],
          "InsetString"], 
         TagBox[InsetBox["", 622],
          "InsetString"], 
         TagBox[InsetBox["", 623],
          "InsetString"], 
         TagBox[InsetBox["", 624],
          "InsetString"], 
         TagBox[InsetBox["", 625],
          "InsetString"], 
         TagBox[InsetBox["", 626],
          "InsetString"], 
         TagBox[InsetBox["", 627],
          "InsetString"], 
         TagBox[InsetBox["", 628],
          "InsetString"], 
         TagBox[InsetBox["", 629],
          "InsetString"], 
         TagBox[InsetBox["", 630],
          "InsetString"], 
         TagBox[InsetBox["", 631],
          "InsetString"], 
         TagBox[InsetBox["", 632],
          "InsetString"], 
         TagBox[InsetBox["", 633],
          "InsetString"], 
         TagBox[InsetBox["", 634],
          "InsetString"], 
         TagBox[InsetBox["", 635],
          "InsetString"], 
         TagBox[InsetBox["", 636],
          "InsetString"], 
         TagBox[InsetBox["", 637],
          "InsetString"], 
         TagBox[InsetBox["", 638],
          "InsetString"], 
         TagBox[InsetBox["", 639],
          "InsetString"], 
         TagBox[InsetBox["", 640],
          "InsetString"], 
         TagBox[InsetBox["", 641],
          "InsetString"], 
         TagBox[InsetBox["", 642],
          "InsetString"], 
         TagBox[InsetBox["", 643],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     AxesOrigin->{3.45*^9, 380.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Google\"", 14, 
               RGBColor[0.6, 0.4, 0.2], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 2}},
     ImageSize->600,
     PlotRange->{All, {373.01368955185296`, 501.98999999999995`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 5},
     Ticks->None], ImageScaled[{0.5, 0.765}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztl2tQVGUYxw+CGOjWei01cw1ryPWCt7zHHi9oNSpKOhtmoiiOSEFFXlFQ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      "], {{{}, {}, {}, {}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{154, 1}], LineBox[{155, 2}],
          LineBox[{156, 3}], LineBox[{157, 4}], LineBox[{158, 5}], 
         LineBox[{159, 6}], LineBox[{160, 7}], LineBox[{161, 8}], 
         LineBox[{162, 9}], LineBox[{163, 10}], LineBox[{164, 11}], 
         LineBox[{165, 12}], LineBox[{166, 13}], LineBox[{167, 14}], 
         LineBox[{168, 15}], LineBox[{169, 16}], LineBox[{170, 17}], 
         LineBox[{171, 18}], LineBox[{172, 19}], LineBox[{173, 20}], 
         LineBox[{174, 21}], LineBox[{175, 22}], LineBox[{176, 23}], 
         LineBox[{177, 24}], LineBox[{178, 25}], LineBox[{179, 26}], 
         LineBox[{180, 27}], LineBox[{181, 28}], LineBox[{182, 29}], 
         LineBox[{183, 30}], LineBox[{184, 31}], LineBox[{185, 32}], 
         LineBox[{186, 33}], LineBox[{187, 34}], LineBox[{188, 35}], 
         LineBox[{189, 36}], LineBox[{190, 37}], LineBox[{191, 38}], 
         LineBox[{192, 39}], LineBox[{193, 40}], LineBox[{194, 41}], 
         LineBox[{195, 42}], LineBox[{196, 43}], LineBox[{197, 44}], 
         LineBox[{198, 45}], LineBox[{199, 46}], LineBox[{200, 47}]}, {}, 
        {GrayLevel[0], Thickness[Large], LineBox[{101, 48}], 
         LineBox[{102, 49}], LineBox[{103, 50}], LineBox[{104, 51}], 
         LineBox[{105, 52}], LineBox[{106, 53}], LineBox[{107, 54}], 
         LineBox[{108, 55}], LineBox[{109, 56}], LineBox[{110, 57}], 
         LineBox[{111, 58}], LineBox[{112, 59}], LineBox[{113, 60}], 
         LineBox[{114, 61}], LineBox[{115, 62}], LineBox[{116, 63}], 
         LineBox[{117, 64}], LineBox[{118, 65}], LineBox[{119, 66}], 
         LineBox[{120, 67}], LineBox[{121, 68}], LineBox[{122, 69}], 
         LineBox[{123, 70}], LineBox[{124, 71}], LineBox[{125, 72}], 
         LineBox[{126, 73}], LineBox[{127, 74}], LineBox[{128, 75}], 
         LineBox[{129, 76}], LineBox[{130, 77}], LineBox[{131, 78}], 
         LineBox[{132, 79}], LineBox[{133, 80}], LineBox[{134, 81}], 
         LineBox[{135, 82}], LineBox[{136, 83}], LineBox[{137, 84}], 
         LineBox[{138, 85}], LineBox[{139, 86}], LineBox[{140, 87}], 
         LineBox[{141, 88}], LineBox[{142, 89}], LineBox[{143, 90}], 
         LineBox[{144, 91}], LineBox[{145, 92}], LineBox[{146, 93}], 
         LineBox[{147, 94}], LineBox[{148, 95}], LineBox[{149, 96}], 
         LineBox[{150, 97}], LineBox[{151, 98}], LineBox[{152, 99}], 
         LineBox[{153, 100}]}}, {
        {Hue[0.67, 0.6, 0.6], 
         TagBox[InsetBox["", 201],
          "InsetString"], 
         TagBox[InsetBox["", 202],
          "InsetString"], 
         TagBox[InsetBox["", 203],
          "InsetString"], 
         TagBox[InsetBox["", 204],
          "InsetString"], 
         TagBox[InsetBox["", 205],
          "InsetString"], 
         TagBox[InsetBox["", 206],
          "InsetString"], 
         TagBox[InsetBox["", 207],
          "InsetString"], 
         TagBox[InsetBox["", 208],
          "InsetString"], 
         TagBox[InsetBox["", 209],
          "InsetString"], 
         TagBox[InsetBox["", 210],
          "InsetString"], 
         TagBox[InsetBox["", 211],
          "InsetString"], 
         TagBox[InsetBox["", 212],
          "InsetString"], 
         TagBox[InsetBox["", 213],
          "InsetString"], 
         TagBox[InsetBox["", 214],
          "InsetString"], 
         TagBox[InsetBox["", 215],
          "InsetString"], 
         TagBox[InsetBox["", 216],
          "InsetString"], 
         TagBox[InsetBox["", 217],
          "InsetString"], 
         TagBox[InsetBox["", 218],
          "InsetString"], 
         TagBox[InsetBox["", 219],
          "InsetString"], 
         TagBox[InsetBox["", 220],
          "InsetString"], 
         TagBox[InsetBox["", 221],
          "InsetString"], 
         TagBox[InsetBox["", 222],
          "InsetString"], 
         TagBox[InsetBox["", 223],
          "InsetString"], 
         TagBox[InsetBox["", 224],
          "InsetString"], 
         TagBox[InsetBox["", 225],
          "InsetString"], 
         TagBox[InsetBox["", 226],
          "InsetString"], 
         TagBox[InsetBox["", 227],
          "InsetString"], 
         TagBox[InsetBox["", 228],
          "InsetString"], 
         TagBox[InsetBox["", 229],
          "InsetString"], 
         TagBox[InsetBox["", 230],
          "InsetString"], 
         TagBox[InsetBox["", 231],
          "InsetString"], 
         TagBox[InsetBox["", 232],
          "InsetString"], 
         TagBox[InsetBox["", 233],
          "InsetString"], 
         TagBox[InsetBox["", 234],
          "InsetString"], 
         TagBox[InsetBox["", 235],
          "InsetString"], 
         TagBox[InsetBox["", 236],
          "InsetString"], 
         TagBox[InsetBox["", 237],
          "InsetString"], 
         TagBox[InsetBox["", 238],
          "InsetString"], 
         TagBox[InsetBox["", 239],
          "InsetString"], 
         TagBox[InsetBox["", 240],
          "InsetString"], 
         TagBox[InsetBox["", 241],
          "InsetString"], 
         TagBox[InsetBox["", 242],
          "InsetString"], 
         TagBox[InsetBox["", 243],
          "InsetString"], 
         TagBox[InsetBox["", 244],
          "InsetString"], 
         TagBox[InsetBox["", 245],
          "InsetString"], 
         TagBox[InsetBox["", 246],
          "InsetString"], 
         TagBox[InsetBox["", 247],
          "InsetString"]}, 
        {Hue[0.9060679774997897, 0.6, 0.6], 
         TagBox[InsetBox["", 248],
          "InsetString"], 
         TagBox[InsetBox["", 249],
          "InsetString"], 
         TagBox[InsetBox["", 250],
          "InsetString"], 
         TagBox[InsetBox["", 251],
          "InsetString"], 
         TagBox[InsetBox["", 252],
          "InsetString"], 
         TagBox[InsetBox["", 253],
          "InsetString"], 
         TagBox[InsetBox["", 254],
          "InsetString"], 
         TagBox[InsetBox["", 255],
          "InsetString"], 
         TagBox[InsetBox["", 256],
          "InsetString"], 
         TagBox[InsetBox["", 257],
          "InsetString"], 
         TagBox[InsetBox["", 258],
          "InsetString"], 
         TagBox[InsetBox["", 259],
          "InsetString"], 
         TagBox[InsetBox["", 260],
          "InsetString"], 
         TagBox[InsetBox["", 261],
          "InsetString"], 
         TagBox[InsetBox["", 262],
          "InsetString"], 
         TagBox[InsetBox["", 263],
          "InsetString"], 
         TagBox[InsetBox["", 264],
          "InsetString"], 
         TagBox[InsetBox["", 265],
          "InsetString"], 
         TagBox[InsetBox["", 266],
          "InsetString"], 
         TagBox[InsetBox["", 267],
          "InsetString"], 
         TagBox[InsetBox["", 268],
          "InsetString"], 
         TagBox[InsetBox["", 269],
          "InsetString"], 
         TagBox[InsetBox["", 270],
          "InsetString"], 
         TagBox[InsetBox["", 271],
          "InsetString"], 
         TagBox[InsetBox["", 272],
          "InsetString"], 
         TagBox[InsetBox["", 273],
          "InsetString"], 
         TagBox[InsetBox["", 274],
          "InsetString"], 
         TagBox[InsetBox["", 275],
          "InsetString"], 
         TagBox[InsetBox["", 276],
          "InsetString"], 
         TagBox[InsetBox["", 277],
          "InsetString"], 
         TagBox[InsetBox["", 278],
          "InsetString"], 
         TagBox[InsetBox["", 279],
          "InsetString"], 
         TagBox[InsetBox["", 280],
          "InsetString"], 
         TagBox[InsetBox["", 281],
          "InsetString"], 
         TagBox[InsetBox["", 282],
          "InsetString"], 
         TagBox[InsetBox["", 283],
          "InsetString"], 
         TagBox[InsetBox["", 284],
          "InsetString"], 
         TagBox[InsetBox["", 285],
          "InsetString"], 
         TagBox[InsetBox["", 286],
          "InsetString"], 
         TagBox[InsetBox["", 287],
          "InsetString"], 
         TagBox[InsetBox["", 288],
          "InsetString"], 
         TagBox[InsetBox["", 289],
          "InsetString"], 
         TagBox[InsetBox["", 290],
          "InsetString"], 
         TagBox[InsetBox["", 291],
          "InsetString"], 
         TagBox[InsetBox["", 292],
          "InsetString"], 
         TagBox[InsetBox["", 293],
          "InsetString"], 
         TagBox[InsetBox["", 294],
          "InsetString"], 
         TagBox[InsetBox["", 295],
          "InsetString"], 
         TagBox[InsetBox["", 296],
          "InsetString"], 
         TagBox[InsetBox["", 297],
          "InsetString"], 
         TagBox[InsetBox["", 298],
          "InsetString"], 
         TagBox[InsetBox["", 299],
          "InsetString"], 
         TagBox[InsetBox["", 300],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->{3.45*^9, 2.},
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"Vol (\[Times] \\!\\(10\\^\\(-6\\)\\))\"", Bold, 
               GrayLevel[0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{Automatic, Automatic}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, Automatic}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, Automatic}],
     GridLinesStyle->Directive[
       GrayLevel[0.9], 
       Dashing[{Small, Small}]],
     ImagePadding->{{50, 10}, {20, 4}},
     ImageSize->600,
     PlotRange->{All, All},
     PlotRangeClipping->True,
     PlotRangePadding->{{200000, 200000}, {0, 1}},
     Ticks->None], ImageScaled[{0.5, 0.465}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmHlYTVsfx0+zEpJ0NUiZuVLIcK+uTsMrc6cMZcpxMxSim1niZLiR4aaS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      "], {{{}, {}, 
        {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
         LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
          18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 
          35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 
          52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 
          69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 
          86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100}]}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[1.], 
         LineBox[{101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 
          113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 
          126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 
          139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 
          152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 
          165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 
          178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 
          191, 192, 193, 194, 195, 196, 197, 198, 199, 
          200}]}, {}, {}, {}, {}, {}, 
        {GrayLevel[0], AbsoluteThickness[1.], LineBox[{308, 209}], 
         LineBox[{309, 210}], LineBox[{310, 211}], LineBox[{311, 212}], 
         LineBox[{312, 213}], LineBox[{313, 214}], LineBox[{314, 215}], 
         LineBox[{315, 216}], LineBox[{330, 231}], LineBox[{331, 232}], 
         LineBox[{332, 233}], LineBox[{333, 234}], LineBox[{334, 235}], 
         LineBox[{335, 236}], LineBox[{336, 237}], LineBox[{337, 238}], 
         LineBox[{338, 239}], LineBox[{339, 240}], LineBox[{340, 241}], 
         LineBox[{341, 242}], LineBox[{342, 243}], LineBox[{343, 244}], 
         LineBox[{344, 245}], LineBox[{345, 246}], LineBox[{346, 247}], 
         LineBox[{347, 248}], LineBox[{348, 249}], LineBox[{349, 250}], 
         LineBox[{374, 275}], LineBox[{375, 276}], LineBox[{376, 277}], 
         LineBox[{377, 278}], LineBox[{378, 279}], LineBox[{384, 285}], 
         LineBox[{385, 286}], LineBox[{386, 287}], LineBox[{387, 288}], 
         LineBox[{388, 289}], LineBox[{389, 290}], LineBox[{390, 291}], 
         LineBox[{391, 292}], LineBox[{392, 293}]}, 
        {GrayLevel[0], AbsoluteThickness[1.], LineBox[{301, 202}], 
         LineBox[{302, 203}], LineBox[{303, 204}], LineBox[{304, 205}], 
         LineBox[{305, 206}], LineBox[{306, 207}], LineBox[{307, 208}], 
         LineBox[{316, 217}], LineBox[{317, 218}], LineBox[{318, 219}], 
         LineBox[{319, 220}], LineBox[{320, 221}], LineBox[{321, 222}], 
         LineBox[{322, 223}], LineBox[{323, 224}], LineBox[{324, 225}], 
         LineBox[{325, 226}], LineBox[{326, 227}], LineBox[{327, 228}], 
         LineBox[{328, 229}], LineBox[{329, 230}], LineBox[{350, 251}], 
         LineBox[{351, 252}], LineBox[{352, 253}], LineBox[{353, 254}], 
         LineBox[{354, 255}], LineBox[{355, 256}], LineBox[{356, 257}], 
         LineBox[{357, 258}], LineBox[{358, 259}], LineBox[{359, 260}], 
         LineBox[{360, 261}], LineBox[{361, 262}], LineBox[{362, 263}], 
         LineBox[{363, 264}], LineBox[{364, 265}], LineBox[{365, 266}], 
         LineBox[{366, 267}], LineBox[{367, 268}], LineBox[{368, 269}], 
         LineBox[{369, 270}], LineBox[{370, 271}], LineBox[{371, 272}], 
         LineBox[{372, 273}], LineBox[{373, 274}], LineBox[{379, 280}], 
         LineBox[{380, 281}], LineBox[{381, 282}], LineBox[{382, 283}], 
         LineBox[{383, 284}], LineBox[{393, 294}], LineBox[{394, 295}], 
         LineBox[{395, 296}], LineBox[{396, 297}], LineBox[{397, 298}], 
         LineBox[{398, 299}], LineBox[{399, 300}]}}, {
        {RGBColor[0, 0, 1], AbsoluteThickness[1.], 
         TagBox[InsetBox["", 1],
          "InsetString"], 
         TagBox[InsetBox["", 2],
          "InsetString"], 
         TagBox[InsetBox["", 3],
          "InsetString"], 
         TagBox[InsetBox["", 4],
          "InsetString"], 
         TagBox[InsetBox["", 5],
          "InsetString"], 
         TagBox[InsetBox["", 6],
          "InsetString"], 
         TagBox[InsetBox["", 7],
          "InsetString"], 
         TagBox[InsetBox["", 8],
          "InsetString"], 
         TagBox[InsetBox["", 9],
          "InsetString"], 
         TagBox[InsetBox["", 10],
          "InsetString"], 
         TagBox[InsetBox["", 11],
          "InsetString"], 
         TagBox[InsetBox["", 12],
          "InsetString"], 
         TagBox[InsetBox["", 13],
          "InsetString"], 
         TagBox[InsetBox["", 14],
          "InsetString"], 
         TagBox[InsetBox["", 15],
          "InsetString"], 
         TagBox[InsetBox["", 16],
          "InsetString"], 
         TagBox[InsetBox["", 17],
          "InsetString"], 
         TagBox[InsetBox["", 18],
          "InsetString"], 
         TagBox[InsetBox["", 19],
          "InsetString"], 
         TagBox[InsetBox["", 20],
          "InsetString"], 
         TagBox[InsetBox["", 21],
          "InsetString"], 
         TagBox[InsetBox["", 22],
          "InsetString"], 
         TagBox[InsetBox["", 23],
          "InsetString"], 
         TagBox[InsetBox["", 24],
          "InsetString"], 
         TagBox[InsetBox["", 25],
          "InsetString"], 
         TagBox[InsetBox["", 26],
          "InsetString"], 
         TagBox[InsetBox["", 27],
          "InsetString"], 
         TagBox[InsetBox["", 28],
          "InsetString"], 
         TagBox[InsetBox["", 29],
          "InsetString"], 
         TagBox[InsetBox["", 30],
          "InsetString"], 
         TagBox[InsetBox["", 31],
          "InsetString"], 
         TagBox[InsetBox["", 32],
          "InsetString"], 
         TagBox[InsetBox["", 33],
          "InsetString"], 
         TagBox[InsetBox["", 34],
          "InsetString"], 
         TagBox[InsetBox["", 35],
          "InsetString"], 
         TagBox[InsetBox["", 36],
          "InsetString"], 
         TagBox[InsetBox["", 37],
          "InsetString"], 
         TagBox[InsetBox["", 38],
          "InsetString"], 
         TagBox[InsetBox["", 39],
          "InsetString"], 
         TagBox[InsetBox["", 40],
          "InsetString"], 
         TagBox[InsetBox["", 41],
          "InsetString"], 
         TagBox[InsetBox["", 42],
          "InsetString"], 
         TagBox[InsetBox["", 43],
          "InsetString"], 
         TagBox[InsetBox["", 44],
          "InsetString"], 
         TagBox[InsetBox["", 45],
          "InsetString"], 
         TagBox[InsetBox["", 46],
          "InsetString"], 
         TagBox[InsetBox["", 47],
          "InsetString"], 
         TagBox[InsetBox["", 48],
          "InsetString"], 
         TagBox[InsetBox["", 49],
          "InsetString"], 
         TagBox[InsetBox["", 50],
          "InsetString"], 
         TagBox[InsetBox["", 51],
          "InsetString"], 
         TagBox[InsetBox["", 52],
          "InsetString"], 
         TagBox[InsetBox["", 53],
          "InsetString"], 
         TagBox[InsetBox["", 54],
          "InsetString"], 
         TagBox[InsetBox["", 55],
          "InsetString"], 
         TagBox[InsetBox["", 56],
          "InsetString"], 
         TagBox[InsetBox["", 57],
          "InsetString"], 
         TagBox[InsetBox["", 58],
          "InsetString"], 
         TagBox[InsetBox["", 59],
          "InsetString"], 
         TagBox[InsetBox["", 60],
          "InsetString"], 
         TagBox[InsetBox["", 61],
          "InsetString"], 
         TagBox[InsetBox["", 62],
          "InsetString"], 
         TagBox[InsetBox["", 63],
          "InsetString"], 
         TagBox[InsetBox["", 64],
          "InsetString"], 
         TagBox[InsetBox["", 65],
          "InsetString"], 
         TagBox[InsetBox["", 66],
          "InsetString"], 
         TagBox[InsetBox["", 67],
          "InsetString"], 
         TagBox[InsetBox["", 68],
          "InsetString"], 
         TagBox[InsetBox["", 69],
          "InsetString"], 
         TagBox[InsetBox["", 70],
          "InsetString"], 
         TagBox[InsetBox["", 71],
          "InsetString"], 
         TagBox[InsetBox["", 72],
          "InsetString"], 
         TagBox[InsetBox["", 73],
          "InsetString"], 
         TagBox[InsetBox["", 74],
          "InsetString"], 
         TagBox[InsetBox["", 75],
          "InsetString"], 
         TagBox[InsetBox["", 76],
          "InsetString"], 
         TagBox[InsetBox["", 77],
          "InsetString"], 
         TagBox[InsetBox["", 78],
          "InsetString"], 
         TagBox[InsetBox["", 79],
          "InsetString"], 
         TagBox[InsetBox["", 80],
          "InsetString"], 
         TagBox[InsetBox["", 81],
          "InsetString"], 
         TagBox[InsetBox["", 82],
          "InsetString"], 
         TagBox[InsetBox["", 83],
          "InsetString"], 
         TagBox[InsetBox["", 84],
          "InsetString"], 
         TagBox[InsetBox["", 85],
          "InsetString"], 
         TagBox[InsetBox["", 86],
          "InsetString"], 
         TagBox[InsetBox["", 87],
          "InsetString"], 
         TagBox[InsetBox["", 88],
          "InsetString"], 
         TagBox[InsetBox["", 89],
          "InsetString"], 
         TagBox[InsetBox["", 90],
          "InsetString"], 
         TagBox[InsetBox["", 91],
          "InsetString"], 
         TagBox[InsetBox["", 92],
          "InsetString"], 
         TagBox[InsetBox["", 93],
          "InsetString"], 
         TagBox[InsetBox["", 94],
          "InsetString"], 
         TagBox[InsetBox["", 95],
          "InsetString"], 
         TagBox[InsetBox["", 96],
          "InsetString"], 
         TagBox[InsetBox["", 97],
          "InsetString"], 
         TagBox[InsetBox["", 98],
          "InsetString"], 
         TagBox[InsetBox["", 99],
          "InsetString"], 
         TagBox[InsetBox["", 100],
          "InsetString"]}, 
        {RGBColor[1, 0, 0], AbsoluteThickness[1.], 
         TagBox[InsetBox["", 101],
          "InsetString"], 
         TagBox[InsetBox["", 102],
          "InsetString"], 
         TagBox[InsetBox["", 103],
          "InsetString"], 
         TagBox[InsetBox["", 104],
          "InsetString"], 
         TagBox[InsetBox["", 105],
          "InsetString"], 
         TagBox[InsetBox["", 106],
          "InsetString"], 
         TagBox[InsetBox["", 107],
          "InsetString"], 
         TagBox[InsetBox["", 108],
          "InsetString"], 
         TagBox[InsetBox["", 109],
          "InsetString"], 
         TagBox[InsetBox["", 110],
          "InsetString"], 
         TagBox[InsetBox["", 111],
          "InsetString"], 
         TagBox[InsetBox["", 112],
          "InsetString"], 
         TagBox[InsetBox["", 113],
          "InsetString"], 
         TagBox[InsetBox["", 114],
          "InsetString"], 
         TagBox[InsetBox["", 115],
          "InsetString"], 
         TagBox[InsetBox["", 116],
          "InsetString"], 
         TagBox[InsetBox["", 117],
          "InsetString"], 
         TagBox[InsetBox["", 118],
          "InsetString"], 
         TagBox[InsetBox["", 119],
          "InsetString"], 
         TagBox[InsetBox["", 120],
          "InsetString"], 
         TagBox[InsetBox["", 121],
          "InsetString"], 
         TagBox[InsetBox["", 122],
          "InsetString"], 
         TagBox[InsetBox["", 123],
          "InsetString"], 
         TagBox[InsetBox["", 124],
          "InsetString"], 
         TagBox[InsetBox["", 125],
          "InsetString"], 
         TagBox[InsetBox["", 126],
          "InsetString"], 
         TagBox[InsetBox["", 127],
          "InsetString"], 
         TagBox[InsetBox["", 128],
          "InsetString"], 
         TagBox[InsetBox["", 129],
          "InsetString"], 
         TagBox[InsetBox["", 130],
          "InsetString"], 
         TagBox[InsetBox["", 131],
          "InsetString"], 
         TagBox[InsetBox["", 132],
          "InsetString"], 
         TagBox[InsetBox["", 133],
          "InsetString"], 
         TagBox[InsetBox["", 134],
          "InsetString"], 
         TagBox[InsetBox["", 135],
          "InsetString"], 
         TagBox[InsetBox["", 136],
          "InsetString"], 
         TagBox[InsetBox["", 137],
          "InsetString"], 
         TagBox[InsetBox["", 138],
          "InsetString"], 
         TagBox[InsetBox["", 139],
          "InsetString"], 
         TagBox[InsetBox["", 140],
          "InsetString"], 
         TagBox[InsetBox["", 141],
          "InsetString"], 
         TagBox[InsetBox["", 142],
          "InsetString"], 
         TagBox[InsetBox["", 143],
          "InsetString"], 
         TagBox[InsetBox["", 144],
          "InsetString"], 
         TagBox[InsetBox["", 145],
          "InsetString"], 
         TagBox[InsetBox["", 146],
          "InsetString"], 
         TagBox[InsetBox["", 147],
          "InsetString"], 
         TagBox[InsetBox["", 148],
          "InsetString"], 
         TagBox[InsetBox["", 149],
          "InsetString"], 
         TagBox[InsetBox["", 150],
          "InsetString"], 
         TagBox[InsetBox["", 151],
          "InsetString"], 
         TagBox[InsetBox["", 152],
          "InsetString"], 
         TagBox[InsetBox["", 153],
          "InsetString"], 
         TagBox[InsetBox["", 154],
          "InsetString"], 
         TagBox[InsetBox["", 155],
          "InsetString"], 
         TagBox[InsetBox["", 156],
          "InsetString"], 
         TagBox[InsetBox["", 157],
          "InsetString"], 
         TagBox[InsetBox["", 158],
          "InsetString"], 
         TagBox[InsetBox["", 159],
          "InsetString"], 
         TagBox[InsetBox["", 160],
          "InsetString"], 
         TagBox[InsetBox["", 161],
          "InsetString"], 
         TagBox[InsetBox["", 162],
          "InsetString"], 
         TagBox[InsetBox["", 163],
          "InsetString"], 
         TagBox[InsetBox["", 164],
          "InsetString"], 
         TagBox[InsetBox["", 165],
          "InsetString"], 
         TagBox[InsetBox["", 166],
          "InsetString"], 
         TagBox[InsetBox["", 167],
          "InsetString"], 
         TagBox[InsetBox["", 168],
          "InsetString"], 
         TagBox[InsetBox["", 169],
          "InsetString"], 
         TagBox[InsetBox["", 170],
          "InsetString"], 
         TagBox[InsetBox["", 171],
          "InsetString"], 
         TagBox[InsetBox["", 172],
          "InsetString"], 
         TagBox[InsetBox["", 173],
          "InsetString"], 
         TagBox[InsetBox["", 174],
          "InsetString"], 
         TagBox[InsetBox["", 175],
          "InsetString"], 
         TagBox[InsetBox["", 176],
          "InsetString"], 
         TagBox[InsetBox["", 177],
          "InsetString"], 
         TagBox[InsetBox["", 178],
          "InsetString"], 
         TagBox[InsetBox["", 179],
          "InsetString"], 
         TagBox[InsetBox["", 180],
          "InsetString"], 
         TagBox[InsetBox["", 181],
          "InsetString"], 
         TagBox[InsetBox["", 182],
          "InsetString"], 
         TagBox[InsetBox["", 183],
          "InsetString"], 
         TagBox[InsetBox["", 184],
          "InsetString"], 
         TagBox[InsetBox["", 185],
          "InsetString"], 
         TagBox[InsetBox["", 186],
          "InsetString"], 
         TagBox[InsetBox["", 187],
          "InsetString"], 
         TagBox[InsetBox["", 188],
          "InsetString"], 
         TagBox[InsetBox["", 189],
          "InsetString"], 
         TagBox[InsetBox["", 190],
          "InsetString"], 
         TagBox[InsetBox["", 191],
          "InsetString"], 
         TagBox[InsetBox["", 192],
          "InsetString"], 
         TagBox[InsetBox["", 193],
          "InsetString"], 
         TagBox[InsetBox["", 194],
          "InsetString"], 
         TagBox[InsetBox["", 195],
          "InsetString"], 
         TagBox[InsetBox["", 196],
          "InsetString"], 
         TagBox[InsetBox["", 197],
          "InsetString"], 
         TagBox[InsetBox["", 198],
          "InsetString"], 
         TagBox[InsetBox["", 199],
          "InsetString"], 
         TagBox[InsetBox["", 200],
          "InsetString"]}, 
        {GrayLevel[0], 
         TagBox[InsetBox["", 400],
          "InsetString"], 
         TagBox[InsetBox["", 401],
          "InsetString"], 
         TagBox[InsetBox["", 402],
          "InsetString"], 
         TagBox[InsetBox["", 403],
          "InsetString"], 
         TagBox[InsetBox["", 404],
          "InsetString"], 
         TagBox[InsetBox["", 405],
          "InsetString"], 
         TagBox[InsetBox["", 406],
          "InsetString"], 
         TagBox[InsetBox["", 407],
          "InsetString"], 
         TagBox[InsetBox["", 408],
          "InsetString"], 
         TagBox[InsetBox["", 409],
          "InsetString"], 
         TagBox[InsetBox["", 410],
          "InsetString"], 
         TagBox[InsetBox["", 411],
          "InsetString"], 
         TagBox[InsetBox["", 412],
          "InsetString"], 
         TagBox[InsetBox["", 413],
          "InsetString"], 
         TagBox[InsetBox["", 414],
          "InsetString"], 
         TagBox[InsetBox["", 415],
          "InsetString"], 
         TagBox[InsetBox["", 416],
          "InsetString"], 
         TagBox[InsetBox["", 417],
          "InsetString"], 
         TagBox[InsetBox["", 418],
          "InsetString"], 
         TagBox[InsetBox["", 419],
          "InsetString"], 
         TagBox[InsetBox["", 420],
          "InsetString"], 
         TagBox[InsetBox["", 421],
          "InsetString"], 
         TagBox[InsetBox["", 422],
          "InsetString"], 
         TagBox[InsetBox["", 423],
          "InsetString"], 
         TagBox[InsetBox["", 424],
          "InsetString"], 
         TagBox[InsetBox["", 425],
          "InsetString"], 
         TagBox[InsetBox["", 426],
          "InsetString"], 
         TagBox[InsetBox["", 427],
          "InsetString"], 
         TagBox[InsetBox["", 428],
          "InsetString"], 
         TagBox[InsetBox["", 429],
          "InsetString"], 
         TagBox[InsetBox["", 430],
          "InsetString"], 
         TagBox[InsetBox["", 431],
          "InsetString"], 
         TagBox[InsetBox["", 432],
          "InsetString"], 
         TagBox[InsetBox["", 433],
          "InsetString"], 
         TagBox[InsetBox["", 434],
          "InsetString"], 
         TagBox[InsetBox["", 435],
          "InsetString"], 
         TagBox[InsetBox["", 436],
          "InsetString"], 
         TagBox[InsetBox["", 437],
          "InsetString"], 
         TagBox[InsetBox["", 438],
          "InsetString"], 
         TagBox[InsetBox["", 439],
          "InsetString"], 
         TagBox[InsetBox["", 440],
          "InsetString"], 
         TagBox[InsetBox["", 441],
          "InsetString"], 
         TagBox[InsetBox["", 442],
          "InsetString"], 
         TagBox[InsetBox["", 443],
          "InsetString"], 
         TagBox[InsetBox["", 444],
          "InsetString"], 
         TagBox[InsetBox["", 445],
          "InsetString"], 
         TagBox[InsetBox["", 446],
          "InsetString"], 
         TagBox[InsetBox["", 447],
          "InsetString"], 
         TagBox[InsetBox["", 448],
          "InsetString"], 
         TagBox[InsetBox["", 449],
          "InsetString"], 
         TagBox[InsetBox["", 450],
          "InsetString"], 
         TagBox[InsetBox["", 451],
          "InsetString"], 
         TagBox[InsetBox["", 452],
          "InsetString"], 
         TagBox[InsetBox["", 453],
          "InsetString"], 
         TagBox[InsetBox["", 454],
          "InsetString"], 
         TagBox[InsetBox["", 455],
          "InsetString"], 
         TagBox[InsetBox["", 456],
          "InsetString"], 
         TagBox[InsetBox["", 457],
          "InsetString"], 
         TagBox[InsetBox["", 458],
          "InsetString"], 
         TagBox[InsetBox["", 459],
          "InsetString"], 
         TagBox[InsetBox["", 460],
          "InsetString"], 
         TagBox[InsetBox["", 461],
          "InsetString"], 
         TagBox[InsetBox["", 462],
          "InsetString"], 
         TagBox[InsetBox["", 463],
          "InsetString"], 
         TagBox[InsetBox["", 464],
          "InsetString"], 
         TagBox[InsetBox["", 465],
          "InsetString"], 
         TagBox[InsetBox["", 466],
          "InsetString"], 
         TagBox[InsetBox["", 467],
          "InsetString"], 
         TagBox[InsetBox["", 468],
          "InsetString"], 
         TagBox[InsetBox["", 469],
          "InsetString"], 
         TagBox[InsetBox["", 470],
          "InsetString"], 
         TagBox[InsetBox["", 471],
          "InsetString"], 
         TagBox[InsetBox["", 472],
          "InsetString"], 
         TagBox[InsetBox["", 473],
          "InsetString"], 
         TagBox[InsetBox["", 474],
          "InsetString"], 
         TagBox[InsetBox["", 475],
          "InsetString"], 
         TagBox[InsetBox["", 476],
          "InsetString"], 
         TagBox[InsetBox["", 477],
          "InsetString"], 
         TagBox[InsetBox["", 478],
          "InsetString"], 
         TagBox[InsetBox["", 479],
          "InsetString"], 
         TagBox[InsetBox["", 480],
          "InsetString"], 
         TagBox[InsetBox["", 481],
          "InsetString"], 
         TagBox[InsetBox["", 482],
          "InsetString"], 
         TagBox[InsetBox["", 483],
          "InsetString"], 
         TagBox[InsetBox["", 484],
          "InsetString"], 
         TagBox[InsetBox["", 485],
          "InsetString"], 
         TagBox[InsetBox["", 486],
          "InsetString"], 
         TagBox[InsetBox["", 487],
          "InsetString"], 
         TagBox[InsetBox["", 488],
          "InsetString"], 
         TagBox[InsetBox["", 489],
          "InsetString"], 
         TagBox[InsetBox["", 490],
          "InsetString"], 
         TagBox[InsetBox["", 491],
          "InsetString"], 
         TagBox[InsetBox["", 492],
          "InsetString"], 
         TagBox[InsetBox["", 493],
          "InsetString"], 
         TagBox[InsetBox["", 494],
          "InsetString"], 
         TagBox[InsetBox["", 495],
          "InsetString"], 
         TagBox[InsetBox["", 496],
          "InsetString"], 
         TagBox[InsetBox["", 497],
          "InsetString"], 
         TagBox[InsetBox["", 498],
          "InsetString"], 
         TagBox[InsetBox["", 499],
          "InsetString"]}, {}}}],
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     Axes->True,
     AxesOrigin->NCache[{3450124800, 0}, {3.4501248*^9, 0}],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"MACD\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{None, None}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, None}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, None}],
     GridLinesStyle->Directive[
       GrayLevel[0.85], 
       Dashing[{Small, Small}], 
       AbsoluteThickness[0.5]],
     ImagePadding->{{50, 10}, {1, 10}},
     ImageSize->600,
     PlotRange->{{3.4501248*^9, 3.4625664*^9}, {-3.5095191153202068`, 
      11.316673137661269`}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 1},
     Ticks->{{{
         NCache[3450124800, 3.4501248*^9], 
         FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3452803200, 3.4528032*^9], 
         FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3455395200, 3.4553952*^9], 
         FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3458073600, 3.4580736*^9], 
         FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3460752000, 3.460752*^9], 
         FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3451334400, 3.4513344*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3454012800, 3.4540128*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3456604800, 3.4566048*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3459283200, 3.4592832*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3461961600, 3.4619616*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, 
       Automatic}], ImageScaled[{0.5, 0.3237}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[{{}, {}, 
      {RGBColor[0.6, 0.24, 0.4429], AbsoluteThickness[1.5], 
       LineBox[CompressedData["
1:eJwl1A1Qi3EcB/AVF67JrluimhtHL3OYUyLJ/odqqUzDVeQWZ8zbJseQGC7n
3aM6pVYtR9E5JW/npVqxWi+nRRmONSl6zuWGWs68/Z7fs7vndp97fs/v/3u+
//89Uzeqkza7czicOLiYf/hRox/T5KNxuj4xQi4BG2eDd/dYlRN2JDF2JIPT
eTdS1mSuYiw8Dg5NOK0eq5ExrnZ7QpOuhhW/tkckMLbPBM+K/PBl4qd4xry1
YOP4f1NV2WiJDvzE5s+TuqE1leCMrnpvozCRMSekliZb9S7DyJ1YxmI52Hmo
IyjUF63IAvO9eZ/5R6WMqQrwt2nlAeutK3D+TrB3cdzf1AbsJ5PV0aToSkHP
94Vo3UFw5dVpO0N4OG/1VbA5MP/2sV583v4c7B8pHaW9G8PYoK2nScstuUkV
iOtbysB+I1Qd37EU520Hf1tgcFVJo3FeJ3hPuLPGFIDzKYRGmuQ0m0tH74zD
PEvA4Sq73CDH9YUt4C8mbkqAGi37AfabLGj6k4756gQNNMkit/YWvWHzjgGv
5s/essXF5tkEls5VhomG2Dwd4HZq15gcAdrg10iT+SXaMttYtGUZuDamS3PI
xOatBj8MWrfhwWHsTw2CF0lnZLh80EbfpzTxeXFClDoZ6x0EfHlgXN7icuwn
3AGe4GXdUCBeifNfYuq1YY3x7H7a+c9o4rI6LXmh2I+3hPHB/LpP7PtJVGDS
ve+E+2V8XpML7jkgKiyNRBtqwSOd/YOe2VgvjjTRJDr1564Dm9EKJbi+OzlK
vw3rKQo8ENv369RGdv5H4ADBmOGUPrSjD5z3Oy6zlbD5bmqiyZtgz6yXV3C/
qs+BL3BzF0ib0fYHYFnbFHlZFJrXCy5Y1+tSXsf3k3CbaWI7nG7uGMD1LafB
c6JslapN2J9zD/zovbp4+X12/h7wsui03BAz1ivGmWkSzz3/56YV+1HzwDku
S9XJuWhhDdg6/KrqTCXmLXsHjtgvp7yGcB6dRwvkk1OTEdWM57NaDA5bmz+4
6hqeN3sqeE6roO2tP3seRrXC/nVnlipe4n3LLDDXQ9x5th/vc5LBk0qUR/cs
wfnEx8DFF/uHigxoI6eNJvphfqGXlM1TBE7amm37eh0tXAOODzx7xsMHvx+y
I2D30Hevx2vRuhtgje5nWms6mhfcTpOKmV4piRz8/kiSwE5jvsezcLnkP+Ql
2Xc=
        "]]}},
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     Axes->True,
     AxesOrigin->NCache[{3450124800, 50}, {3.4501248*^9, 50}],
     AxesStyle->GrayLevel[0.5],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"RSI\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{{{20, 
          FormBox["20", TraditionalForm]}, {50, 
          FormBox["50", TraditionalForm]}, {80, 
          FormBox["80", TraditionalForm]}}, None}, {None, None}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 9, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, {20, 80}}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, {20, 80}}],
     GridLinesStyle->{
       Directive[
        GrayLevel[0.85], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]], 
       Directive[
        GrayLevel[0.45], 
        Dashing[{Small, Small}], 
        AbsoluteThickness[0.5]]},
     ImagePadding->{{50, 10}, {1, 1}},
     ImageSize->600,
     PlotRange->
      NCache[{{3450124800, 3462566400}, {0, 100}}, {{3.4501248*^9, 
        3.4625664*^9}, {0, 100}}],
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 0},
     Ticks->{{{
         NCache[3450124800, 3.4501248*^9], 
         FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3452803200, 3.4528032*^9], 
         FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3455395200, 3.4553952*^9], 
         FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3458073600, 3.4580736*^9], 
         FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3460752000, 3.460752*^9], 
         FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
         NCache[3451334400, 3.4513344*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3454012800, 3.4540128*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3456604800, 3.4566048*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3459283200, 3.4592832*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
         NCache[3461961600, 3.4619616*^9], 
         FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, 
       Automatic}], ImageScaled[{0.5, 0.203}], Automatic, Scaled[{1, 1}]], 
   InsetBox[
    GraphicsBox[{{}, {}, 
      {RGBColor[0.24, 0.6, 0.34], AbsoluteThickness[1.5], 
       LineBox[CompressedData["
1:eJwl0g0w1GkcB/CN0huD1LkpHGKOXN0Kl7DZp1RSab2eiTtLKC+XRceS2lWn
Y5VWWMYc411eKoks8rJHkQ3b3TnV2bOH4m/uzqqrabjL/f4/O7Oz85nf8zz/
7/+7j1lorHe4BoPBCIMv/QsfdkEzRXQM12SVSyxlYMWWFopsMMnWb/iSTXAB
WPRn/3u+1RHazCjwgwoHX/G9/bS5eeDn0gOZejIH2uIu8JV/5jbzE47TNmXd
p0jlyOBEvKMXbc5pcPZhVkN92wHawhywOo/Ba/vZjHZjB1hhI/nh8iFNOo/q
JXjWtEpZ2OBKmxfWSpF/R7cVeih30etLs8E/Be0w2K3vSlshBXPdw8c+UBaY
fwrsctbB65L2Vno/U0dKEU8PjRPjM/a01S3gRO7NwP0xX2BeFTj17Bm5s85n
9JyzoY0iWmZ3E6M3WmNee3Cle4PSJMmDtt49cOiNcjvD19602UpwRP2+3nLb
ANq8te0U2d2kuiXICsG8tmAfZVLBRFoE5g0EtzmsNhJZnsE+X4AbfMyLhryi
sM/VHRSZqNaWexdG0u7ZBf64zs9G9RatDgD38Yos2/vDMf9lsINCKpTz8PzG
VQ8oottdE/smN4y2ygZcse0gY+lVMOb3BxeAV/l/jfmF4AGRpKLujT/mrwPb
lszVd+eGYp/WnRRxBUv2cPE++IA7Pcc3r9ZDcy+A43pjZOb9eL64BjxzPIe/
mIH5ep6CS1jXCvQDMT+H0wX37eCqpYcb8f2FKeBBTT5vYHglfyW46sj1Iadj
pzD/MNg4PS5oTzA+rzSpmyLrWozviJtwrigDm1TFZyZ8HoR5n4CnXYxqRpu/
wrzv6PUFR9eO8vF9uaY9cH9Ew1tCPuB56hLwmff3f5cFYB7Tx+B66UZvooX/
B+cNeKZF2LXA5mFeYxlF9o2kWY58koB5D4ONtdSLJRNx2Ocj8JiObqsbLx77
VINJaFr+b0U4L936I0XmOpOjHfbiXOEGTq3cmaoxi+cxYsFv3QUfJd3+Fvv8
C7yTF5m+rJ2EfRr2UuTVHZ500A+tJuB3BhmeQ058zB8DtkorE2m64ZwjAU+f
v/nrf8pz2OfmPoq4BU95zjMS8T64gi3vb4p5qov72ZFgZvHi7Uke7uflgv3q
05pyJLi+tBN866W5yfZkNNPlIUWK53PPrRtDcyPA3xX6HntF0GIxeLIxZ2o2
Ct3TDl52DEn+Q4pWT4OZZSFBvs1o4alHFEnf8Kn1vB2+f+M1cOfrU9ZXv0Gr
WsFVfVybpamV/JPgMEf39mp+MubX7qdIXnH+dn/meexXBB77u3b2/elU7LcF
PKBxRck/hGZOgIPmlpcSrHE9d/0ARS6yFgotnqHFduD5aHFzfH0K9tsEvsFK
GAw7ifs54+BeC70SeSTOhVqPKfLsufN89QL22cgEZ1pZ2WuMYD7VSfD6TcPy
1Lu4vlRzkCILps5ZJ2JwrtgJfmEUrMdywjkjYMXi9VUreS+B3YZNJemtF7BP
hpwiBmxWcV/tRexzB7iSI1gjVAswrx94b3WG9r63QswrALdXfe8R2YEW1oJT
jur6eeXjfj2rJxSJjrO/brOM+9neYEH51X7ZLxfJ//oAjBI=
        "]]}},
     AspectRatio->NCache[Rational[1, 4]/GoldenRatio, 0.1545084971874737],
     AxesOrigin->NCache[{3450124800, 0}, {3.4501248*^9, 0}],
     Epilog->InsetBox[
       BoxData[
        FormBox[
         PanelBox[
          TagBox[
           GridBox[{{
              StyleBox["\"ADI\"", Bold, 
               RGBColor[0.5450980392156862, 0, 0], StripOnInput -> False]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
          Appearance -> None, Background -> GrayLevel[1]], TraditionalForm]], 
       
       Offset[{0, 0}, 
        Scaled[{0.05, 0.95}]], {Left, Top}],
     Frame->True,
     FrameTicks->{{None, None}, {{{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"May\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"Jun\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"Jul\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"Aug\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"Sep\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}, {{
          NCache[3450124800, 3.4501248*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3452803200, 3.4528032*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3455395200, 3.4553952*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3458073600, 3.4580736*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3460752000, 3.460752*^9], 
          FormBox["\"\"", TraditionalForm], {0.020601132958329826`, 0}}, {
          NCache[3451334400, 3.4513344*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3454012800, 3.4540128*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3456604800, 3.4566048*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3459283200, 3.4592832*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}, {
          NCache[3461961600, 3.4619616*^9], 
          FormBox["\"\"", TraditionalForm], {0.012360679774997897`, 0}}}}},
     FrameTicksStyle->Directive[
       GrayLevel[0], FontFamily -> "Helvetica", 11, Plain],
     GridLines->NCache[{{{3450124800, 
          GrayLevel[0.8]}, {3452803200, 
          GrayLevel[0.8]}, {3455395200, 
          GrayLevel[0.8]}, {3458073600, 
          GrayLevel[0.8]}, {3460752000, 
          GrayLevel[0.8]}}, None}, {{{3.4501248*^9, 
          GrayLevel[0.8]}, {3.4528032*^9, 
          GrayLevel[0.8]}, {3.4553952*^9, 
          GrayLevel[0.8]}, {3.4580736*^9, 
          GrayLevel[0.8]}, {3.460752*^9, 
          GrayLevel[0.8]}}, None}],
     GridLinesStyle->Directive[
       GrayLevel[0.85], 
       Dashing[{Small, Small}], 
       AbsoluteThickness[0.5]],
     ImagePadding->{{50, 10}, {15, 1}},
     ImageSize->600,
     PlotRange->{{3.4501248*^9, 3.4625664*^9}, {-2.0758008658009116`*^6, 
      3.5563457566723295`*^7}},
     PlotRangeClipping->True,
     PlotRangePadding->{200000, 0},
     Ticks->None], ImageScaled[{0.5, 0.075}], Automatic, Scaled[{1, 1}]]},
  ImageSize->600,
  PlotRange->{{0, 1}, {0, 1.25}}]], "Output",
 CellChangeTimes->{
  3.4627249049048843`*^9, {3.46272493569687*^9, 3.462724987274235*^9}, {
   3.46272902211852*^9, 3.46272903447086*^9}},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{911, 971},
WindowMargins->{{89, Automatic}, {Automatic, 0}},
DockedCells->{},
ShowSelection->True,
TaggingRules->{
 "DocuToolsSettings" -> {"$PaletteMode" -> "Internal"}, 
  "SearchTextTranslated" -> ""},
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (February 18, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3462710504-4166664"->{
  Cell[2794, 95, 447, 7, 90, "Print",
   CellTags->"Info3462710504-4166664"]},
 "Info3462710531-6531378"->{
  Cell[4170, 133, 347, 6, 70, "Print",
   CellTags->"Info3462710531-6531378"]},
 "Info3462710570-5772976"->{
  Cell[11470, 304, 435, 7, 90, "Print",
   CellTags->"Info3462710570-5772976"]},
 "Info3462710584-6153215"->{
  Cell[81731, 2122, 424, 7, 90, "Print",
   CellTags->"Info3462710584-6153215"]},
 "Info3462710591-9739051"->{
  Cell[119534, 3254, 293, 5, 70, "Print",
   CellTags->"Info3462710591-9739051"]},
 "Info3462710601-5758356"->{
  Cell[149433, 4053, 298, 5, 70, "Print",
   CellTags->"Info3462710601-5758356"]},
 "Info3462710636-9579012"->{
  Cell[217004, 6005, 468, 7, 90, "Print",
   CellTags->"Info3462710636-9579012"]},
 "Info3462710642-5540596"->{
  Cell[224554, 6175, 354, 6, 70, "Print",
   CellTags->"Info3462710642-5540596"]},
 "Info3462710678-2388317"->{
  Cell[233280, 6367, 550, 8, 110, "Print",
   CellTags->"Info3462710678-2388317"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3462710504-4166664", 966572, 23628},
 {"Info3462710531-6531378", 966680, 23631},
 {"Info3462710570-5772976", 966789, 23634},
 {"Info3462710584-6153215", 966899, 23637},
 {"Info3462710591-9739051", 967010, 23640},
 {"Info3462710601-5758356", 967122, 23643},
 {"Info3462710636-9579012", 967234, 23646},
 {"Info3462710642-5540596", 967346, 23649},
 {"Info3462710678-2388317", 967458, 23652}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 36, 0, 95, "Title"],
Cell[CellGroupData[{
Cell[628, 26, 99, 1, 83, "Section"],
Cell[730, 29, 1206, 34, 126, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1973, 68, 121, 2, 83, "Section",
 CellID->2051879694],
Cell[2097, 72, 188, 4, 33, "Input",
 InitializationCell->True,
 CellID->1649613532]
}, Open  ]],
Cell[CellGroupData[{
Cell[2322, 81, 152, 2, 83, "Section"],
Cell[2477, 85, 114, 1, 32, "Text"],
Cell[CellGroupData[{
Cell[2616, 90, 175, 3, 33, "Input"],
Cell[2794, 95, 447, 7, 90, "Print",
 CellTags->"Info3462710504-4166664"]
}, Open  ]],
Cell[3256, 105, 422, 11, 53, "Input"],
Cell[3681, 118, 198, 5, 32, "Text"],
Cell[CellGroupData[{
Cell[3904, 127, 263, 4, 33, "Input"],
Cell[4170, 133, 347, 6, 70, "Print",
 CellTags->"Info3462710531-6531378"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4554, 144, 221, 5, 33, "Input"],
Cell[4778, 151, 6533, 144, 463, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11348, 300, 119, 2, 33, "Input"],
Cell[11470, 304, 435, 7, 90, "Print",
 CellTags->"Info3462710570-5772976"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11942, 316, 277, 7, 33, "Input"],
Cell[12222, 325, 69345, 1788, 463, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81604, 2118, 124, 2, 33, "Input"],
Cell[81731, 2122, 424, 7, 90, "Print",
 CellTags->"Info3462710584-6153215"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82192, 2134, 304, 8, 33, "Input"],
Cell[82499, 2144, 36876, 1101, 463, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[119412, 3250, 119, 2, 33, "Input"],
Cell[119534, 3254, 293, 5, 70, "Print",
 CellTags->"Info3462710591-9739051"]
}, Open  ]],
Cell[CellGroupData[{
Cell[119864, 3264, 161, 3, 33, "Input"],
Cell[120028, 3269, 28762, 763, 152, "Output"]
}, Open  ]],
Cell[148805, 4035, 483, 10, 32, "Text"],
Cell[CellGroupData[{
Cell[149313, 4049, 117, 2, 33, "Input"],
Cell[149433, 4053, 298, 5, 70, "Print",
 CellTags->"Info3462710601-5758356"]
}, Open  ]],
Cell[CellGroupData[{
Cell[149768, 4063, 106, 2, 33, "Input"],
Cell[149877, 4067, 66945, 1928, 136, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216859, 6000, 142, 3, 33, "Input"],
Cell[217004, 6005, 468, 7, 90, "Print",
 CellTags->"Info3462710636-9579012"]
}, Open  ]],
Cell[CellGroupData[{
Cell[217509, 6017, 263, 6, 33, "Input"],
Cell[217775, 6025, 6626, 141, 125, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[224438, 6171, 113, 2, 33, "Input"],
Cell[224554, 6175, 354, 6, 70, "Print",
 CellTags->"Info3462710642-5540596"]
}, Open  ]],
Cell[CellGroupData[{
Cell[224945, 6186, 108, 2, 33, "Input"],
Cell[225056, 6190, 7878, 162, 142, "Output"]
}, Open  ]],
Cell[232949, 6355, 183, 4, 32, "Text"],
Cell[CellGroupData[{
Cell[233157, 6363, 120, 2, 33, "Input"],
Cell[233280, 6367, 550, 8, 110, "Print",
 CellTags->"Info3462710678-2388317"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233867, 6380, 195, 5, 33, "Input"],
Cell[234065, 6387, 37781, 912, 573, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[271895, 7305, 92, 1, 83, "Section"],
Cell[CellGroupData[{
Cell[272012, 7310, 100, 1, 42, "Subsection"],
Cell[272115, 7313, 420, 13, 51, "Text"],
Cell[CellGroupData[{
Cell[272560, 7330, 968, 24, 129, "Input"],
Cell[273531, 7356, 40854, 955, 573, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[314434, 8317, 101, 1, 42, "Subsection"],
Cell[314538, 8320, 467, 13, 51, "Text"],
Cell[CellGroupData[{
Cell[315030, 8337, 1398, 38, 148, "Input"],
Cell[316431, 8377, 33348, 783, 573, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[349828, 9166, 100, 1, 42, "Subsection"],
Cell[349931, 9169, 371, 14, 32, "Text"],
Cell[CellGroupData[{
Cell[350327, 9187, 387, 10, 33, "Input"],
Cell[350717, 9199, 89611, 2147, 573, "Output"]
}, Open  ]],
Cell[440343, 11349, 354, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[440722, 11362, 366, 9, 33, "Input"],
Cell[441091, 11373, 55081, 1461, 573, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[496221, 12840, 100, 1, 42, "Subsection"],
Cell[496324, 12843, 458, 8, 51, "Text"],
Cell[CellGroupData[{
Cell[496807, 12855, 580, 14, 91, "Input"],
Cell[497390, 12871, 185416, 4109, 956, 116611, 2979, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[682855, 16986, 97, 1, 42, "Subsection"],
Cell[682955, 16989, 426, 8, 51, "Text"],
Cell[CellGroupData[{
Cell[683406, 17001, 531, 14, 91, "Input"],
Cell[683940, 17017, 74418, 2019, 573, "Output"]
}, Open  ]],
Cell[758373, 19039, 118, 1, 32, "Text"],
Cell[CellGroupData[{
Cell[758516, 19044, 913, 23, 129, "Input"],
Cell[759432, 19069, 206512, 4535, 956, 117725, 3077, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
